@@ -1,16 +1,16 @@
 
 setMethod("length", signature(x="SpatRasterDataset"),
 	function(x) {
-		x@ptr$nsds()
+		x@pntr$nsds()
 	}
 )
 
 
 setMethod("sds", signature(x="character"),
-	function(x, ids=0) {
+	function(x, ids=0, opts=NULL, raw=FALSE) {
 
 		if (length(x) > 1) {
-			r <- lapply(x, rast)
+			r <- lapply(x, rast, opts=opts, raw=raw)
 			s <- sds(r)
 			names(s) <- tools::file_path_sans_ext(basename(x))
 			return(s)
@@ -28,7 +28,9 @@ setMethod("sds", signature(x="character"),
 		} else {
 			useids <- TRUE
 		}
-		r@ptr <- SpatRasterStack$new(f, ids, useids)
+		if (is.null(opts)) opts <- ""[0]
+		if (raw) opts <- c(opts, "so=false")		
+		r@pntr <- SpatRasterStack$new(f, ids, useids, opts)
 		messages(r, "sds")
 	}
 )
@@ -36,7 +38,7 @@ setMethod("sds", signature(x="character"),
 setMethod("sds", signature(x="missing"),
 	function(x) {
 		r <- methods::new("SpatRasterDataset")
-		r@ptr <- SpatRasterStack$new()
+		r@pntr <- SpatRasterStack$new()
 		r
 	}
 )
@@ -45,8 +47,8 @@ setMethod("sds", signature(x="missing"),
 setMethod("sds", signature(x="SpatRaster"),
 	function(x, ...) {
 		r <- methods::new("SpatRasterDataset")
-		r@ptr <- SpatRasterStack$new()
-		r@ptr$add(x@ptr, varnames(x)[1], longnames(x)[1], units(x)[1], FALSE)
+		r@pntr <- SpatRasterStack$new()
+		r@pntr$add(x@pntr, varnames(x)[1], longnames(x)[1], units(x)[1], FALSE)
 		dots <- list(...)
 		nms <- names(dots)
 		if (is.null(nms)) nms = ""
@@ -55,7 +57,7 @@ setMethod("sds", signature(x="SpatRaster"),
 			if (inherits(dots[[i]], "SpatRaster")) {
 				vname <- nms[i]
 				if (vname == "") vname = varnames(dots[[i]])[1]
-				r@ptr$add(dots[[i]]@ptr, vname, longnames(dots[[i]])[1], units(dots[[i]])[1], FALSE)
+				r@pntr$add(dots[[i]]@pntr, vname, longnames(dots[[i]])[1], units(dots[[i]])[1], FALSE)
 			}
 		}
 		messages(r, "sds")
@@ -65,12 +67,20 @@ setMethod("sds", signature(x="SpatRaster"),
 setMethod("sds", signature(x="list"),
 	function(x) {
 		r <- methods::new("SpatRasterDataset")
-		r@ptr <- SpatRasterStack$new()
+		r@pntr <- SpatRasterStack$new()
 		nms <- names(x)
 		if (is.null(nms)) nms <- rep("", length(x))
 		for (i in seq_along(x)) {
 			if (inherits(x[[i]], "SpatRaster")) {
-				r@ptr$add(x[[i]]@ptr, nms[i], "", "", FALSE)
+				r@pntr$add(x[[i]]@pntr, nms[i], "", "", FALSE)
+			} else if (inherits(x[[i]], "SpatRasterDataset")) {
+				y <- as.list(x[[i]])
+				ynms <- names(x[[i]])
+				s <- sapply(y, function(j) r@pntr$add(j@pntr, ynms[j], "", "", FALSE))
+			} else {
+				name <- names(x[[i]])
+				cls <- paste(class(x[[i]]), collapse=", ")
+				error("sds", "list element", name, "is a: ", cls)
 			}
 		}
 		messages(r, "sds")
@@ -125,10 +135,11 @@ setMethod("sds", signature(x="stars_proxy"),
 )
 
 
+
 setMethod("c", signature(x="SpatRasterDataset"),
 	function(x, ...) {
 
-		x@ptr <- x@ptr$subset((1:x@ptr$nsds()) -1 ) # why? make a copy?
+		x@pntr <- x@pntr$subset((1:x@pntr$nsds()) -1 ) # why? make a copy?
 
 		dots <- list(...)
 		nms <- names(dots)
@@ -137,14 +148,14 @@ setMethod("c", signature(x="SpatRasterDataset"),
 			if (inherits(dots[[i]], "SpatRasterDataset")) {
 				sdsnms <- names(dots[[i]])
 				for (j in 1:(length(dots[[i]]))) {
-					if (!x@ptr$add(dots[[i]][[j]]@ptr, sdsnms[j], "", "", FALSE)) {
+					if (!x@pntr$add(dots[[i]][[j]]@pntr, sdsnms[j], "", "", FALSE)) {
 						messages(x, "c")
 					}
 				}
 
 			} else if (inherits(dots[[i]], "SpatRaster")) {
 				if (is.null(nms)) error("c", "arguments must be named")
-				if (!x@ptr$add(dots[[i]]@ptr, nms[i], "", "", FALSE)) {
+				if (!x@pntr$add(dots[[i]]@pntr, nms[i], "", "", FALSE)) {
 					messages(x, "c")
 				}
 			} else {
@@ -156,7 +167,7 @@ setMethod("c", signature(x="SpatRasterDataset"),
 )
 
 
-setReplaceMethod("[", c("SpatRasterDataset","numeric","missing"),
+setReplaceMethod("[", c("SpatRasterDataset", "numeric", "missing"),
 	function(x, i, j, value) {
 		if (any(!is.finite(i)) | any(i<1)) {
 			error("`[`", "invalid index")
@@ -165,9 +176,9 @@ setReplaceMethod("[", c("SpatRasterDataset","numeric","missing"),
 		i <- sort(i)
 		for (j in i) {
 			if (j == (length(x)+1)) {
-				x@ptr$add(value@ptr, "", "", "", FALSE)
+				x@pntr$add(value@pntr, "", "", "", FALSE)
 			} else {
-				x@ptr$replace(j-1, value@ptr)
+				x@pntr$replace(j-1, value@pntr, FALSE)
 			}
 		}
 		messages(x, "`[`")
@@ -180,35 +191,56 @@ function(x, i, j, drop=TRUE) {
 	i <- positive_indices(i, length(x), TRUE, "`[`(i)")
 
 	if (drop && (length(i) == 1)) {
-		ptr <- x@ptr$getsds(i-1)
+		tptr <- x@pntr$getsds(i-1)
 		x <- rast()
-		x@ptr <- ptr
+		x@pntr <- tptr
 	} else {
-		x@ptr <- x@ptr$subset(i-1)
+		x@pntr <- x@pntr$subset(i-1)
 	}
 	messages(x, "`[`")
 })
 
 setMethod("[", c("SpatRasterDataset", "numeric", "numeric"),
 function(x, i, j, drop=TRUE) {
-	y <- x[i,drop=drop]
-	if (inherits(y, "SpatRaster")) {
-		return(y[[j]])
+	i <- positive_indices(i, length(x))
+	j <- positive_indices(j, min(nlyr(x)))	
+	nd <- i
+	if (drop) {
+		out <- lapply(nd, function(k) x[k][[j]])
+		out <- rast(out)
+	} else {
+		out <- sds()
+		nms <- x@pntr$names
+		for (k in nd) {
+			r <- x[k][[j]]
+			out@pntr$add(r@pntr, nms[k], "", "", FALSE)
+		}
 	}
-	nd <- y@ptr$nsds()
-	x@ptr <- SpatRasterStack$new()
-	nms <- y@ptr$names
-	for (k in seq_along(1:nd)) {
-		r <- y[k][[j]]
-		x@ptr$add(r@ptr, nms[k], "", "", FALSE)
-	}
-	messages(x, "`[`")
+	messages(out, "`[`")
+})
+
+setMethod("[", c("SpatRasterDataset", "numeric", "logical"),
+function(x, i, j, drop=TRUE) {
+	j <- positive_indices(j, min(nlyr(x)))
+	`[`(x, i=i, j=j, drop=drop)
+})
+
+setMethod("[", c("SpatRasterDataset", "missing", "numeric"),
+function(x, i, j, drop=TRUE) {
+	`[`(x, i=1:x@pntr$nsds(), j=j, drop=drop)
+})
+
+setMethod("[", c("SpatRasterDataset", "missing", "logical"),
+function(x, i, j, drop=TRUE) {
+	j <- positive_indices(j, min(nlyr(x)))
+	`[`(x, i=1:x@pntr$nsds(), j=j, drop=drop)
 })
 
 
 setMethod("[", c("SpatRasterDataset", "logical", "missing"),
 function(x, i, j,drop=TRUE) {
-	x[which(i), drop=drop]
+	i <- positive_indices(j, length(x))
+	x[i, drop=drop]
 })
 
 setMethod("[", c("SpatRasterDataset", "character", "missing"),
@@ -237,6 +269,12 @@ function(x, i, j, drop=TRUE) {
 })
 
 
+setMethod("$", c("SpatRasterDataset"),
+function(x, name) {
+	x[name,drop=TRUE]
+})
+
+
 setMethod("$", "SpatRasterDataset",
 	function(x, name) {
 		x[name]
@@ -247,7 +285,7 @@ setMethod("$", "SpatRasterDataset",
 setMethod("sprc", signature(x="missing"),
 	function(x) {
 		r <- methods::new("SpatRasterCollection")
-		r@ptr <- SpatRasterCollection$new()
+		r@pntr <- SpatRasterCollection$new()
 		r
 	}
 )
@@ -263,26 +301,31 @@ setMethod("sprc", signature(x="SpatRaster"),
 setMethod("sprc", signature(x="list"),
 	function(x) {
 		n <- length(x)
-		ptr <- SpatRasterCollection$new()
+		tptr <- SpatRasterCollection$new()
 		if (n > 0) {
 			for (i in 1:n) {
 				if (inherits(x[[i]], "SpatRaster")) {
-					ptr$add(x[[i]]@ptr, "")
+					tptr$add(x[[i]]@pntr, "")
+				} else if (inherits(x[[i]], "SpatRasterCollection") | 
+							inherits(x[[i]], "SpatRasterDataset")) {
+					y <- as.list(x[[i]])
+					s <- sapply(y, function(j) tptr$add(j@pntr, ""))
 				} else {
 					name <- names(x[[i]])
-					cls <- class(x[[i]])
-					error("sprc", "list elements should be 'SpatRaster'\n", name, "is of class: ", cls)
+					cls <- paste(class(x[[i]]), collapse=", ")
+					error("sprc", "list element", name, "is a: ", cls)
 				}
 			}
 		}
-		x <- new("SpatRasterCollection")
-		x@ptr <- ptr
-		x
+		r <- new("SpatRasterCollection")
+		r@pntr <- tptr
+		if (length(r) == length(x)) names(r) <- names(x)
+		r
 	}
 )
 
 setMethod("sprc", signature(x="character"),
-	function(x, ids=0) {
+	function(x, ids=0, opts=NULL, raw=FALSE) {
 
 		if (length(x) > 1) {
 			r <- lapply(x, rast)
@@ -303,7 +346,10 @@ setMethod("sprc", signature(x="character"),
 		} else {
 			useids <- TRUE
 		}
-		r@ptr <- SpatRasterCollection$new(f, ids, useids)
+		if (is.null(opts)) opts <- ""[0]
+		if (raw) opts <- c(opts, "so=false")		
+		
+		r@pntr <- SpatRasterCollection$new(f, ids, useids, opts)
 		messages(r, "sprc")
 	}
 )
@@ -311,7 +357,7 @@ setMethod("sprc", signature(x="character"),
 
 setMethod("length", signature(x="SpatRasterCollection"),
 	function(x) {
-		x@ptr$length()
+		x@pntr$length()
 	}
 )
 
@@ -319,17 +365,70 @@ setMethod("[", c("SpatRasterCollection", "numeric", "missing"),
 function(x, i, j, ... ,drop=TRUE) {
 	i <- positive_indices(i, length(x), TRUE, "`[`(i)")
 	if (drop && (length(i) == 1)) {
-		ptr <- x@ptr$x[[i]]
+		tptr <- x@pntr$x[[i]]
 		x <- rast()
-		x@ptr <- ptr
+		x@pntr <- tptr
 	} else {
-		s <- x@ptr$x[i]
-		ptr <- SpatRasterCollection$new()
+		s <- x@pntr$x[i]
+		tptr <- SpatRasterCollection$new()
 		for (i in 1:length(s)) {
-			ptr$add(s[[i]], "")
+			tptr$add(s[[i]], "")
 		}
-		x@ptr <- ptr
+		x@pntr <- tptr
 	}
 	messages(x, "`[`")
 })
 
+
+setMethod("add<-", signature("SpatRasterCollection", "SpatRaster"),
+	function(x, value) {
+		x@pntr$add(value@pntr, "")
+		messages(x, "add")
+	}
+)
+
+
+setMethod("add<-", signature("SpatRasterDataset", "SpatRaster"),
+	function(x, value) {
+		x@pntr$add(value@pntr, "")
+		messages(x, "add")
+	}
+)
+
+
+setMethod("c", signature(x="SpatRasterCollection"),
+	function(x, ..., warn=TRUE) {
+		y <- list(...)
+		sprc(c(as.list(x), ...))
+	}
+)
+
+
+setMethod("c", signature(x="SpatRasterCollection"),
+	function(x, ...) {
+
+		x@pntr <- x@pntr$deepcopy()
+
+		dots <- list(...)
+		nms <- names(dots)
+		for (i in seq_along(dots)) {
+			if (inherits(dots[[i]], "SpatRasterCollection") | inherits(dots[[i]], "SpatRasterDataset")) {
+				sdsnms <- names(dots[[i]])
+				for (j in 1:(length(dots[[i]]))) {
+					x@pntr$add(dots[[i]][[j]]@pntr, sdsnms[j])
+				}
+			} else if (inherits(dots[[i]], "SpatRaster")) {
+				if (is.null(nms)) { 
+					name <- paste0("d", i)
+				} else { 
+					name <- nms[i] 
+				}
+				x@pntr$add(dots[[i]]@pntr, name)
+			} else {
+				error("c", "arguments must be SpatRaster, SpatRasterCollection, or SpatRasterDataset")
+			}
+		}
+		messages(x, "c")
+	}
+)
+
