@@ -139,23 +139,57 @@ setAs("ggmap", "SpatRaster",
 
 
 
-as.list.SpatRaster <- function(x, ...) {
-	lapply(1:nlyr(x), function(i) x[[i]])
+as.list.SpatRaster <- function(x, geom=NULL, ...) {
+	if (!is.null(geom)) {
+		e <- as.vector(ext(x))
+		d <- crs(x, describe=TRUE)
+		if (!(is.na(d$authority) || is.na(d$code))) {
+			crs <- paste0(d$authority, ":", d$code)
+		} else {
+			crs <- gsub("\n[ ]+", "", crs(x))
+		}
+		list(
+			ncols=ncol(x),
+			nrows=nrow(x),
+			nlyrs=nlyr(x),
+			xmin=e[1],
+			xmax=e[2],
+			ymin=e[3],
+			ymax=e[4],
+			xres=xres(x),
+			yres=yres(x),
+			nms = paste(names(x), collapse="', '"),
+			units = paste(units(x), collapse="', '"),
+			time = paste(time(x), collapse="', '"),
+			crs=crs
+		)
+	} else {
+		lapply(1:nlyr(x), function(i) x[[i]])
+	}
 }
 setMethod("as.list", signature(x="SpatRaster"), as.list.SpatRaster)
 
+
 as.list.SpatRasterCollection <- function(x, ...) {
-	lapply(1:length(x), function(i) x[i])
+	out <- lapply(1:length(x), function(i) x[i])
+	names(out) <- names(x)
+	out
 }
 setMethod("as.list", signature(x="SpatRasterCollection"), as.list.SpatRasterCollection)
 
+
 as.list.SpatRasterDataset <- function(x, ...) {
-	lapply(1:length(x), function(i) x[i])
+	out <- lapply(1:length(x), function(i) x[i])
+	names(out) <- names(x)
+	out
 }
 setMethod("as.list", signature(x="SpatRasterDataset"), as.list.SpatRasterDataset)
 
+
 as.list.SpatVectorCollection <- function(x, ...) {
-	lapply(1:length(x), function(i) x[i])
+	out <- lapply(1:length(x), function(i) x[i])
+	names(out) <- names(x)
+	out
 }
 setMethod("as.list", signature(x="SpatVectorCollection"), as.list.SpatVectorCollection)
 
@@ -165,8 +199,10 @@ setMethod("as.list", signature(x="SpatVectorCollection"), as.list.SpatVectorColl
 setMethod("as.raster", signature(x="SpatRaster"),
 	function(x, maxcell=500000, col) {
 		if (missing(col)) {
-			col <- rev(grDevices::terrain.colors(255))
+			#col <- rev(grDevices::terrain.colors(255))
+			col <- .default.pal()
 		}
+		
 		x <- spatSample(x, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
 		x <- as.matrix(x, wide=TRUE)
 		r <- range(x, na.rm=TRUE)
@@ -214,11 +250,11 @@ setMethod("as.polygons", signature(x="SpatRaster"),
 	
 		p <- methods::new("SpatVector")
 		if (extent) {
-			p@cpp <- x@cpp$dense_extent(FALSE, FALSE)
+			p@pntr <- x@pntr$dense_extent(FALSE, FALSE)
 			x <- messages(x, "as.polygons")
 		} else {
 			opt <- spatOptions()
-			p@cpp <- x@cpp$as_polygons(round[1], aggregate[1], values[1], na.rm[1], na.all[1], digits, opt)
+			p@pntr <- x@pntr$as_polygons(round[1], aggregate[1], values[1], na.rm[1], na.all[1], digits, opt)
 			x <- messages(x, "as.polygons")
 			if (values) {
 				p <- get_labels(x, p, aggregate[1])
@@ -232,7 +268,7 @@ setMethod("as.lines", signature(x="SpatRaster"),
 	function(x) {
 		p <- methods::new("SpatVector")
 		opt <- spatOptions()
-		p@cpp <- x@cpp$as_lines(opt)
+		p@pntr <- x@pntr$as_lines(opt)
 		messages(p, "as.lines")
 	}
 )
@@ -242,7 +278,7 @@ setMethod("as.polygons", signature(x="SpatExtent"),
 	function(x, crs="") {
 		p <- methods::new("SpatVector")
 		crs <- character_crs(crs, "as.polygons")
-		p@cpp <- SpatVector$new(x@cpp, crs)
+		p@pntr <- SpatVector$new(x@pntr, crs)
 		messages(p, "as.polygons")
 	}
 )
@@ -257,7 +293,7 @@ setMethod("as.lines", signature(x="SpatExtent"),
 
 setMethod("as.points", signature(x="SpatExtent"),
 	function(x, crs="") {
-		#vect(do.call(cbind, x@cpp$as.points()), "points", crs=crs)
+		#vect(do.call(cbind, x@pntr$as.points()), "points", crs=crs)
 		as.points(as.polygons(x, crs))
 	}
 )
@@ -265,17 +301,35 @@ setMethod("as.points", signature(x="SpatExtent"),
 
 setMethod("as.lines", signature(x="SpatVector"),
 	function(x) {
-		x@cpp <- x@cpp$as_lines()
+		x@pntr <- x@pntr$as_lines()
 		messages(x, "as.lines")
 	}
 )
 
+
+setMethod("as.lines", signature(x="matrix"),
+	function(x, crs="") {
+		p <- vect()
+		if (ncol(x) == 2) {
+			nr <- nrow(x)
+			p@pntr$setGeometry("lines", rep(1, nr), rep(1, nr), x[,1], x[,2], rep(FALSE, nr))
+			crs(p) <- crs
+		} else if (ncol(x) == 4) {
+			p@pntr$setLinesStartEnd(x, crs(crs))
+		} else {
+			error("expecting a two or four column matrix", "as.lines")
+		}
+		messages(p, "vect")
+	}
+)
+
+
 setMethod("as.polygons", signature(x="SpatVector"),
 	function(x, extent=FALSE) {
 		if (extent) {
 			as.polygons(ext(x), crs=crs(x))
 		} else {
-			x@cpp <- x@cpp$polygonize()
+			x@pntr <- x@pntr$polygonize()
 			messages(x, "as.polygons")
 		}
 	}
@@ -284,7 +338,7 @@ setMethod("as.polygons", signature(x="SpatVector"),
 setMethod("as.points", signature(x="SpatVector"),
 	function(x, multi=FALSE, skiplast=TRUE) {
 		opt <- spatOptions()
-		x@cpp <- x@cpp$as_points(multi, skiplast)
+		x@pntr <- x@pntr$as_points(multi, skiplast)
 		messages(x, "as.points")
 	}
 )
@@ -294,7 +348,7 @@ setMethod("as.points", signature(x="SpatRaster"),
 	function(x, values=TRUE, na.rm=TRUE, na.all=FALSE) {
 		p <- methods::new("SpatVector")
 		opt <- spatOptions()
-		p@cpp <- x@cpp$as_points(values, na.rm, na.all, opt)
+		p@pntr <- x@pntr$as_points(values, na.rm, na.all, opt)
 		x <- messages(x, "as.points")
 
 		if (values) {
@@ -307,7 +361,7 @@ setMethod("as.points", signature(x="SpatRaster"),
 # mode argument is ignored as mode=mode gave an error on R-devel
 setMethod("as.vector", signature(x="SpatExtent"),
 	function(x, mode="any") {
-		v <- x@cpp$vector
+		v <- x@pntr$vector
 		names(v) <- c("xmin", "xmax", "ymin", "ymax")
 		if (mode == "list") {
 			v <- as.list(v)
@@ -395,18 +449,24 @@ as.data.frame.SpatRaster <- function(x, row.names=NULL, optional=FALSE, xy=FALSE
 			idv <- NULL
 			if (xy) idv <- c("x", "y", idv)
 			if (cells) idv <- c("cell", idv)
+			add <- d[idv]
+			rownames(add) <- NULL
+			for (i in 1:length(idv)) {
+				d[[idv[i]]] <- NULL
+			}
+			d <- data.frame(add, layer=rep(names(x), each=nr), values=as.vector(as.matrix(d)))
 			nms <- names(x)
-			d <- stats::reshape(d, direction="long", idvar=idv, varying=nms, v.names="values")
-			d$time <- nms[d$time]
-			names(d)[names(d) == "time"] <- "layer"
+#			d <- stats::reshape(d, direction="long", idvar=idv, varying=nms, v.names="values")
+#			d$time <- nms[d$time]
+#			names(d)[names(d) == "time"] <- "layer"
 		}
 		rownames(d) <- NULL
 		if (time) {
-			d$time <- NULL
-			vals <- d$values
-			d$values <- NULL
+		#	d$time <- NULL
+		#	vals <- d$values
+		#	d$values <- NULL
 			d$time <- rep(time(x), each=nr)
-			d$values <- vals
+		#	d$values <- vals
 		}
 	} else if (time && has.time(x)) {
 		tm <- as.character(time(x))
@@ -443,11 +503,34 @@ setMethod("as.array", signature(x="SpatRaster"),
 	}
 )
 
+setMethod("as.array", signature(x="SpatRasterDataset"),
+	function(x) {
+		n <- length(x)
+		if (n < 2) return(as.array(x[1]))
+		
+		dm <- sapply(x, dim)
+		udm <- apply(dm, 1, function(i)length(unique))
+		if (!all(udm) == 1) {
+			error("as.array", "cannot make an array from rasters with different dimensions")
+		}
+		
+		a <- array(NA, c(dm[,1], n))
+		dimnames(a) <- list(NULL, NULL, NULL, names(x))
+
+		for (i in 1:n) {
+			a[,,,i] <- as.array(x[i])
+		}
+		a
+	}
+)
+
+
+
 
 # to sf from SpatVector
 # available in sf
 #.v2sf <- function(from) {
-#	txt <- 'sf::st_as_sf(as.data.frame(from, geom=TRUE), wkt="geometry", crs=from@cpp$get_crs("wkt"))'
+#	txt <- 'sf::st_as_sf(as.data.frame(from, geom=TRUE), wkt="geometry", crs=from@pntr$get_crs("wkt"))'
 #	eval(parse(text = txt))
 #}
 
@@ -466,7 +549,7 @@ setMethod("as.array", signature(x="SpatRaster"),
 	#geom <- st_as_text(geom)
 	#v <- vect(geom, crs=crs)
 	v <- vect()
-	v@cpp <- v@cpp$from_hex(sf::rawToHex(sf::st_as_binary(geom)), crs)
+	v@pntr <- v@pntr$from_hex(sf::rawToHex(sf::st_as_binary(geom)), crs)
 	v <- messages(v, "SpatVector from sf")
 	if (ncol(from) > 1) {
 		from[[sfi]] <- NULL
@@ -483,7 +566,7 @@ setMethod("as.array", signature(x="SpatRaster"),
 	#geom <- st_as_text(geom)
 	#v <- vect(geom, crs=crs)
 	v <- svc()
-	v@cpp <- v@cpp$from_hex_col(sf::rawToHex(sf::st_as_binary(geom)), crs)
+	v@pntr <- v@pntr$from_hex_col(sf::rawToHex(sf::st_as_binary(geom)), crs)
 	#if (ncol(from) > 1) {
 	#	from[[sfi]] <- NULL
 	#	values(v) <- as.data.frame(from)
@@ -519,7 +602,7 @@ setAs("sf", "SpatVector",
 
 .from_sfc <- function(from) {
 	v <- vect()
-	v@cpp <- v@cpp$from_hex(sf::rawToHex(sf::st_as_binary(from)), "")
+	v@pntr <- v@pntr$from_hex(sf::rawToHex(sf::st_as_binary(from)), "")
 	crs(v, warn=FALSE) <- attr(from, "crs")$wkt
 	v
 }
@@ -560,6 +643,10 @@ setAs("im", "SpatRaster",
 	function(from) {
 		r <- rast(nrows=from$dim[1], ncols=from$dim[2], xmin=from$xrange[1], xmax=from$xrange[2], ymin=from$yrange[1], ymax=from$yrange[2], crs="")
 		values(r) <- from$v
+		units(r) <- from$units$singular
+		if (from$units$multiplier != 1) {
+			r <- r * from$units$multiplier
+		}
 		flip(r, direction="vertical")
 	}
 )
