@@ -1,23 +1,27 @@
 
 
 fileBlocksize <- function(x) {
-	v <- x@ptr$getFileBlocksize()
+	v <- x@pntr$getFileBlocksize()
 	m <- matrix(v, ncol=2)
 	colnames(m) <- c("rows", "cols")
 	m
 }
 
 
+clearVSIcache <- function() {
+	.clearVSIcache(TRUE)
+}
+
 gdalCache <- function(size=NA) {
-	if (is.na(size)) {
-		.getGDALCacheSizeMB()
-	} else {
-		if (size > 0) {
-			.setGDALCacheSizeMB(size)
-		}
+	vsi <- FALSE # vsi not working
+	if (is.null(size) || is.na(size)) {
+		.getGDALCacheSizeMB(vsi)
+	} else if (size > 0) {
+		.setGDALCacheSizeMB(size, vsi)
 	}
 }
 
+
 getGDALconfig <- function(option) {
 	sapply(option, .gdal_getconfig)
 }
@@ -35,35 +39,51 @@ setGDALconfig <- function(option, value="") {
 }
 
 
+libVersion <- function(lib="all", parse=FALSE) {
+	lib <- tolower(lib)
+	if (lib=="gdal") {
+		out <- .gdal_version()
+	} else if (lib=="proj") {
+		out <- proj_version()
+	} else if (lib=="geos") {
+		out <- .geos_version()
+	} else {
+		out <- c(gdal=.gdal_version(), proj=proj_version(), geos=.geos_version())
+	}
+	if (parse) {
+		nms <- names(out)
+		out <- data.frame(matrix(as.numeric(unlist(strsplit(out, "\\."))), ncol=3, byrow=TRUE), row.names=nms)
+		names(out) <- c("major", "minor", "sub")
+	}
+	out
+}
 
 
-gdal <- function(warn=NA, drivers=FALSE, lib="gdal") {
+
+
+gdal <- function(warn=NA, drivers=FALSE, ...) {
 	if (!is.na(warn)) {
 		warn <- as.integer(warn)
-		stopifnot(warn %in% c(1:4))
+		stopifnot(warn %in% (1:4))
 		.set_gdal_warnings(warn)
 	} else if (drivers) {
 		x <- .gdaldrivers()
 		x <- do.call(cbind, x)
-		x[,2] = c("vector", "raster")[as.integer(x[,2])+1]
-		#x[,3] = c("read", "read/copy-write", "read/write")[as.integer(x[,3])+1]
-		x[,3] = c("read", "read/write", "read/write")[as.integer(x[,3])+1]
-		colnames(x) <- c("name", "type", "can", "vsi", "long.name")
 		x <- data.frame(x)
-		x[,4] <- x[,4] == 1
-		x <- x[order(x$type, x$name), ]
+		x[,2] = c(FALSE, TRUE)[as.integer(x[,2])+1]
+		x[,3] = c(FALSE, TRUE)[as.integer(x[,3])+1]
+		x[,4] = c("read", "read/write", "read/write")[as.integer(x[,4])+1]
+		colnames(x) <- c("name", "raster", "vector", "can", "vsi", "long.name")
+		x[,5] <- x[,5] == 1
+		x <- x[order(x$name), ]
 		rownames(x) <- NULL
 		x
 	} else {
-		lib <- tolower(lib)
-		if (lib=="gdal") {
-			.gdal_version()
-		} else if (lib=="proj") {
-			proj_version()
-		} else if (lib=="geos") {
-			.geos_version()
+		dots <- list(...)
+		if (length(dots) > 0) {
+			libVersion(dots[1])
 		} else {
-			c(gdal=.gdal_version(), proj=proj_version(), geos=.geos_version())
+			libVersion("gdal")		
 		}
 	}
 }
@@ -112,7 +132,8 @@ gdal <- function(warn=NA, drivers=FALSE, lib="gdal") {
 setMethod("describe", signature(x="character"),
 	function(x, sds=FALSE, meta=FALSE, parse=FALSE, options="", print=FALSE, open_opt="") {
 
-		x <- .fullFilename(x[1], FALSE)
+		#x <- .fullFilename(x[1], FALSE)
+		x <- x[1]
 		if (meta) {
 			if (sds) {
 				return(.meta_sds(x, parse))
@@ -148,3 +169,13 @@ setMethod("describe", signature(x="character"),
 	}
 )
 
+setMethod("describe", signature(x="SpatRaster"),
+	function(x, source=1, ...) {
+		if (!hasValues(x)) return(NULL)
+		source <- round(source)
+		if ((source < 1) || (source > nsrc(x))) {
+			error("describe", "source should be >= 1 and <= nsrc()")
+		}
+		describe(sources(x)[source])
+	}
+)
