@@ -5,7 +5,14 @@
 
 setMethod("is.rotated", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$is_rotated()
+		x@pntr$is_rotated()
+	}
+)
+
+
+setMethod("is.flipped", signature(x="SpatRaster"),
+	function(x)  {
+		x@pntr$is_flipped()
 	}
 )
 
@@ -13,7 +20,7 @@ setMethod("is.rotated", signature(x="SpatRaster"),
 setMethod("rangeFill", signature(x="SpatRaster"),
 	function(x, limit, circular=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$fill_range(limit, circular, opt)
+		x@pntr <- x@pntr$fill_range(limit, circular, opt)
 		messages(x, "rangeFill")
 	}
 )
@@ -23,7 +30,7 @@ setMethod("rangeFill", signature(x="SpatRaster"),
 setMethod("weighted.mean", signature(x="SpatRaster", w="numeric"),
 	function(x, w, na.rm=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$wmean_vect(w, na.rm, opt)
+		x@pntr <- x@pntr$wmean_vect(w, na.rm, opt)
 		messages(x, "weighted.mean")
 	}
 )
@@ -32,22 +39,25 @@ setMethod("weighted.mean", signature(x="SpatRaster", w="numeric"),
 setMethod("weighted.mean", signature(x="SpatRaster", w="SpatRaster"),
 	function(x, w, na.rm=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <-x@cpp$wmean_rast(w@cpp, na.rm, opt)
+		x@pntr <-x@pntr$wmean_rast(w@pntr, na.rm, opt)
 		messages(x, "weighted.mean")
 	}
 )
 
-
-
 setMethod("patches", signature(x="SpatRaster"),
-	function(x, directions=4, zeroAsNA=FALSE, allowGaps=TRUE, filename="", ...) {
+	function(x, directions=4, values=FALSE, zeroAsNA=FALSE, allowGaps=TRUE, filename="", ...) {
+		if (values) {
+			opt <- spatOptions(filename, ...)
+			x@pntr <- x@pntr$patches2(directions, opt)
+			return(messages(x, "patches"))			
+		}
 		if (allowGaps) {
 			opt <- spatOptions(filename, ...)
-			x@cpp <- x@cpp$patches(directions[1], zeroAsNA[1], opt)
+			x@pntr <- x@pntr$patches(directions[1], zeroAsNA[1], opt)
 			messages(x, "patches")
 		} else {
 			opt <- spatOptions()
-			x@cpp <- x@cpp$patches(directions[1], zeroAsNA[1], opt)
+			x@pntr <- x@pntr$patches(directions[1], zeroAsNA[1], opt)
 			x <- messages(x, "patches")
 			f <- freq(x)
 			fr <- cbind(f[,2], 1:nrow(f))
@@ -65,7 +75,7 @@ setMethod("patches", signature(x="SpatRaster"),
 
 setMethod("origin", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$origin
+		x@pntr$origin
 	}
 )
 
@@ -103,7 +113,7 @@ setMethod("origin<-", signature("SpatRaster"),
 
 setMethod("align", signature(x="SpatExtent", y="SpatRaster"),
 	function(x, y, snap="near") {
-		x@cpp <- y@cpp$align(x@cpp, tolower(snap))
+		x@pntr <- y@pntr$align(x@pntr, tolower(snap))
 		#messages(x, "align")
 		x
 	}
@@ -111,7 +121,7 @@ setMethod("align", signature(x="SpatExtent", y="SpatRaster"),
 
 setMethod("align", signature(x="SpatExtent", y="numeric"),
 	function(x, y) {
-		x@cpp <- x@cpp$align(y, "")
+		x@pntr <- x@pntr$align(y, "")
 		x
 	}
 )
@@ -120,7 +130,7 @@ setMethod("align", signature(x="SpatExtent", y="numeric"),
 setMethod("intersect", signature(x="SpatRaster", y="SpatRaster"),
 	function(x, y) {
 		opt <- spatOptions() 
-		x@cpp <- x@cpp$intersect(y@cpp, opt)
+		x@pntr <- x@pntr$intersect(y@pntr, opt)
 		messages(x)
 	}
 )
@@ -130,16 +140,25 @@ setMethod("cellSize", signature(x="SpatRaster"),
 	function(x, mask=FALSE, lyrs=FALSE, unit="m", transform=TRUE, rcx=100, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		if (!lyrs) x <- x[[1]]
-		x@cpp <- x@cpp$rst_area(mask, unit, transform, rcx, opt)
+		x@pntr <- x@pntr$rst_area(mask, unit, transform, rcx, opt)
 		messages(x, "cellSize")
 	}
 )
 
 
+setMethod("surfArea", signature(x="SpatRaster"),
+	function(x, filename="", ...) {
+		opt <- spatOptions(filename, ...)
+		x@pntr <- x@pntr$surface_area(opt)
+		messages(x, "surfArea")
+	}
+)
+
+
 setMethod("atan2", signature(y="SpatRaster", x="SpatRaster"),
 	function(y, x) {
 		opt <- spatOptions(filename="", overwrite=TRUE)
-		y@cpp <- y@cpp$atan2(x@cpp, opt)
+		y@pntr <- y@pntr$atan2(x@pntr, opt)
 		messages(y, "atan2")
 	}
 )
@@ -147,7 +166,7 @@ setMethod("atan2", signature(y="SpatRaster", x="SpatRaster"),
 setMethod("atan_2", signature(y="SpatRaster", x="SpatRaster"),
 	function(y, x, filename="", ...) {
 		opt <- spatOptions(filename=filename, ...)
-		y@cpp <- y@cpp$atan2(x@cpp, opt)
+		y@pntr <- y@pntr$atan2(x@pntr, opt)
 		messages(y, "atan_2")
 	}
 )
@@ -157,20 +176,20 @@ setMethod("boundaries", signature(x="SpatRaster"),
 	function(x, classes=FALSE, inner=TRUE, directions=8, falseval=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		type <- ifelse(inner[1], "inner", "outer")
-		x@cpp <- x@cpp$boundaries(classes[1], type, directions[1], falseval[1], opt)
+		x@pntr <- x@pntr$boundaries(classes[1], type, directions[1], falseval[1], opt)
 		messages(x, "boundaries")
 	}
 )
 
 
 .collapseSources <- function(x) {
-	x@cpp <- x@cpp$collapse_sources()
+	x@pntr <- x@pntr$collapse_sources()
 	messages(x, "tighten")
 }
 
 setMethod("deepcopy", signature("SpatRaster"),
 	function(x) {
-		x@cpp <- x@cpp$deepcopy()
+		x@pntr <- x@pntr$deepcopy()
 		x
 	}
 )
@@ -189,18 +208,18 @@ setMethod("split", signature(x="SpatRaster"),
 
 setMethod("add<-", signature("SpatRaster", "SpatRaster"),
 	function(x, value) {
-		if (x@cpp$same(value@cpp)) {
-			x@cpp <- x@cpp$deepcopy()
+		if (x@pntr$same(value@pntr)) {
+			x@pntr <- x@pntr$deepcopy()
 		}
 		opt <- spatOptions()
-		x@cpp$addSource(value@cpp, FALSE, opt)
+		x@pntr$addSource(value@pntr, FALSE, opt)
 		messages(x, "add")
 	}
 )
 
 setMethod("tighten", signature("SpatRaster"),
 	function(x) {
-		x@cpp <- x@cpp$collapse_sources()
+		x@pntr <- x@pntr$collapse_sources()
 		messages(x, "tighten")
 	}
 )
@@ -208,7 +227,7 @@ setMethod("tighten", signature("SpatRaster"),
 setMethod("tighten", signature("SpatRasterDataset"),
 	function(x) {
 		y <- new("SpatRaster")
-		y@cpp <- x@cpp$collapse()
+		y@pntr <- x@pntr$collapse()
 		messages(y, "tighten")
 	}
 )
@@ -217,9 +236,9 @@ setMethod("tighten", signature("SpatRasterDataset"),
 #setMethod("c", signature(x="SpatRaster"),
 #	function(x, ...) {
 #		s <- sds(list(x, ...))
-#		x@cpp <- s@cpp$collapse()
+#		x@pntr <- s@pntr$collapse()
 #		x <- messages(x, "c")
-#		try( x@cpp <- x@cpp$collapse_sources() )
+#		try( x@pntr <- x@pntr$collapse_sources() )
 #		messages(x, "c")
 #	}
 #)
@@ -228,12 +247,12 @@ setMethod("tighten", signature("SpatRasterDataset"),
 
 #cbind.SpatVector <- function(x, y, ...) {
 #	if (inherits(y, "SpatVector")) {
-#		y <- y@cpp$df
+#		y <- y@pntr$df
 #	} else {
 #		stopifnot(inherits(y, "data.frame"))
 #		y <- terra:::.makeSpatDF(y)
 #	}
-#	x@cpp <- x@cpp$cbind(y)
+#	x@pntr <- x@pntr$cbind(y)
 #	messages(x, "cbind")
 #}
 
@@ -241,12 +260,13 @@ cbind.SpatVector <- function(x, y, ...) {
 	dots <- list(y, ...)
 	for (y in dots) {
 		if (inherits(y, "SpatVector")) {
-			y <- y@cpp$df
+			y <- y@pntr$df
 		} else {
-			stopifnot(inherits(y, "data.frame"))
+			# stopifnot(inherits(y, "data.frame"))
+			y <- as.data.frame(y)
 			y <- .makeSpatDF(y)
 		}
-		x@cpp <- x@cpp$cbind(y)
+		x@pntr <- x@pntr$cbind(y)
 		x <- messages(x, "cbind")
 	}
 	x
@@ -257,18 +277,18 @@ rbind.SpatVector <- function(x, y, ...) {
 	if (missing(y) || is.null(y)) return(x)
 	stopifnot(inherits(y, "SpatVector"))
 	s <- svc(x, y, ...)
-	x@cpp <- s@cpp$append()
+	x@pntr <- s@pntr$append()
 	messages(x, "rbind")
 
 #	dots <- list(...)
 #	if (is.null(dots)) {
-#		x@cpp <- x@cpp$rbind(y@cpp, FALSE)
+#		x@pntr <- x@pntr$rbind(y@pntr, FALSE)
 #	} else {
 
 	#if (!is.null(dots)) {
 	#	for (y in dots) {
 	#		stopifnot(inherits(y, "SpatVector"))
-	#		x@cpp <- x@cpp$rbind(y@cpp, FALSE)
+	#		x@pntr <- x@pntr$rbind(y@pntr, FALSE)
 	#		x <- messages(x, "rbind")
 	#	}
 	#}
@@ -303,18 +323,18 @@ setMethod("clamp", signature(x="SpatRaster"),
 		rupp <- inherits(upper, "SpatRaster")
 		r <- rast()
 		if (rlow & rupp) {
-			x@cpp <- x@cpp$clamp_raster(lower@cpp, upper@cpp, NA, NA, values[1], opt)
+			x@pntr <- x@pntr$clamp_raster(lower@pntr, upper@pntr, NA, NA, values[1], opt)
 		} else if (rlow) {
 			if (any(is.na(upper))) error("clamp", "upper limit cannot be NA")
-			x@cpp <- x@cpp$clamp_raster(lower@cpp, r@cpp, NA, upper, values[1], opt)
+			x@pntr <- x@pntr$clamp_raster(lower@pntr, r@pntr, NA, upper, values[1], opt)
 		} else if (rupp) {
 			if (any(is.na(lower))) error("clamp", "lower limit cannot be NA")
-			x@cpp <- x@cpp$clamp_raster(r@cpp, upper@cpp, lower, NA, values[1], opt)
+			x@pntr <- x@pntr$clamp_raster(r@pntr, upper@pntr, lower, NA, values[1], opt)
 		} else {
 			if (any(is.na(lower))) error("clamp", "lower limit cannot be NA")
 			if (any(is.na(upper))) error("clamp", "upper limit cannot be NA")
-			x@cpp <- x@cpp$clamp_raster(r@cpp, r@cpp, lower, upper, values[1], opt)
-			#x@cpp <- x@cpp$clamp(lower, upper, values[1], opt)
+			x@pntr <- x@pntr$clamp_raster(r@pntr, r@pntr, lower, upper, values[1], opt)
+			#x@pntr <- x@pntr$clamp(lower, upper, values[1], opt)
 		}
 		messages(x, "clamp")
 	}
@@ -325,7 +345,7 @@ setMethod("clamp", signature(x="SpatRaster"),
 setMethod("clamp_ts", signature(x="SpatRaster"),
 	function(x, min=FALSE, max=TRUE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$clamp_ts(min, max, opt)
+		x@pntr <- x@pntr$clamp_ts(min, max, opt)
 		messages(x, "clamp_ts")
 	}
 )
@@ -365,7 +385,7 @@ function(x, rcl, include.lowest=FALSE, right=TRUE, others=NULL, brackets=TRUE, f
 		others <- TRUE
 	}
 	keepcats <- FALSE
-    x@cpp <- x@cpp$classify(as.vector(rcl), NCOL(rcl), right, include.lowest, others, othersValue, bylayer[1], brackets[1], keepcats, opt)
+    x@pntr <- x@pntr$classify(as.vector(rcl), NCOL(rcl), right, include.lowest, others, othersValue, bylayer[1], brackets[1], keepcats, opt)
 	messages(x, "classify")
 }
 )
@@ -460,11 +480,11 @@ function(x, from, to, others=NULL, raw=FALSE, filename="", ...) {
 	if (tom) {
 		nms <- colnames(to)
 		if (!is.null(nms)) 	opt$names = nms
-		x@cpp <- x@cpp$replaceValues(from, to, ncol(to), setothers, others, keepcats, opt)
+		x@pntr <- x@pntr$replaceValues(from, to, ncol(to), setothers, others, keepcats, opt)
 	} else if (frm) {
-		x@cpp <- x@cpp$replaceValues(as.vector(t(from)), to, -ncol(from), setothers, others, keepcats, opt)
+		x@pntr <- x@pntr$replaceValues(as.vector(t(from)), to, -ncol(from), setothers, others, keepcats, opt)
 	} else {
-		x@cpp <- x@cpp$replaceValues(from, to, 0, setothers, others, keepcats, opt)
+		x@pntr <- x@pntr$replaceValues(from, to, 0, setothers, others, keepcats, opt)
 	}
 	messages(x, "subst")
 }
@@ -491,14 +511,14 @@ setMethod("crop", signature(x="SpatRaster", y="ANY"),
 		opt <- spatOptions(filename, ...)
 		if (mask) {
 			if (inherits(y, "SpatVector")) {
-				x@cpp <- x@cpp$crop_mask(y@cpp, snap[1], touches[1], extend[1], opt)
+				x@pntr <- x@pntr$crop_mask(y@pntr, snap[1], touches[1], extend[1], opt)
 			} else if (inherits(y, "sf")) {
 				y <- vect(y)
-				x@cpp <- x@cpp$crop_mask(y@cpp, snap[1], touches[1], extend[1], opt)
+				x@pntr <- x@pntr$crop_mask(y@pntr, snap[1], touches[1], extend[1], opt)
 			} else if (inherits(y, "SpatRaster")) {
 				mopt <- spatOptions(filename="", ...)
 				e <- ext(y)
-				x@cpp <- x@cpp$crop(e@cpp, snap[1], extend[1], mopt)
+				x@pntr <- x@pntr$crop(e@pntr, snap[1], extend[1], mopt)
 				x <- messages(x, "crop")
 				if (compareGeom(x, y, lyrs=FALSE, crs=FALSE, warncrs=FALSE, ext=TRUE, rowcol=TRUE, res=FALSE, stopOnError=FALSE, messages=FALSE)) {
 					return(mask(x, y, filename=filename, ...))
@@ -513,11 +533,11 @@ setMethod("crop", signature(x="SpatRaster", y="ANY"),
 			} else {
 				y <- .getExt(y, method="crop")
 				warn("crop", paste("mask=TRUE is ignored for", class(y)[1], "objects"))
-				x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], opt)
+				x@pntr <- x@pntr$crop(y@pntr, snap[1], extend[1], opt)
 			}
 		} else {
 			y <- .getExt(y, method="crop")
-			x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], opt)
+			x@pntr <- x@pntr$crop(y@pntr, snap[1], extend[1], opt)
 		}
 		messages(x, "crop")
 	}
@@ -528,7 +548,7 @@ setMethod("crop", signature(x="SpatRasterDataset", y="ANY"),
 	function(x, y, snap="near", extend=FALSE) {
 		opt <- spatOptions()
 		y <- .getExt(y, method="crop")
-		x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], opt)
+		x@pntr <- x@pntr$crop(y@pntr, snap[1], extend[1], opt)
 		messages(x, "crop")
 	}
 )
@@ -537,7 +557,7 @@ setMethod("crop", signature(x="SpatRasterCollection", y="ANY"),
 	function(x, y, snap="near", extend=FALSE) {
 		y <- .getExt(y, method="crop")
 		opt <- spatOptions()
-		x@cpp <- x@cpp$crop(y@cpp, snap[1], extend[1], double(), opt)
+		x@pntr <- x@pntr$crop(y@pntr, snap[1], extend[1], double(), opt)
 		messages(x, "crop")
 	}
 )
@@ -546,7 +566,7 @@ setMethod("crop", signature(x="SpatRasterCollection", y="ANY"),
 setMethod("selectRange", signature(x="SpatRaster"),
 	function(x, y, z=1, repint=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$selRange(y@cpp, z, repint, opt)
+		x@pntr <- x@pntr$selRange(y@pntr, z, repint, opt)
 		messages(x, "selectRange")
 	}
 )
@@ -554,7 +574,15 @@ setMethod("selectRange", signature(x="SpatRaster"),
 setMethod("cover", signature(x="SpatRaster", y="SpatRaster"),
 	function(x, y, values=NA, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$cover(y@cpp, values, opt)
+		x@pntr <- x@pntr$cover(y@pntr, values, opt)
+		messages(x, "cover")
+	}
+)
+
+setMethod("cover", signature(x="SpatRaster", y="missing"),
+	function(x, y, values=NA, filename="", ...) {
+		opt <- spatOptions(filename, ...)
+		x@pntr <- x@pntr$cover_self(values, opt)
 		messages(x, "cover")
 	}
 )
@@ -570,7 +598,7 @@ setMethod("diff", signature(x="SpatRaster"),
 		y <- x[[-((n-lag+1):n)]]
 		x <- x[[-(1:lag)]]
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$arith_rast(y@cpp, "-", FALSE, opt)
+		x@pntr <- x@pntr$arith_rast(y@pntr, "-", FALSE, opt)
 		messages(x, "diff")
 	}
 )
@@ -585,7 +613,7 @@ setMethod("disagg", signature(x="SpatRaster"),
 			return(r)
 		}
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$disaggregate(fact, opt)
+		x@pntr <- x@pntr$disaggregate(fact, opt)
 		messages(x, "disagg")
 	}
 )
@@ -595,7 +623,7 @@ setMethod("flip", signature(x="SpatRaster"),
 	function(x, direction="vertical", filename="", ...) {
 		d <- match.arg(direction, c("vertical", "horizontal"))
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$flip(d == "vertical", opt)
+		x@pntr <- x@pntr$flip(d == "vertical", opt)
 		messages(x, "flip")
 	}
 )
@@ -605,7 +633,7 @@ setMethod("flip", signature(x="SpatRaster"),
 setMethod("mask", signature(x="SpatRaster", mask="SpatRaster"),
 	function(x, mask, inverse=FALSE, maskvalues=NA, updatevalue=NA, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$mask_raster(mask@cpp, inverse[1], maskvalues, updatevalue[1], opt)
+		x@pntr <- x@pntr$mask_raster(mask@pntr, inverse[1], maskvalues, updatevalue[1], opt)
 		messages(x, "mask")
 	}
 )
@@ -613,11 +641,21 @@ setMethod("mask", signature(x="SpatRaster", mask="SpatRaster"),
 setMethod("mask", signature(x="SpatRaster", mask="SpatVector"),
 	function(x, mask, inverse=FALSE, updatevalue=NA, touches=TRUE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$mask_vector(mask@cpp, inverse[1], updatevalue[1], touches[1], opt)
+		x@pntr <- x@pntr$mask_vector(mask@pntr, inverse[1], updatevalue[1], touches[1], opt)
 		messages(x, "mask")
 	}
 )
 
+setMethod("mask", signature(x="SpatRaster", mask="SpatExtent"),
+	function(x, mask, inverse=FALSE, updatevalue=NA, touches=TRUE, filename="", ...) {
+		opt <- spatOptions(filename, ...)
+		mask <- vect(mask, crs=crs(x))
+		x@pntr <- x@pntr$mask_vector(mask@pntr, inverse[1], updatevalue[1], touches[1], opt)
+		messages(x, "mask")
+	}
+)
+
+
 setMethod("mask", signature(x="SpatRaster", mask="sf"),
    function(x, mask, inverse=FALSE, updatevalue=NA, touches=TRUE, filename="", ...) {
 		mask <- vect(mask)
@@ -626,36 +664,31 @@ setMethod("mask", signature(x="SpatRaster", mask="sf"),
 )
 
 setMethod("project", signature(x="SpatRaster"),
-	function(x, y, method, mask=FALSE, align=FALSE, res=NULL, origin=NULL, threads=FALSE, filename="", ..., use_gdal=TRUE, by_util = FALSE)  {
+	function(x, y, method, mask=FALSE, align_only=FALSE, res=NULL, origin=NULL, threads=FALSE, filename="", ..., use_gdal=TRUE, by_util = FALSE)  {
 
 		if (missing(method)) {
-			if (is.factor(x)[1] || isTRUE(x@cpp$rgb)) {
+			if (is.factor(x)[1] || isTRUE(x@pntr$rgb)) {
 				method <- "near"
 			} else {
 				method <- "bilinear"
 			}
 		} else {
-			method <- method[1]
-		}
-		if (method == "ngb") {
-			method <- "near"
-			warn("project", "argument 'method=ngb' is deprecated, it should be 'method=near'")
+			method <- match.arg(tolower(method[1]), c("near", "bilinear", "cubic", "cubicspline", "lanczos", "average", "sum", "mode", "min", "q1", "median", "q3", "max", "rms"))			
 		}
 		opt <- spatOptions(filename, threads=threads, ...)
 
 		if (inherits(y, "SpatRaster")) {
 			if (use_gdal) {
-
 				if (by_util) {
-						x@cpp <- x@cpp$warp_by_util(y@cpp, "", method, mask[1], align[1], FALSE, opt)
+					x@pntr <- x@pntr$warp_by_util(y@pntr, "", method, mask[1], align_only[1], FALSE, opt)
 				} else {
-					x@cpp <- x@cpp$warp(y@cpp, "", method, mask[1], align[1], FALSE, opt)
+					x@pntr <- x@pntr$warp(y@pntr, "", method, mask[1], align_only[1], FALSE, opt)
 				}
 			} else {
-				if (align) {
-					y <- project(rast(x), y, align=TRUE)
+				if (align_only) {
+					y <- project(rast(x), y, align_only=TRUE)
 				}
-				x@cpp <- x@cpp$resample(y@cpp, method, mask[1], TRUE, opt)
+				x@pntr <- x@pntr$resample(y@pntr, method, mask[1], TRUE, opt)
 			}
 		} else {
 			if (!is.character(y)) {
@@ -666,19 +699,19 @@ setMethod("project", signature(x="SpatRaster"),
 				tmp <- project(rast(x), y)
 				if (!is.null(res)) res(tmp) <- res
 				if (!is.null(origin)) origin(tmp) <- origin
-				return(project(x, tmp, method=method, mask=mask, align=align, filename=filename, use_gdal=use_gdal, by_util=by_util, ...))
+				return(project(x, tmp, method=method, mask=mask, align_only=align_only, filename=filename, use_gdal=use_gdal, by_util=by_util, ...))
 			}
 			if (use_gdal) {
 
 				if (by_util) {
-					x@cpp <- x@cpp$warp_by_util(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
+					x@pntr <- x@pntr$warp_by_util(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
 					
 				} else {
-					x@cpp <- x@cpp$warp(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
+					x@pntr <- x@pntr$warp(SpatRaster$new(), y, method, mask, FALSE, FALSE, opt)
 				}
 			} else {
 				y <- project(rast(x), y)
-				x@cpp <- x@cpp$resample(y@cpp, method, mask[1], TRUE, opt)
+				x@pntr <- x@pntr$resample(y@pntr, method, mask[1], TRUE, opt)
 			}
 		}
 		messages(x, "project")
@@ -691,11 +724,20 @@ setMethod("project", signature(x="SpatVector"),
 		if (!is.character(y)) {
 			y <- as.character(crs(y))
 		}
-		x@cpp <- x@cpp$project(y, partial)
+		x@pntr <- x@pntr$project(y, partial)
 		messages(x, "project")
 	}
 )
 
+setMethod("project", signature(x="SpatVectorCollection"),
+	function(x, y, partial=FALSE)  {
+		x <- lapply(x, function(v) project(v, y, partial=partial))
+		svc(x)
+	}
+)
+
+
+
 setMethod("project", signature(x="SpatExtent"),
 	function(x, from, to)  {
 		if (missing(from)) error("project", "'from' cannot be missing")
@@ -724,9 +766,9 @@ setMethod("project", signature(x="matrix"),
            to <- as.character(crs(to))
         }
 		#v <- vect(x, type="line", crs=from)
-        #v@cpp <- v@cpp$project(to)
+        #v@pntr <- v@pntr$project(to)
         v <- vect()
-		xy <- v@cpp$project_xy(x[,1], x[,2], from, to)
+		xy <- v@pntr$project_xy(x[,1], x[,2], from, to)
 		messages(v, "project")
 		matrix(xy, ncol=2)
     }
@@ -736,7 +778,7 @@ setMethod("project", signature(x="matrix"),
 setMethod("quantile", signature(x="SpatRaster"),
 	function(x, probs=seq(0, 1, 0.25), na.rm=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$quantile(probs, na.rm[1], opt)
+		x@pntr <- x@pntr$quantile(probs, na.rm[1], opt)
 		messages(x, "quantile")
 	}
 )
@@ -772,20 +814,22 @@ setMethod("rectify", signature(x="SpatRaster"),
 			useaoi <- 0
 		}
 		snap <- as.logical(snap)
-		x@cpp <- x@cpp$rectify(method, aoi@cpp, useaoi, snap, opt)
+		x@pntr <- x@pntr$rectify(method, aoi@pntr, useaoi, snap, opt)
 		messages(x, "rectify")
 	}
 )
 
 setMethod("resample", signature(x="SpatRaster", y="SpatRaster"),
-	function(x, y, method, threads=FALSE, filename="", ...)  {
+	function(x, y, method, threads=FALSE, by_util=FALSE, filename="", ...)  {
 
 		if (missing(method)) {
-			method <- ifelse(is.factor(x)[1], "near", "bilinear")
-		}
-		if (method == "ngb") {
-			method <- "near"
-			warn("resample", "argument 'method=ngb' is deprecated, it should be 'method=near'")
+			if (is.factor(x)[1] || isTRUE(x@pntr$rgb)) {
+				method <- "near"
+			} else {
+				method <- "bilinear"
+			}
+		} else {
+			method <- match.arg(tolower(method[1]), c("near", "bilinear", "cubic", "cubicspline", "lanczos", "average", "sum", "mode", "min", "q1", "median", "q3", "max", "rms"))			
 		}
 		xcrs = crs(x)
 		ycrs = crs(y)
@@ -796,11 +840,12 @@ setMethod("resample", signature(x="SpatRaster", y="SpatRaster"),
 			crs(y) <- xcrs
 		}
 		opt <- spatOptions(filename, threads=threads, ...)
-#		if (gdal) {
-			x@cpp <- x@cpp$warp(y@cpp, "", method, FALSE, FALSE, TRUE, opt)
-#		} else {
-#			x@cpp <- x@cpp$resample(y@cpp, method, FALSE, TRUE, opt)
-#		}
+
+		if (by_util) {
+			x@pntr <- x@pntr$warp_by_util(y@pntr, "", method, FALSE, FALSE, TRUE, opt)
+		} else {
+			x@pntr <- x@pntr$warp(y@pntr, "", method, FALSE, FALSE, TRUE, opt)
+		}
 		messages(x, "resample")
 	}
 )
@@ -812,7 +857,7 @@ setMethod("impose", signature(x="SpatRasterCollection"),
 		stopifnot(inherits(y, "SpatRaster"))
 		opt <- spatOptions(filename, ...)
 		r <- rast()
-		r@cpp <- x@cpp$morph(y@cpp, opt)
+		r@pntr <- x@pntr$morph(y@pntr, opt)
 		messages(r, "impose")
 	}
 )
@@ -822,7 +867,7 @@ setMethod("impose", signature(x="SpatRasterCollection"),
 setMethod("rev", signature(x="SpatRaster"),
 	function(x) {
 		opt <- spatOptions("", FALSE, list())
-		x@cpp <- x@cpp$reverse(opt)
+		x@pntr <- x@pntr$reverse(opt)
 		messages(x, "rev")
 	}
 )
@@ -830,13 +875,13 @@ setMethod("rev", signature(x="SpatRaster"),
 setMethod("rotate", signature(x="SpatRaster"),
 	function(x, left=TRUE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$rotate(left, opt)
+		x@pntr <- x@pntr$rotate(left, opt)
 		messages(x, "rotate")
 	}
 )
 
 setMethod("rotate", signature(x="SpatVector"),
-	function(x, longitude=0, split=FALSE, left=TRUE, normalize=FALSE) {
+	function(x, longitude=0, split=TRUE, left=TRUE, normalize=FALSE) {
 		if (split) {
 			e <- ext(x)
 			if ((longitude < e$xmin) || (longitude > e$xmax)) {
@@ -850,6 +895,8 @@ setMethod("rotate", signature(x="SpatVector"),
 			ew <- ext(c(e[1], longitude, e[3:4]))
 			ee <- ext(c(longitude, e[2:4]))
 			x$unique_id_for_aggregation <- 1:nrow(x)
+			xcrs <- crs(x)
+			crs(x) <- NULL  # avoid wrapping
 			xw <- crop(x, ew)
 			xe <- crop(x, ee)
 			if (left) {
@@ -865,8 +912,9 @@ setMethod("rotate", signature(x="SpatVector"),
 				x <- out
 			} 
 			x$unique_id_for_aggregation <- NULL
+			crs(x) <- xcrs
 		} else {
-			x@cpp <- x@cpp$rotate_longitude(longitude, left)
+			x@pntr <- x@pntr$rotate_longitude(longitude, left)
 			x <- messages(x, "rotate")
 		}
 		if (normalize) {
@@ -880,7 +928,7 @@ setMethod("segregate", signature(x="SpatRaster"),
 	function(x, classes=NULL, keep=FALSE, other=0, round=FALSE, digits=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
 		if (is.null(classes)) classes <- 1[0]
-		x@cpp <- x@cpp$separate(classes, keep[1], other[1], round[1], digits[1], opt)
+		x@pntr <- x@pntr$separate(classes, keep[1], other[1], round[1], digits[1], opt)
 		messages(x, "segregate")
 	}
 )
@@ -889,7 +937,7 @@ setMethod("segregate", signature(x="SpatRaster"),
 setMethod("shift", signature(x="SpatRaster"),
 	function(x, dx=0, dy=0, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$shift(dx, dy, opt)
+		x@pntr <- x@pntr$shift(dx, dy, opt)
 		messages(x, "shift")
 	}
 )
@@ -905,7 +953,7 @@ setMethod("shift", signature(x="SpatExtent"),
 
 setMethod("shift", signature(x="SpatVector"),
 	function(x, dx=0, dy=0) {
-		x@cpp <- x@cpp$shift(dx, dy)
+		x@pntr <- x@pntr$shift(dx, dy)
 		messages(x, "shift")
 	}
 )
@@ -923,7 +971,7 @@ setMethod("rescale", signature(x="SpatRaster"),
 		}
 		ex = x0 + fx * (e[1:2] - x0);
 		ey = y0 + fy * (e[3:4] - y0);
-		x@cpp <- x@cpp$deepcopy()
+		x@pntr <- x@pntr$deepcopy()
 		ext(x) <- ext(c(ex, ey))
 		messages(x, "rescale")
 	}
@@ -940,11 +988,18 @@ setMethod("rescale", signature(x="SpatVector"),
 		if (missing(y0)) {
 			y0 <- mean(e[3:4])
 		}
-		x@cpp <- x@cpp$rescale(fx, fy, x0[1], y0[1])
+		x@pntr <- x@pntr$rescale(fx, fy, x0[1], y0[1])
 		messages(x, "rescale")
 	}
 )
 
+setMethod("scale_linear", signature(x="SpatRaster"),
+	function(x, min=0, max=1, filename="", ...) {
+		opt <- spatOptions(filename=filename, ...)
+		x@pntr <- x@pntr$scale_linear(min, max, opt)
+		messages(x, "scale_linear")
+	}
+)
 
 setMethod("scale", signature(x="SpatRaster"),
 	function(x, center=TRUE, scale=TRUE) {
@@ -963,7 +1018,7 @@ setMethod("scale", signature(x="SpatRaster"),
 		} else {
 			doscale = TRUE;
 		}
-		x@cpp <- x@cpp$scale(center, docenter, scale, doscale, opt)
+		x@pntr <- x@pntr$scale(center, docenter, scale, doscale, opt)
 		messages(x, "scale")
 	}
 )
@@ -999,7 +1054,7 @@ setMethod("stretch", signature(x="SpatRaster"),
 			app(x, ecdfun, filename=filename, wopt=wopt)
 		} else {
 			opt <- spatOptions(filename, ...)
-			x@cpp <- x@cpp$stretch(minv, maxv, minq, maxq, smin, smax, opt)
+			x@pntr <- x@pntr$stretch(minv, maxv, minq, maxq, smin, smax, opt)
 			messages(x, "stretch")
 		}
 	}
@@ -1032,14 +1087,14 @@ setMethod("summary", signature(object="SpatVector"),
 setMethod("t", signature(x="SpatRaster"),
 	function(x) {
 		opt <- spatOptions()
-		x@cpp <- x@cpp$transpose(opt)
+		x@pntr <- x@pntr$transpose(opt)
 		messages(x, "t")
 	}
 )
 
 setMethod("t", signature(x="SpatVector"),
 	function(x) {
-		x@cpp <- x@cpp$transpose()
+		x@pntr <- x@pntr$transpose()
 		messages(x, "t")
 	}
 )
@@ -1050,7 +1105,7 @@ setMethod("terrain", signature(x="SpatRaster"),
 		unit <- match.arg(unit, c("degrees", "radians"))
 		opt <- spatOptions(filename, ...)
 		seed <- ifelse("flowdir" %in% v, .seed(), 0)
-		x@cpp <- x@cpp$terrain(v, neighbors[1], unit=="degrees", seed, opt)
+		x@pntr <- x@pntr$terrain(v, neighbors[1], unit=="degrees", seed, opt)
 		messages(x, "terrain")
 	}
 )
@@ -1066,7 +1121,7 @@ setMethod("viewshed", signature(x="SpatRaster"),
 		outops <- c("yes/no", "sea", "land")
 		output <- match.arg(tolower(output), outops)
 		output <- match(output, outops)
-		z@cpp <- x@cpp$view(c(loc[1:2], observer[1], target[1]), c(1,0,2,3), curvcoef, 2, 0, output, opt)
+		z@pntr <- x@pntr$view(c(loc[1:2], observer[1], target[1]), c(1,0,2,3), curvcoef, 2, 0, output, opt)
 		messages(z, "viewshed")
 	}
 )
@@ -1074,7 +1129,7 @@ setMethod("viewshed", signature(x="SpatRaster"),
 setMethod("sieve", signature(x="SpatRaster"),
 	function(x, threshold, directions=8, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$sieve(threshold[1], directions[1], opt)
+		x@pntr <- x@pntr$sieve(threshold[1], directions[1], opt)
 		messages(x, "sieve")
 	}
 )
@@ -1087,7 +1142,7 @@ setMethod("trim", signature(x="SpatRaster"),
 		if (padding < 0) {
 			error("trim", "padding must be a non-negative integer")
 		}
-		x@cpp <- x@cpp$trim(value[1], padding, opt)
+		x@pntr <- x@pntr$trim(value[1], padding, opt)
 		messages(x, "trim")
 	}
 )
@@ -1095,7 +1150,7 @@ setMethod("trim", signature(x="SpatRaster"),
 setMethod("trans", signature(x="SpatRaster"),
 	function(x, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$transpose(opt)
+		x@pntr <- x@pntr$transpose(opt)
 		messages(x, "trans")
 	}
 )
@@ -1107,7 +1162,7 @@ setMethod("unique", signature(x="SpatRaster", incomparables="ANY"),
 		opt <- spatOptions()
 
 		if (as.raster) incomparables = FALSE
-		u <- x@cpp$unique(incomparables, digits, na.rm[1], opt)
+		u <- x@pntr$unique(incomparables, digits, na.rm[1], opt)
 
 		if (!as.raster) {
 			u <- get_labels(x, u)
@@ -1165,7 +1220,7 @@ setMethod("labels", signature(object="SpatRaster"),
 
 setMethod("scoff", signature(x="SpatRaster"),
 	function(x) {
-		out <- x@cpp$getScaleOffset()
+		out <- x@pntr$getScaleOffset()
 		names(out) <- c("scale", "offset")
 		do.call(cbind, out)
 	}
@@ -1174,17 +1229,17 @@ setMethod("scoff", signature(x="SpatRaster"),
 setMethod("scoff<-", signature("SpatRaster"),
 	function(x, value) {
 		if (is.null(value)) {
-			x@cpp <- x@cpp$deepcopy()
-			x@cpp$setScaleOffset(1, 0)
+			x@pntr <- x@pntr$deepcopy()
+			x@pntr$setScaleOffset(1, 0)
 		} else {
 			if (NCOL(value) != 2) {
 				error("scoff<-", "value must be a 2-column matrix")
 			}
-			x@cpp <- x@cpp$deepcopy()
+			x@pntr <- x@pntr$deepcopy()
 			value[is.na(value[,1]),1] <- 1
 			value[is.na(value[,2]),2] <- 0
-			x@cpp$setScaleOffset(value[,1], value[,2])
-			x@cpp$setValueType(0)
+			x@pntr$setScaleOffset(value[,1], value[,2])
+			x@pntr$setValueType(0)
 		}
 		messages(x, "scoff<-")
 	}
@@ -1193,7 +1248,7 @@ setMethod("scoff<-", signature("SpatRaster"),
 setMethod("sort", signature(x="SpatRaster"),
 	function (x, decreasing=FALSE, order=FALSE, filename="", ...) {
 		opt <- spatOptions(filename, ...)
-		x@cpp <- x@cpp$sort(decreasing[1], order[1], opt)
+		x@pntr <- x@pntr$sort(decreasing[1], order[1], opt)
 		messages(x, "sort")
 	}
 )
@@ -1228,3 +1283,25 @@ setMethod("sort", signature(x="data.frame"),
 		x[i, ]
 	}
 )
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
