@@ -121,7 +121,7 @@ setMethod("layerCor", signature(x="SpatRaster"),
 				old_pearson(x, asSample=asSample, na.rm=na.rm, nl=nl, n=n, mat=mat)
 			} else {			
 				opt <- spatOptions()
-				m <- x@cpp$layerCor("cor", use, asSample, opt)
+				m <- x@pntr$layerCor("cor", use, asSample, opt)
 				x <- messages(x)
 				m <- lapply(m, function(i) {
 					matrix(i, nrow=nl, byrow=TRUE, dimnames=list(names(x), names(x)))
@@ -134,6 +134,7 @@ setMethod("layerCor", signature(x="SpatRaster"),
 			if (use %in% c("complete.obs", "complete.masked")) {
 				v <- na.omit(v)
 			}
+			mat <- matrix(NA, nrow=nl, ncol=nl)			
 			for(i in 1:nl) {
 				for(j in i:nl) {
 					if (use == "pairwise.complete.obs") {
