@@ -15,7 +15,7 @@
 
 setMethod("density", signature(x="SpatRaster"),
 	function(x, maxcells=100000, plot=TRUE, main, ...) {
-		x <- spatSample(x, maxcells, method="regular", as.raster=TRUE)
+		x <- spatSample(x, maxcells, method="regular", as.raster=TRUE, warn=FALSE)
 		res <- list()
 		nl <- nlyr(x)
 		if (missing(main)) {
@@ -54,7 +54,7 @@ setMethod("density", signature(x="SpatRaster"),
 
 setMethod("persp", signature(x="SpatRaster"),
 	function(x, maxcells=100000, ...)  {
-		x <- spatSample(x, size=maxcells, method="regular", as.raster=TRUE)
+		x <- spatSample(x, size=maxcells, method="regular", as.raster=TRUE, warn=FALSE)
 		value <- t(as.matrix(x, wide=TRUE)[nrow(x):1,])
 		y <- yFromRow(x, nrow(x):1)
 		x <- xFromCol(x, 1:ncol(x))
@@ -65,7 +65,7 @@ setMethod("persp", signature(x="SpatRaster"),
 
 .plot.filled.contour <- function(x, maxcells=100000, ...) {
 
-	x <- spatSample(x[[1]], maxcells, method="regular", as.raster=TRUE)
+	x <- spatSample(x[[1]], maxcells, method="regular", as.raster=TRUE, warn=FALSE)
 	X <- xFromCol(x, 1:ncol(x))
 	Y <- yFromRow(x, nrow(x):1)
 	Z <- t( matrix( values(x), ncol=ncol(x), byrow=TRUE)[nrow(x):1,] )
@@ -84,7 +84,7 @@ setMethod("contour", signature(x="SpatRaster"),
 		if (filled) {
 			.plot.filled.contour(x, maxcells=maxcells, ...)
 		} else {
-			x <- spatSample(x[[1]], maxcells, method="regular", as.raster=TRUE)
+			x <- spatSample(x[[1]], maxcells, method="regular", as.raster=TRUE, warn=FALSE)
 			if (is.null(list(...)$asp)) {
 				asp <- ifelse(is.lonlat(x, perhaps=TRUE, warn=FALSE), 1/cos((mean(as.vector(ext(x))[3:4]) * pi)/180), 1)
 				graphics::contour(x=xFromCol(x,1:ncol(x)), y=yFromRow(x, nrow(x):1), z=t(as.matrix(x, wide=TRUE)[nrow(x):1,]), asp=asp, ...)
@@ -98,7 +98,7 @@ setMethod("contour", signature(x="SpatRaster"),
 
 setMethod("as.contour", signature(x="SpatRaster"),
 	function(x, maxcells=100000, ...) {
-		x <- spatSample(x[[1]], size=maxcells, method="regular", as.raster=TRUE)
+		x <- spatSample(x[[1]], size=maxcells, method="regular", as.raster=TRUE, warn=FALSE)
 		z <- grDevices::contourLines(x=xFromCol(x,1:ncol(x)), y=yFromRow(x, nrow(x):1), z=t(as.matrix(x, wide=TRUE)[nrow(x):1,]), ...)
 		y <- lapply(1:length(z), function(i) cbind(z[[i]]$level, i, z[[i]]$x, z[[i]]$y))
 		y <- do.call(rbind, y)
@@ -146,8 +146,7 @@ setMethod("pairs", signature(x="SpatRaster"),
 		if (hist) {dp <- panelhist} else {dp <- NULL}
 		if (cor) {up <- panelcor} else {up <- NULL}
 
-
-		d <- spatSample(x, maxcells, method="regular", as.raster=FALSE)
+		d <- spatSample(x, maxcells, method="regular", as.raster=FALSE, warn=FALSE)
 
 		dots <- list(...)
 		cex <- dots$cex
@@ -159,23 +158,40 @@ setMethod("pairs", signature(x="SpatRaster"),
 	}
 )
 
+.textbox <- function(x, y=NULL, labels, col="black", hc="white", hw=0.1, ... ) {
+	xy <- grDevices::xy.coords(x, y)
+	hw <- hw[1]
+	n <- nchar(labels)
+	x0 <- hw * graphics::strwidth("A")
+	y0 <- hw * graphics::strheight("A")
+	x1 <- n * x0
+	v <- vect(cbind(xy$x + c(0, x1), xy$y), crs="local")
+	b <- buffer(v, y0)
+}
 
 
 .halo <- function(x, y=NULL, labels, col="black", hc="white", hw=0.1, ... ) {
 	xy <- grDevices::xy.coords(x, y)
+	hw <- hw[1]
 	xo <- hw * graphics::strwidth("A")
 	yo <- hw * graphics::strheight("A")
 	n <- nchar(labels)
-	theta <- seq(pi/4, 2*pi, length.out=8*hw*10)
+	fact <- 100 * max(1, hw*2)
+	
+	theta <- seq(pi/4, 2*pi, length.out=hw*fact)
 	for (i in theta) {
 		text( xy$x + cos(i)*xo, xy$y + sin(i)*yo, labels, col=hc, ... )
 	}
 	graphics::text(xy$x, xy$y, labels, col=col, ... )
 }
 
+halo <- function(x, y=NULL, labels, col="black", hc="white", hw=0.1, ... ) {
+	.halo(x=x, y=y, labels=labels, col=col, hc=hc, hw=hw, ...) 
+}
+
 
 setMethod("text", signature(x="SpatRaster"),
-	function(x, labels, digits=0, halo=FALSE, ...) {
+	function(x, labels, digits=0, halo=FALSE, hc="white", hw=0.1, ...) {
 		if (missing(labels)) {
 			labels <- 1
 		}
@@ -193,14 +209,20 @@ setMethod("text", signature(x="SpatRaster"),
 		} else {
 			p <- as.points(x, values=FALSE, na.rm=FALSE)
 		}
-		xy <- geom(p)[, c("x", "y")]
+		xy <- geom(p)[, c("x", "y"), drop=FALSE]
 		if (is.factor(labels)) {
 			labels <- substr(as.character(labels), 1, max(1, digits))
 		} else if (is.numeric(labels)) {
 			labels <- as.character(round(labels, digits=digits) )
 		}
-		if (halo) {
-			.halo(xy[,1], xy[,2], labels, ...)
+		if (length(labels) < nrow(xy)) {
+			labels <- rep(labels, nrow(xy))
+		}
+#		if (!overlap) {
+#			xy <- getLabelXY(xy, labels, cex)
+#		}
+		if (halo && (isTRUE(hw > 0))) {
+			.halo(xy[,1], xy[,2], labels, hc=hc, hw=hw, ...)
 		} else {
 			text(xy[,1], xy[,2], labels, ...)
 		}
@@ -209,7 +231,7 @@ setMethod("text", signature(x="SpatRaster"),
 
 
 setMethod("text", signature(x="SpatVector"),
-	function(x, labels, halo=FALSE, inside=FALSE, ...) {
+	function(x, labels, halo=FALSE, inside=FALSE, hc="white", hw=0.1, ...) {
 		if (missing(labels)) {
 			labels <- 1:nrow(x)
 		} else if (length(labels) == 1) {
@@ -226,8 +248,8 @@ setMethod("text", signature(x="SpatVector"),
 			}
 		}
 		xy <- geom(centroids(x, inside=inside))[,c("x","y"),drop=FALSE]
-		if (halo) {
-			.halo(xy[,1], xy[,2], labels, ...)
+		if (halo && (isTRUE(hw > 0))) {
+			.halo(xy[,1], xy[,2], labels, hc=hc, hw=hw, ...)
 		} else {
 			text(xy[,1], xy[,2], labels, ...)
 		}
@@ -242,7 +264,7 @@ setMethod("boxplot", signature(x="SpatRaster"),
 			cn <- names(x)
 			if ( ncell(x) > maxcell) {
 				warn("boxplot", "taking a sample of ", maxcell, " cells")
-				x <- spatSample(x, maxcell, method="regular", as.raster=TRUE)
+				x <- spatSample(x, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
 			}
 			names(x) <- cn
 			boxplot(values(x), ...)
@@ -250,7 +272,7 @@ setMethod("boxplot", signature(x="SpatRaster"),
 			s <- c(x[[1]], y[[1]])
 			if ( ncell(x) > maxcell) {
 				warn("boxplot", "taking a regular sample of ", maxcell, " cells")
-				s <- spatSample(s, maxcell, method="regular", as.raster=TRUE)
+				s <- spatSample(s, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
 			}
 			s <- values(s, dataframe=TRUE)
 			cn <- colnames(s)
@@ -276,8 +298,8 @@ setMethod("barplot", "SpatRaster",
 		}
 		height <- height[[1]]
 		f <- is.factor(height)
-		x <- spatSample(height[[1]], maxcell, method="regular", as.raster=FALSE, as.df=f)
-		x <- spatSample(height[[1]], maxcell, method="regular", as.raster=FALSE, as.df=FALSE)
+#		x <- spatSample(height[[1]], maxcell, method="regular", as.raster=FALSE, as.df=f)
+		x <- spatSample(height[[1]], maxcell, method="regular", as.raster=FALSE, as.df=FALSE, warn=FALSE)
 		adj <- nrow(x) / ncell(height)
 		if (adj < 1) {
 			warn("barplot", "a sample of ", round(100*adj, 1), "% of the raster cells were used to estimate frequencies")
@@ -303,20 +325,36 @@ setMethod("barplot", "SpatRaster",
 shade <- function(slope, aspect, angle=45, direction=0, normalize=FALSE, filename="", overwrite=FALSE, ...) {
 	stopifnot(inherits(slope, "SpatRaster"))
 	opt <- spatOptions(filename, overwrite=overwrite, ...)
-	slope@ptr <- slope@ptr$hillshade(aspect@ptr, angle, direction, normalize[1], opt)
+	slope@pntr <- slope@pntr$hillshade(aspect@pntr, angle, direction, normalize[1], opt)
 	messages(slope, "shade")
 }
 
 
 map.pal <- function(name, n=50, ...) { 
+	n <- round(n)
+	if (n < 1) {
+		error("map.pal", "n should be >= 1")	
+	}
 	f <- system.file("colors/palettes.rds", package="terra")
 	v <- readRDS(f)
 	if (missing(name)) {
 		return(names(v))
 	}
 	if (name %in% names(v)) {
-		r <- grDevices::colorRampPalette(v[[name]], ...)
-		r(n)
+		v <- v[[name]]
+		if ((n > 0) && ((length(v) != n))) {
+			grDevices::colorRampPalette(v, ...)(n)
+		} else {
+			v
+		}
+	} else if (name == "random") {
+		if (n > 433) {
+			warning("map.pal", "you cannot get > 433 random colors; using n=433 instead")
+			n <- 433
+		}
+		s <- sample(grDevices::colors()[grep('gr(a|e)y', grDevices::colors(), invert = TRUE)], n)
+		rgb(t(grDevices::col2rgb(s))/255)
+
 	} else {
 		error("map.pal", paste(name, "is not a known palette"))
 	}
