@@ -1,14 +1,25 @@
 
 .get_breaks <- function(x, n, method, r=NULL) {
 	#x <- x[!is.na(x)]
+	
 	if (is.function(method)) {
 		if (!is.null(r)) {
-			x[(x<r[1]) | (x>r[2])] <- NA
+			if (!is.na(r[1])) { 
+				x[ x < r[1] ] <- NA
+			} 
+			if (!is.na(r[2])) { 
+				x[ x > r[2] ] <- NA
+			} 
 		}
 		breaks <- method(x)
 	} else if (method[1]=="cases") {
 		if (!is.null(r)) {
-			x[(x<r[1]) | (x>r[2])] <- NA
+			if (!is.na(r[1])) { 
+				x[ x < r[1] ] <- NA
+			} 
+			if (!is.na(r[2])) { 
+				x[ x > r[2] ] <- NA
+			} 
 		}
 		n <- n+1
 		i <- seq(0, 1, length.out=n)
@@ -23,13 +34,16 @@
 	} else { # if (method=="eqint") {
 		if (is.null(r)) {
 			r <- c(min(x, na.rm=TRUE), max(x, na.rm=TRUE))
+		} else if (any(is.na(r))) {
+			if (is.na(r[1])) r[1] <- min(x, na.rm=TRUE)
+			if (is.na(r[2])) r[2] <- max(x, na.rm=TRUE)
 		}
 		small <- 1e-16
 		if ((r[1] %% 1) != 0) { r[1] <- r[1] - small }
 		if ((r[2] %% 1) != 0) { r[2] <- r[2] + small }
 		breaks <- seq(r[1] , r[2], length.out=n+1)
 	}
-	breaks
+	unique(breaks)
 }
 
 .get_nrnc <- function(nr, nc, nl) {
@@ -271,8 +285,23 @@ retro_labels <- function(x, lat=TRUE) {
 }
 
 
+
+
 .plot.cont.legend <- function(x, ...) {
 
+	if (!is.null(x$leg$tic)) {
+		accepted <- c("in", "out", "none", "through", "throughout")
+		tics <- accepted[pmatch(x$leg$tic[1], accepted[-5], 5)]
+	} else {
+		tics <- "throughout"
+	}
+
+	if (!is.null(x$leg$tic.box.col)) {
+		ticboxcol <- x$leg$tic.box.col
+	} else {
+		ticboxcol <- "black"
+	}
+	
 	if (is.null(x$leg$x)) {
 		x$leg$x <- "right"
 	} else if (!(x$leg$x %in% c("left", "right", "top", "bottom"))) {
@@ -307,6 +336,10 @@ retro_labels <- function(x, lat=TRUE) {
 	zztxt <- x$leg$labels
 	if (is.null(zztxt)) {
 		zztxt <- formatC(zz, digits=x$leg$digits, format = "f")
+		if (x$fill_range) {
+			if (isTRUE(x$range_filled[1])) zztxt[1] <- paste0("< ", zztxt[1])		
+			if (isTRUE(x$range_filled[2])) zztxt[length(zztxt)] <- paste0("> ", zztxt[length(zztxt)])		
+		}
 	}
 	e <- x$leg$ext
 	if (x$leg$x %in% c("left", "right")) {
@@ -314,10 +347,26 @@ retro_labels <- function(x, lat=TRUE) {
 		graphics::rect(e$xmin, Y[-(nc + 1)], e$xmax, Y[-1], col=rev(cols), border=NA, xpd=NA)
 		ypos <- e$ymin + (zz - zlim[1])/(zlim[2] - zlim[1]) * e$dy
 		if (x$leg$x == "right") {
-			graphics::segments(e$xmin, ypos, e$xmax+e$dx*0.25, ypos, xpd=NA)
+			if (tics == "throughout") {
+				graphics::segments(e$xmin, ypos, e$xmax+e$dx*0.25, ypos, xpd=NA)
+			} else if (tics == "through") {
+				graphics::segments(e$xmin, ypos, e$xmax, ypos, xpd=NA)
+			} else if (tics == "in") {
+				graphics::segments(e$xmax-e$dx*0.25, ypos, e$xmax, ypos, xpd=NA)			
+			} else if (tics == "out") {
+				graphics::segments(e$xmax, ypos, e$xmax+e$dx*0.25, ypos, xpd=NA)
+			}
 			text(e$xmax, ypos, zztxt, pos=4, xpd=NA, cex=cex, ...)
 		} else {
-			graphics::segments(e$xmin-e$dx*0.25, ypos, e$xmax, ypos, xpd=NA)
+			if (tics == "throughout") {
+				graphics::segments(e$xmin-e$dx*0.25, ypos, e$xmax, ypos, xpd=NA)
+			} else if (tics == "through") {
+				graphics::segments(e$xmin, ypos, e$xmax, ypos, xpd=NA)
+			} else if (tics == "in") {
+				graphics::segments(e$xmin, ypos, e$xmin+e$dx*0.25, ypos, xpd=NA)
+			} else if (tics == "out") {
+				graphics::segments(e$xmin-e$dx*0.25, ypos, e$xmin, ypos, xpd=NA)
+			}
 			text(e$xmin, ypos, zztxt, pos=2, xpd=NA, cex=cex, ...)
 		}
 	} else {
@@ -325,17 +374,33 @@ retro_labels <- function(x, lat=TRUE) {
 		graphics::rect(X[-(nc + 1)], e$ymin, X[-1], e$ymax, col=rev(cols), border=NA, xpd=NA)
 		xpos <- e$xmin + (zz - zlim[1])/(zlim[2] - zlim[1]) * e$dx
 		if (x$leg$x == "bottom") {
-			graphics::segments(xpos, e$ymin-e$dy*0.25, xpos, e$ymax, xpd=NA)
+			if (tics == "throughout") {
+				graphics::segments(xpos, e$ymin-e$dy*0.25, xpos, e$ymax, xpd=NA)
+			} else if (tics == "through") {
+				graphics::segments(xpos, e$ymin, xpos, e$ymax, xpd=NA)
+			} else if (tics == "in") {
+				graphics::segments(xpos, e$ymin+e$dy*0.25, xpos, e$ymin, xpd=NA)
+			} else if (tics == "out") {
+				graphics::segments(xpos, e$ymin-e$dy*0.25, xpos, e$ymin, xpd=NA)
+			}
 			text(xpos, e$ymin, zztxt, pos=1, xpd=NA, cex=cex)
 		} else {
-			graphics::segments(xpos, e$ymin, xpos, e$ymax+e$dy*0.25, xpd=NA)
+			if (tics == "throughout") {
+				graphics::segments(xpos, e$ymin, xpos, e$ymax+e$dy*0.25, xpd=NA)
+			} else if (tics == "through") {
+				graphics::segments(xpos, e$ymin, xpos, e$ymax, xpd=NA)
+			} else if (tics == "in") {
+				graphics::segments(xpos, e$ymax, xpos, e$ymax-e$dy*0.25, xpd=NA)
+			} else if (tics == "out") {
+				graphics::segments(xpos, e$ymax, xpos, e$ymax+e$dy*0.25, xpd=NA)
+			}
 			text(xpos, e$ymax+e$dy*0.25, zztxt, pos=3, xpd=NA, cex=cex)
 		}
 	}
-	graphics::rect(e$xmin, e$ymin, e$xmax, e$ymax, border ="black", xpd=NA)
+	graphics::rect(e$xmin, e$ymin, e$xmax, e$ymax, border=ticboxcol, xpd=NA)
 
 
-    if (!is.null(x$leg$title)) {
+    if (isTRUE("title" %in% names(x$leg))) {
 		leg_i <- x$leg$leg_i
 		if (is.null(leg_i)) leg_i = 1
 	    if (leg_i <= length(x$leg$title)) {
@@ -344,13 +409,15 @@ retro_labels <- function(x, lat=TRUE) {
 			legtitle <- x$leg$title[1]		
 		}
 		e <- x$leg$ext
-		if (x$leg$x %in% c("top", "bottom")) {
-			txt <- paste(legtitle, collapse=" ")
-		} else {
-			txt <- paste(legtitle, collapse="\n")		
-		}
+		if (length(legtitle) > 1) { # or perhaps !inherits(legtitle, "expression")
+			if (x$leg$x %in% c("top", "bottom")) {
+				legtitle <- paste(legtitle, collapse=" ")
+			} else {
+				legtitle <- paste(legtitle, collapse="\n")		
+			}
+		} 
 		# offset=.5*graphics::strheight("a",cex=x$leg$title.cex)
-		text(x=e$xmax, y=e$ymax, labels=txt, pos=3, cex=x$leg$title.cex, xpd=NA)
+		text(x=e$xmax, y=e$ymax, labels=legtitle, pos=3, cex=x$leg$title.cex, xpd=NA)
 	}
 	x
 }
@@ -442,3 +509,72 @@ add_legend <- function(x, y, ...) {
 }
 
 
+add_box <- function(...) {
+	e <- unlist(get.clip())
+	if (!is.null(e)) {
+		bx <- rbind(
+			cbind(e[1], e[3:4]),
+			cbind(e[2], e[4:3]),
+			cbind(e[1], e[3])
+		)
+		if (is.null(list(...)$xpd)) {
+			lines(bx, xpd=TRUE, ...)
+		} else {
+			lines(bx, ...)		
+		}
+	}
+}
+
+
+add_grid <- function(nx=NULL, ny=nx, col="lightgray", lty="dotted", lwd=1) {
+
+	p <- get.clip()
+
+	## adapted from graphics::grid 
+	g.grid.at <- function (side, n, axp, usr2) {
+		if (is.null(n)) {
+			stopifnot(is.numeric(ax <- axp), length(ax) == 3L)
+			graphics::axTicks(side, axp=ax, usr=usr2, log=FALSE)
+		}
+		else if (!is.na(n) && (n <- as.integer(n)) >= 1L) {
+			at <- seq.int(usr2[1L], usr2[2L], length.out = n + 1L)
+			at[-c(1L, n + 1L)]
+		}
+	}
+
+    atx <- if (is.null(nx) || (!is.na(nx) && nx >= 1)) 
+        g.grid.at(1L, nx, axp = graphics::par("xaxp"), usr2 = p[1:2])
+    aty <- if (is.null(ny) || (!is.na(ny) && ny >= 1)) 
+        g.grid.at(2L, ny, axp = graphics::par("yaxp"), usr2 = p[3:4])
+    graphics::abline(v = atx, h = aty, col = col, lty = lty, lwd = lwd)
+    invisible(list(atx = atx, aty = aty))
+}
+
+
+add_mtext <- function(text, side=3, line=0, ...) {
+
+	stopifnot(side %in% 1:4)
+
+	p <- unlist(get.clip())
+	h <- graphics::strheight(text, units = "user", ...)	
+	
+	srt <- 0
+	if (side==1) {
+		x <- mean(p[1:2])
+		y <- p[3] - h - line * h
+	} else 	if (side==2) {
+		x <- p[1] -1.25 * h - line * h
+		y <- mean(p[3:4])
+		srt <- 90
+	} else 	if (side==3) {
+		x <- mean(p[1:2])
+		y <- p[4] + h + line * h
+	} else {
+		x <- p[2] + 1.25 * h + line * h
+		y <- mean(p[3:4])
+		srt <- 270
+	} 
+	
+	text(x=x, y=y, labels=text, xpd=TRUE, srt=srt, ...)
+}
+
