@@ -68,11 +68,14 @@ baselayers <- function(tiles, wrap=TRUE) {
 	out <- list(v=v, leg=list())
 	
 	if (is.null(type)) type <- ""
-	if (type == "continuous") type <- "interval"	
-	if ((!is.numeric(v)) || (length(unique(v)) < 11)) {
-		type <- "classes"
+	if (type == "continuous") {
+		type <- "interval"	
 	} else if (type == "") {
-		type <- "interval"
+		if ((!is.numeric(v)) || (length(unique(v)) < 11)) {
+			type <- "classes"
+		} else {
+			type <- "interval"
+		}
 	} else {
 		type <- match.arg(type, c("interval", "classes"))
 	}
@@ -103,7 +106,7 @@ baselayers <- function(tiles, wrap=TRUE) {
 setMethod("plet", signature(x="SpatVector"),
 	function(x, y="", col, fill=0.2, main=y, cex=1, lwd=2, border="black", alpha=1, popup=TRUE, label=FALSE, split=FALSE, tiles=c("Streets", "Esri.WorldImagery", "OpenTopoMap"), wrap=TRUE, legend="bottomright", collapse=FALSE, type=NULL, breaks=NULL, breakby="eqint", sort=TRUE, decreasing=FALSE, map=NULL, ...)  {
 
-		checkLeafLetVersion()
+		#checkLeafLetVersion()
 		y <- unique(y)
 		if (length(y) > 1) {
 			y = y[1]
@@ -117,7 +120,7 @@ setMethod("plet", signature(x="SpatVector"),
 #type=type, breaks=breaks, breakby=breakby, sort=sort, decreasing=decreasing,
 		}
 		
-		if (missing(col)) col <- grDevices::rainbow
+		if (missing(col)) col <- function(n) grDevices::rainbow(n, start=.2)
 		alpha <- max(0, min(1, alpha))
 		fill <- max(0, min(1, fill))
 		x <- makelonlat(x)
@@ -132,7 +135,7 @@ setMethod("plet", signature(x="SpatVector"),
 		g <- geomtype(x)
 		leg <- NULL
 		if (y == "") { # no legend
-			group <- x@cpp$layer
+			group <- x@pntr$layer
 			if (group == "") group = g
 			cols <- .getCols(nrow(x), col)
 			pop  <- lab <- NULL
@@ -240,7 +243,7 @@ setMethod("plet", signature(x="SpatVector"),
 setMethod("plet", signature(x="SpatVectorCollection"),
 	function(x, col, fill=0, cex=1, lwd=2, border="black", alpha=1, popup=TRUE, label=FALSE, tiles=c("Streets", "Esri.WorldImagery", "OpenTopoMap"), wrap=TRUE, legend="bottomright", collapse=FALSE, map=NULL)  {
 
-		checkLeafLetVersion()
+		#checkLeafLetVersion()
 
 		if (is.null(map)) {
 			tiles <- unique(as.character(tiles))
@@ -264,9 +267,9 @@ setMethod("plet", signature(x="SpatVectorCollection"),
 		lwd <- rep_len(lwd, n) 
 
 		alpha <- rep_len(alpha, n) 
-		alpha <- pmax(0, min(1, alpha))
+		alpha <- pmax(0, pmin(1, alpha))
 		fill <- rep_len(fill, n) 
-		fill <- pmax(0, min(1, fill))
+		fill <- pmax(0, pmin(1, fill))
 		popup <- rep_len(popup, n) 
 		label <- rep_len(label, n) 
 		border <- rep_len(border, n) 
@@ -306,6 +309,52 @@ setMethod("plet", signature(x="SpatVectorCollection"),
 )
 
 
+
+setMethod("polys", signature(x="leaflet"),
+	function(x, y, col, fill=0.2, lwd=2, border="black", alpha=1, popup=TRUE, label=FALSE, ...)  {
+
+		if (inherits(y, "SpatVector")) {
+			if (nrow(y) == 0) return(x)
+			y <- makelonlat(y)
+			if (missing(col)) col <- "black"
+			if (geomtype(y) != "polygons") {
+				error("polys", "SpatVector y must have polygons geometry")
+			}
+			leaflet::addPolygons(x, data=y, weight=lwd, fillColor=col, 
+					fillOpacity=fill, col=border, opacity=alpha, popup=popup, 
+					label=label, ...)			
+			
+		} else if (inherits(y, "SpatVectorCollection")) {
+			nms <- names(y)
+			n <- length(y)
+			nms[nchar(nms) == 0] <- "X"
+			nms <- make.unique(nms)
+			if (is.function(col)) {
+				cols <- col(n)
+			} else {
+				cols <- rep_len(col, n) 
+			}	
+			lwd <- rep_len(lwd, n) 
+			alpha <- rep_len(alpha, n) 
+			fill <- rep_len(fill, n) 
+			border <- rep_len(border, n) 
+			popup <- rep_len(popup, n) 
+			label <- rep_len(label, n) 
+
+			for (i in 1:length(nms)) {
+				x <- leaflet::addPolygons(x, data=y[i], weight=lwd[i], fillColor=cols[i], 
+					fillOpacity=fill[i], col=border[i], opacity=alpha[i], popup=popup[i], 
+					label=label[i], group=nms[i], ...)			
+			}
+			collapse=FALSE
+			leaflet::addLayersControl(x, overlayGroups = nms, options = leaflet::layersControlOptions(collapsed=collapse))
+		} else {
+			error("plet", "y should be a SpatVector or SpatVectorCollection")
+		}	
+	}
+)
+
+
 setMethod("lines", signature(x="leaflet"),
 	function(x, y, col, lwd=2, alpha=1, ...)  {
 		if (inherits(y, "SpatVector")) {
@@ -333,12 +382,14 @@ setMethod("lines", signature(x="leaflet"),
 			}
 			collapse=FALSE
 			leaflet::addLayersControl(x, overlayGroups = nms, options = leaflet::layersControlOptions(collapsed=collapse))
-		}
+		} else {
+			error("plet", "y should be a SpatVector or SpatVectorCollection")
+		}	
 	}
 )
 
 setMethod("points", signature(x="leaflet"),
-	function(x, y, col, cex=1, alpha=1, popup=FALSE, ...)  {
+	function(x, y, col, cex=1, alpha=1, label=1:nrow(y), popup=FALSE, ...)  {
 		stopifnot(inherits(y, "SpatVector"))
 		if (nrow(y) == 0) return(x)
 		y <- makelonlat(y)
@@ -355,7 +406,7 @@ setMethod("points", signature(x="leaflet"),
 		} else {
 			popup <- NULL
 		}
-		leaflet::addCircleMarkers(x, data=y, radius=cex, popup=popup, label=1:nrow(y), opacity=alpha, col=col, ...)
+		leaflet::addCircleMarkers(x, data=y, radius=cex, popup=popup, label=label, opacity=alpha, col=col, ...)
 	}
 )
 
@@ -363,6 +414,8 @@ setMethod("points", signature(x="leaflet"),
 
 
 make.panel <- function(x, maxcell) {
+
+	nl <- nlyr(x)
 	x <- spatSample(x, maxcell/nl, "regular", as.raster=TRUE, warn=FALSE)
 	if (is.lonlat(x)) {
 		asp <- 1/cos((mean(ext(x)[3:4]) * pi)/180)
@@ -379,7 +432,6 @@ make.panel <- function(x, maxcell) {
 	#	}
 	#}
 
-	nl <- nlyr(x)
 	asp <- asp * nrow(x) / ncol(x)
 	nc <- ceiling(2*sqrt(nl/2) * asp)
 	nr <- ceiling(nl / nc)
@@ -411,15 +463,81 @@ make.panel <- function(x, maxcell) {
 }
 
 
-setMethod("plet", signature(x="SpatRaster"),
-	function(x, y=1, col, alpha=0.8, main=names(x), tiles=NULL, wrap=TRUE, maxcell=500000, legend="bottomright", shared=FALSE, panel=FALSE, collapse=TRUE, map=NULL)  {
+.get_cls <- function(x, type="", dig.lab=3, cols, breaks=NULL, breakby="eqint", sort=TRUE, decreasing=FALSE,  ...) {
 
-		checkLeafLetVersion()
 
-		if (is.na(crs(x))) {
-			error("plet", "x must have a crs")
+	if (is.null(type) ||(type == "")) {
+		if (is.factor(x) || is.bool(x)) {
+			type <- "classes"		
+		} else if (is.null(breaks)) {
+			type <- "continuous"
+		} else {
+			type <- "interval"
 		}
+	} else {
+		type <- match.arg(type, c("continuous", "interval", "classes"))
+	}
+	if ((type == "interval") && (is.null(breaks))) {
+		
+	}
+	if (type == "continuous") {
+		if (inherits(cols, "function")) {
+			cols <- cols(100)
+		}
+		return(list(type=type, x=x, cols=cols))
+	}
+	if (type == "classes") {
+		if (!is.factor(x)) {
+			uv <- unique(values(x))
+			uv <- sort(uv)
+			uv <- uv[!is.na(uv)]
+			levels(x) <- data.frame(ID=uv, value=uv)
+		} else {
+			uv <- levels(x)[[1]][,2]
+		}
+		ncols <- length(uv)
+	} else {
+		if (is.null(breaks)) breaks <- 5
+		if (length(breaks) == 1) {
+			breaks <- .get_breaks(values(x), n=breaks, breakby, r=NULL)
+		}
+		x <- classify(x, breaks)
+		ncols <- length(breaks)-1
+	}
 
+	if (inherits(cols, "function")) {
+		cols <- cols(ncols)
+	} else {
+		cols <- grDevices::colorRampPalette(cols)(ncols)		
+	}
+	return(list(type=type, x=x, cols=cols))
+
+#	out$legend_type <- type
+#	out$uv <- unique(v)
+}
+
+
+
+setMethod("plet", signature(x="SpatRaster"),
+	function(x, y=1, col, alpha=0.8, main=names(x), tiles=c("Streets", "Esri.WorldImagery", "OpenTopoMap"), 
+		wrap=TRUE, maxcell=500000, stretch=NULL, legend="bottomright", shared=FALSE, panel=FALSE, collapse=TRUE, 
+		type=NULL, breaks=NULL, breakby="eqint", map=NULL, ...)  {
+
+		#checkLeafLetVersion()
+		
+		if (is.na(crs(x)) | (grepl("^Cartesian", .name_or_proj4(x)))) {
+			tiles <- ""
+			e <- ext(x)
+			rx <- diff(e[1:2])
+			ry <- diff(e[3:4])
+			m <- max(rx, ry)
+			ext(x) <- c(0, rx/m, 0, ry/m)
+			crs(x) <- "EPSG:3857"
+			notmerc <- FALSE
+
+		} else {
+			notmerc <- isTRUE(crs(x, describe=TRUE)$code != "3857")
+		}
 
 #		if (!is.null(add)) {
 #			if (inherits(add, "SpatVector")) {
@@ -435,6 +553,16 @@ setMethod("plet", signature(x="SpatRaster"),
 
 		alpha <- max(0, min(1, alpha))
 		
+		hasRGB <- has.RGB(x)
+
+		if (hasRGB) {
+			y <- RGB(x)
+			legend <- NULL		
+		}
+		# else if (!is.null(terra::coltab(x)[[1]])) {
+		#	legend <- NULL		
+		#}
+		
 		e <- ext(x)
 		if (is.lonlat(x) && ((e$ymin < -85) || (e$ymax > 85))) {
 			yr1 <- e$ymax - e$ymin
@@ -461,14 +589,15 @@ setMethod("plet", signature(x="SpatRaster"),
 			tiles <- tiles[tiles!=""]
 			if (length(tiles) > 1) {
 				tiles <- tiles[1]
-				warn("plet", "only a single tileset can be used with raster data")
+				#warn("plet", "only a single tileset can be used with raster data")
 			}
 			map <- baselayers(tiles, wrap)
 		} else {
 			tiles <- NULL
 		}
 		if (missing(col)) {
-			col <- rev(grDevices::terrain.colors(255))
+			col <- .default.pal()
+			#col <- rev(grDevices::terrain.colors(255))
 		}
 
 		main <- gsub("\n", "</br>", main)
@@ -476,18 +605,36 @@ setMethod("plet", signature(x="SpatRaster"),
 			main <- rep_len(main, length(x))[y]
 		}
 
-
-		notmerc <- isTRUE(crs(x, describe=TRUE)$code != "3857")
-
+		if (hasRGB) {
+			if (!is.null(stretch)) {
+				if (stretch == "lin") {
+					x <- stretch(x, minq=0.02, maxq=0.98)
+				} else {
+					x <- stretch(x, histeq=TRUE, scale=255)
+				}
+			}
+			RGB(x) <- 1:length(y)
+			x <- colorize(x, "col")
+		} else {
+			leg <- .get_cls(x, type=type, dig.lab=3, cols=col, breaks=breaks, breakby=breakby, ...)
+			x <- leg$x
+			col <- leg$cols
+		}
+		
 		if (nlyr(x) == 1) {
 			map <- leaflet::addRasterImage(map, x, colors=col, opacity=alpha, project=notmerc)
 			if (!is.null(legend)) {
-				if (!all(hasMinMax(x))) setMinMax(x)
-				r <- minmax(x)
-				v <- seq(r[1], r[2], length.out=5)
-				pal <- leaflet::colorNumeric(col, v, reverse = TRUE)
-				map <- leaflet::addLegend(map, legend, pal=pal, values=v, opacity=1, title=main[1],
-					  labFormat = leaflet::labelFormat(transform = function(x) sort(x, decreasing = TRUE)))
+				if (leg$type == "continuous") {
+					if (!all(hasMinMax(x))) setMinMax(x)
+					r <- minmax(x)
+					v <- seq(r[1], r[2], length.out=5)
+					pal <- leaflet::colorNumeric(col, v, reverse = TRUE)
+					map <- leaflet::addLegend(map, legend, pal=pal, values=v, opacity=1, title=main[1],
+						  labFormat = leaflet::labelFormat(transform = function(x) sort(x, decreasing = TRUE)))
+				} else {
+					map <- leaflet::addLegend(map, position=legend, colors=leg$cols, 
+							labels=levels(x)[[1]][,2], opacity=alpha, title=main)
+				}
 			}
 			if (panel) {
 				#map <- leaflet::addCircleMarkers(map, data=p, label=p$label, radius=1, opacity=1, col="red")
