@@ -1,4 +1,39 @@
 
+
+hexcols <- function(out) {
+
+	get_col <- function(cols, alpha) {
+		if (isTRUE(alpha < 255)) {
+			grDevices::rgb(t(grDevices::col2rgb(cols, alpha=TRUE)), alpha=alpha, maxColorValue=255)
+		} else {
+			i <- !grepl("^#", cols)
+			cols[i] <- grDevices::rgb(t(grDevices::col2rgb(cols[i], alpha=FALSE)), maxColorValue=255)	
+			cols
+		}
+	}
+
+	if (NCOL(out$cols) == 1) {
+		out$cols <- get_col(out$cols, out$alpha)
+	} else if (NCOL(out$cols) == 2) {
+		out$cols[,2] <- get_col(out$cols[,2], out$alpha)
+	} else if (NCOL(out$cols) == 3) {
+		out$cols[,3] <- get_col(out$cols[,3], out$alpha)
+	}
+	
+	out
+
+}
+
+
+.default.pal <- function() {
+	opt.pal <- options("terra.pal")[[1]]
+	if (is.null(opt.pal))  {
+		map.pal("viridis", 100)
+	} else {
+		opt.pal
+	}
+}
+
 .as.raster.rgb <- function(out, x) {
 
 	if (is.null(out$rgb$scale)) {
@@ -51,9 +86,11 @@
 	RGB <- stats::na.omit(RGB)
 	naind <- as.vector( attr(RGB, "na.action") )
 
-	if (ncol(RGB) == 4){
-		alpha <- RGB[,4] * 255
+	if (ncol(RGB) == 4) {
+		alpha <- RGB[,4]
 		RGB <- RGB[,-4]
+	} else {
+		alpha <- out$alpha
 	}
 
 	if (!is.null(naind)) {
@@ -61,9 +98,9 @@
 		if (is.null(out$rgb$bgalpha)) out$rgb$bgalpha <- 255
 		bg <- grDevices::rgb(bg[1], bg[2], bg[3], alpha=out$rgb$bgalpha, maxColorValue=255)
 		z <- rep( bg, times=ncell(x))
-		z[-naind] <- grDevices::rgb(RGB[,1], RGB[,2], RGB[,3], alpha=out$alpha, maxColorValue=scale)
+		z[-naind] <- grDevices::rgb(RGB[,1], RGB[,2], RGB[,3], alpha=alpha, maxColorValue=scale)
 	} else {
-		z <- grDevices::rgb(RGB[,1], RGB[,2], RGB[,3], alpha=out$alpha, maxColorValue=scale)
+		z <- grDevices::rgb(RGB[,1], RGB[,2], RGB[,3], alpha=alpha, maxColorValue=scale)
 	}
 	
 	out$r <- matrix(z, nrow=nrow(x), ncol=ncol(x), byrow=TRUE)
@@ -75,7 +112,8 @@
 
 	Z <- as.matrix(x, wide=TRUE)
 	Z[is.nan(Z) | is.infinite(Z)] <- NA
-
+	Z[] <- round(Z, 12)
+	
 # loss of precision
 #	z <- stats::na.omit(round(as.vector(Z), 12))
 	z <- stats::na.omit(as.vector(Z))
@@ -87,14 +125,14 @@
 		return(out)
 	}
 
-	uzi <- round(unique(z), 12)
+	uzi <- unique(z)
 
 	if (type == "depends") {
 		if (length(uzi) < 9) {
-			return (.as.raster.classes(out, x))
+			return (.as.raster.classes(out, x, Z=Z))
 		}
 	} else if ((length(uzi) == 1) && is.null(out$range)) {
-		return (.as.raster.classes(out, x))
+		return (.as.raster.classes(out, x, Z=Z))
 	}
 
 	if (is.null(out$range)) {
@@ -102,7 +140,35 @@
 #		out$fill_range <- FALSE
 	} else {
 		stopifnot(length(out$range) == 2)
-		stopifnot(out$range[2] > out$range[1])
+		if (out$fill_range) {
+			out$range_filled <- c(FALSE, FALSE)
+			if (!is.na(out$range[1])) {
+				if (out$range[1] > min(z)) {
+					out$range_filled[1] <- TRUE
+					Z[ Z < out$range[1] ] <- out$range[1]
+				} 
+			} else {
+				out$range[1] <- min(z, na.rm=TRUE)
+			}
+			if (!is.na(out$range[2])) {
+				if (out$range[2] < max(z)) {
+					Z[ Z > out$range[2] ] <- out$range[2]
+					out$range_filled[2] <- TRUE
+				} 
+			} else {
+				out$range[2] <- max(z, na.rm=TRUE)
+			}
+		} else {
+			if (all(is.na(out$range))) {
+				out$range <- range(z)
+			} else if (is.na(out$range[1])) {
+				out$range[1] <- min(z)
+			} else if (is.na(out$range[2])) {
+				out$range[2] <- max(z)
+			}
+		}
+		
+		if (!any(out$range_filled)) out$fill_range <- FALSE
 	}
 
 	breaks <- .get_breaks(z, length(out$cols), "eqint", out$range)
@@ -114,8 +180,13 @@
 #	} else {
 #		out$frange <- out$range	
 #	}
-	Z[] <- out$cols[as.integer(cut(Z, breaks, include.lowest=TRUE, right=FALSE))]
 
+	if (length(breaks) == 1) {
+		Z[] <- out$cols[ceiling(length(out$cols)/2)]
+	} else {
+		Z[] <- out$cols[as.integer(cut(as.numeric(Z), breaks, include.lowest=TRUE, right=FALSE))]
+	}
+	
 	out$r <- as.raster(Z)
 
 	out$legend_type <- "continuous"
@@ -142,10 +213,12 @@ prettyNumbs <- function(x, digits) {
 	gsub("\\.$", "", x)
 }
 
-.as.raster.classes <- function(out, x, ...) {
+.as.raster.classes <- function(out, x, Z=NULL, ...) {
 
-	Z <- as.matrix(x, wide=TRUE)
-	Z[is.nan(Z) | is.infinite(Z)] <- NA
+	if (is.null(Z)) {
+		Z <- as.matrix(x, wide=TRUE)
+		Z[is.nan(Z) | is.infinite(Z)] <- NA
+	}
 	if (all(is.na(Z))) {
 		#out$values = FALSE
 		out$range <- c(NA, NA)
@@ -197,8 +270,8 @@ prettyNumbs <- function(x, digits) {
 	stopifnot(length(out$leg$legend) == length(out$levels))
 	nlevs <- length(levs)
 
+
 	if (NCOL(out$cols) == 2) {
-		out$cols[,2] <- as.character(out$cols[,2])
 		i <- match(Z, as.numeric(levs))
 		Z[] <- out$cols[,2][i]
 		i <- match(as.numeric(levs), out$cols[,1])
@@ -362,9 +435,9 @@ prettyNumbs <- function(x, digits) {
 
 	if (!is.null(out$leg$digits)) {
 #		out$leg$legend <- substr(formatC(levs, digits=digits, format = "f", flag="#"), 1, digits+1)
-		fz <- cut(Z, out$breaks, include.lowest=TRUE, right=FALSE, dig.lab=out$leg$digits)
+		fz <- cut(as.numeric(Z), out$breaks, include.lowest=TRUE, right=FALSE, dig.lab=out$leg$digits)
 	} else {
-		fz <- cut(Z, out$breaks, include.lowest=TRUE, right=FALSE)
+		fz <- cut(as.numeric(Z), out$breaks, include.lowest=TRUE, right=FALSE)
 	}
 
 
@@ -380,6 +453,7 @@ prettyNumbs <- function(x, digits) {
 	} else {
 		cols <- rep_len(cols, nlevs)
 	}
+	
 	#out$cols <- cols
 	out$leg$fill <- cols
 	#out$leg$levels <- levels(fz)
@@ -393,6 +467,12 @@ prettyNumbs <- function(x, digits) {
 			m <- prettyNumbs(m, out$leg$digits)
 		}
 		m <- apply(m, 1, function(i) paste(i, collapse=" - "))
+		m <- gsub("-Inf -", "<=", m)
+		i <- grep("- Inf", m)
+		if (length(i) == 1) {
+			m[i] <- gsub("- Inf", "", m[i])
+			m[i] <- paste(">", m[i])				
+		}	
 		out$leg$legend <- m
 	}
 	out$leg$digits <- NULL
@@ -400,13 +480,11 @@ prettyNumbs <- function(x, digits) {
 }
 
 
-
 .as.raster.interval <- function(out, x, ...) {
 
 	out$legend_type <- "classes"
 
 	if (NCOL(out$cols) == 3) {
-		out$cols[,3] <- as.character(out$cols[,3])
 		rcl <- cbind(as.matrix(out$cols[,1:2]), 1:nrow(out$cols))
 		x <- classify(x, rcl, include.lowest=TRUE, others=NA)
 		m <- apply(out$cols[,1:2], 1, function(i) paste(i, collapse=" - "))
@@ -462,7 +540,7 @@ prettyNumbs <- function(x, digits) {
 		arglist <- c(list(x=x$lim[1:2], y=x$lim[3:4], type="n", xlab="", ylab="", asp=x$asp, xaxs=x$xaxs, yaxs=x$yaxs, axes=FALSE), x$dots)
 		do.call(plot, arglist)
 		if (!is.null(x$background)) {
-			graphics::rect(x$lim[1], x$lim[3], x$lim[2], x$lim[4], col=x$background)			
+			graphics::rect(x$lim[1], x$lim[3], x$lim[2], x$lim[4], col=x$background, border=TRUE)			
 		}
 	}
 	if (!x$values) {
@@ -524,7 +602,7 @@ prettyNumbs <- function(x, digits) {
 		}
 		if (isTRUE(x$halo.main)) {
 			.halo(x$loc.main[1], x$loc.main[2], x$main, pos=pos, offset=x$line.main, cex=x$cex.main, 
-				font=x$font.main, col=x$col.main, xpd=TRUE)
+				font=x$font.main, col=x$col.main, xpd=TRUE, hc=x$halo.main.hc, hw=x$halo.main.hw)
 		} else {
 			text(x$loc.main[1], x$loc.main[2], x$main, pos=pos, offset=x$line.main, cex=x$cex.main, 
 				font=x$font.main, col=x$col.main, xpd=TRUE)
@@ -540,24 +618,28 @@ prettyNumbs <- function(x, digits) {
 
 .prep.plot.data <- function(x, type, cols, mar=NULL, draw=FALSE,
   interpolate=FALSE, legend=TRUE, legend.only=FALSE, pax=list(), plg=list(),
-  levels=NULL, add=FALSE, range=NULL, breaks=NULL, breakby="eqint",
+  levels=NULL, add=FALSE, range=NULL, fill_range=FALSE, breaks=NULL, breakby="eqint",
   coltab=NULL, cats=NULL, xlim=NULL, ylim=NULL, ext=NULL, colNA=NA, alpha=NULL, reset=FALSE,
   sort=TRUE, decreasing=FALSE, grid=FALSE, las=0, all_levels=FALSE, decimals=NULL, background=NULL,
   xlab="", ylab="", cex.lab=0.8, line.lab=1.5, asp=NULL, yaxs="i", xaxs="i", main="", cex.main=1.2, 
   line.main=0.5, font.main=graphics::par()$font.main, col.main = graphics::par()$col.main, loc.main=NULL, 
-  halo=FALSE, axes=TRUE, box=TRUE, cex=1, maxcell=500000, buffer=FALSE, clip=TRUE, 
+  halo=FALSE, hc="white", hw=0.1, axes=TRUE, box=TRUE, cex=1, maxcell=500000, buffer=FALSE, clip=TRUE, 
   # for rgb 
-  stretch=NULL, scale=NULL, bgalpha=NULL, zlim=NULL, zcol=NULL,  ...) {
+  stretch=NULL, scale=NULL, bgalpha=NULL, zlim=NULL, zcol=NULL, overview=NULL, ...) {
 #cex is catch and kill
 
 	out <- list()
 	e <- out$lim <- out$ext <- as.vector(ext(x))
 	hadWin <- hasWin <- FALSE
+	if (add && is.null(ext)) {
+		ext <- unlist(get.clip())[1:4]
+	}
+	
 	if ((!is.null(ext)) || (!is.null(xlim)) || (!is.null(ylim))) {
 		if (!is.null(ext)) {
 			ext <- ext(ext)
-			#e <- as.vector(align( intersect(ext, ext(x)), x))
-			e <- as.vector(align(ext, x))
+			#e <- as.vector(align(ext, x))
+			e <- as.vector(ext)
 			out$lim <- out$ext <- e
 		} 
 		if (!is.null(xlim)) {
@@ -577,13 +659,25 @@ prettyNumbs <- function(x, digits) {
 		window(x) <- out$ext <- w
 	} 
 	if (ncell(x) > 1.1 * maxcell) {
+			
+		if (is.null(overview)) {	
+			if (grepl("https://", tolower(sources(x))[1])) {
+				overview <- TRUE
+			} else {
+				overview <- FALSE
+			}
+		}
+	
 		if (inherits(alpha, "SpatRaster")) {
 			if (nlyr(alpha) > 1) {
 				alpha <- alpha[[1]]
 			}
-			alpha <- spatSample(alpha, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
+#			alpha <- spatSample(alpha, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
+			alpha <- sampleRaster(alpha, maxcell, method="regular", replace=FALSE, ext=NULL, warn=FALSE, overview=overview)
 		}
-		x <- spatSample(x, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
+#		x <- spatSample(x, maxcell, method="regular", as.raster=TRUE, warn=FALSE)
+		x <- sampleRaster(x, maxcell, method="regular", replace=FALSE, ext=NULL, warn=FALSE, overview=overview)
+
 		out$lim <- out$ext <- as.vector(ext(x))
 	}
 	
@@ -617,13 +711,21 @@ prettyNumbs <- function(x, digits) {
 		out$asp <- asp
 		out$lonlat <- FALSE
 	}
+	out$cols <- cols
 	if (!is.null(alpha)) {
 		if (!inherits(alpha, "SpatRaster")) {
-			alpha <- alpha[1] * 255
-			cols <- grDevices::rgb(t(grDevices::col2rgb(cols)), alpha=alpha, maxColorValue=255)
-		} 
+			out$alpha <- alpha[1]
+			if ((alpha < 0) || (alpha > 1)) {
+				warn("plot", "alpha should be between 0 and 1")
+				out$alpha <- 255
+			} else {
+				out$alpha <- out$alpha[1] * 255
+			}
+			out <- hexcols(out)
+		}
 	} else {
-		alpha <- 255
+		out$alpha <- 255
+		out <- hexcols(out)
 	}
 	out$rgb$stretch <- stretch
 	out$rgb$scale <- scale
@@ -640,6 +742,9 @@ prettyNumbs <- function(x, digits) {
 	out$reset <- reset
 	out$main  <- main
 	out$halo.main <- halo
+	out$halo.main.hc <- hc
+	out$halo.main.hw <- hw
+
 	out$loc.main  <- loc.main
 	out$cex.main  <- cex.main
 	out$font.main <- font.main
@@ -650,7 +755,6 @@ prettyNumbs <- function(x, digits) {
 	out$yaxs <- yaxs
 	out$xlab <- xlab
 	out$ylab <- ylab 
-	out$cols <- cols
 	out$coltab <- coltab
 	out$cats <- cats
 	out$breaks <- breaks
@@ -696,7 +800,7 @@ prettyNumbs <- function(x, digits) {
 		} else {
 			out$interpolate <- isTRUE(interpolate)
 			out$range <- range
-			#out$fill_range <- fill_range
+			out$fill_range <- fill_range
 			out <- .as.raster.continuous(out, x, type)
 		}
 
@@ -728,7 +832,7 @@ prettyNumbs <- function(x, digits) {
 
 		if (!is.null(colNA)) {
 			if (!is.na(colNA) && out$values) {
-				out$colNA <- grDevices::rgb(t(grDevices::col2rgb(colNA)), alpha=alpha, maxColorValue=255)
+				out$colNA <- grDevices::rgb(t(grDevices::col2rgb(colNA)), alpha=out$alpha, maxColorValue=255)
 				out$r[is.na(out$r)] <- out$colNA
 			}
 		}
@@ -754,13 +858,15 @@ prettyNumbs <- function(x, digits) {
 
 
 setMethod("plot", signature(x="SpatRaster", y="numeric"),
-	function(x, y=1, col, type=NULL, mar=NULL, legend=TRUE, axes=!add, plg=list(), pax=list(), maxcell=500000, smooth=FALSE, range=NULL, levels=NULL, all_levels=FALSE, breaks=NULL, breakby="eqint", fun=NULL, colNA=NULL, alpha=NULL, sort=FALSE, decreasing=FALSE, grid=FALSE, ext=NULL, reset=FALSE, add=FALSE, buffer=FALSE, background=NULL, box=axes, clip=TRUE, ...) {
+	function(x, y=1, col, type=NULL, mar=NULL, legend=TRUE, axes=!add, plg=list(), pax=list(), maxcell=500000, smooth=FALSE, range=NULL, fill_range=FALSE, levels=NULL, all_levels=FALSE, breaks=NULL, breakby="eqint", fun=NULL, colNA=NULL, alpha=NULL, sort=FALSE, decreasing=FALSE, grid=FALSE, ext=NULL, reset=FALSE, add=FALSE, buffer=FALSE, background=NULL, box=axes, clip=TRUE, overview=NULL, ...) {
 
 		old.mar <- graphics::par()$mar
 		on.exit(graphics::par(mar=old.mar))
 
+		
 		y <- round(y)
 		hasRGB <- FALSE		
+
 		if (has.RGB(x) && ((is.null(type) && (y[1] < 0)))) {
 			type <- "rgb"
 			legend <- FALSE
@@ -771,16 +877,18 @@ setMethod("plot", signature(x="SpatRaster", y="numeric"),
 			hasRGB <- TRUE
 			y <- RGB(x)
 		}
-		stopifnot((min(y) > 0) & (max(y) <= nlyr(x)))
+		nlx <- nlyr(x)
+		stopifnot((min(y) > 0) & (max(y) <= nlx))
 
 		if ((!hasRGB) && (length(y) > 1)) {
+
 			x <- x[[y]]
 			if (inherits(alpha, "SpatRaster")) {
 				if (nlyr(alpha) > 1) {
 					alpha <- alpha[[y]]
 				}
 			}
-			plot(x, col=col, type=type, mar=mar, legend=legend, axes=axes, plg=plg, pax=pax, maxcell=2*maxcell/length(y), smooth=smooth, range=range, levels=levels, all_levels=all_levels, breaks=breaks, breakby=breakby, fun=fun, colNA=colNA, alpha=alpha, grid=grid, sort=sort, decreasing=decreasing, ext=ext, reset=reset, add=add, buffer=buffer, background=background, box=box, clip=clip, ...)
+			plot(x, col=col, type=type, mar=mar, legend=legend, axes=axes, plg=plg, pax=pax, maxcell=2*maxcell/length(y), smooth=smooth, range=range, fill_range=fill_range, levels=levels, all_levels=all_levels, breaks=breaks, breakby=breakby, fun=fun, colNA=colNA, alpha=alpha, grid=grid, sort=sort, decreasing=decreasing, ext=ext, reset=reset, add=add, buffer=buffer, background=background, box=box, clip=clip, overview=overview, ...)
 			return(invisible())
 		} else {
 			x <- x[[y]]
@@ -798,7 +906,8 @@ setMethod("plot", signature(x="SpatRaster", y="numeric"),
 		}
 
 		if (missing(col)) {
-			col <- rev(grDevices::terrain.colors(255))
+			col <- .default.pal()
+			#col <- rev(grDevices::terrain.colors(255))
 		} else if (inherits(col, "data.frame")) {
 			if (ncol(col) == 2) {
 				type <- "classes"
@@ -855,7 +964,7 @@ setMethod("plot", signature(x="SpatRaster", y="numeric"),
 			}
 		}
 
-		x <- .prep.plot.data(x, type=type, cols=col, mar=mar, draw=TRUE, plg=plg, pax=pax, legend=isTRUE(legend), axes=isTRUE(axes), coltab=coltab, cats=cats, interpolate=smooth, levels=levels, range=range, colNA=colNA, alpha=alpha, reset=reset, grid=grid, sort=sort, decreasing=decreasing, ext=ext, all_levels=all_levels, breaks=breaks, breakby=breakby, add=add, buffer=buffer, background=background, box=box, maxcell=maxcell, clip=clip, ...)
+		x <- .prep.plot.data(x, type=type, cols=col, mar=mar, draw=TRUE, plg=plg, pax=pax, legend=isTRUE(legend), axes=isTRUE(axes), coltab=coltab, cats=cats, interpolate=smooth, levels=levels, range=range, fill_range=fill_range, colNA=colNA, alpha=alpha, reset=reset, grid=grid, sort=sort, decreasing=decreasing, ext=ext, all_levels=all_levels, breaks=breaks, breakby=breakby, add=add, buffer=buffer, background=background, box=box, maxcell=maxcell, clip=clip, overview=overview, ...)
 
 		if (!is.null(fun)) {
 			if (!is.null(formals(fun))) {
@@ -870,18 +979,24 @@ setMethod("plot", signature(x="SpatRaster", y="numeric"),
 
 
 setMethod("plot", signature(x="SpatRaster", y="missing"),
-	function(x, y, main, mar=NULL, nc, nr, maxnl=16, maxcell=500000, ...)  {
+	function(x, y, main, mar=NULL, nc, nr, maxnl=16, maxcell=500000, add=FALSE, ...)  {
 
 		if (has.RGB(x)) {
 			if (missing(main)) main = ""
-			p <- plot(x, -1, main=main, mar=mar, maxcell=maxcell, ...)
+			p <- plot(x, -1, main=main, mar=mar, maxcell=maxcell, add=add, ...)
 			return(invisible(p))
 		}
 
 		nl <- max(1, min(nlyr(x), maxnl))
+
+		if (add && (nl > 1)) {
+			nl <- 1
+			warn("plot", "adding the first layer of x")
+		}
+
 		if (nl==1) {
 			if (missing(main)) main = ""
-			out <- plot(x, 1, maxcell=maxcell, main=main[1], mar=mar, ...)
+			out <- plot(x, 1, maxcell=maxcell, main=main[1], mar=mar, add=add, ...)
 			return(invisible(out))
 		}
 
@@ -905,7 +1020,7 @@ setMethod("plot", signature(x="SpatRaster", y="missing"),
 			main <- rep_len(main, nl)
 		}
 		for (i in 1:nl) {
-			plot(x, i, main=main[i], mar=mar, maxcell=maxcell, ...)
+			plot(x, i, main=main[i], mar=mar, maxcell=maxcell, add=add, ...)
 		}
 	}
 )
