@@ -43,14 +43,14 @@ setMethod("dots", signature(x="SpatVector"),
 #	cols <- out$cols
 #	if (is.null(cols)) cols = rep("black", n)
 
-#	g <- lapply(x@ptr$linesList(), function(i) { names(i)=c("x", "y"); i } )
+#	g <- lapply(x@pntr$linesList(), function(i) { names(i)=c("x", "y"); i } )
 
 #	g <- geom(x, df=TRUE)
 #	g <- split(g, g[,1])
 #	g <- lapply(g, function(x) split(x[,3:4], x[,2]))
 #	n <- length(g)
 
-	g <- x@ptr$linesList()
+	g <- x@pntr$linesList()
 	lty <- rep_len(lty, n)
 	lwd <- rep_len(lwd, n)
 	for (i in 1:n) {
@@ -119,7 +119,7 @@ setMethod("dots", signature(x="SpatVector"),
 #				# g[[i]][[1]] <- a
 #			}
 
-	g <- x@ptr$polygonsList()
+	g <- x@pntr$polygonsList()
 	if (is.null(out$leg$density)) {
 		for (i in seq_along(g)) {
 			for (j in seq_along(g[[i]])) {
@@ -144,18 +144,15 @@ setMethod("dots", signature(x="SpatVector"),
 
 	if (out$leg$geomtype == "points") {
 		points(x, col=out$main_cols, cex=out$cex, pch=pch, ...)
-		#if (!out$add) {
-		#	e <- out$lim
-		#}
-		out$leg$pch = pch
-		out$leg$pt.cex = out$cex
+		if (is.null(out$leg$pch)) out$leg$pch <- pch
+		if (is.null(out$leg$pt.cex)) out$leg$pt.cex = out$cex
 	} else if (out$leg$geomtype == "polygons") {
 		out <- .plotPolygons(x, out, density=out$leg$density, angle=out$leg$angle, lty=lty, lwd=lwd, ...)
 	} else {
 		# out <- .plotLines(x, out, ...)
 		lines(x, col=out$main_cols, lty=lty, lwd=lwd, ...)
-		out$leg$lwd = lwd
-		out$leg$lty = lty
+		if (is.null(out$leg$lwd)) out$leg$lwd = lwd
+		if (is.null(out$leg$lty)) out$leg$lty = lty
 	}
 	out
 }
@@ -433,11 +430,11 @@ setMethod("dots", signature(x="SpatVector"),
 
 
 .prep.vect.data <- function(x, y, type=NULL, cols=NULL, mar=NULL, legend=TRUE,
-	legend.only=FALSE, levels=NULL, add=FALSE, range=NULL, breaks=NULL, breakby="eqint",
+	legend.only=FALSE, levels=NULL, add=FALSE, range=NULL, fill_range=FALSE, breaks=NULL, breakby="eqint",
 	xlim=NULL, ylim=NULL, colNA=NA, alpha=NULL, axes=TRUE, buffer=TRUE, background=NULL,
 	pax=list(), plg=list(), ext=NULL, grid=FALSE, las=0, sort=TRUE, decreasing=FALSE, values=NULL,
 	box=TRUE, xlab="", ylab="", cex.lab=0.8, line.lab=1.5, yaxs="i", xaxs="i", main="", cex.main=1.2, line.main=0.5, font.main=graphics::par()$font.main, col.main = graphics::par()$col.main, 
-	density=NULL, angle=45, border="black", dig.lab=3, cex=1, clip=TRUE, leg_i=1, asp=NULL, ...) {
+	density=NULL, angle=45, border="black", dig.lab=3, cex=1, clip=TRUE, leg_i=1, asp=NULL, xpd=NULL, ...) {
 
 	out <- list()
 	out$blank <- FALSE
@@ -483,7 +480,9 @@ setMethod("dots", signature(x="SpatVector"),
 			stopifnot(length(ylim) == 2)
 			out$lim[3:4] <- sort(ylim)
 		}
-	} 
+	} else if ((!add) && is.null(xpd)) {
+		xpd <- TRUE
+	}
 	out$ngeom <- nrow(x)
 	out$clip <- isTRUE(clip)
 
@@ -579,6 +578,7 @@ setMethod("dots", signature(x="SpatVector"),
 		}
 		out$range_set <- FALSE
 	}
+	out$fill_range <- fill_range
 	out$v <- v
 
 	if (!is.logical(sort)) {
@@ -612,7 +612,8 @@ setMethod("dots", signature(x="SpatVector"),
 				cols <- "black"
 			}
 		} else {
-			cols <- rev(grDevices::rainbow(100, start=.1, end=0.9))
+			cols <- .default.pal()
+#			cols <- rev(grDevices::rainbow(100, start=.1, end=0.9))
 		}
 	}
 	if (!is.null(alpha)) {
@@ -649,7 +650,7 @@ setMethod("dots", signature(x="SpatVector"),
 		}
 	}
 
-	.plot.vect.map(x, out, ...)
+	.plot.vect.map(x, out, xpd=xpd, ...)
 }
 
 
@@ -752,3 +753,46 @@ setMethod("plot", signature(x="SpatVectorProxy", y="missing"),
 	}
 )
 
+
+
+setMethod("plot", signature(x="SpatVectorCollection", y="missing"),
+	function(x, y, main, mar=NULL, nc, nr, maxnl=16, ...) {
+		nl <- max(1, min(length(x), maxnl))
+
+		if (nl==1) {
+			if (missing(main)) main = ""
+			out <- plot(x[1], main=main[1], mar=mar, ...)
+			return(invisible(out))
+		}
+
+		nrnc <- .get_nrnc(nr, nc, nl)
+		old.par <- graphics::par(no.readonly = TRUE)
+		on.exit(graphics::par(old.par))
+		if (is.null(mar)) {
+			mar=c(1.5, 1, 2.5, 3)
+		}
+		graphics::par(mfrow=nrnc)
+
+		if (missing("main")) {
+			main <- names(x)
+		} else {
+			main <- rep_len(main, nl)
+		}
+		for (i in 1:nl) {
+			plot(x[i], main=main[i], mar=mar, ...)
+		}
+	}
+)
+
+setMethod("plot", signature(x="SpatVectorCollection", y="numeric"),
+	function(x, y, main, mar=NULL, ext=NULL, ...) {
+		y <- round(y)
+		if ((y > 0) && (y <= length(x))) {
+			if (is.null(ext)) ext <- ext(x)
+			plot(x[y], main=main, mar=mar, ext=ext, ...)
+		} else {
+			error("plot", "y should be between 1 and length(x)")
+		}
+	}
+)
+
