@@ -32,23 +32,29 @@ new_rast <- function(nrows=10, ncols=10, nlyrs=1, xmin=0, xmax=1, ymin=0, ymax=1
 	#check_proj4_datum(crs)
 
 	r <- methods::new("SpatRaster")
-	r@cpp <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
+	r@pntr <- SpatRaster$new(c(nrows, ncols, nlyrs), e, crs)
 	r <- messages(r, "rast")
 
 	if (!missing(resolution)) {
 		res(r) <- resolution
 	}
-	if (!missing(names)) {
-		names(r) <- names
-	}
 	if (!missing(vals)) {
 		if (length(vals) == 1) {
 			if (is.na(vals[1])) {
 				vals <- as.numeric(NA)
 			}
 		}
+		if (!is.null(ncol(vals))) {
+			nms <- colnames(vals)
+			if (length(nms) == nlyr(r)) {
+				names(r) <- nms
+			}
+		}
 		values(r) <- vals
 	}
+	if (!missing(names)) {
+		names(r) <- names
+	}
 	if (!missing(time)) {
 		time(r) <- time
 	}
@@ -66,11 +72,49 @@ setMethod("rast", signature(x="missing"),
 )
 
 
+
+
+rast_from_image <- function(x) { 
+	# list representing an "image"
+	if (! all(dim(x$z) == c(length(x$x), length(x$y)))) {
+		error("rast", '"z" does not have the right dimensions') 
+	}
+	# omitted "-1" bug fix by Barry Rowlingson 
+	resx <- ( x$x[length(x$x)] - x$x[1] ) / (length(x$x)-1)
+	resy <- ( x$y[length(x$y)] - x$y[1] ) / (length(x$y)-1)
+	xmn <- min(x$x) - 0.5 * resx
+	xmx <- max(x$x) + 0.5 * resx
+	ymn <- min(x$y) - 0.5 * resy
+	ymx <- max(x$y) + 0.5 * resy
+
+	dx <- abs(max(abs((x$x[-1] - x$x[-length(x$x)])) / resx) - 1)
+	dy <- abs(max(abs((x$y[-1] - x$y[-length(x$y)])) / resy) - 1)
+	if (is.na(dx) | is.na(dy)) {
+		error("rast", "missing values in coordinates")
+	} 
+	if (dx > 0.01 | dy > 0.01) {
+		error("rast", "data are not on a regular grid")
+	}
+#	if (xmn > -360.1 & xmx < 360.1 & ymn > -90.1 & ymx < 90.1) { 
+#		crs <- "+proj=longlat +datum=WGS84"
+#	} else {
+#		crs <- ""
+#	}
+	x <- t(x$z)
+	x <- x[nrow(x):1, ]
+	rast(x, extent=ext(xmn, xmx, ymn, ymx), crs="")
+}
+
+
+
 setMethod("rast", signature(x="list"),
 	function(x, warn=TRUE) {
 		i <- sapply(x, function(i) inherits(i, "SpatRaster"))
 		if (!all(i)) {
 			if (!any(i)) {
+				if ((length(x) == 3) && all(c("x", "y", "z") %in% names(x))) {
+					return(rast_from_image(x))
+				}
 				error("rast,list", "none of the elements of x are a SpatRaster")
 			} else {
 				warn("rast", sum(!i), " out of ", length(x), " elements of x are not a SpatRaster")
@@ -84,7 +128,7 @@ setMethod("rast", signature(x="list"),
 		}
 		opt <- spatOptions()
 		for (i in 2:length(x)) {
-			out@cpp$addSource(x[[i]]@cpp, warn, opt)
+			out@pntr$addSource(x[[i]]@pntr, warn, opt)
 		}
 		out <- messages(out, "rast")
 		lnms <- names(x)
@@ -113,13 +157,25 @@ setMethod("rast", signature(x="SpatExtent"),
 		dots$xmax=x[2]
 		dots$ymin=x[3]
 		dots$ymax=x[4]
-		do.call(new_rast, dots)
+		r <- do.call(new_rast, dots)
+		# re-apply extent to avoid mimute differences
+		ext(r) <- x
+		r
 	}
 )
 
 
 setMethod("rast", signature(x="SpatVector"),
-	function(x, ...) {
+	function(x, type="", ...) {
+	
+		if (type == "xyz") {
+			if (geomtype(x) != "points") {
+				error("rast", "xyz can only be used with points")
+			}
+			x <- data.frame(crds(x), data.frame(x))
+			return(rast(x, type="xyz", ...))
+		}
+	
 		dots <- list(...)
 		e <- ext(x)
 		dots$xmin=e[1]
@@ -158,6 +214,8 @@ setMethod("rast", signature(x="SpatVector"),
 		j <- file.exists(dirname(p))
 		x[j] <- p[j]
 	} else {
+		i <- !file.exists(p)
+		p[i] <- x[i]
 		return(p)
 	}
 	#if (identical(basename(x), x)) {
@@ -170,7 +228,7 @@ setMethod("rast", signature(x="SpatVector"),
 }
 
 setMethod("rast", signature(x="character"),
-	function(x, subds=0, lyrs=NULL, drivers=NULL, opts=NULL, win=NULL, snap="near", vsi=FALSE, raw=FALSE) {
+	function(x, subds=0, lyrs=NULL, drivers=NULL, opts=NULL, win=NULL, snap="near", vsi=FALSE, raw=FALSE, noflip=FALSE) {
 
 		f <- .fullFilename(x, vsi=vsi)
 		if (length(f) == 0) {
@@ -192,13 +250,13 @@ setMethod("rast", signature(x="character"),
 		if (is.null(drivers)) drivers <- ""[0]
 		if (length(subds) == 0) subds = 0
 		if (is.character(subds)) {
-			#r@cpp <- SpatRaster$new(f, -1, subds, FALSE, 0[])
-			r@cpp <- SpatRaster$new(f, -1, subds, FALSE, drivers, opts, 0[])
+			#r@pntr <- SpatRaster$new(f, -1, subds, FALSE, 0[])
+			r@pntr <- SpatRaster$new(f, -1, subds, FALSE, drivers, opts, 0[], noflip)
 		} else {
-			r@cpp <- SpatRaster$new(f, subds-1, "", FALSE, drivers, opts, 0[])
+			r@pntr <- SpatRaster$new(f, subds-1, "", FALSE, drivers, opts, 0[], noflip)
 		}
 		r <- messages(r, "rast")
-		if (r@cpp$getMessage() == "ncdf extent") {
+		if (r@pntr$getMessage() == "ncdf extent") {
 			# could have used opts="IGNORE_XY_AXIS_NAME_CHECKS=YES"
 			test <- try(r <- .ncdf_extent(r, f), silent=TRUE)
 			if (inherits(test, "try-error")) {
@@ -229,6 +287,8 @@ setMethod("rast", signature(x="character"),
 
 multi <- function(x, subds=0, xyz=3:1, drivers=NULL, opts=NULL) {
 
+	noflip <- FALSE
+
 	x <- trimws(x)
 	x <- x[x!=""]
 	if (length(x) == 0) {
@@ -242,11 +302,11 @@ multi <- function(x, subds=0, xyz=3:1, drivers=NULL, opts=NULL) {
 	subds <- subds[1]
 
 	if (is.character(subds)) {
-		r@cpp <- SpatRaster$new(f, -1, subds, TRUE, drivers, opts, xyz-1)
+		r@pntr <- SpatRaster$new(f, -1, subds, TRUE, drivers, opts, xyz-1, isTRUE(noflip[1]))
 	} else {
-		r@cpp <- SpatRaster$new(f, subds-1, ""[0], TRUE, drivers, opts, xyz-1)
+		r@pntr <- SpatRaster$new(f, subds-1, ""[0], TRUE, drivers, opts, xyz-1, isTRUE(noflip[1]))
 	}
-	if (r@cpp$getMessage() == "ncdf extent") {
+	if (r@pntr$getMessage() == "ncdf extent") {
 		test <- try(r <- .ncdf_extent(r), silent=TRUE)
 		if (inherits(test, "try-error")) {
 			warn("rast", "GDAL did not find an extent. Cells not equally spaced?")
@@ -268,7 +328,7 @@ setMethod("rast", signature(x="SpatRaster"),
 		if (inherits(nlyrs, "SpatRaster")) {
 			error("rast", "use 'c()' to combine SpatRasters")
 		}
-		x@cpp <- x@cpp$geometry(nlyrs, props, keeptime, keepunits, tags)
+		x@pntr <- x@pntr$geometry(nlyrs, props, keeptime, keepunits, tags)
 		x <- messages(x, "rast")
 		if (!missing(names)) {
 			if (length(names) == nlyr(x)) names(x) <- names
@@ -289,7 +349,7 @@ setMethod("rast", signature(x="SpatRasterDataset"),
 			x[1]
 		} else {
 			r <- methods::new("SpatRaster")
-			r@cpp <- x@cpp$collapse()
+			r@pntr <- x@pntr$collapse()
 			nms <- names(x)
 			if (any(nms != "")) {
 				names(r) <- paste(rep(nms, nlyr(x)), names(r), sep="_")
@@ -324,7 +384,7 @@ setMethod("rast", signature(x="array"),
 			e <- c(0, dims[2], 0, dims[1])
 		}
 		crs <- character_crs(crs, "rast")
-		r@cpp <- SpatRaster$new(dims, e, crs)
+		r@pntr <- SpatRaster$new(dims, e, crs)
 		values(r) <- x
 		messages(r, "rast")
 	}
@@ -360,7 +420,10 @@ setMethod("rast", signature(x="ANY"),
 		xyz <- as.matrix(xyz)
 		xyz <- matrix(as.numeric(xyz), ncol=ncol(xyz), nrow=nrow(xyz))
 	}
+	
 	x <- sort(unique(xyz[,1]))
+#	x <- sort(unique(round(xyz[,1], digits+2)))
+	
 	if (length(x) == 1) {
 		error("rast", "cannot create a raster geometry from a single x coordinate")
 	}
@@ -379,13 +442,11 @@ setMethod("rast", signature(x="ANY"),
 	}
 
 	y <- sort(unique(xyz[,2]))
+#	y <- sort(unique(round(xyz[,2], digits+2)))
 	if (length(y) == 1) {
 		error("rast", "cannot create a raster geometry from a single y coordinate")
 	}
 	dy <- y[-1] - y[-length(y)]
-	# probably a mistake to use the line below
-	# Gareth Davies suggested that it be removed
-	# dy <- round(dy, digits)
 
 	ry <- min(dy)
 	for (i in 1:5) {
@@ -434,6 +495,7 @@ setMethod("rast", signature(x="ANY"),
 	}
 	nms <- names(x)
 	names(x)[1:3] <- c("x", "y", "l")
+	x <- as.data.frame(x) # so that this also works for tibble #1582
 	w <- stats::reshape(x, timevar="l", idvar=c("x", "y"), direction="wide")
 	w <- rast(w, type="xyz", digits=digits, crs=crs, extent=extent)
 	names(w) <- gsub(paste0(nms[4], "."), "", names(w))
@@ -518,7 +580,7 @@ setMethod("rast", signature(x="stars_proxy"),
 setMethod("NAflag<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		value <- as.numeric(value)
-		if (!(x@cpp$setNAflag(value))) {
+		if (!(x@pntr$setNAflag(value))) {
 			error("NAflag<-", "cannot set this value")
 		}
 		x
@@ -527,7 +589,7 @@ setMethod("NAflag<-", signature(x="SpatRaster"),
 
 setMethod("NAflag", signature(x="SpatRaster"),
 	function(x)  {
-		x@cpp$getNAflag()
+		x@pntr$getNAflag()
 	}
 )
 
