@@ -1,4 +1,108 @@
 
+regular_exact <- function(r, size) {
+	size <- round(size)
+	stopifnot(size > 0)
+	if (size >= ncell(r)) {
+		return(xyFromCell(r, 1:ncell(r)))
+	}
+	x <- (ymax(r) - ymin(r)) / (xmax(r) - xmin(r))
+	if (x < 1) {
+		nc <- min(max(1, size * 1/(1 + 1/x)), ncol(r))
+		nr <- (size / nc)
+	} else {
+		nr <- min(max(1, size * 1/(1 + x)), nrow(r))
+		nc <- (size / nr)
+	}
+
+	addrow <- addcol <- 0
+	floor <- TRUE
+
+	if ((nc%%1) >= (nr%%1)) {
+		nc <- round(nc)
+		if ((nc * ceiling(nr)) <= size) {
+			nr <- ceiling(nr)
+			floor <- FALSE
+		} else {
+			nr <- floor(nr)
+			if ((nr * nc) > size) {
+				nc <- nc-1
+			}
+		}
+		d <- size - nr * nc
+		if (d > 0) {
+			if (floor) {
+				addcol <- d
+			} else {
+				addrow <- d 
+			}
+		}
+	} else {
+		nr <- round(nr)
+		if ((nr * ceiling(nc)) <= size) {
+			nc <- ceiling(nc)
+			floor <- FALSE
+		} else {
+			nc <- floor(nc)
+			if ((nr * nc) > size) {
+				nr <- nr-1
+			}
+		}
+		d <- size - nr * nc
+		if (d > 0) {
+			if (floor) {
+				addcol <- d
+			} else {
+				addrow <- d	
+			}
+		}
+	}
+
+	if (addcol > 0) {
+		ncs <- rep(nr, nc+1)
+		b <- (nr+addcol)/2
+		ncs[1] <- ceiling(b)
+		ncs[nc+1] <- floor(b)
+		nc=ncs
+	} else if (addrow > 0) {
+		nrs <- rep(nc, nr+1)
+		b <- (nc+addrow)/2
+		nrs[1] <- ceiling(b)
+		nrs[nr+1] <- floor(b)
+		nr=nrs
+	}
+	nnr <- length(nr)
+	nnc <- length(nc)
+	
+	if ((nnr == 1) && (nnc == 1)) {
+		dx <- (xmax(r) - xmin(r)) / nc
+		x <- xmin(r) + dx/2 + (0:(nc-1)) * dx 
+		dy <- (ymax(r) - ymin(r)) / nr
+		y <- ymin(r) + dy/2 + (0:(nr-1)) * dy 
+		expand.grid(x=x, y=y)
+	} else if (nnc == 1) {	
+		dy <- (ymax(r) - ymin(r)) / nnr
+		y <- ymin(r) + dy/2 + (0:(nnr-1)) * dy 
+		out <- vector("list", nnr)
+		for (i in 1:nnr) {
+			dx <- (xmax(r) - xmin(r)) / nr[i]
+			x <- xmin(r) + dx/2 + (0:(nr[i]-1)) * dx 
+			out[[i]] <- expand.grid(x=x, y=y[i])
+		}
+		do.call(rbind, out)
+	} else {
+		dx <- (xmax(r) - xmin(r)) / nnc
+		x <- xmin(r) + dx/2 + (0:(nnc-1)) * dx 
+		out <- vector("list", nnc)
+		for (i in 1:nnc) {
+			dy <- (ymax(r) - ymin(r)) / nc[i]
+			y <- ymin(r) + dy/2 + (0:(nc[i]-1)) * dy 
+			out[[i]] <- expand.grid(x=x[i], y=y)
+		}
+		do.call(rbind, out)
+	}
+}
+
+
 sampleWeights <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE, values=TRUE, cells=FALSE, xy=FALSE, ext=NULL) {
 	if (!is.null(ext)) {
 		x <- crop(x, ext)
@@ -29,14 +133,20 @@ sampleWeights <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE, v
 }
 
 
-sampleStratMemory <- function(x, size, replace, lonlat, ext=NULL, weights=NULL, warn=TRUE) {
+sampleStratMemory <- function(x, size, replace, lonlat, ext=NULL, weights=NULL, warn=TRUE, each) {
 	if (!is.null(ext)) {
+		xold <- rast(x)
 		x <- crop(x, ext)
+		cells <- cells(xold, ext)
 		if (!is.null(weights)) {
 			weights <- crop(weights, ext)
 		}
+	} else {
+		cells <- 1:ncell(x)	
 	}
 	
+	doprob <- TRUE
+	prob <- NULL
 	if (!is.null(weights)) {
 		if (!inherits(weights, "SpatRaster")) {
 			error("spatSample", "weights must be a SpatRaster")
@@ -44,62 +154,55 @@ sampleStratMemory <- function(x, size, replace, lonlat, ext=NULL, weights=NULL,
 		if (!compareGeom(x, weights)) {
 			error("spatSample", "geometry of weights does not match the geometry of x")
 		}
-		v <- cbind(cell=1:ncell(x), values(x), values(weights))
-		v <- v[!is.na(v[,2]), ]
-		uv <- sort(unique(v[,2]))
-		ys <- vector(mode="list", length=length(uv))
-		for (i in seq_len(length(uv))) {
-			vv <- v[v[,2] == uv[i], ]
-			if (replace) {
-				s <- sample.int(nrow(vv), size, prob=vv[,3], replace=TRUE)
-			} else {
-				s <- sample.int(nrow(vv), min(size, nrow(vv)), prob=vv[,3], replace=FALSE)
-			}
-			ys[[i]] <- vv[s,-3]
-		}
+		v <- na.omit(cbind(cell=cells, values(x), values(weights)))		
 	} else if (lonlat) {
-		v <- cbind(cell=1:ncell(x), values(x), abs(cos(pi * values(init(x, "y")) / 360)))
-		v <- v[!is.na(v[,2]), ]
-		uv <- sort(unique(v[,2]))
-		ys <- vector(mode="list", length=length(uv))
-		for (i in seq_len(length(uv))) {
-			vv <- v[v[,2] == uv[i], ]
-			if (replace) {
-				s <- sample.int(nrow(vv), size, prob=vv[,3], replace=TRUE)
-			} else {
-				s <- sample.int(nrow(vv), min(size, nrow(vv)), prob=vv[,3], replace=FALSE)
-			}
-			ys[[i]] <- vv[s,-3]
-		}
+		v <- cbind(cell=cells, values(x), abs(cos(pi * values(init(x, "y")) / 360)))
 	} else {
-		v <- cbind(cell=1:ncell(x), values(x))
-		v <- v[!is.na(v[,2]), ]
-		uv <- sort(unique(v[,2]))
-		ys <- vector(mode="list", length=length(uv))
-		for (i in seq_len(length(uv))) {
-			vv <- v[v[,2] == uv[i], ]
-			if (replace) {
-				s <- sample.int(nrow(vv), size, replace=TRUE)
-			} else {
-				s <- sample.int(nrow(vv), min(size, nrow(vv)), replace=FALSE)
-			}
-			ys[[i]] <- vv[s,]
+		v <- cbind(cell=cells, values(x))
+		doprob <- FALSE
+	}
+
+	v <- v[!is.na(v[,2]), ]
+	uv <- sort(unique(v[,2]))
+	nuv <- length(uv)
+	if (each) {
+		sz <- rep(size, nuv)
+	} else {	
+		sz <- rep(floor(size / nuv), nuv)
+		d <- size - sum(sz)
+		i <- sample(nuv, d)
+		sz[i] <- sz[i] + 1
+	}
+	ys <- vector(mode="list", length=nuv)
+		
+	for (i in seq_len(length(uv))) {
+		if (sz[i] == 0) next
+		vv <- v[v[,2] == uv[i], ]
+		if (doprob) prob <- vv[,3]
+		if (replace) {
+			s <- sample.int(nrow(vv), sz[i], prob=prob, replace=TRUE)
+		} else {
+			s <- sample.int(nrow(vv), min(sz[i], nrow(vv)), prob=prob, replace=FALSE)
 		}
+		ys[[i]] <- vv[s,-3]
 	}
 	ys <- do.call(rbind, ys)
+
 	if (warn) {
 		ta <- tapply(ys[,1], ys[,2], length)
-		ta <- names(ta)[ta < size]
+		sz <- sz[sz > 0]
+		ta <- names(ta)[ta < sz]
 		if (length(ta) > 0) {
 			warn("spatSample", 'fewer samples than requested are available for group(s): ', paste(ta, collapse=', '))
 		}
 	}
+
 	ys
 }
 
 
 
-sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE, values=TRUE, cells=TRUE, xy=FALSE, ext=NULL, warn=TRUE, exp=5, weights=NULL) {
+sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE, values=TRUE, cells=TRUE, xy=FALSE, ext=NULL, warn=TRUE, exp=5, weights=NULL, exhaustive=FALSE, lonlat, each) {
 
 	if (nlyr(x) > 1) {
 		x <- x[[1]]
@@ -109,14 +212,10 @@ sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE
 		error("spatSample", "x has no values")
 	}
 
-	lonlat <- is.lonlat(x, perhaps=TRUE, warn=FALSE)
-
-	if (blocks(x, n=4)$n == 1) {
-	
-		res <- sampleStratMemory(x, size, replace, lonlat, ext, weights, warn)
 
+	if ((blocks(x, n=4)$n == 1) || exhaustive) {
+		res <- sampleStratMemory(x, size, replace, lonlat, ext, weights, warn, each=each)
 	} else {
-
 		f <- unique(x)[,1]
 		exp <- max(1, exp)
 		ss <- exp * size * length(f)
@@ -148,11 +247,20 @@ sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE
 			sr <- cbind(cell=sr, extract(x, sr))
 		}
 		uv <- unique(sr[,2])
+		nuv <- length(uv)
+		if (each) {
+			sz <- rep(size, nuv)
+		} else {
+			sz <- rep(floor(size / nuv), nuv)
+			d <- size - sum(sz)
+			i <- sample(nuv, d)
+			sz[i] <- sz[i] + 1
+		}
 		ys <- vector(mode="list", length=length(uv))
 		for (i in seq_len(length(uv))) {
 			y <- sr[sr[, 2] == uv[i], ,drop=FALSE]
-			if (nrow(y) > size) {
-				y <- y[sample.int(nrow(y), size),  ,drop=FALSE]
+			if (nrow(y) > sz[i]) {
+				y <- y[sample.int(nrow(y), sz[i]),  ,drop=FALSE]
 			}
 			ys[[i]] <- y
 		}
@@ -161,7 +269,8 @@ sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE
 
 		if (warn) {
 			ta <- table(res[,2])
-			ta <- names(ta[ta < size])
+			sz <- sz[sz > 0]
+			ta <- names(ta[ta < sz])
 			tb <- f[!(f %in% unique(res[,2]))]
 			tba <- c(tb, ta)
 			if ((length(tba) > 0)) {
@@ -206,20 +315,23 @@ sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE
 	if (nlyr(x) > 1) {
 		x <- subst(anyNA(x), 1, NA)
 	}
+
 	if (lonlat) {
 		v <- cbind(cell=1:ncell(x), abs(cos(pi * values(init(x, "y")) / 360)), values(x))
 		v <- v[!is.na(v[,3]),]
-		i <- sample.int(nrow(v), min(size, nrow(v)), prob=v[,2], replace=replace)
+		ssize <- ifelse(replace, size, min(size, nrow(v)))
+		i <- sample.int(nrow(v), ssize, prob=v[,2], replace=replace)
 	} else {
 		v <- cbind(cell=1:ncell(x), values(x))
 		v <- v[!is.na(v[,2]),]
-		i <- sample.int(nrow(v), min(size, nrow(v)), replace=replace)
+		ssize <- ifelse(replace, size, min(size, nrow(v)))
+		i <- sample.int(nrow(v), ssize, replace=replace)
 	}
 	v[i,1]
 }
 
 
-.sampleCells <- function(x, size, method, replace, na.rm=FALSE, ext=NULL, exp=5) {
+.sampleCells <- function(x, size, method, replace, na.rm=FALSE, ext=NULL, exp=5, exact=FALSE) {
 	r <- rast(x)
 	lonlat <- is.lonlat(r, perhaps=TRUE, warn=TRUE)
 	if (!is.null(ext)) {
@@ -235,9 +347,9 @@ sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE
 			esize <- size
 		}
 		if (na.rm && (blocks(x, n=4)$n == 1)) {
-			cells <- .sampleCellsMemory(x, size, replace, lonlat, ext)
+			cells <- .sampleCellsMemory(x, esize, replace, lonlat, ext)
 		} else if (lonlat) {
-			m <- ifelse(replace, 1.5, 1.25)
+			m <- ifelse(replace, 1.5, 2)
 			n <- m * esize
 			y <- yFromRow(r, 1:nrow(r))
 			rows <- sample.int(nrow(r), n, replace=TRUE, prob=abs(cos(pi*y/180)))
@@ -251,9 +363,14 @@ sampleStratified <- function(x, size, replace=FALSE, as.df=TRUE, as.points=FALSE
 			cells <- sample.int(ncell(r), esize, replace=replace)
 		}
 	} else { # regular
+	
 		if (TRUE) {
-			xy <- spatSample(ext(r), size, method, lonlat, FALSE)
-			cells <- cellFromXY(r, xy)
+			if (exact) {
+				xy <- regular_exact(r, size)	
+			} else {
+				xy <- spatSample(ext(r), size, method, lonlat, FALSE)
+			}
+			cells <- cellFromXY(r, xy)		
 		} else {
 			if (lonlat) {
 				#ratio <- ncol(r)/nrow(r)
@@ -391,24 +508,27 @@ set_factors <- function(x, ff, cts, asdf) {
 	s
 }
 
-sampleRaster <- function(x, size, method, replace, ext=NULL, warn) {
+sampleRaster <- function(x, size, method, replace, ext=NULL, warn, overview=FALSE) {
 #	hadWin <- hasWin <- FALSE
 	if (!is.null(ext)) {
 #		hasWin <- TRUE
 #		hadWin <- window(x)
 #		oldWin <- ext(x)
-		w <- intersect(ext(x), ext(ext))		
+		w <- intersect(ext(x), ext(ext))
+		if (is.null(w)) {
+			error("sampleRaster", "x and ext do not intersect")
+		}
 		x <- deepcopy(x)
 		window(x) <- w
 	}
 	if (method == "regular") {
 		if (length(size) > 1) {
-			x@cpp <- x@cpp$sampleRowColRaster(size[1], size[2], warn[1])
+			x@pntr <- x@pntr$sampleRowColRaster(size[1], size[2], warn[1])
 		} else {
-			x@cpp <- x@cpp$sampleRegularRaster(size)
+			x@pntr <- x@pntr$sampleRegularRaster(size, overview)
 		}
 	} else if (method == "random") {
-		x@cpp <- x@cpp$sampleRandomRaster(size, replace, .seed())
+		x@pntr <- x@pntr$sampleRandomRaster(size, replace, .seed())
 	} else {
 		error("spatSample", "method must be 'regular' or 'random' if as.raster=TRUE")
 	}
@@ -424,7 +544,10 @@ sampleRaster <- function(x, size, method, replace, ext=NULL, warn) {
 
 
 setMethod("spatSample", signature(x="SpatRaster"),
-	function(x, size, method="random", replace=FALSE, na.rm=FALSE, as.raster=FALSE, as.df=TRUE, as.points=FALSE, values=TRUE, cells=FALSE, xy=FALSE, ext=NULL, warn=TRUE, weights=NULL, exp=5, exhaustive=FALSE) {
+	function(x, size, method="random", replace=FALSE, na.rm=FALSE, as.raster=FALSE, as.df=TRUE, as.points=FALSE, values=TRUE, cells=FALSE, xy=FALSE, ext=NULL, warn=TRUE, weights=NULL, exp=5, exhaustive=FALSE, exact=FALSE, each=TRUE) {
+
+
+		if (method == "display") return(sampleRaster(x, size, "regular", FALSE, ext=ext, warn=FALSE, overview=TRUE))
 
 		if (!as.points) {
 			if (!(values || cells || xy)) {
@@ -451,8 +574,11 @@ setMethod("spatSample", signature(x="SpatRaster"),
 
 		if (as.raster) return(sampleRaster(x, size, method, replace, ext, warn))
 
+		lonlat <- is.lonlat(x, perhaps=TRUE, warn=FALSE)
+		if (lonlat) exact <- FALSE
+
 		if (method == "stratified") {
-			return( sampleStratified(x, size, replace=replace, as.df=as.df, as.points=as.points, cells=cells, values=values, xy=xy, ext=ext, warn=warn, exp=exp, weights=weights) )
+			return( sampleStratified(x, size, replace=replace, as.df=as.df, as.points=as.points, cells=cells, values=values, xy=xy, ext=ext, warn=warn, exp=exp, weights=weights, exhaustive=exhaustive, lonlat=lonlat, each=each) )
 		} else if (!is.null(weights)) {
 			error("spatSample", "argument weights is only used when method='stratified'")
 		}
@@ -481,7 +607,7 @@ setMethod("spatSample", signature(x="SpatRaster"),
 			if (exhaustive && (method=="random") && na.rm) {
 				cnrs <- .sampleCellsExhaustive(x, size, replace, ext, weights=NULL, warn=FALSE)
 			} else {
-				cnrs <- .sampleCells(x, size, method, replace, na.rm, ext)
+				cnrs <- .sampleCells(x, size, method, replace, na.rm, ext, exp=exp, exact=exact)
 			}
 			
 			if (method == "random") {
@@ -533,18 +659,24 @@ setMethod("spatSample", signature(x="SpatRaster"),
 		if (!is.null(ext)) x <- crop(x, ext)
 
 		if (method == "regular") {
-			opt <- spatOptions()
-			if (length(size) > 1) {
-				v <- x@cpp$sampleRowColValues(size[1], size[2], opt)
+			if (exact && (length(size) == 1)) {
+				xy <- regular_exact(x, size)
+				v <- extract(x, xy, ID=FALSE)
 			} else {
-				v <- x@cpp$sampleRegularValues(size, opt)
+				opt <- spatOptions()
+				if (length(size) > 1) {
+					v <- x@pntr$sampleRowColValues(size[1], size[2], opt)
+				} else {
+					v <- x@pntr$sampleRegularValues(size, opt)
+				}
+				x <- messages(x, "spatSample")
+				
+				if (length(v) > 0) {
+					v <- do.call(cbind, v)
+					colnames(v) <- names(x)
+				}
+				v <- set_factors(v, ff, lv, as.df)
 			}
-			x <- messages(x, "spatSample")
-			if (length(v) > 0) {
-				v <- do.call(cbind, v)
-				colnames(v) <- names(x)
-			}
-			v <- set_factors(v, ff, lv, as.df)
 			return(v)
 		} else { # random
 			size <- size[1]
@@ -553,13 +685,17 @@ setMethod("spatSample", signature(x="SpatRaster"),
 				cnrs <- .sampleCellsExhaustive(x, size, replace, ext, weights=NULL, warn=FALSE)
 				out <- x[cnrs]	
 			} else {
-				#v <- x@cpp$sampleRandomValues(size, replace, seed)
+				#v <- x@pntr$sampleRandomValues(size, replace, seed)
 				if (size > 0.75 * ncell(x)) {
 					if (na.rm) {
 						out <- stats::na.omit(values(x))
-						attr(x, "na.action") <- NULL
+						attr(out, "na.action") <- NULL
 						if (nrow(out) < size) {
-							warn("spatSample", "more non-NA cells requested than available")
+							if (replace) {
+								out <- out[sample.int(nrow(out), size, replace=TRUE), ,drop=FALSE]
+							} else {
+								warn("spatSample", "more non-NA cells requested than available")
+							}
 						} else {
 							out <- out[sample.int(nrow(out), size), ,drop=FALSE]
 						}
@@ -575,7 +711,7 @@ setMethod("spatSample", signature(x="SpatRaster"),
 					scells <- NULL
 					ssize <- size*2
 					for (i in 1:10) {
-						scells <- c(scells, .sampleCells(x, ssize, method, replace, na.rm))
+						scells <- c(scells, .sampleCells(x, ssize, method, replace, na.rm, exact=exact))
 						if ((i>1) && (!replace)) {
 							scells <- unique(scells)
 						}
@@ -589,7 +725,7 @@ setMethod("spatSample", signature(x="SpatRaster"),
 						}
 					}
 				} else {
-					scells <- .sampleCells(x, size, method, replace)
+					scells <- .sampleCells(x, size, method, replace, exact=exact)
 					out <- extractCells(x, scells, raw=!as.df)   
 				}
 				if (NROW(out) < size) {
@@ -608,7 +744,7 @@ setMethod("spatSample", signature(x="SpatRaster"),
 
 
 setMethod("spatSample", signature(x="SpatExtent"),
-	function(x, size, method="random", lonlat, as.points=FALSE) {
+	function(x, size, method="random", lonlat, as.points=FALSE, exact=FALSE) {
 		if (missing(lonlat)) {
 			error("spatSample", "provide a lonlat argument")
 		}
@@ -619,9 +755,16 @@ setMethod("spatSample", signature(x="SpatExtent"),
 		size <- round(size)
 		stopifnot(size > 0)
 		if (method=="random") {
-			s <- x@cpp$sampleRandom(size, lonlat, .seed())
+			s <- x@pntr$sampleRandom(size, lonlat, .seed())
+		} else if (exact) {
+			s <- regular_exact(x, size)
+			colnames(s) <- c("x", "y")
+			if (as.points) {
+				s <- vect(s)
+			}
+			return(s)
 		} else {
-			s <- x@cpp$sampleRegular(size, lonlat)
+			s <- x@pntr$sampleRegular(size, lonlat)
 		}
 		s <- do.call(cbind, s)
 		colnames(s) <- c("x", "y")
@@ -689,7 +832,7 @@ setMethod("spatSample", signature(x="SpatExtent"),
 
 
 #coordinates <- function(x) {
-#	do.call(cbind, x@cpp$coordinates())
+#	do.call(cbind, x@pntr$coordinates())
 #}
 
 get_field_name <- function(x, nms, sender="") {
@@ -743,9 +886,9 @@ setMethod("spatSample", signature(x="SpatVector"),
 			}
 			out <- vect()
 			if (length(size) == 1) {
-				out@cpp <- x@cpp$sample(size, method[1], .seed())
+				out@pntr <- x@pntr$sample(size, method[1], .seed())
 			} else {
-				out@cpp <- x@cpp$sampleGeom(size, method[1], .seed())	
+				out@pntr <- x@pntr$sampleGeom(size, method[1], .seed())	
 			}
 			messages(x, "spatSample")
 			return(messages(out, "spatSample"))
