@@ -5,7 +5,7 @@
 
 
 win_basename <- function(x) {
-	if (grepl("Windows", utils::osVersion)) {
+	if (isTRUE(grepl("Windows", utils::osVersion))) {
 		large <- nchar(x) > 256
 		if (any(large)) {
 			for (i in 1:length(large)) {
@@ -158,17 +158,17 @@ setMethod ("show" , "SpatVector",
 		cat(" geometry    :", geomtype(object), "\n")
 		cat(" dimensions  : ", d[1], ", ", d[2], "  (geometries, attributes)\n", sep="" )
 		cat(" extent      : ", e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
-		if (object@cpp$source != "") {
-			if (object@cpp$layer != tools::file_path_sans_ext(win_basename(object@cpp$source))) {
-				cat(" source      : ", win_basename(object@cpp$source), " (", object@cpp$layer, ")\n", sep="")
+		if (object@pntr$source != "") {
+			if (object@pntr$layer != tools::file_path_sans_ext(win_basename(object@pntr$source))) {
+				cat(" source      : ", win_basename(object@pntr$source), " (", object@pntr$layer, ")\n", sep="")
 			} else {
-				cat(" source      : ", win_basename(object@cpp$source), "\n", sep="")
+				cat(" source      : ", win_basename(object@pntr$source), "\n", sep="")
 			}
 		}
 		cat(" coord. ref. :", .name_or_proj4(object), "\n")
 		if (d[2] > 0) {
 			nr <- min(d[1], 3)
-			dd <- as.data.frame(object[1:nr,])
+			dd <- as.data.frame(object)[1:nr, , drop=FALSE]
 			printDF(dd, 3, TRUE)
 		}
 	}
@@ -183,12 +183,11 @@ setMethod ("show" , "SpatVectorProxy",
 		cat(" geometry    :", geomtype(object), "\n")
 		cat(" dimensions  : ", d[1], ", ", d[2], "  (geometries, attributes)\n", sep="" )
 		cat(" extent      : ", e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
-		if (object@cpp$v$layer != tools::file_path_sans_ext(win_basename(object@cpp$v$source))) {
-			cat(" source      : ", win_basename(object@cpp$v$source), " (", object@cpp$v$layer, ")\n", sep="")
+		if (object@pntr$v$layer != tools::file_path_sans_ext(win_basename(object@pntr$v$source))) {
+			cat(" source      : ", win_basename(object@pntr$v$source), " (", object@pntr$v$layer, ")\n", sep="")
 		} else {
-			cat(" source      : ", win_basename(object@cpp$v$source), "\n", sep="")
+			cat(" source      : ", win_basename(object@pntr$v$source), "\n", sep="")
 		}
-		cat(" layer       :", object@cpp$v$layer, "\n")
 		cat(" coord. ref. :", .name_or_proj4(object), "\n")
 		dd <- get.data.frame(object)
 		printDF(dd, 0, TRUE)
@@ -215,7 +214,7 @@ setMethod ("show" , "SpatRaster",
 				txt <- "extent (win): "
 			}
 			cat(txt, w[1], ", ", w[2], ", ", w[3], ", ", w[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
-			#e <- as.vector(object@cpp$source[[1]]$window$full_extent$vector)
+			#e <- as.vector(object@pntr$source[[1]]$window$full_extent$vector)
 			#cat("full extent : " , e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
 		} else {
 			e <- as.vector(ext(object))
@@ -286,20 +285,20 @@ setMethod ("show" , "SpatRaster",
 						if (nsr == (mxsrc+1)) {
 							cat("             ", sources[mxsrc+1], lbsprint[mxsrc+1], "\n")
 						} else {
-							cat("             ", "... and", nsr-mxsrc, "more source(s)\n")
+							cat("             ", "... and", nsr-mxsrc, "more sources\n")
 						}
 					}
 				} else {
 					cat("source      :", sources[1], "\n")
 				}
 			}
-			rgbtype <- object@cpp$rgbtype
+			rgbtype <- object@pntr$rgbtype
 			if (rgbtype != "") {
 				rdgb <- RGB(object)
 				if (is.null(rdgb)) rdgb <- 1:3
-				cat(paste("colors", toupper(object@cpp$rgbtype), " :"), paste(rdgb, collapse=", "), "\n")
+				cat(paste("colors", toupper(object@pntr$rgbtype), " :"), paste(rdgb, collapse=", "), "\n")
 			}
-			hasct <- object@cpp$hasColors()
+			hasct <- object@pntr$hasColors()
 			if (any(hasct)) {
 				cat("color table :", paste(which(hasct), collapse=", "), "\n")
 			}
@@ -335,7 +334,7 @@ setMethod ("show" , "SpatRaster",
 			isB <- is.bool(object)
 			if (any(hMM) || any(is.factor(object))) {
 				#r <- minmax(object)
-				rr <- r <- rbind(object@cpp$range_min, object@cpp$range_max)
+				rr <- r <- rbind(object@pntr$range_min, object@pntr$range_max)
 				r[,!hMM] <- c(Inf, -Inf)
 				#sc <- scoff(object)
 				#r <- r * sc[,1] + sc[,2]
@@ -425,10 +424,10 @@ setMethod ("show" , "SpatRaster",
 
 		}
 
-		if (object@cpp$hasTime) {
+		if (object@pntr$hasTime) {
 			label <- "time        "
 			rtim <- range(time(object))
-			tims <- object@cpp$timestep
+			tims <- object@pntr$timestep
 			if (tims == "yearmonths") {
 				rtim <- format_ym(rtim)
 				label <- "time (ymnts)"
@@ -544,7 +543,7 @@ setMethod("show" , "SpatRasterCollection",
 		cat("extent      : " , e[1], ", ", e[2], ", ", e[3], ", ", e[4], "  (xmin, xmax, ymin, ymax)\n", sep="")
 		
 		
-		crs <- .name_or_proj4(object@cpp$x[[1]])
+		crs <- .name_or_proj4(object@pntr$x[[1]])
 		if (crs != "") cat("crs (first) :", crs,	 "\n")
 		ln <- names(object)
 		if (any(ln != "")) {
@@ -561,7 +560,7 @@ setMethod("show" , "SpatGraticule",
 	function(object) {
 		cat("class       :" , class(object), "\n")
 		v <- vect()
-		v@cpp <- object@cpp
+		v@pntr <- object@pntr
 		cat("lon         :" , stats::na.omit(v$lon), "\n")		
 		cat("lat         :" , stats::na.omit(v$lat), "\n")		
 		cat("coord. ref. :", .name_or_proj4(v), "\n")
