@@ -53,16 +53,23 @@ setMethod("subset", signature(x="SpatRaster"),
 		if (negate) subset = -subset
 		subset <- positive_indices(subset, nlyr(x), TRUE, "subset")
 		opt <- spatOptions(filename, overwrite, ...)
-		x@cpp <- x@cpp$subset(subset-1, opt)
+		x@pntr <- x@pntr$subset(subset-1, opt)
 		messages(x, "subset")
 	}
 )
 
 
-
 ## exact matching
 setMethod("$", "SpatRaster",
 	function(x, name) {
+		if ((nlyr(x) == 1) && is.factor(x)) {
+			factnms <- names(cats(x)[[1]])
+			i <- match(name, factnms[-1])
+			if (!is.na(i)) {
+				activeCat(x) <- i		
+				return(x)
+			}
+		}
 		subset(x, name, NSE=FALSE)
 	}
 )
@@ -101,7 +108,10 @@ setMethod("subset", signature(x="SpatVector"),
 			r <- if (missing(subset)) {
 					TRUE
 				} else {
-					r <- eval(substitute(subset), d, parent.frame())
+					e <- substitute(subset)
+					# r <- eval(e, d, parent.frame())
+					# for #1600
+					r <- eval(e, d, globalenv())
 					if (!is.logical(r)) error("subset", "argument 'subset' must be logical")
 					r & !is.na(r)
 				}
@@ -110,7 +120,8 @@ setMethod("subset", signature(x="SpatVector"),
 				} else {
 					nl <- as.list(seq_along(d))
 					names(nl) <- names(d)
-					eval(substitute(select), nl, parent.frame())
+					# eval(substitute(select), nl, parent.frame())
+					eval(substitute(select), nl, globalenv())
 				}
 			x <- x[r, v, drop=drop]
 		} else {
@@ -140,10 +151,10 @@ setMethod("subset", signature(x="SpatVector"),
 			i <- 0
 		}
 	}
-	x@cpp <- x@cpp$subset_cols(i-1)
+	x@pntr <- x@pntr$subset_cols(i-1)
 	x <- messages(x, "subset")
 	if (drop) {	# drop geometry
-		.getSpatDF(x@cpp$df)
+		.getSpatDF(x@pntr$df)
 	} else {
 		x
 	}
@@ -153,7 +164,7 @@ setMethod("subset", signature(x="SpatVector"),
 setMethod("[", c("SpatVector", "numeric", "missing"),
 function(x, i, j, ... , drop=FALSE) {
 	i <- positive_indices(i, nrow(x), TRUE, "`[`")
-	x@cpp <- x@cpp$subset_rows(i-1)
+	x@pntr <- x@pntr$subset_rows(i-1)
 	x <- messages(x, "[")
 	if (drop) {
 		as.data.frame(x)
@@ -172,7 +183,7 @@ function(x, i, j, drop=FALSE) {
 setMethod("[", c("SpatVector", "logical", "missing"),
 function(x, i, j, drop=FALSE) {
 	i <- which(rep_len(i, nrow(x)))
-	x@cpp <- x@cpp$subset_rows(i-1)
+	x@pntr <- x@pntr$subset_rows(i-1)
 	x <- messages(x, "[")
 	if (drop) {
 		as.data.frame(x)
@@ -185,8 +196,8 @@ setMethod("[", c("SpatVector", "numeric", "numeric"),
 function(x, i, j, drop=FALSE) {
 	i <- positive_indices(i, nrow(x), TRUE, "`[`")
 	j <- positive_indices(j, ncol(x), TRUE, "`[`")
-	p <- x@cpp$subset_rows(i-1)
-	x@cpp <- p$subset_cols(j-1)
+	p <- x@pntr$subset_rows(i-1)
+	x@pntr <- p$subset_cols(j-1)
 	x <- messages(x, "'['")
 	if (drop) {
 		as.data.frame(x)
@@ -199,7 +210,7 @@ function(x, i, j, drop=FALSE) {
 setMethod("[", c("SpatVector", "missing", "numeric"),
 function(x, i, j, drop=FALSE) {
 	j <- positive_indices(j, ncol(x), TRUE, "`[`")
-	x@cpp <- x@cpp$subset_cols(j-1)
+	x@pntr <- x@pntr$subset_cols(j-1)
 	x <- messages(x, "[")
 	if (drop) {
 		as.data.frame(x)
