@@ -27,18 +27,18 @@ yearweek <- function(d) {
 
 setMethod("has.time", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$hasTime
+		x@pntr$hasTime
 	}
 )
 
 
 setMethod("timeInfo", signature(x="SpatRaster"),
 	function(x) {
-		time <- x@cpp$hasTime
+		time <- x@pntr$hasTime
 		if (time) {
-			step <- x@cpp$timestep
+			step <- x@pntr$timestep
 			if (step == "seconds") {
-				data.frame(time=time, step=step, zone=x@cpp$timezone)
+				data.frame(time=time, step=step, zone=x@pntr$timezone)
 			} else {
 				data.frame(time=time, step=step, zone="")
 			}
@@ -57,9 +57,9 @@ setMethod("timeInfo", signature(x="SpatRasterDataset"),
 
 
 time_as_seconds <- function(x) {
-	d <- x@cpp$time
+	d <- x@pntr$time
 	d <- strptime("1970-01-01", "%Y-%m-%d", tz="UTC") + d
-	tz <- x@cpp$timezone
+	tz <- x@pntr$timezone
 	if (!(tz %in% c("", "UTC"))) {
 		attr(d, "tzone") = tz
 	}
@@ -67,14 +67,54 @@ time_as_seconds <- function(x) {
 }
 
 
+#setMethod("time", signature(x="SpatVector"),
+#	function(x, format="") {
+#		cls <- sapply(values(x[1,]), function(i) { a = class(i); a[length(a)] })
+#		i <- which(cls %in% c("Date", "POSIXt"))[1]
+#		if (is.na(i)) {
+#			return(rep(NA, nrow(x)))
+#		} else {
+#			d <- x[,i,drop=TRUE][,,drop=TRUE]
+#			if (format != "") {
+#				steps <- c("seconds", "days", "months", "years", "yearmonths")
+#				format <- match.arg(tolower(format), steps)
+#				if (!(format %in% steps)) {
+#					error("time", "not a valid time format")
+#				}
+#				tstep <- ifelse(cls[i]=="Date", "days", "seconds")
+#				if (format == "seconds") {
+#					if (tstep != "seconds") {
+#						error("time", "cannot extract seconds from Dates")
+#					}
+#					d
+#				} else if (format == "days") {
+#					as.Date(d)
+#				} else if (format == "yearmonths") {
+#					y <- as.integer(format(d, "%Y"))
+#					y + (as.integer(format(d, "%m"))-1)/12
+#				} else if (format == "months") {
+#					as.integer(format(d, "%m"))
+#				} else if (format == "years") {
+#					as.integer(format(d, "%Y"))
+#				}
+#			} else {
+#				d
+#			}
+#		}
+#	}
+#)
+
 setMethod("time", signature(x="SpatRaster"),
 	function(x, format="") {
-		if (!x@cpp$hasTime) {
+		if (!x@pntr$hasTime) {
 			return(rep(NA, nlyr(x)))
 		}
-		d <- x@cpp$time
-		tstep <- x@cpp$timestep
+		d <- x@pntr$time
+		tstep <- x@pntr$timestep
+		
 		if (format != "") {
+			steps <- c("seconds", "days", "months", "years", "yearmonths")
+			format <- match.arg(tolower(format), steps)
 			if ((format == "months") && (tstep == "years")) {
 				error("time", "cannot extract months from years-time")
 			} else if ((format == "years") && (tstep %in% c("months"))) {
@@ -87,28 +127,30 @@ setMethod("time", signature(x="SpatRaster"),
 				error("time", "cannot extract days from this type of time data")
 			}
 			tstep <- format
-		} 
+		} else if (tstep == "raw") {
+			return(d)
+		}
+		
+		
+		d <- strptime("1970-01-01", "%Y-%m-%d", tz="UTC") + d
 		if (tstep == "seconds") {
-			d <- strptime("1970-01-01", "%Y-%m-%d", tz="UTC") + d
-			tz <- x@cpp$timezone
+			tz <- x@pntr$timezone
 			if (!(tz %in% c("", "UTC"))) {
 				attr(d, "tzone") = tz
 			}
 			d
 		} else if (tstep == "days") {
-			d <- strptime("1970-01-01", "%Y-%m-%d", tz = "UTC") + d
 			as.Date(d)
 		} else if (tstep == "yearmonths") {
-			d <- strptime("1970-01-01", "%Y-%m-%d", tz = "UTC") + d
 			y <- as.integer(format(d, "%Y"))
 			y + (as.integer(format(d, "%m"))-1)/12
 		} else if (tstep == "months") {
-			d <- strptime("1970-01-01", "%Y-%m-%d", tz = "UTC") + d
 			as.integer(format(d, "%m"))
 		} else if (tstep == "years") {
-			d <- strptime("1970-01-01", "%Y-%m-%d", tz = "UTC") + d
 			as.integer(format(d, "%Y"))
-		} else { # raw
+#		} else if (tstep == "yearweeks") {
+#			yearweek(as.Date(d))
+		} else { # ???
 			d
 		}
 	}
@@ -144,8 +186,9 @@ setMethod("time<-", signature(x="SpatRaster"),
 			value <- tstep
 			tstep <- ""
 		}
+		x@pntr <- x@pntr$deepcopy()
 		if (is.null(value)) {
-			x@cpp$setTime(0[0], "remove", "")
+			x@pntr$setTime(0[0], "remove", "")
 			return(x)
 		}
 		if (inherits(value, "character")) {
@@ -167,8 +210,8 @@ setMethod("time<-", signature(x="SpatRaster"),
 			if (tstep == "") stept <- "days"
 		} else if (inherits(value, "POSIXt")) {
 			if (tstep == "") stept <- "seconds"
-			tzone <- attr(value, "tzone")
-			if (is.null(tzone)) tzone = ""
+			tzone <- attr(value, "tzone")[1]
+			if (is.null(tzone)) tzone = "UTC"
 		} else if (inherits(value, "yearmon")) {
 			value <- as.numeric(value)
 			year <- floor(value)
@@ -183,14 +226,14 @@ setMethod("time<-", signature(x="SpatRaster"),
 				if (is.numeric(value)) {
 					value <- posix_from_ym(value, "6")
 				} else {
-					value <- as.integer(strftime(value, format = "%Y"))
+					value <- as.integer(strftime(value, format = "%Y", tz=tzone))
 					value <- posix_from_ym(value, "6")
 				}
 			} else if (tstep == "months") {
 				if (is.numeric(value)) {
 					value <- floor(value)
 				} else {
-					value <- as.integer(strftime(value, format = "%m"))
+					value <- as.integer(strftime(value, format = "%m", tz=tzone))
 				}
 				if (!all(value %in% 1:12)) {
 					error("date<-", "months should be between 1 and 12")
@@ -201,8 +244,8 @@ setMethod("time<-", signature(x="SpatRaster"),
 					y <- as.integer(substr(value, 1, 4))
 					m <- value - (y * 100)
 				} else {
-					y <- as.integer(strftime(value, format = "%Y"))
-					m <- as.integer(strftime(value, format = "%m"))
+					y <- as.integer(strftime(value, format = "%Y", tz=tzone))
+					m <- as.integer(strftime(value, format = "%m", tz=tzone))
 				}
 				if (!all(m %in% 1:12)) {
 					error("date<-", "months should be between 1 and 12")
@@ -216,7 +259,7 @@ setMethod("time<-", signature(x="SpatRaster"),
 				stept <- "raw"
 			}
 		}
-		if (!x@cpp$setTime(as.numeric(value), stept, tzone)) {
+		if (!x@pntr$setTime(as.numeric(value), stept, tzone)) {
 			error("time<-", "cannot set these values")
 		}
 		return(x)
@@ -256,7 +299,7 @@ setMethod("time<-", signature(x="SpatRasterDataset"),
 
 setMethod("depth", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$depth
+		x@pntr$depth
 	}
 )
 
@@ -264,11 +307,11 @@ setMethod("depth", signature(x="SpatRaster"),
 setMethod("depth<-", signature(x="SpatRaster"),
 	function(x, value)  {
 		if (is.null(value)) {
-			x@cpp$setDepth(0[0])
+			x@pntr$setDepth(0[0])
 			return(x)
 		}
 		value <- as.numeric(value)
-		if (! x@cpp$setDepth(value)) {
+		if (! x@pntr$setDepth(value)) {
 			error("depth<-", "cannot set these  values")
 		}
 		return(x)
@@ -289,7 +332,7 @@ setMethod("linearUnits", signature(x="SpatVector"),
 
 setMethod("units", signature(x="SpatRaster"),
 	function(x) {
-		x@cpp$units
+		x@pntr$units
 	}
 )
 
@@ -300,7 +343,7 @@ setMethod("units<-", signature(x="SpatRaster"),
 		} else {
 			value <- as.character(value)
 		}
-		if (! x@cpp$set_units(value)) {
+		if (! x@pntr$set_units(value)) {
 			error("units<-", "cannot set these  values")
 		}
 		return(x)
@@ -310,14 +353,14 @@ setMethod("units<-", signature(x="SpatRaster"),
 
 setMethod("units", signature(x="SpatRasterDataset"),
 	function(x) {
-		x@cpp$units
+		x@pntr$units
 	}
 )
 
 setMethod("units<-", signature(x="SpatRasterDataset"),
 	function(x, value)  {
 		value <- as.character(value)
-		x@cpp$units <- value
+		x@pntr$units <- value
 		return(x)
 	}
 )
