@@ -1,15 +1,15 @@
 
 
-setMethod("window<-", signature(x="SpatRaster"),
+setMethod("set.window", signature(x="SpatRaster"),
 	function(x, value)  {
 		if (inherits(value, "SpatExtent")) {
-			value <- value * ext(x)
-			if (!(x@ptr$setWindow(value@ptr))) {
-				error("window<-,SpatRaster", "could not set window")
+			window(x) <- NULL
+			value <- intersect(ext(x), value)
+			if (!(x@pntr$setWindow(value@pntr))) {
+				error("window<-,SpatRaster", "cannot set this window")
 			}
-			#warn("window<-", "using a window is experimental")
 		} else if (is.null(value) || is.na(value)) {
-			x@ptr$removeWindow()
+			x@pntr$removeWindow()
 		} else {
 			error("window<-", "'value' should be a SpatExtent, NULL or NA")
 		}
@@ -18,9 +18,16 @@ setMethod("window<-", signature(x="SpatRaster"),
 )
 
 
+setMethod("window<-", signature(x="SpatRaster"),
+	function(x, value)  {
+		set.window(deepcopy(x), value)
+	}
+)
+
+
 setMethod("window", signature(x="SpatRaster"),
 	function(x)  {
-		x@ptr$hasWindow()
+		x@pntr$hasWindow()
 	}
 )
 
