@@ -3,6 +3,8 @@
 #include "string_utils.h"
 #include "math_utils.h"
 
+#include "sort.h"
+
 #include "gdal_priv.h"
 #include "gdalio.h"
 #include "ogr_spatialref.h"
@@ -11,6 +13,7 @@
 #include <geos_c.h>
 
 
+
 #if GDAL_VERSION_MAJOR >= 3
 #include "proj.h"
 #define projh
@@ -30,10 +33,12 @@
 #if PROJ_VERSION_MAJOR >=8
 #include "proj.h"
 #else
+#define ACCEPT_USE_OF_DEPRECATED_PROJ_API_H
 #include <proj_api.h>
 #endif
 #endif
 
+
 //from sf
 
 #ifdef projh
@@ -57,6 +62,7 @@ std::string proj_version() {
 #endif
 
 
+
 // [[Rcpp::export]]
 std::vector<unsigned char> hex2rgb(std::string s) {
 	unsigned char r, g, b;
@@ -66,6 +72,20 @@ std::vector<unsigned char> hex2rgb(std::string s) {
 	return x;
 }
 
+/*
+// [[Rcpp::export]]
+std::vector<size_t> terra_order(std::vector<double> v) {
+	return sort_order_a(v);
+}
+
+// [[Rcpp::export]]
+std::vector<double> terra_permute(std::vector<double> v, std::vector<size_t> p) {
+	permute(v, p);
+	return v;
+}
+*/
+
+
 // [[Rcpp::export]]
 std::string rgb2hex(std::vector<unsigned char> x) {
 	std::stringstream ss;
@@ -130,7 +150,7 @@ std::vector<std::string> getCRSname(std::string s) {
 		if (value != NULL) {
 			if (west > -1000) {
 				aoi	= value;
-				box = std::to_string(west) + ", " + std::to_string(east) + ", " + std::to_string(north) + ", " + std::to_string(south);
+				box = std::to_string(west) + ", " + std::to_string(east) + ", " + std::to_string(south) + ", " + std::to_string(north);
 			}
 		}
 	}
@@ -148,6 +168,7 @@ double getLinearUnits(std::string s) {
 
 
 
+
 // [[Rcpp::export(name = ".geotransform")]]
 std::vector<double> geotransform(std::string fname) {
 	std::vector<double> out;
@@ -246,7 +267,7 @@ std::vector<std::vector<std::string>> sdsmetatdataparsed(std::string filename) {
 // [[Rcpp::export(name = ".gdaldrivers")]]
 std::vector<std::vector<std::string>> gdal_drivers() {
 	size_t n = GetGDALDriverManager()->GetDriverCount();
-	std::vector<std::vector<std::string>> s(5, std::vector<std::string>(n));
+	std::vector<std::vector<std::string>> s(6, std::vector<std::string>(n));
     GDALDriver *poDriver;
     char **papszMetadata;
 	for (size_t i=0; i<n; i++) {
@@ -254,16 +275,18 @@ std::vector<std::vector<std::string>> gdal_drivers() {
 		const char* ss = poDriver->GetDescription();
 		if (ss != NULL ) s[0][i] = ss;
 		ss = poDriver->GetMetadataItem( GDAL_DMD_LONGNAME );
-		if (ss != NULL ) s[4][i] = ss;
+		if (ss != NULL ) s[5][i] = ss;
 
 		papszMetadata = poDriver->GetMetadata();
 		bool rst = CSLFetchBoolean( papszMetadata, GDAL_DCAP_RASTER, FALSE);
+		bool vct = CSLFetchBoolean( papszMetadata, GDAL_DCAP_VECTOR, FALSE);
 		s[1][i] = std::to_string(rst);
+		s[2][i] = std::to_string(vct);
 		bool create = CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATE, FALSE);
 		bool copy = CSLFetchBoolean( papszMetadata, GDAL_DCAP_CREATECOPY, FALSE);
-		s[2][i] = std::to_string(create + copy);
+		s[3][i] = std::to_string(create + copy);
 		bool vsi = CSLFetchBoolean( papszMetadata, GDAL_DCAP_VIRTUALIO, FALSE);
-		s[3][i] = std::to_string(vsi);
+		s[4][i] = std::to_string(vsi);
 	}
 	return s;
 }
@@ -355,6 +378,22 @@ void set_gdal_warnings(int level) {
 	}
 }
 
+#include "common.h"
+
+std::mt19937 my_rgen;
+
+// [[Rcpp::export(name = ".seedinit")]]
+void seed_init(uint32_t seed_val) {
+  my_rgen.seed(seed_val);
+}
+
+/*
+[[Rcpp::export(name = ".rnumb")]]
+int rnumb() {
+    std::uniform_int_distribution<std::mt19937::result_type> rand_nr(0, 9); 
+	return  rand_nr(my_rgen);
+}
+*/
 
 // [[Rcpp::export(name = ".gdalinit")]]
 void gdal_init(std::string projpath, std::string datapath) {
@@ -434,16 +473,43 @@ std::vector<double> percRank(std::vector<double> x, std::vector<double> y, doubl
 }
 
 
+// [[Rcpp::export(name = ".clearVSIcache")]]
+void clearVSIcache(bool vsi) {
+	//if (vsi)
+	VSICurlClearCache();
+}
+
 // [[Rcpp::export(name = ".setGDALCacheSizeMB")]]
-void setGDALCacheSizeMB(double x) {
-  GDALSetCacheMax64(static_cast<int64_t>(x) * 1024 * 1024);
+void setGDALCacheSizeMB(double x, bool vsi) {
+	if (vsi) {
+		int64_t v = x * 1024 * 1024;
+		CPLSetConfigOption("CPL_VSIL_CURL_CACHE_SIZE", std::to_string(v).c_str());
+	} else {
+		GDALSetCacheMax64(static_cast<int64_t>(x) * 1024 * 1024);
+	}
+
 }
 
 // [[Rcpp::export(name = ".getGDALCacheSizeMB")]]
-double getGDALCacheSizeMB() {
-  return static_cast<double>(GDALGetCacheMax64() / 1024 / 1024);
+double getGDALCacheSizeMB(bool vsi) {
+	if (vsi) {
+		std::string out = gdal_getconfig("CPLGetConfigOption");
+		Rcpp::Rcout << out << std::endl;
+		if (out == "") return NAN;
+		double v = -1; 
+		try {
+			v = stod(out) / (1024 * 1024);
+		} catch(...){
+			return(NAN);
+		}
+		return(v);
+		
+	} else {
+		return static_cast<double>(GDALGetCacheMax64() / 1024 / 1024);
+	}
 }
 
+
 // convert NULL-terminated array of strings to std::vector<std::string>
 std::vector<std::string> charpp2vect(char **cp) {
 	std::vector<std::string> out;
@@ -636,3 +702,4 @@ double stattest2(std::vector<double> x, std::string fun, bool narm) {
 }
 
 */
+
