@@ -6,22 +6,30 @@
 #include "NA.h"
 #include "spatTime.h"
 //#include "spatVector2.h"
-
 //static void SpatRaster_finalizer( SpatRaster* ptr ){
 //}
 
+/*
 Rcpp::List getBlockSizeR(SpatRaster* r, unsigned n, double frac) {
 	SpatOptions opt;
 	opt.ncopies = n;
 	opt.set_memfrac(frac);
-    BlockSize bs = r->getBlockSize(opt);
+	BlockSize bs = r->getBlockSize(opt);
+	Rcpp::List L = Rcpp::List::create(Rcpp::Named("row") = bs.row, Rcpp::Named("nrows") = bs.nrows, Rcpp::Named("n") = bs.n);
+	return(L);
+}
+*/
+
+
+Rcpp::List getBlockSizeR(SpatRaster* r, SpatOptions* opt) {
+	BlockSize bs = r->getBlockSize(*opt);
 	Rcpp::List L = Rcpp::List::create(Rcpp::Named("row") = bs.row, Rcpp::Named("nrows") = bs.nrows, Rcpp::Named("n") = bs.n);
 	return(L);
 }
 
 
 Rcpp::List getBlockSizeWrite(SpatRaster* r) {
-    BlockSize bs = r->bs;
+	BlockSize bs = r->bs;
 	Rcpp::List L = Rcpp::List::create(Rcpp::Named("row") = bs.row, Rcpp::Named("nrows") = bs.nrows, Rcpp::Named("n") = bs.n);
 	return(L);
 }
@@ -34,7 +42,7 @@ bool sameObject(SpatRaster* a, SpatRaster* b) {
 
 
 Rcpp::List getDataFrame(SpatDataFrame* v) {
-	unsigned n = v->ncol();
+	size_t n = v->ncol();
 	Rcpp::List out(n);
 	if (n == 0) {
 		return(out);
@@ -185,15 +193,15 @@ RCPP_EXPOSED_CLASS(SpatRasterStack)
 RCPP_EXPOSED_CLASS(SpatVector)
 RCPP_EXPOSED_CLASS(SpatVectorProxy)
 RCPP_EXPOSED_CLASS(SpatVectorCollection)
-RCPP_EXPOSED_CLASS(SpatGraph)
+//RCPP_EXPOSED_CLASS(SpatGraph)
 //RCPP_EXPOSED_CLASS(SpatVector2)
 
 RCPP_MODULE(spat){
 
-    using namespace Rcpp;
+	using namespace Rcpp;
 
 /*
-    class_<SpatVector2>("SpatVector2")
+	class_<SpatVector2>("SpatVector2")
 	
 		.constructor()
 		.field("x", &SpatVector2::X)
@@ -207,14 +215,14 @@ RCPP_MODULE(spat){
 	;
 */
 
-    class_<SpatTime_v>("SpatTime_v")
+	class_<SpatTime_v>("SpatTime_v")
 		.constructor()
 		.field("step", &SpatTime_v::step)
 		.field("zone", &SpatTime_v::zone)
 		.field("x", &SpatTime_v::x)
 	;
 
-    class_<SpatFactor>("SpatFactor")
+	class_<SpatFactor>("SpatFactor")
 		.constructor()
 		.constructor<std::vector<unsigned>, std::vector<std::string>, bool>()
 //		.constructor<std::vector<unsigned>, std::vector<std::string>>()
@@ -224,18 +232,21 @@ RCPP_MODULE(spat){
 	;
 
 
-    class_<SpatSRS>("SpatSRS")
+	class_<SpatSRS>("SpatSRS")
 		.constructor()
 		.method("set", &SpatSRS::set)
 		.method("is_lonlat", &SpatSRS::is_lonlat)
 		.method("to_meter", &SpatSRS::to_meter)
 	;
 
-    class_<SpatGraph>("SpatGraph")
+
+/*
+	class_<SpatGraph>("SpatGraph")
 		.constructor()
 	;
+*/
 
-    class_<SpatExtent>("SpatExtent")
+	class_<SpatExtent>("SpatExtent")
 		.constructor()
 		.constructor<double, double, double, double>()
 		.method("deepcopy", &SpatExtent::deepCopy, "deepCopy")
@@ -257,7 +268,7 @@ RCPP_MODULE(spat){
 	;
 
 /*
-    class_<SpatWindow>("SpatWindow")
+	class_<SpatWindow>("SpatWindow")
 		.field_readonly("full_extent", &SpatWindow::full_extent)
 		.field_readonly("full_nrow", &SpatWindow::full_nrow)
 		.field_readonly("full_ncol", &SpatWindow::full_ncol)
@@ -267,16 +278,19 @@ RCPP_MODULE(spat){
 	;
 */
 
-    class_<SpatMessages>("SpatMessages")
+
+
+	class_<SpatMessages>("SpatMessages")
 		.constructor()
 		//.field("success", &SpatMessages::success)
 		.field("has_error", &SpatMessages::has_error)
 		.field("has_warning", &SpatMessages::has_warning)
-		.method("getError", &SpatMessages::getError)
-		.method("getWarnings", &SpatMessages::getWarnings)
+		//.method("has_error", &SpatMessages::hasError)
+		//.method("has_warning", &SpatMessages::hasWarning)
 	;
 
-    class_<SpatOptions>("SpatOptions")
+
+	class_<SpatOptions>("SpatOptions")
 		.constructor()
 		.method("deepcopy", &SpatOptions::deepCopy, "deepCopy")
 		.property("tempdir", &SpatOptions::get_tempdir, &SpatOptions::set_tempdir )
@@ -308,7 +322,7 @@ RCPP_MODULE(spat){
 		.property("todisk", &SpatOptions::get_todisk, &SpatOptions::set_todisk)
 		.field("messages", &SpatOptions::msg, "messages")
 		.field("gdal_options", &SpatOptions::gdal_options)
-		.field("pid", &SpatOptions::pid )
+		.field("tmpfile", &SpatOptions::tmpfile )
 		.field("names", &SpatOptions::names)
 		.property("steps", &SpatOptions::get_steps, &SpatOptions::set_steps)
 	//	.property("overwrite", &SpatOptions::set_overwrite, &SpatOptions::get_overwrite )
@@ -317,9 +331,15 @@ RCPP_MODULE(spat){
 		.property("scale", &SpatOptions::get_scale, &SpatOptions::set_scale)
 		.property("offset", &SpatOptions::get_offset, &SpatOptions::set_offset)
 
+		.method("has_error", &SpatOptions::hasError)
+		.method("has_warning", &SpatOptions::hasWarning)
+
+		.method("getError", &SpatOptions::getError)
+		.method("getWarnings", &SpatOptions::getWarnings)
+
 	;
 
-    class_<SpatDataFrame>("SpatDataFrame")
+	class_<SpatDataFrame>("SpatDataFrame")
 		.constructor()
 
 		.field_readonly("itype", &SpatDataFrame::itype)
@@ -338,7 +358,8 @@ RCPP_MODULE(spat){
 		.method("add_column_long", (bool (SpatDataFrame::*)(std::vector<long>, std::string name))( &SpatDataFrame::add_column))
 		.method("add_column_string", (bool (SpatDataFrame::*)(std::vector<std::string>, std::string name))( &SpatDataFrame::add_column))
 		.method("add_column_factor", (bool (SpatDataFrame::*)(SpatFactor, std::string name))( &SpatDataFrame::add_column))
-		.method("add_column_bool", &SpatDataFrame::add_column_bool)
+		.method("add_column_bool", (bool (SpatDataFrame::*)(std::vector<int>, std::string name))( &SpatDataFrame::add_column_bool))
+
 		.method("add_column_time", &SpatDataFrame::add_column_time)
 
 		.method("remove_column", (bool (SpatDataFrame::*)(std::string field))( &SpatDataFrame::remove_column))
@@ -364,7 +385,7 @@ RCPP_MODULE(spat){
 	;
 
 
-    class_<SpatVectorCollection>("SpatVectorCollection")
+	class_<SpatVectorCollection>("SpatVectorCollection")
 		.constructor()
 		.constructor<std::string, std::string, std::string, std::vector<double>, SpatVector>()
 
@@ -389,7 +410,7 @@ RCPP_MODULE(spat){
 	;
 
 
-    class_<SpatCategories>("SpatCategories")
+	class_<SpatCategories>("SpatCategories")
 		.constructor()
 		.field_readonly("df", &SpatCategories::d)
 		.field("index", &SpatCategories::index)
@@ -397,21 +418,26 @@ RCPP_MODULE(spat){
 	;
 
 
-    class_<SpatVector>("SpatVector")
+/*
+	class_<SpatVector>("SpatVector")
 		.constructor()
 		.constructor<SpatExtent, std::string>()
 		.constructor<std::vector<std::string>>()
 	;
+*/
 
-
-    class_<SpatVector>("SpatVector")
+	class_<SpatVector>("SpatVector")
 		.constructor()
 		.constructor<SpatExtent, std::string>()
 		.constructor<std::vector<std::string>>()
 
+
+//		.method("pointInPolygon", &SpatVector::pointInPolygon)
+		
 		.method("deepcopy", &SpatVector::deepCopy)
 		.method("wkt", &SpatVector::wkt)
 		.method("wkb", &SpatVector::wkb)
+		.method("wkb_raw", &SpatVector::wkb_raw)
 		.method("hex", &SpatVector::hex)
 		.method("from_hex", &SpatVector::from_hex)
 		.method("make_nodes", &SpatVector::make_nodes)
@@ -451,7 +477,6 @@ RCPP_MODULE(spat){
 		.method("linesList", &SpatVector::linesList)
 		.method("polygonsList", &SpatVector::polygonsList)
 		.method("linesNA", &SpatVector::linesNA)
-		.method("linedistlonlat", &SpatVector::linedistLonLat)
 
 		.method("add_column_empty", (void (SpatVector::*)(unsigned dtype, std::string name))( &SpatVector::add_column))
 		.method("add_column_double", (bool (SpatVector::*)(std::vector<double>, std::string name))( &SpatVector::add_column))
@@ -482,19 +507,21 @@ RCPP_MODULE(spat){
 		.method("set_crs", (bool (SpatVector::*)(std::string crs))( &SpatVector::setSRS))
 		//.method("prj", &SpatVector::getPRJ)
 
-		.method("distance_self", (std::vector<double> (SpatVector::*)(bool, std::string))( &SpatVector::distance))
-		.method("distance_other", (std::vector<double> (SpatVector::*)(SpatVector, bool, std::string))( &SpatVector::distance))
+//		.method("get_index", &SpatVector::get_index)
 
+		.method("distance_self", (std::vector<double> (SpatVector::*)(bool, std::string, const std::string))( &SpatVector::distance))
+		.method("distance_other", (std::vector<double> (SpatVector::*)(SpatVector, bool, std::string, const std::string))( &SpatVector::distance))
 		.method("point_distance", &SpatVector::pointdistance)
 
-		.method("geosdist_self", (std::vector<double> (SpatVector::*)(bool, std::string))( &SpatVector::geos_distance))
-		.method("geosdist_other", (std::vector<double> (SpatVector::*)(SpatVector, bool, std::string))( &SpatVector::geos_distance))
+//		.method("geosdist_self", (std::vector<double> (SpatVector::*)(bool, std::string))( &SpatVector::geos_distance))
+//		.method("geosdist_other", (std::vector<double> (SpatVector::*)(SpatVector, bool, std::string))( &SpatVector::geos_distance))
 
 		.method("extent", &SpatVector::getExtent)
 		.method("getDF", &getVectorAttributes)
 		.method("getGeometryWKT", &SpatVector::getGeometryWKT)
 		.method("isLonLat", &SpatVector::is_lonlat)
 		.method("length", &SpatVector::length)
+		.method("nsegments", &SpatVector::nseg)
 //		.field("srs", &SpatVector::srs, "srs")
 		.field("messages", &SpatVector::msg)
 		.property("names", &SpatVector::get_names, &SpatVector::set_names)
@@ -506,9 +533,10 @@ RCPP_MODULE(spat){
 		.method("setGeometry", &SpatVector::setGeometry)
 		.method("setPointsXY", &SpatVector::setPointsGeometry)
 		.method("setPointsDF", &SpatVector::setPointsDF)
+		.method("setLinesStartEnd", &SpatVector::setLinesStartEnd)
 		.method("size", &SpatVector::size)
-		.method("subset_cols", ( SpatVector (SpatVector::*)(std::vector<int>))( &SpatVector::subset_cols ))
-		.method("subset_rows", ( SpatVector (SpatVector::*)(std::vector<int>))( &SpatVector::subset_rows ))
+		.method("subset_cols", ( SpatVector (SpatVector::*)(std::vector<long>))( &SpatVector::subset_cols ))
+		.method("subset_rows", ( SpatVector (SpatVector::*)(std::vector<long>))( &SpatVector::subset_rows ))
 		.method("remove_rows", &SpatVector::remove_rows)
 		.method("type", &SpatVector::type)
 		.method("naGeoms", &SpatVector::naGeoms)
@@ -528,8 +556,8 @@ RCPP_MODULE(spat){
 
 		.method("disaggregate", &SpatVector::disaggregate)
 		.method("buffer", &SpatVector::buffer)
-		.method("buffer2", &SpatVector::buffer2)
-		.method("buffer3", &SpatVector::buffer3)
+//		.method("buffer2", &SpatVector::buffer2)
+//		.method("buffer3", &SpatVector::buffer3)
 		.method("centroid", &SpatVector::centroid)
 		.method("point_on_surface", &SpatVector::point_on_surface)
 		.method("make_valid2", &SpatVector::make_valid2)
@@ -552,6 +580,7 @@ RCPP_MODULE(spat){
 		.method("intersect", &SpatVector::intersect)
 		.method("delaunay", &SpatVector::delaunay)
 		.method("voronoi", &SpatVector::voronoi)
+//		.method("voronoi_sphere", &SpatVector::voronoi_sphere)
 		.method("hull", &SpatVector::hull)
 
 		.method("width", &SpatVector::width)
@@ -564,7 +593,7 @@ RCPP_MODULE(spat){
 		.method("related_within", ( std::vector<std::vector<double>> (SpatVector::*)(std::string, bool))( &SpatVector::which_relate))
 
 //		.method("relate_first", &SpatVector::relateFirst)
-		.method("relate_between", ( std::vector<int> (SpatVector::*)(SpatVector, std::string, bool, bool))( &SpatVector::relate ))
+//		.method("relate_between", ( std::vector<int> (SpatVector::*)(SpatVector, std::string, bool, bool))( &SpatVector::relate ))
 		.method("relate_within", ( std::vector<int> (SpatVector::*)(std::string, bool))( &SpatVector::relate ))
 		.method("equals_between", ( std::vector<unsigned> (SpatVector::*)(SpatVector, double))( &SpatVector::equals_exact ))
 		.method("equals_within", ( std::vector<unsigned> (SpatVector::*)(bool, double))( &SpatVector::equals_exact ))
@@ -573,8 +602,10 @@ RCPP_MODULE(spat){
 		.method("crop_ext", ( SpatVector (SpatVector::*)(SpatExtent, bool))( &SpatVector::crop ))
 		.method("crop_vct", ( SpatVector (SpatVector::*)(SpatVector))( &SpatVector::crop ))
 
-		.method("near_between", (SpatVector (SpatVector::*)(SpatVector, bool))( &SpatVector::nearest_point))
-		.method("near_within", (SpatVector (SpatVector::*)())( &SpatVector::nearest_point))
+		.method("near_between", (SpatVector (SpatVector::*)(SpatVector, bool, const std::string))( &SpatVector::nearest_point))
+		.method("near_within", (SpatVector (SpatVector::*)(const std::string))( &SpatVector::nearest_point))
+
+// not used?
 		.method("near_geom", &SpatVector::nearest_geometry)
 
 		//.method("knearest", &SpatVector::knearest)
@@ -594,7 +625,7 @@ RCPP_MODULE(spat){
 	;
 
 
-//    class_<SpatRasterSource>("SpatRasterSource")
+//	class_<SpatRasterSource>("SpatRasterSource")
 //		.field_readonly("cats", &SpatRasterSource::cats)
 ///		.field_readonly("has_scale_offset", &SpatRasterSource::has_scale_offset)
 //		.field_readonly("scale", &SpatRasterSource::scale)
@@ -620,7 +651,7 @@ RCPP_MODULE(spat){
 //;
 
 
-    class_<SpatVectorProxy>("SpatVectorProxy")
+	class_<SpatVectorProxy>("SpatVectorProxy")
 		.constructor()
 		.field("v", &SpatVectorProxy::v )
 		.method("deepcopy", &SpatVectorProxy::deepCopy, "deepCopy")
@@ -628,12 +659,12 @@ RCPP_MODULE(spat){
 
 
 
-    class_<SpatRaster>("SpatRaster")
+	class_<SpatRaster>("SpatRaster")
 		.constructor()
 	 // .constructor<std::string, int>()
-	    .constructor<std::vector<std::string>, std::vector<int>, std::vector<std::string>, bool, std::vector<std::string>, std::vector<std::string>, std::vector<size_t>>()
+		.constructor<std::vector<std::string>, std::vector<int>, std::vector<std::string>, bool, std::vector<std::string>, std::vector<std::string>, std::vector<size_t>, bool>()
 		
-		.constructor<std::vector<unsigned>, std::vector<double>, std::string>()
+		.constructor<std::vector<size_t>, std::vector<double>, std::string>()
 		//.finalizer(&SpatRaster_finalizer)
 
 		//.method("fromFiles", &SpatRaster::fromFiles)
@@ -648,6 +679,8 @@ RCPP_MODULE(spat){
 		.method("addLyrTags", &SpatRaster::addLyrTags)
 		.method("getLyrTags", &SpatRaster::getLyrTags)
 
+		.method("getAllFiles", &SpatRaster::getAllFiles)
+
 
 		//.field("name", &SpatRaster::name)
 		.method("getFileBlocksize", &SpatRaster::getFileBlocksize)
@@ -668,6 +701,7 @@ RCPP_MODULE(spat){
 		.property("extent", &SpatRaster::getExtent, &SpatRaster::setExtent )
 
 		.method("is_rotated", &SpatRaster::is_rotated)
+		.method("is_flipped", &SpatRaster::is_flipped)
 
 		.method("setWindow", &SpatRaster::setWindow, "")
 		.method("removeWindow", &SpatRaster::removeWindow, "")
@@ -676,6 +710,7 @@ RCPP_MODULE(spat){
 		//.method("getRasterAtt", &getRasterAttributes, "get attributes")
 
 		.method("filenames", &SpatRaster::filenames )
+//		.method("shared_basegeom", &SpatRaster::shared_basegeom)
 
 		.field_readonly("rgb", &SpatRaster::rgb)
 		.field_readonly("rgbtype", &SpatRaster::rgbtype)
@@ -829,9 +864,10 @@ RCPP_MODULE(spat){
 		.method("rst_area", &SpatRaster::rst_area)
 		.method("sum_area", &SpatRaster::sum_area)
 		.method("sum_area_group", &SpatRaster::sum_area_group)
+		.method("surface_area", &SpatRaster::surfaceArea)
 
 		.method("as_points", &SpatRaster::as_points)
-		.method("as_points_value", &SpatRaster::as_points_value)
+//		.method("as_points_value", &SpatRaster::as_points_value)
 		.method("cells_notna", &SpatRaster::cells_notna)
 		.method("cells_notna_novalues", &SpatRaster::cells_notna_novalues)
 		.method("as_multipoints", &SpatRaster::as_multipoints)
@@ -845,43 +881,54 @@ RCPP_MODULE(spat){
 		.method("bilinearValues", &SpatRaster::bilinearValues)
 
 		.method("patches", &SpatRaster::clumps)
+		.method("patches2", &SpatRaster::patches)
 		.method("boundaries", &SpatRaster::edges)
 		.method("buffer", &SpatRaster::buffer)
 		.method("gridDistance", &SpatRaster::gridDistance)
 		.method("costDistance", &SpatRaster::costDistance)
 		.method("rastDistance", &SpatRaster::distance)
+		.method("nearest", &SpatRaster::nearest)
+		.method("vectDistance", &SpatRaster::distance_vector)
 		.method("rastDirection", &SpatRaster::direction)
+		.method("vectDirectionRasterize", &SpatRaster::direction_rasterize)
+		
+//		.method("vectDistanceDirect", &SpatRaster::distance_spatvector)
+//		.method("vectDistanceRasterize", &SpatRaster::distance_rasterize)
+		
+		.method("get_tiles_ext", &SpatRaster::get_tiles_extent)
+		.method("get_tiles_ext_vect", &SpatRaster::get_tiles_extent_vect)
 		.method("make_tiles", &SpatRaster::make_tiles)
 		.method("make_tiles_vect", &SpatRaster::make_tiles_vect)
 		.method("ext_from_rc", &SpatRaster::ext_from_rc)
 
 		.method("combineCats", &SpatRaster::combineCats)
 		.method("droplevels", &SpatRaster::dropLevels)
-		.method("vectDistanceDirect", &SpatRaster::distance_spatvector)
-		.method("vectDistanceRasterize", &SpatRaster::distance_rasterize)
-		.method("vectDirectionRasterize", &SpatRaster::direction_rasterize)
 		.method("clamp", &SpatRaster::clamp)
 		.method("clamp_raster", &SpatRaster::clamp_raster)
 		.method("clamp_ts", &SpatRaster::clamp_ts)
 		.method("replaceValues", &SpatRaster::replaceValues)
-		.method("classify", ( SpatRaster (SpatRaster::*)(std::vector<double>, unsigned, unsigned, bool, bool, double, bool, bool, bool, SpatOptions&) )( &SpatRaster::reclassify))
+		.method("classify", ( SpatRaster (SpatRaster::*)(std::vector<double>, size_t, unsigned, bool, bool, double, bool, bool, bool, SpatOptions&) )( &SpatRaster::reclassify))
 		//.method("source_collapse", &SpatRaster::collapse, "collapse")
 		.method("selRange", &SpatRaster::selRange)
 		.method("separate", &SpatRaster::separate)
 		.method("sort", &SpatRaster::sort)
 		.method("intersect", &SpatRaster::intersect)
 
-		.method("cover", &SpatRaster::cover)
+		.method("cover", ( SpatRaster (SpatRaster::*)(SpatRaster, std::vector<double>, SpatOptions&) )( &SpatRaster::cover))
+		.method("cover_self", ( SpatRaster (SpatRaster::*)(std::vector<double>, SpatOptions&) )( &SpatRaster::cover))
+
 		.method("crop", &SpatRaster::crop)
 		.method("crop_mask", &SpatRaster::cropmask)
 		.method("cum", &SpatRaster::cum)
 		.method("disaggregate", &SpatRaster::disaggregate)
 		.method("expand", &SpatRaster::extend)
 		.method("extractCell", &SpatRaster::extractCell)
-		.method("extractXY", &SpatRaster::extractXY)
+//		.method("extractXY", &SpatRaster::extractXY)
 //		.method("extractXYFlat", &SpatRaster::extractXYFlat, "extractXYflat")
 		.method("extractVector", &SpatRaster::extractVector)
 		.method("extractVectorFlat", &SpatRaster::extractVectorFlat)
+		.method("extractBuffer", &SpatRaster::extractBuffer)
+			
 		.method("flip", &SpatRaster::flip)
 		.method("focal", &SpatRaster::focal)
 		.method("focalValues", &SpatRaster::focal_values)
@@ -891,7 +938,7 @@ RCPP_MODULE(spat){
 
 		.method("get_aggregates", &SpatRaster::get_aggregates)
 		.method("get_aggregate_dims", &SpatRaster::get_aggregate_dims2)
-	//	.method("global", &SpatRaster::global)
+		.method("globalTF", &SpatRaster::globalTF)
 		.method("mglobal", &SpatRaster::mglobal)
 		.method("layerCor", &SpatRaster::layerCor)
 		.method("global_weighted_mean", &SpatRaster::global_weighted_mean)
@@ -938,6 +985,7 @@ RCPP_MODULE(spat){
 		.method("reverse", &SpatRaster::reverse)
 		.method("rotate", &SpatRaster::rotate)
 		//.method("sampleCells", &SpatRaster::sampleCells, "sampleCells")
+
 		.method("sampleRegularRaster", &SpatRaster::sampleRegularRaster)
 		.method("sampleRowColRaster", &SpatRaster::sampleRowColRaster)
 		.method("sampleRegularValues", &SpatRaster::sampleRegularValues)
@@ -945,6 +993,7 @@ RCPP_MODULE(spat){
 		.method("sampleRandomRaster", &SpatRaster::sampleRandomRaster)
 		.method("sampleRandomValues", &SpatRaster::sampleRandomValues)
 		.method("scale", &SpatRaster::scale)
+		.method("scale_linear", &SpatRaster::scale_linear)
 		.method("shift", &SpatRaster::shift)
 		.method("similarity", &SpatRaster::similarity)
 		.method("terrain", &SpatRaster::terrain)
@@ -955,16 +1004,18 @@ RCPP_MODULE(spat){
 		.method("trig", &SpatRaster::trig)
 		.method("trim1", &SpatRaster::trim1)
 		.method("trim", &SpatRaster::trim2)
+
 		.method("unique", &SpatRaster::unique)
 		.method("where", &SpatRaster::where)
 		.method("sieve", &SpatRaster::sieveFilter)
 		.method("view", &SpatRaster::viewshed)
 		.method("proximity", &SpatRaster::proximity)
+		.method("fillNA", &SpatRaster::fillNA)
 
 		.method("rectify", &SpatRaster::rectify)
 		.method("stretch", &SpatRaster::stretch)
 		.method("warp", &SpatRaster::warper)
-    .method("warp_by_util", &SpatRaster::warper_by_util)
+		.method("warp_by_util", &SpatRaster::warper_by_util)
 		.method("resample", &SpatRaster::resample)
 		.method("zonal", &SpatRaster::zonal)
 		.method("zonal_weighted", &SpatRaster::zonal_weighted)
@@ -972,11 +1023,16 @@ RCPP_MODULE(spat){
 		.method("zonal_poly_table", &SpatRaster::zonal_poly_table)		
 		.method("zonal_poly_weighted", &SpatRaster::zonal_poly_weighted)		
 //		.method("zonal_old", &SpatRaster::zonal_old)
+		.method("watershed2", &SpatRaster::watershed2, "watershed2") //EC 20210311 // EC 20210702
+		.method("pitfinder2", &SpatRaster::pitfinder2, "pitfinder2") //EC 20220810 // EC 20220810	
+		.method("NIDP2", &SpatRaster::NIDP2, "NIDP2") //EC 20231031
+		.method("flowAccu2", &SpatRaster::flowAccu2) //, "flowAccu2") //EC 20231031
+		.method("flowAccu2_weight", &SpatRaster::flowAccu2_weight) //, "flowAccu2_weight") //EC 20231114
 	;
 
-    class_<SpatRasterCollection>("SpatRasterCollection")
+	class_<SpatRasterCollection>("SpatRasterCollection")
 		.constructor()
-	    .constructor<std::string, std::vector<int>, bool, std::vector<std::string>>()
+		.constructor<std::string, std::vector<int>, bool, std::vector<std::string>>()
 
 		.property("names", &SpatRasterCollection::get_names, &SpatRasterCollection::set_names)
 
@@ -984,8 +1040,8 @@ RCPP_MODULE(spat){
 		.method("dims", &SpatRasterCollection::dims)
 		.method("extent", &SpatRasterCollection::getExtent)
 
-		.method("has_error", &SpatRasterCollection::has_error)
-		.method("has_warning", &SpatRasterCollection::has_warning)
+		.method("has_error", &SpatRasterCollection::hasError)
+		.method("has_warning", &SpatRasterCollection::hasWarning)
 		.method("getError", &SpatRasterCollection::getError)
 		.method("getWarnings", &SpatRasterCollection::getWarnings)
 		//.field("messages", &SpatRasterCollection::msg, "messages")
@@ -998,21 +1054,25 @@ RCPP_MODULE(spat){
 		.method("mosaic", &SpatRasterCollection::mosaic)
 		.method("morph", &SpatRasterCollection::morph)
 		.method("crop", &SpatRasterCollection::crop)
+		.method("addTag", &SpatRasterCollection::addTag)
+		.method("getTags", &SpatRasterCollection::getTags)
+		.method("make_vrt", &SpatRasterCollection::make_vrt)
 	;
 
-    class_<SpatRasterStack>("SpatRasterStack")
+	class_<SpatRasterStack>("SpatRasterStack")
 		.constructor()
-	    .constructor<std::string, std::vector<int>, bool, std::vector<std::string>>()
-	    .constructor<SpatRaster, std::string, std::string, std::string>()
+		.constructor<std::string, std::vector<int>, bool, std::vector<std::string>>()
+		.constructor<SpatRaster, std::string, std::string, std::string>()
 		.method("deepcopy", &SpatRasterStack::deepCopy)
 
-		.method("has_error", &SpatRasterStack::has_error)
-		.method("has_warning", &SpatRasterStack::has_warning)
+		.method("has_error", &SpatRasterStack::hasError)
+		.method("has_warning", &SpatRasterStack::hasWarning)
 		.method("getError", &SpatRasterStack::getError)
 		.method("getWarnings", &SpatRasterStack::getWarnings)
 
 		.method("readStart", &SpatRasterStack::readStart)
 		.method("readStop", &SpatRasterStack::readStop)
+		.method("readAll", &SpatRasterStack::readAll)
 		.method("nsds", &SpatRasterStack::nsds)
 		.method("ncol", &SpatRasterStack::ncol)
 		.method("nrow", &SpatRasterStack::nrow)
@@ -1036,6 +1096,9 @@ RCPP_MODULE(spat){
 		.method("extractCell", &SpatRasterStack::extractCell)
 		.method("extractVector", &SpatRasterStack::extractVector)
 		.method("crop", &SpatRasterStack::crop)
+		.method("addTag", &SpatRasterStack::addTag)
+		.method("getTags", &SpatRasterStack::getTags)
+
 	;
 }
 
