@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -33,6 +33,35 @@
 #include <sstream>
 
 
+/*
+GDAL 3.10
+
+std::vector<double> SpatRaster::extract_interpolate(std::vector<double> x, std::vector<double> y, std::string algo) {
+
+	GDALRIOResampleAlg eInterpolation = GRIORA_Bilinear ;
+	size_t n = x.size();
+	std::vector<double> out(n, NAN);
+	double value;
+	GDALDatasetH hDs;
+	
+	SpatOptions opt;
+	if (!open_gdal(hDs, 0, false, opt)) {
+		setError("cannot open dataset");
+		return(out);
+	}
+	
+	GDALRasterBandH poBand = GDALGetRasterBand(hDs, 1);
+	for (size_t i=0; i<n; i++) {
+		if (GDALInterpolateAtPoint(pbBand, x[i], y[1],  eInterpolation, &value) == CE_None) {
+			out[i] = value;
+		}
+	}
+	GDALClose( hDs );
+	return out;
+}	
+*/
+
+
 SpatGeom getPolygonsGeom2(OGRGeometry *poGeometry) {
 	SpatGeom g(polygons);
 	OGRPoint ogrPt;
@@ -164,7 +193,7 @@ SpatVector SpatRaster::dense_extent(bool inside, bool geobounds) {
 
 #if GDAL_VERSION_MAJOR <= 2 && GDAL_VERSION_MINOR < 2
 
-SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method, bool mask, bool align, SpatOptions &opt) {
+SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method, bool mask, bool align, bool resample, SpatOptions &opt) {
 	SpatRaster out;
 	out.setError("Not supported for this old version of GDAL");
 	return(out);
@@ -326,14 +355,13 @@ bool getAlgo(GDALResampleAlg &alg, std::string m) {
 		alg = GRA_Max;
 	} else if (m=="min") {
 		alg = GRA_Min;
-	} else if (m=="med") {
+	} else if (m=="median") {
 		alg = GRA_Med;
 	} else if (m=="q1") {
 		alg = GRA_Q1;
 	} else if (m=="q3") {
 		alg = GRA_Q3;
 	} else {
-		alg = GRA_NearestNeighbour;
 		return false;
 	}
 	return true;
@@ -346,7 +374,7 @@ bool is_valid_warp_method(const std::string &method) {
 }
 
 
-bool set_warp_options(GDALWarpOptions *psWarpOptions, GDALDatasetH &hSrcDS, GDALDatasetH &hDstDS, std::vector<unsigned> srcbands, std::vector<unsigned> dstbands, std::string method, std::string srccrs, std::string msg, bool verbose, bool threads) {
+bool set_warp_options(GDALWarpOptions *psWarpOptions, GDALDatasetH &hSrcDS, GDALDatasetH &hDstDS, std::vector<size_t> srcbands, std::vector<size_t> dstbands, std::string method, std::string srccrs, std::string msg, bool verbose, bool threads) {
 
 	if (srcbands.size() != dstbands.size()) {
 		msg = "number of source bands must match number of dest bands";
@@ -356,11 +384,7 @@ bool set_warp_options(GDALWarpOptions *psWarpOptions, GDALDatasetH &hSrcDS, GDAL
 
 	GDALResampleAlg a;
 	if (!getAlgo(a, method)) {
-		if ((method=="sum") || (method=="rms")) {
-			msg = method + " not available in your version of GDAL";
-		} else {
-			msg = "unknown resampling algorithm";
-		}
+		msg = method + " is not a valid method";
 		return false;
 	}
 
@@ -432,20 +456,247 @@ bool gdal_warper(GDALWarpOptions *psWarpOptions, GDALDatasetH &hSrcDS, GDALDatas
 */
 
 
+
 SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method, bool mask, bool align, bool resample, SpatOptions &opt) {
 
-/*
-	if (extset) {
-		std::vector<bool> m = inMemory();
-		if (!vall(m)) {
-			std::string fname = tempFile(opt.get_tempdir(), opt.pid, "_temp_rasterize.tif");
-			SpatOptions xopt(opt);
-			xopt.set_filenames({fname});
-			SpatRaster y = x.writeRaster(opt);
-			return warper(y, crs, method, mask, align, resample, opt);
+	size_t ns = nsrc();
+	bool fixext = false;
+	for (size_t j=0; j<ns; j++) {
+		if (source[j].extset && (!source[j].memory)) {
+			fixext = true;
+			break;
 		}
 	}
-*/
+	if (fixext) {
+		SpatRaster r = *this;
+		for (size_t j=0; j<ns; j++) {
+			if (r.source[j].extset && (!r.source[j].memory)) {
+				SpatRaster tmp(source[j]);
+				//if (tmp.canProcessInMemory(opt)) {
+				//	tmp.readAll();
+				//} else {
+				tmp = tmp.writeTempRaster(opt);
+				r.source[j] = tmp.source[0]; 
+			}
+		}
+		return r.warper(x, crs, method, mask, align, resample, opt);
+	}
+
+
+	SpatRaster out = x.geometry(nlyr(), false, false);
+	if (!is_valid_warp_method(method)) {
+		out.setError("not a valid warp method");
+		return out;
+	}
+	std::string srccrs = getSRS("wkt");
+	if (resample) {
+		out.setSRS(srccrs);
+	}
+
+	out.setNames(getNames());
+	if ((method == "near") || (method == "mode")) {
+		out.source[0].hasColors = hasColors();
+		out.source[0].cols = getColors();
+		out.source[0].hasCategories = hasCategories();
+		out.source[0].cats = getCategories();
+		out.rgb = rgb;
+		out.rgblyrs = rgblyrs;
+		out.rgbtype = rgbtype;
+	}
+	if (hasTime()) {
+		out.source[0].hasTime = true;
+		out.source[0].timestep = getTimeStep();
+		out.source[0].timezone = getTimeZone();
+		out.source[0].time = getTime();
+	}
+	if (hasUnit()) {
+		out.source[0].hasUnit = true;
+		out.source[0].unit = getUnit();
+	}
+
+	bool use_crs = !crs.empty();
+	if (use_crs) {
+		align = false;
+		resample = false;
+	} else if (!hasValues()) {
+		std::string fname = opt.get_filename();
+		if (!fname.empty()) {
+			out.addWarning("raster has no values, not writing to file");
+		}
+		return out;
+	}
+	if (align) {
+		crs = out.getSRS("wkt");
+	}
+
+	if (!resample) {
+		if (srccrs.empty()) {
+			out.setError("input raster CRS not set");
+			return out;
+		}
+	}
+
+	lrtrim(crs);
+	SpatOptions sopt(opt);
+	if (use_crs || align) {
+		GDALDatasetH hSrcDS;
+		if (!open_gdal(hSrcDS, 0, false, sopt)) {
+			out.setError("cannot create dataset from source");
+			return out;
+		}
+		out.setSRS(crs);
+		if (!get_output_bounds(hSrcDS, srccrs, crs, out)) {
+			GDALClose( hSrcDS );
+			out.setError("cannot get output boundaries for the target crs");
+			return out;
+		}
+		GDALClose( hSrcDS );
+	} else if (!resample) {
+		OGRSpatialReference source, target;
+		const char *pszDefFrom = srccrs.c_str();
+		OGRErr erro = source.SetFromUserInput(pszDefFrom);
+		if (erro != OGRERR_NONE) {
+			out.setError("input crs is not valid");
+			return out;
+		}
+		std::string targetcrs = out.getSRS("wkt");
+		const char *pszDefTo = targetcrs.c_str();
+		erro = target.SetFromUserInput(pszDefTo);
+		if (erro != OGRERR_NONE) {
+			out.setError("output crs is not valid");
+			return out;
+		}
+		OGRCoordinateTransformation *poCT;
+		poCT = OGRCreateCoordinateTransformation(&source, &target);
+		if( poCT == NULL )	{
+			out.setError( "Cannot do this transformation" );
+			return(out);
+		}
+		OCTDestroyCoordinateTransformation(poCT);
+	}
+
+	if (align) {
+		SpatExtent e = out.getExtent();
+		e = x.align(e, "out");
+		out.setExtent(e, false, true, "");
+		std::vector<double> res = x.resolution();
+		out = out.setResolution(res[0], res[1]);
+	}
+	if (!hasValues()) {
+		return out;
+	}
+
+	SpatOptions mopt;
+	if (mask) {
+		mopt = opt;
+		opt = SpatOptions(opt);
+	}
+
+	opt.ncopies += 4;
+	if (!out.writeStart(opt, filenames())) {
+		return out;
+	}
+
+	std::string errmsg;
+	SpatExtent eout = out.getExtent();
+
+
+
+//	std::vector<bool> has_so = source[0].has_scale_offset;
+//	std::vector<double> scale = source[0].scale;
+//	std::vector<double> offset = source[0].offset;
+
+//	for (size_t i=1; i<ns; i++) {
+//		has_so.insert(has_so.end(), source[0].has_scale_offset.begin(), source[0].has_scale_offset.end());
+//		scale.insert(scale.end(), source[0].scale.begin(), source[0].scale.end());
+//		offset.insert(offset.end(), source[0].offset.begin(), source[0].offset.end());
+//	}
+
+
+	std::vector<bool> has_so(nlyr(), false);
+	std::vector<double> scale(nlyr(), 1);
+	std::vector<double> offset(nlyr(), 0);
+
+
+	double halfy = out.yres() / 2;
+	for (size_t i = 0; i < out.bs.n; i++) {
+		eout.ymax = out.yFromRow(out.bs.row[i]) + halfy;
+		eout.ymin = out.yFromRow(out.bs.row[i] + out.bs.nrows[i]-1) - halfy;
+		SpatRaster crop_out = out.crop(eout, "near", false, sopt);
+		GDALDatasetH hDstDS;
+
+		if (!crop_out.create_gdalDS(hDstDS, "", "MEM", false, NAN, has_so, scale, offset, sopt)) {
+			return crop_out;
+		}
+
+		int bandstart = 0;
+		for (size_t j=0; j<ns; j++) {
+			GDALDatasetH hSrcDS;
+			if (!open_gdal(hSrcDS, j, false, sopt)) {
+				out.setError("cannot create dataset from source");
+				if( hDstDS != NULL ) GDALClose( (GDALDatasetH) hDstDS );
+				return out;
+			}
+			std::vector<size_t> srcbands = source[j].layers;
+			std::vector<size_t> dstbands(srcbands.size());
+			std::iota (dstbands.begin(), dstbands.end(), bandstart);
+			bandstart += dstbands.size();
+
+			GDALWarpOptions *psWarpOptions = GDALCreateWarpOptions();
+			if (!set_warp_options(psWarpOptions, hSrcDS, hDstDS, srcbands, dstbands, method, srccrs, errmsg, opt.get_verbose(), opt.threads)) {
+				if (hDstDS != NULL ) GDALClose((GDALDatasetH) hDstDS);
+				out.setError(errmsg);
+				return out;
+			}
+			//ok = gdal_warper(psWarpOptions, hSrcDS, hDstDS);
+			
+			bool ok=true;
+			GDALWarpOperation oOperation;
+			if (oOperation.Initialize(psWarpOptions) != CE_None) {
+				ok = false;
+			} else if (oOperation.ChunkAndWarpImage(0, 0, GDALGetRasterXSize(hDstDS), GDALGetRasterYSize(hDstDS)) != CE_None) {
+				ok = false;
+			}
+			GDALDestroyGenImgProjTransformer(psWarpOptions->pTransformerArg);
+			GDALDestroyWarpOptions(psWarpOptions);
+
+			if (hSrcDS != NULL) GDALClose((GDALDatasetH) hSrcDS);
+			if (!ok) {
+				if (hDstDS != NULL) GDALClose((GDALDatasetH) hDstDS);
+				out.setError("warp failure");
+				return out;
+			}
+		}
+
+		bool ok = crop_out.from_gdalMEM(hDstDS, false, true);
+		if (hDstDS != NULL) GDALClose((GDALDatasetH) hDstDS);
+		if (!ok) {
+			out.setError("cannot do this transformation (warp)");
+			return out;
+		}
+//		std::vector<double> v = crop_out.getValues(-1, opt);
+//		if (!out.writeBlock(v, i)) return out;
+		if (!out.writeBlock(crop_out.source[0].values, i)) return out;
+
+	}
+	out.writeStop();
+	if (mask) {
+		SpatVector v = dense_extent(true, true);
+		v = v.project(out.getSRS("wkt"), true);
+		if (v.nrow() > 0) {
+			out = out.mask(v, false, NAN, true, mopt);
+		} else {
+			out.addWarning("masking failed");
+		}
+	}
+	return out;
+}
+
+
+
+/*
+SpatRaster SpatRaster::oldwarper(SpatRaster x, std::string crs, std::string method, bool mask, bool align, bool resample, SpatOptions &opt) {
+
 	size_t ns = nsrc();
 	bool fixext = false;
 	for (size_t j=0; j<ns; j++) {
@@ -617,8 +868,8 @@ SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method,
 				if( hDstDS != NULL ) GDALClose( (GDALDatasetH) hDstDS );
 				return out;
 			}
-			std::vector<unsigned> srcbands = source[j].layers;
-			std::vector<unsigned> dstbands(srcbands.size());
+			std::vector<size_t> srcbands = source[j].layers;
+			std::vector<size_t> dstbands(srcbands.size());
 			std::iota (dstbands.begin(), dstbands.end(), bandstart);
 			bandstart += dstbands.size();
 
@@ -672,6 +923,7 @@ SpatRaster SpatRaster::warper(SpatRaster x, std::string crs, std::string method,
 	return out;
 }
 
+*/
 
 
 SpatRaster SpatRaster::warper_by_util(SpatRaster x, std::string crs, std::string method, bool mask, bool align, bool resample, SpatOptions &opt) {
@@ -711,7 +963,7 @@ SpatRaster SpatRaster::warper_by_util(SpatRaster x, std::string crs, std::string
 	}
 	
 	out.setNames(getNames());
-	if (method == "near") {
+	if ((method == "near") || (method == "mode")) {
 		out.source[0].hasColors = hasColors();
 		out.source[0].cols = getColors();
 		out.source[0].hasCategories = hasCategories();
@@ -753,15 +1005,14 @@ SpatRaster SpatRaster::warper_by_util(SpatRaster x, std::string crs, std::string
 	SpatOptions sopt(opt);
 	if (use_crs || align) {
 		GDALDatasetH hSrcDS;
-		SpatRaster g = geometry(1);
-		if (!g.open_gdal(hSrcDS, 0, false, sopt)) {
+		if (!open_gdal(hSrcDS, 0, false, sopt)) {
 			out.setError("cannot create dataset from source");
 			return out;
 		}
 		out.setSRS(crs);
 		if (!get_output_bounds(hSrcDS, srccrs, crs, out)) {
 			GDALClose( hSrcDS );
-			out.setError("cannot get output boundaries");
+			out.setError("cannot get output boundaries for the target crs");
 			return out;
 		}
 		GDALClose( hSrcDS );
@@ -843,8 +1094,8 @@ SpatRaster SpatRaster::warper_by_util(SpatRaster x, std::string crs, std::string
 				if( hDstDS != NULL ) GDALClose( (GDALDatasetH) hDstDS );
 				return out;
 			}
-			std::vector<unsigned> srcbands = source[j].layers;
-			std::vector<unsigned> dstbands(srcbands.size());
+			std::vector<size_t> srcbands = source[j].layers;
+			std::vector<size_t> dstbands(srcbands.size());
 			std::iota (dstbands.begin(), dstbands.end(), bandstart);
 			bandstart += dstbands.size();
 			
@@ -917,7 +1168,7 @@ SpatRaster SpatRaster::warper_by_util(SpatRaster x, std::string crs, std::string
 
 SpatRaster SpatRaster::resample(SpatRaster x, std::string method, bool mask, bool agg, SpatOptions &opt) {
 
-	unsigned nl = nlyr();
+	size_t nl = nlyr();
 	SpatRaster out = x.geometry(nl);
 	out.setNames(getNames());
 
@@ -954,12 +1205,12 @@ SpatRaster SpatRaster::resample(SpatRaster x, std::string method, bool mask, boo
 			//    a) disaggregate "x", warp, and aggregate the results
 			//    b) or give a warning?
 		} else {
-			unsigned xq = x.xres() / xres();
-			unsigned yq = x.yres() / yres();
+			size_t xq = x.xres() / xres();
+			size_t yq = x.yres() / yres();
 			if (std::max(xq, yq) > 1) {
 				xq = xq == 0 ? 1 : xq;
 				yq = yq == 0 ? 1 : yq;
-				std::vector<unsigned> agf = {yq, xq, 1};
+				std::vector<size_t> agf = {yq, xq, 1};
 				SpatOptions agopt(opt);
 				SpatRaster xx;
 				if (method == "bilinear") {
@@ -978,11 +1229,11 @@ SpatRaster SpatRaster::resample(SpatRaster x, std::string method, bool mask, boo
 		opt = SpatOptions(opt);
 	}
 
-	unsigned nc = out.ncol();
+	size_t nc = out.ncol();
   	if (!out.writeStart(opt, filenames())) { return out; }
 	for (size_t i = 0; i < out.bs.n; i++) {
-        unsigned firstcell = out.cellFromRowCol(out.bs.row[i], 0);
-		unsigned lastcell  = out.cellFromRowCol(out.bs.row[i]+out.bs.nrows[i]-1, nc-1);
+        size_t firstcell = out.cellFromRowCol(out.bs.row[i], 0);
+		size_t lastcell  = out.cellFromRowCol(out.bs.row[i]+out.bs.nrows[i]-1, nc-1);
 		std::vector<double> cells(1+lastcell-firstcell);
 		std::iota (std::begin(cells), std::end(cells), firstcell);
         std::vector<std::vector<double>> xy = out.xyFromCell(cells);
@@ -1047,24 +1298,8 @@ SpatRaster SpatRaster::rectify(std::string method, SpatRaster aoi, unsigned usea
 	}
 	double gt[6];
 	if( poDataset->GetGeoTransform(gt) != CE_None ) {
-	
 		if (GCP_geotrans(poDataset, gt)) {
-		
-			GDALClose( (GDALDatasetH) poDataset );
-			std::string tmpfile = tempFile(opt.get_tempdir(), opt.pid, "_rect.tif");
-			//++17 
-			//std::filesystem::copy_file(source[0].filename, tmpfile);	
-
-			std::ifstream  src(source[0].filename, std::ios::binary);
-			std::ofstream  dst(tmpfile,   std::ios::binary);
-			dst << src.rdbuf();	
-		
-			GDALDataset *poDataset = openGDAL(tmpfile, GDAL_OF_RASTER | GDAL_OF_READONLY, source[0].open_drivers, source[0].open_ops);
 			poDataset->SetGeoTransform(gt);
-			GDALClose( (GDALDatasetH) poDataset );
-			SpatRaster tmp(tmpfile,  {-1}, {""}, {}, {});
-			return tmp.rectify(method, aoi, useaoi, snap, opt);
-
 		} else {
 			out.setError("can't get the geotransform");
 			GDALClose( (GDALDatasetH) poDataset );
@@ -1147,7 +1382,10 @@ SpatVector SpatRaster::polygonize(bool round, bool values, bool narm, bool aggre
 	if (round && (digits > 0)) {
 		tmp = tmp.math2("round", digits, topt);
 		round = false;
+	} else if (tmp.source[0].extset) { 
+		tmp = tmp.hardCopy(topt);
 	}
+
 /*
 	} else if (tmp.sources_from_file()) {
 		// for NAN and INT files. Should have a check for that
@@ -1157,9 +1395,6 @@ SpatVector SpatRaster::polygonize(bool round, bool values, bool narm, bool aggre
 	}
 */
 	
-	if (tmp.source[0].extset) {
-		tmp = tmp.hardCopy(topt);
-	}
 
 	GDALDatasetH rstDS;
 	if (!tmp.open_gdal(rstDS, 0, false, topt)) {
@@ -1278,6 +1513,18 @@ SpatRaster SpatRaster::rgb2col(size_t r,  size_t g, size_t b, SpatOptions &opt)
 		return out;
 	}
 
+	std::vector<size_t> lyrs = {r, g, b};
+	SpatOptions ops(opt);
+	SpatRaster tmp = subset(lyrs, ops);
+
+	if (source[0].extset) {
+		SpatOptions topt(opt);
+		tmp = tmp.hardCopy(topt);
+		return tmp.rgb2col(r, g, b, opt);
+	} else {
+		tmp = tmp.collapse_sources();
+	}
+
 	std::string filename = opt.get_filename();
 	opt.set_datatype("INT1U");
 	std::string driver;
@@ -1285,7 +1532,7 @@ SpatRaster SpatRaster::rgb2col(size_t r,  size_t g, size_t b, SpatOptions &opt)
 		if (canProcessInMemory(opt)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		}
@@ -1304,12 +1551,8 @@ SpatRaster SpatRaster::rgb2col(size_t r,  size_t g, size_t b, SpatOptions &opt)
 		}
 	}
 
-	std::vector<unsigned> lyrs = {(unsigned)r, (unsigned)g, (unsigned)b};
-	SpatOptions ops(opt);
-	*this = subset(lyrs, ops);
-	*this = collapse_sources();
 	GDALDatasetH hSrcDS, hDstDS;
-	if (!open_gdal(hSrcDS, 0, false, ops)) {
+	if (!tmp.open_gdal(hSrcDS, 0, false, ops)) {
 		out.setError("cannot create dataset from source");
 		return out;
 	}
@@ -1457,7 +1700,7 @@ SpatRaster SpatRaster::viewshed(const std::vector<double> obs, const std::vector
 		}
 	}
 
-	std::string filename = tempFile(topt.get_tempdir(), topt.pid, ".tif");
+	std::string filename = tempFile(topt.get_tempdir(), topt.tmpfile, ".tif");
 	driver = "GTiff";
 
 	GDALDatasetH hSrcDS;
@@ -1531,6 +1774,18 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 		out.setError("input raster has no values");
 		return out;
 	}
+	
+	if (source[0].extset) { 
+		SpatOptions topt(opt);
+		SpatRaster etmp;
+		if (nlyr() > 1) {
+			etmp = etmp.subset({0}, topt);
+			etmp = etmp.hardCopy(topt);
+		} else {
+			etmp = hardCopy(topt);
+		}
+		return etmp.proximity(target, exclude, keepNA, unit, buffer, maxdist, remove_zero, opt);
+	}
 
 	std::string filename = opt.get_filename();
 	std::string driver;
@@ -1538,7 +1793,7 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 		if (canProcessInMemory(opt)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		}
@@ -1575,6 +1830,10 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 	SpatRaster x;
 	bool mask = false;
 	std::vector<double> mvals;
+	if ((!buffer) && (maxdist > 0)) {
+		papszOptions = CSLSetNameValue(papszOptions, "MAXDIST", doubleToAlmostChar(maxdist).c_str());
+	}
+
 	if (buffer) {
 		if (remove_zero) {
 			x = isnotnan(true, ops);
@@ -1592,6 +1851,7 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 	} else {
 		//option for keepNA does not work, perhaps because of int conversion
 		//papszOptions = CSLSetNameValue(papszOptions, "USE_INPUT_NODATA", "YES");
+		
 		if (std::isnan(exclude)) {
 			if (keepNA) {
 				x = replaceValues({target, NAN}, {0, 0}, 1, true, 1, false, ops);
@@ -1616,7 +1876,7 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 		return out;
 	}
 
-	std::string tmpfile = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+	std::string tmpfile = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 	std::string fname = mask ? tmpfile : filename;
 	if (!out.create_gdalDS(hDstDS, fname, driver, false, 0, {false}, {1}, {0}, ops)) {
 		out.setError("cannot create new dataset");
@@ -1663,6 +1923,14 @@ SpatRaster SpatRaster::proximity(double target, double exclude, bool keepNA, std
 
 SpatRaster SpatRaster::sieveFilter(int threshold, int connections, SpatOptions &opt) {
 
+	if (nlyr() > 1) {
+		SpatOptions sopt(opt);
+		SpatRaster tmp = subset({0}, sopt);
+		tmp = tmp.sieveFilter(threshold, connections, opt);
+		tmp.addWarning("only the first layer was used");
+		return tmp;
+	}
+
 	SpatRaster out = geometry(1, true, true, true);
 
 	if (!hasValues()) {
@@ -1677,75 +1945,79 @@ SpatRaster SpatRaster::sieveFilter(int threshold, int connections, SpatOptions &
 		out.setError("a threshold < 2 is not meaningful");
 		return out;
 	}
-
-	std::string filename = opt.get_filename();
-	std::string driver;
-	if (filename.empty()) {
-		if (canProcessInMemory(opt)) {
-			driver = "MEM";
-		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
-			opt.set_filenames({filename});
-			driver = "GTiff";
-		}
-	} else {
-		driver = opt.get_filetype();
-		getGDALdriver(filename, driver);
-		if (driver.empty()) {
-			setError("cannot guess file type from filename");
-			return out;
-		}
-		std::string errmsg;
-		if (!can_write({filename}, filenames(), opt.get_overwrite(), errmsg)) {
-			out.setError(errmsg);
-			return out;
-		}
+	
+		
+	if (source[0].extset) { 
+		SpatOptions topt(opt);
+		SpatRaster etmp = hardCopy(topt);
+		return etmp.sieveFilter(threshold, connections, opt);
 	}
 
+
+	std::string tmp_filename = "";
+	std::string driver = "MEM";
 	SpatOptions ops(opt);
-	GDALDatasetH hSrcDS, hDstDS;
+	if (!canProcessInMemory(ops)) {
+		tmp_filename = tempFile(ops.get_tempdir(), ops.tmpfile, "_sieve.tif");
+		driver = "GTiff";
+		ops.set_filenames({tmp_filename});
+	}
+
+	SpatOptions mopt(opt);
+	SpatRaster mask = isnotnan(false, mopt);
+
+	GDALDatasetH hSrcDS, hMskDS, hDstDS;
+
 	if (!open_gdal(hSrcDS, 0, false, ops)) {
 		out.setError("cannot open input dataset");
 		return out;
 	}
+	if (!mask.open_gdal(hMskDS, 0, false, ops)) {
+		out.setError("cannot open mask dataset");
+		return out;
+	}
 
 	GDALDriverH hDriver = GDALGetDriverByName( driver.c_str() );
 	if ( hDriver == NULL ) {
 		out.setError("empty driver");
 		return out;
 	}
-
 	//opt.datatype = "INT4S";
-	if (!out.create_gdalDS(hDstDS, filename, driver, true, 0, source[0].has_scale_offset, source[0].scale, source[0].offset, opt)) {
+	if (!out.create_gdalDS(hDstDS, tmp_filename, driver, true, 0, source[0].has_scale_offset, source[0].scale, source[0].offset, ops)) {
 		out.setError("cannot create new dataset");
 		GDALClose(hSrcDS);
 		return out;
 	}
 
 	GDALRasterBandH hSrcBand = GDALGetRasterBand(hSrcDS, 1);
+	GDALRasterBandH hMskBand = GDALGetRasterBand(hMskDS, 1);
 	GDALRasterBandH hTargetBand = GDALGetRasterBand(hDstDS, 1);
 
-	if (GDALSieveFilter(hSrcBand, nullptr, hTargetBand, threshold, connections, nullptr, NULL, NULL) != CE_None) {
-		out.setError("sieve failed");
+	if (GDALSieveFilter(hSrcBand, hMskBand, hTargetBand, threshold, connections, nullptr, NULL, NULL) != CE_None) {
 		GDALClose(hSrcDS);
+		GDALClose(hMskDS);
 		GDALClose(hDstDS);
+		out.setError("sieve failed");
 		return out;
 	}
 
 	GDALClose(hSrcDS);
+	GDALClose(hMskDS);
+
 	if (driver == "MEM") {
 		if (!out.from_gdalMEM(hDstDS, false, true)) {
 			out.setError("conversion failed (mem)");
 		}
 		GDALClose(hDstDS);
-		return out;
+
+	} else {
+		GDALClose(hDstDS);
+		out = SpatRaster(tmp_filename, {-1}, {""}, {}, {});
 	}
 	
-	double adfMinMax[2];
-	GDALComputeRasterMinMax(hTargetBand, true, adfMinMax);
-	GDALSetRasterStatistics(hTargetBand, adfMinMax[0], adfMinMax[1], -9999, -9999);
-	GDALClose(hDstDS);
-	return SpatRaster(filename, {-1}, {""}, {}, {});
+	opt.names = getNames();
+	out.source[0].source_name = {""};
+	return out.mask(mask, false, 0, NAN, opt);
 }
 
 
@@ -1892,7 +2164,6 @@ void *LinearOps(std::vector<double> op) {
 SpatRaster SpatRaster::rasterizeWindow(std::vector<double> x, std::vector<double> y, std::vector<double> z, std::string algo, std::vector<double> algops, SpatOptions &opt) {
 
 	SpatRaster out = geometry(1);
-
 	GDALGridAlgorithm eAlg;
 		if (!getGridderAlgo(algo, eAlg)) {
 		out.setError("unknown algorithm");
@@ -1945,6 +2216,10 @@ SpatRaster SpatRaster::rasterizeWindow(std::vector<double> x, std::vector<double
 		return out;
 	}
 
+	const char *old_count_value = CPLGetConfigOption("GDAL_GRID_POINT_COUNT_THRESHOLD", NULL);
+	std::string n = std::to_string(x.size());
+	CPLSetConfigOption("GDAL_GRID_POINT_COUNT_THRESHOLD", n.c_str());
+
 	GUInt32 npts = x.size();
 	GDALGridContext *ctxt = GDALGridContextCreate(eAlg, poOptions, npts, &x[0], &y[0], &z[0], true);
 	CPLFree( poOptions );
@@ -1963,6 +2238,7 @@ SpatRaster SpatRaster::rasterizeWindow(std::vector<double> x, std::vector<double
 		if ( eErr != CE_None ) {
 			out.setError("something went wrong");
 			GDALGridContextFree(ctxt);
+			CPLSetConfigOption("GDAL_GRID_POINT_COUNT_THRESHOLD", old_count_value);
 			return out;
 		}
 
@@ -1974,10 +2250,13 @@ SpatRaster SpatRaster::rasterizeWindow(std::vector<double> x, std::vector<double
 		}
 		if (!out.writeBlock(f, i)) {
 			GDALGridContextFree(ctxt);
+			CPLSetConfigOption("GDAL_GRID_POINT_COUNT_THRESHOLD", old_count_value);
 			return out;
 		}
 	}
 	GDALGridContextFree(ctxt);
+	CPLSetConfigOption("GDAL_GRID_POINT_COUNT_THRESHOLD", old_count_value);
+
 	out.writeStop();
 	return out;
 }
@@ -1989,6 +2268,97 @@ SpatRaster SpatRaster::fillna(int threshold, int connections, SpatOptions &opt)
 CPLErr GDALFillNodata(GDALRasterBandH hTargetBand, GDALRasterBandH hMaskBand, doubledfMaxSearchDist, intbDeprecatedOption, intnSmoothingIterations, char**papszOptions, GDALProgressFuncpfnProgress, void*pProgressArg)
 */
 
+
+SpatRaster SpatRaster::fillNA(double missing, double maxdist, int niter, SpatOptions &opt) {
+
+	SpatRaster out = geometry(1, true, true, true);
+
+	if (!hasValues()) {
+		out.setError("input raster has no values");
+		return out;
+	}
+	if (maxdist <= 0) {
+		out.setError("maxdist should be > 0");
+		return out;
+	}
+	if (niter < 0) {
+		out.setError("niter should be >= 0");
+		return out;
+	}
+
+	std::string filename = opt.get_filename();
+	std::string driver;
+	if (filename.empty()) {
+		if (canProcessInMemory(opt)) {
+			driver = "MEM";
+		} else {
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
+			opt.set_filenames({filename});
+			driver = "GTiff";
+		}
+	} else {
+		driver = opt.get_filetype();
+		getGDALdriver(filename, driver);
+		if (driver.empty()) {
+			setError("cannot guess file type from filename");
+			return out;
+		}
+		std::string errmsg;
+		if (!can_write({filename}, filenames(), opt.get_overwrite(), errmsg)) {
+			out.setError(errmsg);
+			return out;
+		}
+	}
+
+	SpatOptions ops(opt);
+	GDALDatasetH hSrcDS, hDstDS;
+	if (!open_gdal(hSrcDS, 0, false, ops)) {
+		out.setError("cannot open input dataset");
+		return out;
+	}
+
+	GDALDriverH hDriver = GDALGetDriverByName( driver.c_str() );
+	if ( hDriver == NULL ) {
+		out.setError("empty driver");
+		return out;
+	}
+
+	//opt.datatype = "INT4S";
+	if (!out.create_gdalDS(hDstDS, filename, driver, true, 0, source[0].has_scale_offset, source[0].scale, source[0].offset, opt)) {
+		out.setError("cannot create new dataset");
+		GDALClose(hSrcDS);
+		return out;
+	}
+
+	GDALRasterBandH hSrcBand = GDALGetRasterBand(hSrcDS, 1);
+	GDALRasterBandH hTargetBand = GDALGetRasterBand(hDstDS, 1);
+
+	if (GDALFillNodata(hTargetBand, hSrcBand, maxdist, 0, niter, NULL, NULL, NULL) != CE_None) {
+		out.setError("fillNA failed");
+		GDALClose(hSrcDS);
+		GDALClose(hDstDS);
+		return out;
+	}
+
+	GDALClose(hSrcDS);
+	if (driver == "MEM") {
+		if (!out.from_gdalMEM(hDstDS, false, true)) {
+			out.setError("conversion failed (mem)");
+		}
+		GDALClose(hDstDS);
+		return out;
+	}
+	
+	double adfMinMax[2];
+	GDALComputeRasterMinMax(hTargetBand, true, adfMinMax);
+	GDALSetRasterStatistics(hTargetBand, adfMinMax[0], adfMinMax[1], -9999, -9999);
+	GDALClose(hDstDS);
+	return SpatRaster(filename, {-1}, {""}, {}, {});
+}
+
+
+
+
 /*
 #include <gdalpansharpen.h>
 SpatRaster SpatRaster::panSharpen(SpatRaster pan, SpatOptions &opt) {
