@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -22,7 +22,6 @@
 #include "recycle.h"
 #include "string_utils.h"
 
-
 void callbck(void *item, void *userdata) { // callback function for tree selection
 	std::vector<size_t> *ret = (std::vector<size_t> *) userdata;
 	ret->push_back(*((size_t *) item));
@@ -48,7 +47,7 @@ SpatVectorCollection SpatVector::bienvenue() {
 std::vector<std::string> SpatVector::wkt() {
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
-	std::vector<std::string> out;
+	std::vector<std::string> out::crop
 	out.reserve(g.size());
 	char * wkt;
 	for (size_t i = 0; i < g.size(); i++) {
@@ -95,7 +94,21 @@ std::vector<std::string> SpatVector::wkb() {
 	return out;
 }
 
-	
+std::vector<std::vector<unsigned char>> SpatVector::wkb_raw() {
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
+	std::vector<std::vector<unsigned char>> out; 
+	size_t len = 0;
+	for (size_t i = 0; i < g.size(); i++) {
+		unsigned char *hex = GEOSGeomToWKB_buf_r(hGEOSCtxt, g[i].get(), &len);
+		std::vector<unsigned char> raw; 
+		raw = std::vector<unsigned char>(hex, hex+len);
+		out.push_back(raw);
+		free(hex);
+	}
+	geos_finish(hGEOSCtxt);
+	return out;
+}	
 	
 std::vector<std::string> SpatVector::hex() {
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
@@ -214,7 +227,7 @@ SpatVector SpatVector::make_valid2() {
 		if (r != NULL) {
 			if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
 				x[i] = geos_ptr(r, hGEOSCtxt);
-				ids.push_back(i);
+				//ids.push_back(i);
 			} else {
 				GEOSGeom_destroy_r(hGEOSCtxt, r);
 			}
@@ -224,11 +237,11 @@ SpatVector SpatVector::make_valid2() {
 	out = coll.get(0);
 	geos_finish(hGEOSCtxt);
 	out.srs = srs;
-	if (ids.size() != n) {
-		out.df = df.subset_rows(out.df.iv[0]);
-	} else {
+//	if (ids.size() != n) {
+//		out.df = df.subset_rows(out.df.iv[0]);
+//	} else {
 		out.df = df;
-	}
+//	}
 #endif
 	return out;
 }
@@ -314,6 +327,10 @@ std::vector<std::vector<unsigned>> SpatVector::index_sparse(SpatVector v) {
 SpatVector SpatVector::crop(SpatExtent e, bool wrap) {
 
 	SpatVector out;
+	if ((e.xmin == e.xmax) || (e.ymin == e.ymax)) {
+		return *this;	
+	}
+
 
 #ifndef GEOS350
 	out.setError("GEOS 3.5 required for crop");
@@ -705,6 +722,56 @@ SpatVector SpatVector::shared_paths(SpatVector x, bool index) {
 }
 
 
+bool find_segments(GEOSContextHandle_t hGEOSCtxt, std::vector<double> &x, std::vector<double> &y, std::vector<double> &cx, std::vector<double> &cy, std::vector<size_t> &si, std::vector<double> &sx, std::vector<double> &sy) {
+	size_t n = x.size() - 1;
+	size_t m = cx.size() - 1;
+	double ix, iy;
+	si.resize(0);
+	sx.resize(0);
+	sy.resize(0);
+	for (size_t i=0; i<n; i++) {
+		for (size_t j=0; j<m; j++) {
+			if (GEOSSegmentIntersection_r(hGEOSCtxt, x[i], y[i], x[i+1], cy[j+1], cx[j], cy[j], cx[j+1], cy[j+1], &ix, &iy) == 1) {
+				si.push_back(i);
+				sx.push_back(ix);
+				sy.push_back(iy);
+			}
+		}
+	}
+	return si.size() > 0;
+}
+
+
+SpatVector SpatVector::split_lines(SpatVector v) {
+
+// check GEOS version 
+
+	SpatVector out;
+	return out;
+
+/*
+	SpatVector out = *this;
+	std::vector<size_t> si;
+	std::vector<double> sx, sy;
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+
+	for (size_t i=0; i<v.size(); i++) {
+		SpatVector tmp = v.subset_rows(i);
+		std::vector<int> x = out.relate(tmp, "intersects", true, true);
+		std::vector<std::vector<double>> xy1 = tmp.coordinates();
+		for (size_t j=0; j<x.size(); j++) {
+			if (x[j] == 1) {
+				std::vector<std::vector<double>> xy2 = tmp.coordinates();
+//				if (find_segments(hGEOSCtxt, xy1[0], xy1[1], xy2[0], xy2[1], si, sx, sy)) {
+					
+//				}
+			}
+		}
+	}
+	return out;
+*/
+}	
+
 
 /*
 SpatVector SpatVector::split_polygons(SpatVector lns) {
@@ -722,37 +789,57 @@ SpatVector SpatVector::split_polygons(SpatVector lns) {
 		g = v.geoms[0];
 */
 
+
+
+SpatVector polygonize_one(const GEOSGeometry* gi, GEOSContextHandle_t hGEOSCtxt) {
+
+	size_t ngeoms = 1;
+	std::vector<GeomPtr> p(1);
+	SpatVector out;
+
+	GEOSGeometry* r = GEOSPolygonize_r(hGEOSCtxt, &gi, ngeoms);
+	if (r == NULL) {
+		out.setError("something bad happened");
+		geos_finish(hGEOSCtxt);
+		return out;
+	}
+	if (GEOSisEmpty_r(hGEOSCtxt, r)) {
+		GEOSGeom_destroy_r(hGEOSCtxt, r);
+	} else {
+		p[0] = geos_ptr(r, hGEOSCtxt);
+		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
+		out = coll.get(0);
+		out.aggregate(false);
+	}
+	return out;
+}
+
 SpatVector SpatVector::polygonize() {
 
+	if (type() == "polygons") {
+		return *this;
+	}
+
 	SpatVector out;
 	out.srs = srs;
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 
 	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
-	std::vector<GeomPtr> p;
-	p.reserve(g.size());
-	size_t ngeoms = 1;
-	for (size_t i = 0; i < g.size(); i++) {
+
+	const GEOSGeometry* gi = g[0].get();
+	out = polygonize_one(gi, hGEOSCtxt);
+
+	for (size_t i = 1; i < g.size(); i++) {
 		const GEOSGeometry* gi = g[i].get();
-		GEOSGeometry* r = GEOSPolygonize_r(hGEOSCtxt, &gi, ngeoms);
-		if (r == NULL) {
-			out.setError("something bad happened");
-			geos_finish(hGEOSCtxt);
-			return out;
-		}
-		if (!GEOSisEmpty_r(hGEOSCtxt, r)) {
-			p.push_back(geos_ptr(r, hGEOSCtxt));
-		} else {
-			GEOSGeom_destroy_r(hGEOSCtxt, r);
-		}
-	}
-	if (!p.empty()) {
-		SpatVectorCollection coll = coll_from_geos(p, hGEOSCtxt);
-		out = coll.get(0);
-		out.srs = srs;
-		out.df = df;
+		SpatVector onegeom = polygonize_one(gi, hGEOSCtxt);
+		out.addGeom(onegeom.getGeom(0));
 	}
 	geos_finish(hGEOSCtxt);
+
+	out.srs = srs;
+	if (df.nrow() == out.size()) {
+		out.df = df;
+	}
 	return out;
 }
 
@@ -851,6 +938,7 @@ SpatVector SpatVector::crop(SpatVector v) {
 
 	SpatVector out;
 	out.srs = srs;
+	if (v.empty()) return *this;
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 
@@ -870,9 +958,7 @@ SpatVector SpatVector::crop(SpatVector v) {
 	ids.reserve(nx);
 
 	for (size_t i = 0; i < nx; i++) {
-	//	Rcpp::Rcout << "intersection " << i;
 		GEOSGeometry* geom = GEOSIntersection_r(hGEOSCtxt, x[i].get(), y[0].get());
-	//	Rcpp::Rcout << " done" << std::endl;
 		if (geom == NULL) {
 			out.setError("GEOS exception");
 			geos_finish(hGEOSCtxt);
@@ -890,8 +976,11 @@ SpatVector SpatVector::crop(SpatVector v) {
 
 	if (!result.empty()) {
 //		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
-		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids);
-		out = coll.get(0);
+//		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids);
+		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids, true, false);
+		out = coll.get(0);	
+//		std::vector<std::string> nms = out.get_names();
+//		out = out.aggregate(nms[0], true);
 		out.df = df.subset_rows(out.df.iv[0]);
 		out.srs = srs;
 	}
@@ -901,9 +990,19 @@ SpatVector SpatVector::crop(SpatVector v) {
 
 
 
-SpatVector SpatVector::hull(std::string htype, std::string by) {
+SpatVector SpatVector::hull(std::string htype, std::string by, double param, bool allowHoles, bool tight) {
 
 	SpatVector out;
+	if (nrow() == 0) {
+		out.srs = srs;
+		return out;
+	}
+
+	std::vector<std::string> methods = {"convex", "rectangle", "circle", "concave_ratio", "concave_length"};
+	if (std::find(methods.begin(), methods.end(), htype) == methods.end()) {
+		out.setError("unknown hull type");
+		return out;
+	}
 
 	if (!by.empty()) {
 		SpatVector tmp = aggregate(by, false);
@@ -930,47 +1029,73 @@ SpatVector SpatVector::hull(std::string htype, std::string by) {
 
 	out.reserve(size());
 
-	if (htype != "convex") {
-		#ifndef GEOS361
-		out.setError("GEOS 3.6.1 required for rotated rectangle");
-		return out;
-		#endif
-		if (is_lonlat()) {
-			if ((extent.ymin > -85) && (extent.ymax < 85)) {
-				SpatVector tmp = project("+proj=merc", false);
-				tmp = tmp.hull(htype, "");
-				tmp = tmp.project(srs.wkt, false);
-				return tmp;
-			}
+/*
+	if (is_lonlat()) {
+		if ((extent.ymin > -85) && (extent.ymax < 85)) {
+			SpatVector tmp = project("+proj=merc", false);
+			tmp = tmp.hull(htype, "");
+			tmp = tmp.project(srs.wkt, false);
+			return tmp;
 		}
 	}
+*/
 
+	SpatVector a = aggregate(false);
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
-	SpatVector a = aggregate(false);
 	std::vector<GeomPtr> g = geos_geoms(&a, hGEOSCtxt);
 	//std::string vt = type();
 	GEOSGeometry* h;
 	if (htype == "convex") {
-		h = GEOSConvexHull_r(hGEOSCtxt, g[0].get());
+		h = GEOSConvexHull_r(hGEOSCtxt, g[0].get());	
 	} else if (htype == "circle") {
 	#ifndef GEOS380
+		geos_finish(hGEOSCtxt);
 		out.setError("GEOS 3.8 required for bounding circle");
 		return out;
 	#else
 		h = GEOSMinimumBoundingCircle_r(hGEOSCtxt, g[0].get(), NULL, NULL);
 	#endif
-	} else {
+	} else if (htype == "rectangle") {
 	#ifndef GEOS361
+		geos_finish(hGEOSCtxt);
 		out.setError("GEOS 3.6.1 required for rotated rectangle");
 		return out;
 	#else
 		h = GEOSMinimumRotatedRectangle_r(hGEOSCtxt, g[0].get());
 	#endif
+	} else if (htype == "concave_ratio") {
+	#ifndef GEOS3110
+		geos_finish(hGEOSCtxt);
+		out.setError("GEOS 3.11 required for concave hull");
+		return out;
+	#else
+		h = GEOSConcaveHull_r(hGEOSCtxt, g[0].get(), param, allowHoles);
+	#endif
+	} else if (htype == "concave_length") {
+	#ifndef GEOS3110
+		geos_finish(hGEOSCtxt);
+		out.setError("GEOS 3.11 required for concave_length hull");
+		return out;
+	#else
+		if (type() == "polygons") {
+			h = GEOSConcaveHullOfPolygons_r(hGEOSCtxt, g[0].get(), param, tight, allowHoles);
+		} else {
+		#ifndef GEOS3120
+				geos_finish(hGEOSCtxt);
+				out.setError("GEOS 3.12 required for concave_length hull for points and lines");
+				return out;
+		#else 
+				h = GEOSConcaveHullByLength_r(hGEOSCtxt, g[0].get(), param, allowHoles);
+		#endif	
+		}
+	#endif
+	} else {
+		geos_finish(hGEOSCtxt);
+		out.setError("unknown hull type");
+		return out;
 	}
 	
-	
-	
 	std::vector<GeomPtr> b(1);
 	b[0] = geos_ptr(h, hGEOSCtxt);
 	SpatVectorCollection coll = coll_from_geos(b, hGEOSCtxt);
@@ -1048,7 +1173,7 @@ SpatVector SpatVector::voronoi(SpatVector bnd, double tolerance, int onlyEdges)
 
 
 
-SpatVector SpatVector::delaunay(double tolerance, int onlyEdges) {
+SpatVector SpatVector::delaunay(double tolerance, int onlyEdges, bool constrained) {
 	SpatVector out;
 	if (nrow() == 0) {
 		out.addWarning("input SpatVector has no geometries");
@@ -1058,12 +1183,25 @@ SpatVector SpatVector::delaunay(double tolerance, int onlyEdges) {
 #ifndef GEOS350
 	out.setError("GEOS 3.5 required for delaunay");
 	return out;
+#endif 
+
+#ifndef GEOS3100
+	if (constrained) {
+		out.setError("GEOS 3.10 required for constrained delaunay");
+		return out;
+	}
 #else
 
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	SpatVector a = aggregate(false);
 	std::vector<GeomPtr> g = geos_geoms(&a, hGEOSCtxt);
-	GEOSGeometry* v = GEOSDelaunayTriangulation_r(hGEOSCtxt, g[0].get(), tolerance, onlyEdges);
+
+	GEOSGeometry* v;
+	if (constrained) {
+		v = GEOSConstrainedDelaunayTriangulation_r(hGEOSCtxt, g[0].get());
+	} else {
+		v = GEOSDelaunayTriangulation_r(hGEOSCtxt, g[0].get(), tolerance, onlyEdges);
+	}	
 	if (v == NULL) {
 		out.setError("GEOS exception");
 		geos_finish(hGEOSCtxt);
@@ -1092,6 +1230,11 @@ SpatVector SpatVector::buffer(std::vector<double> d, unsigned quadsegs, std::str
 	if (srs.is_empty()) {
 		out.addWarning("unknown CRS. Results may be wrong");
 	} 
+	if (d.empty()) {
+		out.setError("no buffer distance provided");
+		return out;
+	} 
+	
 	bool islonlat = is_lonlat();
 	if (d.size() == 1 && d[0] == 0) {
 		islonlat = false; //faster
@@ -1147,13 +1290,22 @@ SpatVector SpatVector::buffer(std::vector<double> d, unsigned quadsegs, std::str
 		}
 		b[i] = geos_ptr(pt, hGEOSCtxt);
 	}
-	SpatVectorCollection coll = coll_from_geos(b, hGEOSCtxt);
+	const std::vector<long> ids = std::vector<long>();
+	SpatVectorCollection coll = coll_from_geos(b, hGEOSCtxt, ids, false);
 
 	GEOSBufferParams_destroy_r(hGEOSCtxt, bufparms);	
 	geos_finish(hGEOSCtxt);
 	out = coll.get(0);
 	out.srs = srs;
 	out.df = df;
+
+// revdep problem with ShapePattern
+	if (std::isnan(out.extent.xmin)) {
+		SpatVector empty;
+		empty.srs = srs;
+		return empty;
+	}
+	
 	return out;
 }
 
@@ -1168,9 +1320,9 @@ SpatVector SpatVector::buffer2(std::vector<double> d, unsigned quadsegs) {
 	std::vector<GeomPtr> b(size());
 	for (size_t i = 0; i < g.size(); i++) {
 		
-		Rcpp::Rcout << "buffer " << i;		
+//		Rcpp::Rcout << "buffer " << i;		
 		GEOSGeometry* pt = GEOSBuffer_r(hGEOSCtxt, g[i].get(), d[i], (int) quadsegs);
-		Rcpp::Rcout << " done" << std::endl;
+//		Rcpp::Rcout << " done" << std::endl;
 
 		if (pt == NULL) {
 			out.setError("GEOS exception");
@@ -1311,9 +1463,14 @@ SpatVector SpatVector::intersect(SpatVector v, bool values) {
 
 
 
+
 std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> getRelateFun(const std::string rel) {
 	std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> rfun;
-	if (rel == "intersects") {
+	if (rel == "equals") {
+		rfun = GEOSEquals_r;
+//	} else if (rel == "equalidentical") {
+//		rfun = GEOSEqualsIdentical_r;
+	} else if (rel == "intersects") {
 		rfun = GEOSIntersects_r;
 	} else if (rel == "disjoint") {
 		rfun = GEOSDisjoint_r;
@@ -1369,7 +1526,7 @@ int getRel(std::string &relation) {
 	int pattern = 1;
 	std::string rel = relation;
 	std::transform(rel.begin(), rel.end(), rel.begin(), ::tolower);
-	std::vector<std::string> f {"rook", "queen", "intersects", "touches", "crosses", "overlaps", "within", "contains", "covers", "coveredby", "disjoint"};
+	std::vector<std::string> f {"rook", "queen", "intersects", "touches", "crosses", "overlaps", "within", "contains", "covers", "coveredby", "disjoint", "equals"}; // tbd: "equalsexact", "equals"
 	if (std::find(f.begin(), f.end(), rel) == f.end()) {
 		if (relation.size() != 9) {
 			pattern = 2;
@@ -1393,7 +1550,38 @@ int getRel(std::string &relation) {
 	return pattern;
 }
 
+
+std::vector<int> SpatVector::pointInPolygon(std::vector<double> &x, std::vector<double> &y) {
+
+	std::vector<int> out;
+
+#ifdef GEOS3120
+	size_t ng = size();
+	size_t np = x.size();
+	out.reserve(np);
+	GEOSContextHandle_t hGEOSCtxt = geos_init();
+	std::vector<GeomPtr> g = geos_geoms(this, hGEOSCtxt);
+
+	for (size_t i = 0; i < ng; i++) {
+		PrepGeomPtr pr = geos_ptr(GEOSPrepare_r(hGEOSCtxt, g[i].get()), hGEOSCtxt);
+		for (size_t j = 0; j < np; j++) {
+			out.push_back( GEOSPreparedIntersectsXY_r(hGEOSCtxt, pr.get(), x[j], y[j]));
+		}
+	}
+
+# else 
+	SpatVector pnts;
+	pnts.srs = srs;
+	pnts.setPointsGeometry(x, y);
+	out = relate(pnts, "intersects", true, true);
+
+# endif
+
+	return out;
+}
+
 std::vector<int> SpatVector::relate(SpatVector v, std::string relation, bool prepared, bool index) {
+
 	// this method is redundant with "which_relate")
 	std::vector<int> out;
 	int pattern = getRel(relation);
@@ -1402,13 +1590,18 @@ std::vector<int> SpatVector::relate(SpatVector v, std::string relation, bool pre
 		return out;
 	}
 	if ((relation == "FF*FF****") || (relation == "disjoint")) index = false;
-
+	if (relation.substr(0, 5) == "equal") {
+		prepared = false;
+	}
+	
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
 	size_t nx = size();
 	size_t ny = v.size();
 
+	out.reserve(nx * ny);
+
 	if (!index) {
 		out.reserve(nx*ny);
 		if (pattern == 1) {
@@ -1520,7 +1713,24 @@ std::vector<std::vector<double>> SpatVector::which_relate(SpatVector v, std::str
 	out[0].reserve(nx * 1.5);
 	out[1].reserve(nx * 1.5);
 
-	if (!index) {
+
+	if (relation.substr(0, 5) == "equal") {
+		std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+		for (size_t i = 0; i < nx; i++) {
+			bool none = !narm;
+			for (size_t j = 0; j < ny; j++) {
+				if ( relFun(hGEOSCtxt, x[i].get(), y[j].get())) {
+					out[0].push_back(i);
+					out[1].push_back(j);
+					none = false;
+				}
+			}
+			if (none) {
+				out[0].push_back(i);
+				out[1].push_back(NAN);
+			}
+		}
+	} else if (!index) {
 		if (pattern == 1) {
 			for (size_t i = 0; i < nx; i++) {
 				bool none = !narm;
@@ -1633,8 +1843,23 @@ std::vector<std::vector<double>> SpatVector::which_relate(std::string relation,
 	out[0].reserve(nx * 1.5);
 	out[1].reserve(nx * 1.5);
 
-
-	if (!index) {
+	if (relation.substr(0, 5) == "equal") {
+		std::function<char(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeometry *)> relFun = getRelateFun(relation);
+		for (size_t i = 0; i < nx; i++) {
+			bool none = !narm;
+			for (size_t j = 0; j < nx; j++) {
+				if ( relFun(hGEOSCtxt, x[i].get(), x[j].get())) {
+					out[0].push_back(i);
+					out[1].push_back(j);
+					none = false;
+				}
+			}
+			if (none) {
+				out[0].push_back(i);
+				out[1].push_back(NAN);
+			}
+		}
+	} else if (!index) {
 		if (pattern == 1) {
 			for (size_t i=0; i<nx; i++) {
 				bool none = !narm;
@@ -2159,7 +2384,7 @@ SpatVector SpatVector::mask(SpatVector x, bool inverse) {
 			b[i] = !b[i];
 		}
 	}
-	std::vector<int> r;
+	std::vector<unsigned> r;
 	r.reserve(b.size());
 	for (size_t i=0; i<b.size(); i++) {
 		if (b[i]) r.push_back(i);
@@ -2175,9 +2400,10 @@ bool get_dist_fun(dist_fn &f, std::string s) {
 		f = GEOSDistance_r;
 	else if (s == "Hausdorff")
 		f = GEOSHausdorffDistance_r;
-#ifdef GEOS370
+#ifdef GEOS380
 	else if (s == "Frechet")
 		f = GEOSFrechetDistance_r;
+		// GEOSFrechetDistanceDensify_r
 #endif
 	else {
 		return false;
@@ -2186,7 +2412,7 @@ bool get_dist_fun(dist_fn &f, std::string s) {
 }
 
 
-std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel, std::string fun) {
+std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel, std::string fun, double m) {
 
 	std::vector<double> out;
 
@@ -2252,10 +2478,13 @@ std::vector<double> SpatVector::geos_distance(SpatVector v, bool parallel, std::
 		}
 	}
 	geos_finish(hGEOSCtxt);
+	if (m != 1) {
+		for (double &d : out) d *= m;
+	}
 	return out;
  }
 
-std::vector<double> SpatVector::geos_distance(bool sequential, std::string fun) {
+std::vector<double> SpatVector::geos_distance(bool sequential, std::string fun, double m) {
 
 	std::vector<double> out;
 	dist_fn distfun;
@@ -2294,6 +2523,9 @@ std::vector<double> SpatVector::geos_distance(bool sequential, std::string fun)
 		out.push_back(0);
 	}
 	geos_finish(hGEOSCtxt);
+	if (m != 1) {
+		for (double &d : out) d *= m;
+	}
 	return out;
  }
 
@@ -2369,6 +2601,7 @@ SpatVector SpatVector::unite() {
 
 
 SpatVector SpatVector::symdif(SpatVector v) {
+
 	if ((type() != "polygons") || (v.type() != "polygons")) {
 		SpatVector out;
 		out.setError("expected two polygon geometries");
@@ -2462,6 +2695,11 @@ SpatVector SpatVector::cover(SpatVector v, bool identity, bool expand) {
 
 SpatVector SpatVector::erase_agg(SpatVector v) {
 
+	if ((nrow()==0) || (v.nrow() == 0)) {
+		return(*this);
+	}
+
+
 	if ((type() == "points") || (v.type() == "points")) {
 		std::vector<bool> b = is_related(v, "intersects");
 		std::vector<unsigned> r;
@@ -2503,12 +2741,13 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 		}
 	}
 	if (!result.empty()) {
-		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt);
+		std::vector<long> ids;
+		SpatVectorCollection coll = coll_from_geos(result, hGEOSCtxt, ids, true, false);
 		out = coll.get(0);
 		out.srs = srs;
 		out.df = df.subset_rows(rids);
 	} else {
-		std::vector<int> none(1, -1);
+		std::vector<long> none(1, -1);
 		out = subset_rows(none);
 	}
 	geos_finish(hGEOSCtxt);
@@ -2522,6 +2761,10 @@ SpatVector SpatVector::erase_agg(SpatVector v) {
 
 SpatVector SpatVector::erase(SpatVector v) {
 
+	if ((nrow()==0) || (v.nrow() == 0)) {
+		return(*this);
+	}
+
 	if ((type() == "points") || (v.type() == "points")) {
 		std::vector<bool> b = is_related(v, "intersects");
 		std::vector<unsigned> r;
@@ -2539,7 +2782,7 @@ SpatVector SpatVector::erase(SpatVector v) {
 	std::vector<GeomPtr> y = geos_geoms(&v, hGEOSCtxt);
 	size_t nx = size();
 	size_t ny = v.size();
-	std::vector<int> rids;
+	std::vector<long> rids;
 	rids.reserve(nx);
 
 	for (size_t i = 0; i < nx; i++) {
@@ -2562,7 +2805,7 @@ SpatVector SpatVector::erase(SpatVector v) {
 	}
 
 	if (rids.empty()) {
-		std::vector<int> none(1, -1);
+		std::vector<long> none(1, -1);
 		out = subset_rows(none);
 	} else {
 		SpatVectorCollection coll = coll_from_geos(x, hGEOSCtxt);
@@ -2641,6 +2884,11 @@ SpatVector SpatVector::erase(SpatVector v) {
 */
 
 SpatVector SpatVector::erase(bool sequential) {
+
+	if (nrow()==0) {
+		return(*this);
+	}
+
 	SpatVector out;
 
 	if (type() != "polygons") {
@@ -2749,9 +2997,12 @@ SpatVector SpatVector::gaps() {
 
 
 
-SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
+// also use GEOSPreparedNearestPoints_r()
+
+SpatVector SpatVector::nearest_point(SpatVector v, bool parallel, const std::string method) {
 	SpatVector out;
 
+
 	if ((size() == 0) || v.empty()) {
 		out.setError("empty SpatVecor(s)");
 		return out;
@@ -2760,14 +3011,22 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 		out.setError("CRSs do not match");
 		return out;
 	}
+	bool lonlat = is_lonlat();
+	if (lonlat) {
+		std::vector<std::string> methods = {"geo", "cosine", "haversine"};
+		if (!is_in_vector(method, methods)) {
+			out.setError("invalid distance method");
+		}
+	}
 	out.srs = srs;
 
-	if ((is_lonlat()) && (type() == "points") && (v.type() == "points")) {
+	
+	if (lonlat && (type() == "points") && (v.type() == "points")) {
 		std::vector<double> nlon, nlat, dist;
 		std::vector<long> id;
 		std::vector<std::vector<double>> p = coordinates();
 		std::vector<std::vector<double>> pv = v.coordinates();
-		nearest_lonlat(id, dist, nlon, nlat, p[0], p[1], pv[0], pv[1]);
+		nearest_lonlat(id, dist, nlon, nlat, p[0], p[1], pv[0], pv[1], method);
 		out.setPointsGeometry(nlon, nlat);
 		std::vector<long> fromid(id.size());
 		std::iota(fromid.begin(), fromid.end(), 0);
@@ -2814,7 +3073,7 @@ SpatVector SpatVector::nearest_point(SpatVector v, bool parallel) {
 	return out;
 }
 
-SpatVector SpatVector::nearest_point() {
+SpatVector SpatVector::nearest_point(const std::string method) {
 	SpatVector out;
 	if ((size() == 0)) {
 		out.addWarning("empty SpatVecor");
@@ -2827,12 +3086,22 @@ SpatVector SpatVector::nearest_point() {
 	size_t n = size();
 	out.srs = srs;
 
-	if (is_lonlat()) {
+
+	bool lonlat = is_lonlat();
+	if (lonlat) {
+		std::vector<std::string> methods = {"geo", "cosine", "haversine"};
+		if (!is_in_vector(method, methods)) {
+			out.setError("invalid distance method");
+		}
+	}
+
+
+	if (lonlat) {
 		if (type() == "points") {
 			std::vector<double> nlon, nlat, dist;
 			std::vector<long> id;
 			std::vector<std::vector<double>> p = coordinates();
-			nearest_lonlat_self(id, dist, nlon, nlat, p[0], p[1]);
+			nearest_lonlat_self(id, dist, nlon, nlat, p[0], p[1], method);
 			out.setPointsGeometry(nlon, nlat);
 			out.df.add_column(id, "id");
 			out.df.add_column(dist, "distance");
@@ -2843,7 +3112,6 @@ SpatVector SpatVector::nearest_point() {
 		}
 	}
 
-
 	GEOSContextHandle_t hGEOSCtxt = geos_init();
 	std::vector<GeomPtr> x = geos_geoms(this, hGEOSCtxt);
 	std::vector<GeomPtr> b(n);
@@ -2963,7 +3231,6 @@ SpatVector SpatVector::cross_dateline(bool &fixed) {
 
 
 
-
 SpatVector SpatVector::centroid(bool check_lonlat) {
 
 	SpatVector out;
