@@ -22,25 +22,37 @@
 # if GEOS_VERSION_MINOR >= 5
 #  define GEOS350
 # endif
-# if GEOS_VERSION_MINOR == 6
-#  if GEOS_VERSION_PATCH >= 1
-#   define GEOS361
-#  endif
+# if GEOS_VERSION_MINOR >= 6
+// for #1744
+//#  if GEOS_VERSION_PATCH >= 1
+#   define GEOS360
+//#  endif
 # endif
 # if GEOS_VERSION_MINOR >= 7
 #  define GEOS361
 #  define GEOS370
 # endif
 # if GEOS_VERSION_MINOR >= 8
-#  define GEOS361
-#  define GEOS370
 #  define GEOS380
 # endif
+# if GEOS_VERSION_MINOR >= 10
+#  define GEOS3100
+# endif
+# if GEOS_VERSION_MINOR >= 11
+#  define GEOS3110
+# endif
+# if GEOS_VERSION_MINOR >= 12
+#  define GEOS3120
+# endif
 #else
 # if GEOS_VERSION_MAJOR > 3
 #  define GEOS350
-#  define GEOS370
 #  define GEOS361
+#  define GEOS370
+#  define GEOS380
+#  define GEOS310
+#  define GEOS3110
+#  define GEOS3120
 # endif
 #endif
 
@@ -316,6 +328,7 @@ inline std::vector<GeomPtr> geos_geoms(SpatVector *v, GEOSContextHandle_t hGEOSC
 			geoms.reserve(np);
 			for (size_t j=0; j < np; j++) {
 				//SpatPart svp = svg.getPart(j);
+//				if (svg.parts[j].x.size() < 3) continue;
 				GEOSGeometry* gp = geos_line(svg.parts[j].x, svg.parts[j].y, hGEOSCtxt); 
 				if (gp != NULL) {
 					geoms.push_back(gp);
@@ -336,6 +349,7 @@ inline std::vector<GeomPtr> geos_geoms(SpatVector *v, GEOSContextHandle_t hGEOSC
 			geoms.reserve(np);
 			for (size_t j=0; j < np; j++) {
 				SpatPart svp = svg.getPart(j);
+//				if (svp.x.size() < 3) continue;
 				GEOSGeometry* gp = geos_polygon(svp, hGEOSCtxt);
 
 				if (gp != NULL) {
@@ -644,7 +658,7 @@ inline SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSCont
 
 		} else if (gt == "GeometryCollection") {
 
-			bool first=true;
+			bool firstlin=true, firstpol=true, firstpnt=true;
 			size_t kk = 0; // introduced for intersect
 			for(size_t j = 0; j<np; j++) {
 
@@ -680,18 +694,18 @@ inline SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSCont
 							out.setError(msg);
 							return out;
 						}
-						if (track_ids && first) {
+						if (track_ids && firstpol) {
 							pol_ids.push_back(ids[i]);
-							first=false;
+							firstpol=false;
 						}
 					} else if (ggt == "Point" || ggt == "MultiPoint") {
 						if (!pointsFromGeom(hGEOSCtxt, part, f, k, pt_x, pt_y, pt_gid, pt_gp, pt_hole, msg)) {
 							out.setError(msg);
 							return out;
 						}
-						if (track_ids && first) {
+						if (track_ids && firstpnt) {
 							pts_ids.push_back(ids[i]);
-							first=false;
+							firstpnt=false;
 						}
 
 					} else if (ggt == "LineString" || ggt == "MultiLineString") {
@@ -699,9 +713,9 @@ inline SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSCont
 							out.setError(msg);
 							return out;
 						}
-						if (track_ids && first) {
+						if (track_ids && firstlin) {
 							lin_ids.push_back(ids[i]);
-							first=false;
+							firstlin=false;
 						}
 
 					} else {
