@@ -9,52 +9,61 @@
 
 //#include "spatFactor.h"
 #include "recycle.h"
+#include "sort.h"
 #include "gdalio.h"
 
 
-SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vector<double> values, double background, SpatOptions &opt) {
+SpatRaster SpatRaster::rasterizePoints(std::vector<double>&x, std::vector<double> &y, std::string fun, std::vector<double> &values, bool narm, double background, SpatOptions &opt) {
 
 	SpatRaster out = geometry(1, false, false, false);
+	
 	if (!out.writeStart(opt, filenames())) {
 		return out;
 	}
-	if (fun != "count" && (values.size() != x.size())) {
-		out.setError("values do not match geometries");
+	
+	if (y.size() != x.size()) {
+		out.setError("number of x and y coordinates do not match");
+		return out;
+	}
+	if ((fun == "count") && (values.size() != x.size()) && (!values.empty())) {
+		out.setError("number of values does not match the number of points");
+		return out;
+	} else if (values.size() != x.size()) {
+		out.setError("number of values does not match the number of points");
 		return out;
 	}
 
 	size_t nc = ncol();
-	std::vector<std::vector<double>> pxy = x.coordinates();
-	std::vector<double> cells = cellFromXY(pxy[0], pxy[1]);
+	std::vector<double> cells = cellFromXY(x, y, -9);
+	// order for multiple chunks, but also to remove NAs (-9)
+	std::vector<std::size_t> so = sort_order_a(cells);
+	permute(cells, so);
+	permute(values, so);
+
+	size_t cellcnt = 0;
+	for (size_t i=0; i<cells.size(); i++) {
+		if (cells[i] < 0) {
+			cellcnt++;
+		} else {
+			break;
+		}
+	}
+
 	if (fun == "count") {
+		bool dotest = (!values.empty()) && narm;
 		for (size_t i=0; i < out.bs.n; i++) {
 			double cmin = out.bs.row[i] * nc;
 			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
+
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (dotest && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
 					size_t k = cells[j] - cmin;
 					v[k]++;
-				}
-			}
-			if (background != 0) {
-				for (size_t j=0; j<v.size(); j++) {
-					if (v[j] == 0) {
-						v[j] = background;
-					}
-				}
-			}
-			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
-		}
-	} else if (fun == "sum") {
-		for (size_t i=0; i < out.bs.n; i++) {
-			double cmin = out.bs.row[i] * nc;
-			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
-			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
-					size_t k = cells[j] - cmin;
-					v[k] += values[j];
+				} else {
+					cellcnt = j;
+					break;
 				}
 			}
 			if (background != 0) {
@@ -72,24 +81,162 @@ SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vecto
 			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
 			std::vector<double> cnt = v;
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
 					size_t k = cells[j] - cmin;
 					v[k] += values[j];
 					cnt[k]++;
-				}
-			}
-			for (size_t j=0; j<cnt.size(); j++) {
-				if (cnt[j] > 0) {
-					v[j] /= cnt[j];
+				} else {
+					cellcnt = j;
+					break;
 				}
 			}
 			if (background != 0) {
 				for (size_t j=0; j<v.size(); j++) {
-					if (v[j] == 0) {
+					if (cnt[j] == 0) {
 						v[j] = background;
+					} else {
+						v[j] /= cnt[j];						
 					}
 				}
+			} else {
+				for (size_t j=0; j<cnt.size(); j++) {
+					if (cnt[j] > 0) {
+						v[j] /= cnt[j];
+					}
+				}				
+			}			
+			
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "sum") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] += values[j];
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "min") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] = std::min(v[k], values[j]);
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "max") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] = std::max(v[k], values[j]);
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "prod") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} else {
+						v[k] *= values[j];
+					}
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "pa") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					v[k] = 1;
+				} else {
+					cellcnt = j;
+					break;
+				}
+			}
+			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
+		}
+	} else if (fun == "first") {
+		for (size_t i=0; i < out.bs.n; i++) {
+			double cmin = out.bs.row[i] * nc;
+			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
+			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
+			std::vector<bool> newcell(out.bs.nrows[i] * out.ncol(), true);;
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
+					size_t k = cells[j] - cmin;
+					if (newcell[k]) {
+						v[k] = values[j];
+						newcell[k] = false;
+					} 
+				} else {
+					cellcnt = j;
+					break;
+				}
 			}
 			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
 		}
@@ -98,10 +245,14 @@ SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vecto
 			double cmin = out.bs.row[i] * nc;
 			double cmax = (out.bs.row[i]+out.bs.nrows[i]) * nc - 1;
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), background);
-			for (size_t j=0; j<cells.size(); j++) {
-				if (cells[j] >= cmin && cells[j] <= cmax) {
+			for (size_t j=cellcnt; j<cells.size(); j++) {
+				if (narm && std::isnan(values[j])) continue;
+				if (cells[j] <= cmax) {
 					size_t k = cells[j] - cmin;
 					v[k] = values[j];
+				} else {
+					cellcnt = j;
+					break;
 				}
 			}
 			if (!out.writeValues(v, out.bs.row[i], out.bs.nrows[i]))  return out;
@@ -112,9 +263,22 @@ SpatRaster SpatRaster::rasterizePoints(SpatVector x, std::string fun, std::vecto
 }
 
 
+SpatRaster SpatRaster::rasterizePoints(SpatVector &x, std::string fun, std::vector<double> &values, bool narm, double background, SpatOptions &opt) {
+	if (values.empty()) {
+		values = std::vector<double>(x.nrow(), 1);
+	}
+	std::vector<std::vector<double>> pxy = x.coordinates();
+	return rasterizePoints(pxy[0], pxy[1], fun, values, narm, background, opt);
+}
+
+
 SpatRaster SpatRaster::rasterizeGeom(SpatVector x, std::string unit, std::string fun, SpatOptions &opt) {
 
-	if (x.type() != "points") {
+	if (x.type() == "points") {
+		std::vector<double> v;
+		return rasterizePoints(x, "count", v, false, 0.0, opt);
+
+	} else {	
 
 		SpatRaster out = geometry(1, false, false, false);
 		SpatOptions ops(opt);
@@ -148,13 +312,8 @@ SpatRaster SpatRaster::rasterizeGeom(SpatVector x, std::string unit, std::string
 			tom = std::isnan(tom) ? 1 : tom;
 			m *= tom;
 		}
-		if (x.type() == "lines") {
-			out.setNames({"length"});
-		} else {
-			out.setNames({"area"});
-			m *= m;
-		}
-		opt.ncopies = std::max(opt.ncopies, (unsigned)4) * 8;
+		out.setNames({fun});
+		opt.ncopies = std::max(opt.ncopies, (size_t)4) * 8;
 		if (!out.writeStart(opt, filenames())) {
 			return out;
 		}
@@ -163,7 +322,7 @@ SpatRaster SpatRaster::rasterizeGeom(SpatVector x, std::string unit, std::string
 			e.ymin = yFromRow(out.bs.row[i] + out.bs.nrows[i] - 1) - rsy;
 			SpatRaster tmp = empty.crop(e, "near", false, ops);
 
-			SpatVector p = tmp.as_polygons(true, false, false, false, false, ops);
+			SpatVector p = tmp.as_polygons(true, false, false, false, false, 0, ops);
 			std::vector<double> v(out.bs.nrows[i] * out.ncol(), 0);
 
 			if (fun == "crosses") {
@@ -201,8 +360,6 @@ SpatRaster SpatRaster::rasterizeGeom(SpatVector x, std::string unit, std::string
 		out.writeStop();
 		return(out);
 
-	} else {
-		return rasterizePoints(x, "count", {}, 0.0, opt);
 	}
 }
 
@@ -252,18 +409,18 @@ bool SpatRaster::getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filen
 	filename = opt.get_filename();
 	SpatOptions ops(opt);
 	ops.ncopies += 4;
-	if (filename == "") {
+	if (filename.empty()) {
 		if (canProcessInMemory(ops)) {
 			driver = "MEM";
 		} else {
-			filename = tempFile(opt.get_tempdir(), opt.pid, ".tif");
+			filename = tempFile(opt.get_tempdir(), opt.tmpfile, ".tif");
 			opt.set_filenames({filename});
 			driver = "GTiff";
 		}
 	} else {
 		driver = opt.get_filetype();
 		getGDALdriver(filename, driver);
-		if (driver == "") {
+		if (driver.empty()) {
 			out.setError("cannot guess file type from filename");
 			return false;
 		}
@@ -300,6 +457,26 @@ bool SpatRaster::getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filen
 }
 
 
+bool SpatRaster::getDShMEM(GDALDatasetH &rstDS, SpatRaster &out, double &naval, double background, SpatOptions &opt) {
+
+	SpatOptions ops(opt);
+	if (opt.names.size() == nlyr()) {
+		out.setNames(opt.names);
+	}
+	if (!out.create_gdalDS(rstDS, "", "MEM", true, background, source[0].has_scale_offset, source[0].scale, source[0].offset, ops)) {
+		out.setError("cannot create dataset");
+		return false;
+	}
+	GDALRasterBandH hBand = GDALGetRasterBand(rstDS, 1);
+	GDALDataType gdt = GDALGetRasterDataType(hBand);
+	getNAvalue(gdt, naval);
+	int hasNA;
+	double naflag = GDALGetRasterNoDataValue(hBand, &hasNA);
+	naval = hasNA ? naflag : naval;
+
+	return true;
+}
+
 
 
 
@@ -335,6 +512,7 @@ SpatRaster SpatRaster::rasterizeLyr(SpatVector x, double value, double backgroun
 	std::string driver, filename;
 	GDALDatasetH rstDS;
 	double naval;
+	
 	if (!getDSh(rstDS, out, filename, driver, naval, update, background, opt)) {
 		return out;
 	}
@@ -388,8 +566,8 @@ SpatRaster SpatRaster::rasterizeLyr(SpatVector x, double value, double backgroun
 
 
 SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<double> values,
-	double background, bool touches, bool add, bool weights, bool update, bool minmax, SpatOptions &opt) {
-
+	double background, bool touches, std::string fun, bool weights, bool update, bool minmax, SpatOptions &opt) {
+	
 	std::string gtype = x.type();
 	bool ispol = gtype == "polygons";
 	if (weights) update = false;
@@ -406,29 +584,35 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		//unsigned agy = 100;
 		wout = wout.disaggregate({agx, agy}, sopts);
 		double f = agx * agy;
-		wout = wout.rasterize(x, field, {1/f}, background, touches, add, false, false, false, sopts);
+		wout = wout.rasterize(x, field, {1/f}, background, touches, fun, false, false, false, sopts);
 		wout = wout.aggregate({agx, agy}, "sum", true, opt);
 		return wout;
 	}
 
+//	Rcpp::Rcout << "x" << std::endl;
+
+	bool add = fun == "sum";
+
 	SpatRaster out;
 	if ( !hasValues() ) update = false;
 	if (update) {
-		out = hardCopy(opt);
+		out = geometry();
 	} else {
 		out = geometry(1);
-		if (field == "") {
-			out.setNames({"layer"});
-		} else {
-			out.setNames({field});
-		}
+	}
+	if (field.empty()) {
+		out.setNames({"layer"});
+	} else {
+		out.setNames({field});
 	}
 
 	size_t nGeoms = x.size();
 	if (nGeoms == 0) {
-		if (!update) {
+		if (update) {
+			out = *this;
+		} else {
 			out = out.init({background}, opt);
-		}
+		} 
 		return out;
 	}
 
@@ -437,7 +621,7 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 		out.addWarning("you cannot use 'sum' and 'touches' at the same time");
 	}
 
-	if (field != "") {
+	if (!field.empty()) {
 		int i = x.df.get_fieldindex(field);
 		if (i < 0) {
 			out.setError("field " + field + " not found");
@@ -505,98 +689,122 @@ SpatRaster SpatRaster::rasterize(SpatVector x, std::string field, std::vector<do
 	GDALDatasetH rstDS;
 	double naval;
 	if (add) {	background = 0;	}
-
-	if (!out.getDSh(rstDS, out, filename, driver, naval, update, background, opt)) {
-		return out;
-	}
-	for (double &d : values) d = std::isnan(d) ? naval : d;
-		// passing NULL instead may also work.
-
 	std::vector<int> bands(out.nlyr());
 	std::iota(bands.begin(), bands.end(), 1);
 	rep_each(values, out.nlyr());
+	
+	SpatRaster temp = out;
+  	if (!out.writeStart(opt, filenames())) {
+		return out;
+	}
+	
+	bool hasError = false;
+	SpatExtent e = temp.getExtent();
+	SpatRaster tmp;
+	SpatOptions topt(opt);
 
 	char** papszOptions = NULL;
-	CPLErr err;
-	if (ispol && touches && (nGeoms > 1)) {
-		// first to get the touches
-		papszOptions = CSLSetNameValue(papszOptions, "ALL_TOUCHED", "TRUE");
-		err = GDALRasterizeGeometries(rstDS,
-				static_cast<int>(bands.size()), &(bands[0]),
-				static_cast<int>(ogrGeoms.size()),
-				(OGRGeometryH *) &(ogrGeoms[0]),
-				NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
-		CSLDestroy(papszOptions);
+
+	for (size_t i = 0; i < out.bs.n; i++) {
+
+		if (out.bs.n > 1) {
+			double halfres = temp.yres() / 2;
+			e.ymax = temp.yFromRow(out.bs.row[i]) + halfres;
+			e.ymin = temp.yFromRow(out.bs.row[i] + out.bs.nrows[i] - 1) - halfres;
+			if (update) {
+				tmp = crop(e, "near", false, topt);	
+			} else {
+				tmp = temp.crop(e, "near", false, topt);
+			}
+		} else {
+			if (update) {
+				tmp = hardCopy(topt);
+			} else {
+				tmp = temp;
+			}
+		}
+		if (update) {
+			std::string filename;
+			if (!tmp.getDSh(rstDS, tmp, filename, driver, naval, update, background, topt)) {
+				return tmp;
+			}
+		} else if (!tmp.getDShMEM(rstDS, tmp, naval, background, opt)) {
+			return tmp;
+		}
+		
+	
+		if (i==1) for (double &d : values) d = std::isnan(d) ? naval : d;
+
+		CPLErr err;
+		if (ispol && touches && (nGeoms > 1)) {
+			// first to get the touches
+			if (i == 0) papszOptions = CSLSetNameValue(papszOptions, "ALL_TOUCHED", "TRUE");
+			err = GDALRasterizeGeometries(rstDS,
+					static_cast<int>(bands.size()), &(bands[0]),
+					static_cast<int>(ogrGeoms.size()),
+					(OGRGeometryH *) &(ogrGeoms[0]),
+					NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
+
+			if ( err != CE_None ) {
+				tmp.setError("rasterization failed");
+				GDALClose(rstDS);
+				hasError = true; break;
+			}
+			//GDALFlushCache(rstDS);
+			// second time to fix the internal area
+			err = GDALRasterizeGeometries(rstDS,
+					static_cast<int>(bands.size()), &(bands[0]),
+					static_cast<int>(ogrGeoms.size()),
+					(OGRGeometryH *) &(ogrGeoms[0]),
+					NULL, NULL, &(values[0]), NULL, NULL, NULL);
+
+		} else {
+			if (i == 0)  {
+				if (touches) {
+					papszOptions = CSLSetNameValue(papszOptions, "ALL_TOUCHED", "TRUE");
+				} else if (add) {
+					papszOptions = CSLSetNameValue(papszOptions, "MERGE_ALG", "ADD");
+				}
+			}
+			err = GDALRasterizeGeometries(rstDS,
+					static_cast<int>(bands.size()), &(bands[0]),
+					static_cast<int>(ogrGeoms.size()),
+					(OGRGeometryH *) &(ogrGeoms[0]),
+					NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
+
+		}
 
 		if ( err != CE_None ) {
-			out.setError("rasterization failed");
+			tmp.setError("rasterization failed");
 			GDALClose(rstDS);
-			for (size_t i=0; i<ogrGeoms.size(); i++) {
-				OGR_G_DestroyGeometry(ogrGeoms[i]);
-			}
-			return out;
+			hasError = true; break;
 		}
-		//GDALFlushCache(rstDS);
-		// second time to fix the internal area
-		err = GDALRasterizeGeometries(rstDS,
-				static_cast<int>(bands.size()), &(bands[0]),
-				static_cast<int>(ogrGeoms.size()),
-				(OGRGeometryH *) &(ogrGeoms[0]),
-				NULL, NULL, &(values[0]), NULL, NULL, NULL);
 
-	} else {
-		if (touches) {
-			papszOptions = CSLSetNameValue(papszOptions, "ALL_TOUCHED", "TRUE");
-		} else if (add) {
-			papszOptions = CSLSetNameValue(papszOptions, "MERGE_ALG", "ADD");
+		if (!tmp.from_gdalMEM(rstDS, false, true)) {
+			tmp.setError("rasterization failed");
+			GDALClose(rstDS);
+			hasError = true; break;
 		}
-		err = GDALRasterizeGeometries(rstDS,
-				static_cast<int>(bands.size()), &(bands[0]),
-				static_cast<int>(ogrGeoms.size()),
-				(OGRGeometryH *) &(ogrGeoms[0]),
-				NULL, NULL, &(values[0]), papszOptions, NULL, NULL);
+		GDALClose(rstDS);
 
-		CSLDestroy(papszOptions);
+		std::vector<double> v = tmp.getValues(-1, topt);
+		if (!out.writeBlock(v, i)) return out;
 	}
-
+	
+	CSLDestroy(papszOptions);	
 	for (size_t i=0; i<ogrGeoms.size(); i++) {
 		OGR_G_DestroyGeometry(ogrGeoms[i]);
 	}
-
-	if ( err != CE_None ) {
-		out.setError("rasterization failed");
-		GDALClose(rstDS);
-		return out;
-	}
-
-	if (driver == "MEM") {
-		if (!out.from_gdalMEM(rstDS, false, true)) {
-			out.setError("rasterization failed (mem)");
-		}
-	}
-
-	GDALRasterBandH band = GDALGetRasterBand(rstDS, 1);
-
-	if (minmax) {
-		double adfMinMax[2];
-		GDALComputeRasterMinMax(band, false, adfMinMax);
-		GDALSetRasterStatistics(band, adfMinMax[0], adfMinMax[1], -9999, -9999);
-	}
-
-	GDALClose(rstDS);
-	if (driver != "MEM") {
-		out = SpatRaster(filename, {-1}, {""}, {}, {});
-	} else {
-		std::string fname = opt.get_filename();
-		if ((fname != "") && (!update)) {
-			out = out.writeRaster(opt);
-		}
-	}
+	if (update) readStop();
+	out.writeStop();
+	
+	if (hasError) return tmp; 
+		
 	return out;
 }
 
 
-std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, SpatOptions &opt) {
+std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, bool small, SpatOptions &opt) {
 // note that this is only for lines and polygons
     SpatOptions ropt(opt);
 	SpatRaster r = geometry(1);
@@ -621,21 +829,25 @@ std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, Spat
 
 	SpatRaster rc = r.crop(e, "out", false, ropt);
 	std::vector<double> feats(1, 1) ;
-    SpatRaster rcr = rc.rasterize(v, "", feats, NAN, touches, false, false, false, false, ropt);
+    SpatRaster rcr = rc.rasterize(v, "", feats, NAN, touches, "", false, false, false, ropt);
 	SpatVector pts = rcr.as_points(false, true, false, ropt);
 	std::vector<double> cells;
-	if (pts.size() == 0) {
-		pts = v.as_points(false, true);
-		SpatDataFrame vd = pts.getGeometryDF();
-		std::vector<double> x = vd.getD(0);
-		std::vector<double> y = vd.getD(1);
-		cells = r.cellFromXY(x, y);	
-		cells.erase(std::remove_if(cells.begin(), cells.end(),
-                    [](const double& value) { return std::isnan(value); }), cells.end());
-		std::sort( cells.begin(), cells.end() );
-		cells.erase(std::unique(cells.begin(), cells.end()), cells.end());
-		if (cells.size() == 0) {
-			cells.resize(1, NAN);
+	if (pts.empty()) {
+		if (small) {
+			pts = v.as_points(false, true);
+			SpatDataFrame vd = pts.getGeometryDF();
+			std::vector<double> x = vd.getD(0);
+			std::vector<double> y = vd.getD(1);
+			cells = r.cellFromXY(x, y);	
+			cells.erase(std::remove_if(cells.begin(), cells.end(),
+						[](const double& value) { return std::isnan(value); }), cells.end());
+			std::sort( cells.begin(), cells.end() );
+			cells.erase(std::unique(cells.begin(), cells.end()), cells.end());
+			if (cells.empty()) {
+				cells.resize(1, NAN);
+			}
+		} else {
+			cells.resize(1, NAN);			
 		}
 	} else {
 		SpatDataFrame vd = pts.getGeometryDF();
@@ -643,7 +855,7 @@ std::vector<double> SpatRaster::rasterizeCells(SpatVector &v, bool touches, Spat
 		std::vector<double> y = vd.getD(1);
 		cells = r.cellFromXY(x, y);
 //		cells.erase(std::unique(cells.begin(), cells.end()), cells.end());
-		if (cells.size() == 0) {
+		if (cells.empty()) {
 			cells.resize(1, NAN);
 		}
 	}
@@ -670,10 +882,10 @@ void SpatRaster::rasterizeCellsWeights(std::vector<double> &cells, std::vector<d
 		r = rc;
 	}
 	std::vector<double> feats;
-	r = r.rasterize(v, "", feats, NAN, false, false, true, false, false, ropt);
+	r = r.rasterize(v, "", feats, NAN, false, "", true, false, false, ropt);
 	std::vector<std::vector<double>> cv = r.cells_notna(ropt);
 
-	if (cv[0].size() == 0) {
+	if (cv[0].empty()) {
 		weights.resize(1);
 		weights[0] = NAN;
 		cells.resize(1);
@@ -699,10 +911,10 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 
 //	if (r.ncell() < 1000) {
 		std::vector<double> feats(1, 1) ;
-		r = r.rasterize(v, "", feats, NAN, true, false, false, false, false, ropt);
+		r = r.rasterize(v, "", feats, NAN, true, "", false, false, false, ropt);
 
 		SpatVector pts = r.as_points(true, true, false, ropt);
-		if (pts.size() == 0) {
+		if (pts.empty()) {
 			weights.resize(1);
 			weights[0] = NAN;
 			cells.resize(1);
@@ -713,7 +925,7 @@ void SpatRaster::rasterizeCellsExact(std::vector<double> &cells, std::vector<dou
 			std::vector<double> y = vd.getD(1);
 			cells = cellFromXY(x, y);
 
-			SpatVector rv = r.as_polygons(false, false, false, true, false, ropt);
+			SpatVector rv = r.as_polygons(false, false, false, true, false, 0, ropt);
 			std::vector<double> csize = rv.area("m", true, {});
 			rv.df.add_column(csize, "area");
 			rv.df.add_column(cells, "cells");
@@ -793,7 +1005,7 @@ void SpatRaster::rasterizeLinesLength(std::vector<double> &cells, std::vector<do
 */
 
 	SpatOptions xopt(opt);
-	xopt.ncopies = std::max(xopt.ncopies, (unsigned)4) * 8;
+	xopt.ncopies = std::max(xopt.ncopies, (size_t)4) * 8;
 	SpatRaster x = geometry(1);
 
 	SpatExtent ev = v.getExtent();
@@ -810,7 +1022,7 @@ void SpatRaster::rasterizeLinesLength(std::vector<double> &cells, std::vector<do
 		std::iota(cell.begin(), cell.end(), 0);
 		std::vector<std::vector<double>> xy = tmp.xyFromCell(cell);
 		cell = cellFromXY(xy[0], xy[1]);
-		SpatVector p = tmp.as_polygons(true, false, false, false, false, xopt);
+		SpatVector p = tmp.as_polygons(true, false, false, false, false, 0, xopt);
 		p.df.add_column(cell, "cell");
 		p = p.intersect(v, true);
 		if (p.nrow() > 1) {
