@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -256,7 +256,7 @@ bool GetVAT(std::string filename, SpatCategories &vat) {
 	SpatVector v, fvct;
 	std::vector<double> fext;
 
-	v.read(filename, "", "", fext, fvct, false, ""); 
+	v.read(filename, "", "", fext, fvct, false, "", {}); 
 	if (v.df.nrow() == 0) return false;
 
 
@@ -392,8 +392,8 @@ bool colsFromRat(SpatDataFrame &d, SpatDataFrame &out) {
 		if (k >= 0) {
 			if (!setIntCol(d, out, k, cols1[i])) return false;
 		} else {
-				int k = where_in_vector(cols2[i], ss, true);
-		if (k >= 0) {
+			int k = where_in_vector(cols2[i], ss, true);
+			if (k >= 0) {
 				if (!setIntCol(d, out, k, cols1[i])) return false;
 			} else {
 				return false;
@@ -580,6 +580,41 @@ inline std::string dtypename(const std::string &d) {
 }
 
 
+void get_tags(std::vector<std::string> meta, std::string prefix,  std::vector<std::string> &name, std::vector<std::string> &value) {
+	if (!meta.empty()) {
+		for (size_t i=0; i<meta.size(); i++) {
+			size_t tagpos = meta[i].find(prefix);
+			if (tagpos != std::string::npos) {		
+				size_t pos = meta[i].find("=");
+				if (pos != std::string::npos) {
+					std::string mn = meta[i].substr(prefix.size(), pos-tagpos-prefix.size());
+					if (!((mn == "_FillValue") || (mn == "grid_mapping") || (mn == "Conventions") || (mn == "created_by") || (mn == "created_date"))) {
+						name.push_back(mn);
+						value.push_back(meta[i].substr(pos+1, meta[i].length()));
+					}
+				}
+			}
+		}
+	}
+	
+}
+
+std::vector<std::string> get_metadata(std::string filename, std::vector<std::string> options) {
+	std::vector<std::string> metadata;
+    GDALDataset *poDataset = openGDAL(filename, GDAL_OF_RASTER | GDAL_OF_READONLY | GDAL_OF_VERBOSE_ERROR, {}, options);
+    if( poDataset == NULL )  {
+		return metadata;
+	}
+	char **m = poDataset->GetMetadata();
+	if (m) {
+		while (*m != nullptr) {
+			metadata.push_back(*m++);
+		}
+	}
+	GDALClose( (GDALDatasetH) poDataset );
+	return metadata;
+}
+
 
 SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool useids, std::vector<std::string> options) {
 
@@ -626,7 +661,7 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 			if (pos != std::string::npos) {
 				s.erase(0, pos + delim.length());
 				SpatRaster sub;
-				if (sub.constructFromFile(s, {-1}, {""}, {}, options)) {
+				if (sub.constructFromFile(s, {-1}, {""}, {}, options, false)) {
 					std::string sname = sub.source[0].source_name.empty() ? basename_sds(s) : sub.source[0].source_name;
 					if (!push_back(sub, sname, sub.source[0].source_name_long, sub.source[0].unit[0], true)) {
 						addWarning("skipped (different geometry): " + s);
@@ -637,7 +672,20 @@ SpatRasterStack::SpatRasterStack(std::string fname, std::vector<int> ids, bool u
 			}
 		}
 	}
+	meta.resize(0);
+	char **m = poDataset->GetMetadata();
+	if (m) {
+		while (*m != nullptr) {
+			meta.push_back(*m++);
+		}
+	}
 	GDALClose( (GDALDatasetH) poDataset );
+
+	std::vector<std::string> tagnames, tagvalues;
+//	get_tags(meta, "NC_GLOBAL#TAG_", tagnames, tagvalues);
+	get_tags(meta, "NC_GLOBAL#", tagnames, tagvalues);
+	for (size_t i=0; i<tagnames.size(); i++) addTag(tagnames[i], tagvalues[i]);
+
 }
 
 
@@ -687,7 +735,7 @@ SpatRasterCollection::SpatRasterCollection(std::string fname, std::vector<int> i
 			if (pos != std::string::npos) {
 				s.erase(0, pos + delim.length());
 				SpatRaster sub;
-				if (sub.constructFromFile(s, {-1}, {""}, {}, options)) {
+				if (sub.constructFromFile(s, {-1}, {""}, {}, options, false)) {
 					push_back(sub, basename_sds(s));
 				} else {
 					addWarning("skipped (fail): " + s);
@@ -695,7 +743,20 @@ SpatRasterCollection::SpatRasterCollection(std::string fname, std::vector<int> i
 			}
 		}
 	}
+	meta.resize(0);
+	char **m = poDataset->GetMetadata();
+	if (m) {
+		while (*m != nullptr) {
+			meta.push_back(*m++);
+		}
+	}
 	GDALClose( (GDALDatasetH) poDataset );
+
+	std::vector<std::string> tagnames, tagvalues;
+//	get_tags(meta, "NC_GLOBAL#TAG_", tagnames, tagvalues);
+	get_tags(meta, "NC_GLOBAL#", tagnames, tagvalues);
+	for (size_t i=0; i<tagnames.size(); i++) addTag(tagnames[i], tagvalues[i]);
+
 }
 
 
@@ -764,8 +825,15 @@ bool getGCPs(GDALDataset *poDataset, SpatRasterSource &s) {
 }
 
 
-bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options) {
 
+bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options, bool noflip) {
+
+	if (fname == "WCS:") {
+		// for https://github.com/rspatial/terra/issues/1505
+		setError("no raster data in WCS:");
+		return false;
+	}
+	
 	std::vector<std::string> clean_ops = options;
 	bool app_so = true;
 	size_t opsz = options.size();
@@ -791,13 +859,14 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	std::string gdrv = poDataset->GetDriver()->GetDescription();
 
 	char **metasds = poDataset->GetMetadata("SUBDATASETS");
+	
 	if (metasds != NULL) {
 		std::vector<std::string> meta;
 		for (size_t i=0; metasds[i] != NULL; i++) {
 			meta.push_back(metasds[i]);
 		}
 		GDALClose( (GDALDatasetH) poDataset );
-		return constructFromSDS(fname, meta, subds, subdsname, options, gdrv);
+		return constructFromSDS(fname, meta, subds, subdsname, options, gdrv, noflip);
 
 	} else if (nl==0) {
 		setError("no raster data in " + fname);
@@ -805,6 +874,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	}
 
 	char **meterra = poDataset->GetMetadata("USER_TAGS");
+
 	if (meterra != NULL) {
 		std::vector<std::string> meta;
 		for (size_t i=0; meterra[i] != NULL; i++) {
@@ -845,7 +915,6 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		//ymin = roundn(ymin, 9);
 
 		if (adfGeoTransform[5] > 0) {
-			s.flipped = true;
 			std::swap(ymin, ymax);
 		}
 
@@ -861,19 +930,33 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			addWarning("the data in this file are rotated. Use 'rectify' to fix that");
 		}
 	} else {
-
+		bool warn=true;
 		hasExtent = false;
+		if (adfGeoTransform[5] > 0) {
+			if (noflip) {
+				warn = false;
+				s.extset = true;
+			} else {
+				s.flipped = true;
+			}
+		}
 		SpatExtent e(0, s.ncol, 0, s.nrow);
 		s.extent = e;
+
 		if ((gdrv=="netCDF") || (gdrv == "HDF5")) {
 			#ifndef standalone
 			setMessage("ncdf extent");
 			#else
 			addWarning("unknown extent. Cells not equally spaced?");
 			#endif
-		} else {
+		} else if (warn) {
 			addWarning("unknown extent");
 		}
+
+		// seems to cause more harm then benefit #1627
+		//try {
+		//	s.flipped = adfGeoTransform[5] > 0;
+		//} catch(...) {}
 	}
 
 	s.memory = false;
@@ -894,6 +977,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	}
 */
 
+
 	std::string crs = getDsWKT(poDataset);
 	if (crs.empty()) {
 		if (hasExtent && s.extent.xmin >= -180 && s.extent.xmax <= 360 && s.extent.ymin >= -90 && s.extent.ymax <= 90) {
@@ -906,27 +990,6 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		addWarning(msg);
 	}
 
-	std::vector<int_64> timestamps;
-	std::string timestep="raw";
-	std::vector<std::string> units;
-
-	try {
-		read_aux_json(fname, timestamps, timestep, units, s.nlyr);
-	} catch(...) {
-		timestamps.resize(0);
-		units.resize(0);
-		addWarning("could not parse aux.json");
-	}
-	if (!timestamps.empty()) {
-		s.time = timestamps;
-		s.timestep = timestep;
-		s.hasTime = true;
-	}
-	if (!units.empty()) {
-		s.unit = units;
-		s.hasUnit = true;
-	}
-
 	GDALRasterBand  *poBand;
 	//int nBlockXSize, nBlockYSize;
 	double adfMinMax[2];
@@ -940,29 +1003,57 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	bool getCols = s.nlyr == 3;
 	std::vector<unsigned> rgb_lyrs(3, -99);
 
+	s.hasUnit = true;
+	s.hasTime = true;
+	std::vector<std::string> datm, unts;
+	datm.reserve(s.nlyr);
+	unts.reserve(s.nlyr);
+	
 	int bs1, bs2;
 	for (size_t i = 0; i < s.nlyr; i++) {
+		
 		poBand = poDataset->GetRasterBand(i+1);
 
-		if ((gdrv=="netCDF") || (gdrv == "HDF5") || (gdrv == "GRIB") || (gdrv == "GTiff")) {
+		if (s.hasTime) {
+			const char* dtm = poBand->GetMetadataItem("DATE_TIME");
+			if (dtm != NULL) {
+				datm.push_back(dtm);
+			} else {
+				s.hasTime = false;
+			}
+		}
+		if (s.hasUnit) {
+			const char* ut = poBand->GetMetadataItem("UNIT");
+			if (ut != NULL) {
+				unts.push_back(ut);
+			} else {
+				s.hasUnit = false;
+			}
+		}
+
+		// if ((gdrv=="netCDF") || (gdrv == "HDF5") || (gdrv == "GRIB") || (gdrv == "GTiff")) {
 			char **m = poBand->GetMetadata();
 			while (m != nullptr && *m != nullptr) {
 				bandmeta[i].push_back(*m++);
 			}
-			char **meterra = poBand->GetMetadata("LYR_TAGS");
+			//for (size_t j = 0; j<bandmeta[i].size(); j++) {
+			//	Rcpp::Rcout << bandmeta[i][j] << std::endl;
+			//}
+			
+			char **meterra = poBand->GetMetadata("USER_TAGS");
 			if (meterra != NULL) {
 //				std::vector<std::string> meta;
-				for (size_t i=0; meterra[i] != NULL; i++) {
-					std::string s = meterra[i];
-					size_t pos = s.find("=");
+				for (size_t j=0; meterra[j] != NULL; j++) {
+					std::string ms = meterra[j];
+					size_t pos = ms.find("=");
 					if (pos != std::string::npos) {
-						std::string name = s.substr(0, pos);
-						std::string value = s.substr(pos+1); 
-						addLyrTags({i}, {name}, {value});
+						std::string name = ms.substr(0, pos);
+						std::string value = ms.substr(pos+1); 
+						s.addLyrTag(i, name, value);
 					}
 				}
 			}
-		}
+		// }
 
 		int success;
 	//	double naflag = poBand->GetNoDataValue(&success);
@@ -1021,6 +1112,7 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		}
 
 		std::string bandname = poBand->GetDescription();
+
 		char **cat = poBand->GetCategoryNames();
 		if( cat != NULL )	{
 			SpatCategories scat = GetCategories(cat, bandname);
@@ -1089,10 +1181,88 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 		s.names[i] = nm;
 	}
 
+	if (s.hasTime) {
+			
+		if (datm[0].find('T') != std::string::npos) {
+			s.timestep = "seconds";
+		} else {
+			// backwards compatibility
+			if (datm[0].length() == 4) {
+				s.timestep = "years";
+			} else if (datm[0].length() == 7) {
+				if (datm[0].substr(0, 5) == "0000-") {
+					s.timestep = "months";					
+				} else {
+					s.timestep = "yearmonths";
+				}
+			} else if (datm[0].length() == 10) {
+			// current formats, always xxxx-xx-xx where possible
+				if (datm[0].substr(7, 3) == "-00") {
+					if (datm[0].substr(4, 3) == "-00") {
+						s.timestep = "years";
+					} else if (datm[0].substr(0, 4) == "0000") {
+						s.timestep = "months";		
+					} else {
+						s.timestep = "yearmonths";
+					}
+				} else {
+					s.timestep = "days";
+				}
+			}
+		}
+		for (size_t i=0; i<datm.size(); i++) {
+			s.time[i] = parse_time(datm[i]);
+		}
+		
+
+// try units from json
+		std::vector<int_64> timestamps;
+		std::string timestep="raw";
+		//std::vector<std::string> units;
+		try {
+			read_aux_json(fname, timestamps, timestep, unts, s.nlyr);
+		} catch(...) {
+			unts.resize(0);
+			addWarning("could not parse aux.json");
+		}
+		if (!unts.empty()) {
+			s.hasUnit = true;
+		}
+		
+		
+	} else {
+
+		std::vector<int_64> timestamps;
+		std::string timestep="raw";
+//		std::vector<std::string> units;
+		if (unts.empty()) {
+			try {
+				read_aux_json(fname, timestamps, timestep, unts, s.nlyr);
+			} catch(...) {
+				timestamps.resize(0);
+				unts.resize(0);
+				addWarning("could not parse aux.json");
+			}
+			if (!timestamps.empty()) {
+				s.time = timestamps;
+				s.timestep = timestep;
+				s.hasTime = true;
+			}
+			if (!unts.empty()) {
+				s.hasUnit = true;
+			}
+		}
+	}
+	if (s.hasUnit) {
+		s.unit = unts;
+	}
+
 
 	msg = "";
+	std::vector<std::string> metadata;
+
 	if ((gdrv=="netCDF") || (gdrv == "HDF5"))  {
-		std::vector<std::string> metadata;
+		
 		char **m = poDataset->GetMetadata();
 		if (m) {
 			while (*m != nullptr) {
@@ -1100,11 +1270,26 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 			}
 		}
 		s.set_names_time_ncdf(metadata, bandmeta, msg);
+
+		if (s.srs.is_empty()) {
+
+			bool lat = false;
+			bool lon = false;
+			for (size_t i=0; i<metadata.size(); i++) {
+				if (!lat) lat = metadata[i].find("long_name=latitude");
+				if (!lon) lon = metadata[i].find("long_name=longitude");
+			}
+			if (lon && lat) {
+				if (s.srs.set("+proj=longlat", msg)) {
+					s.parameters_changed = true;
+				}
+			}
+		}
 	} else if (gdrv == "GRIB") {	
 		s.set_names_time_grib(bandmeta, msg);
 	} else if (gdrv == "GTiff") {	
 	// needs to get its own generic one 
-		s.set_names_time_tif(bandmeta, msg);
+	//	s.set_names_time_tif(bandmeta, msg);
 	}
 	s.bmdata = bandmeta;
 	if (msg.size() > 1) {
@@ -1115,6 +1300,14 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 	s.hasValues = true;
 	setSource(s);
 
+	if ((!metadata.empty())) {
+		std::vector<std::string> tagnames, tagvalues;
+//		std::string stag = s.source_name + "#TAG_";
+		std::string stag = s.source_name + "#";
+		get_tags(metadata, stag, tagnames, tagvalues);
+		for (size_t i=0; i<tagnames.size(); i++) addTag(tagnames[i], tagvalues[i]);
+
+	}
 	if (getCols) {
 		setRGB(rgb_lyrs[0], rgb_lyrs[1], rgb_lyrs[2], -99, "rgb");
 	}
@@ -1123,15 +1316,21 @@ bool SpatRaster::constructFromFile(std::string fname, std::vector<int> subds, st
 }
 
 
-bool SpatRaster::readStartGDAL(unsigned src) {
+bool SpatRaster::readStartGDAL(size_t src) {
 
     GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_drivers, source[src].open_ops);
 
+
     if( poDataset == NULL )  {
-		if (!file_exists(source[src].filename )) {
+		size_t ncolon = std::count(source[src].filename.begin(), source[src].filename.end(), ':');
+		if ((ncolon < 2) && (!file_exists(source[src].filename ))) {
 			setError("file does not exist: " + source[src].filename);
 		} else {
-			setError("cannot read from " + source[src].filename  );
+			if (source[src].filename.substr(0, 4) == "HDF4") {
+				setError("cannot read from " + source[src].filename + "\n(Only 32 open datasets allowed with HDF4)");
+			} else {
+				setError("cannot read from " + source[src].filename);
+			}
 		}
 		return false;
 	}
@@ -1141,7 +1340,7 @@ bool SpatRaster::readStartGDAL(unsigned src) {
 	return(true);
 }
 
-bool SpatRaster::readStopGDAL(unsigned src) {
+bool SpatRaster::readStopGDAL(size_t src) {
 	if (source[src].gdalconnection != NULL) {
 		GDALClose( (GDALDatasetH) source[src].gdalconnection);
 	}
@@ -1199,7 +1398,7 @@ void vflip(std::vector<double> &v, const size_t &ncell, const size_t &nrows, con
 }
 
 
-void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t row, unsigned nrows, size_t col, unsigned ncols) {
+void SpatRaster::readChunkGDAL(std::vector<double> &data, size_t src, size_t row, size_t nrows, size_t col, size_t ncols) {
 
 	if (source[src].flipped) {
 		row = nrow() - row - nrows;
@@ -1226,8 +1425,8 @@ void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t r
 		return;
 	}
 
-	unsigned ncell = ncols * nrows;
-	unsigned nl = source[src].nlyr;
+	size_t ncell = ncols * nrows;
+	size_t nl = source[src].nlyr;
 	std::vector<double> out(ncell * nl);
 	int hasNA;
 	std::vector<double> naflags(nl, NAN);
@@ -1285,7 +1484,7 @@ void SpatRaster::readChunkGDAL(std::vector<double> &data, unsigned src, size_t r
 
 
 
-std::vector<double> SpatRaster::readValuesGDAL(unsigned src, size_t row, size_t nrows, size_t col, size_t ncols, int lyr) {
+std::vector<double> SpatRaster::readValuesGDAL(size_t src, size_t row, size_t nrows, size_t col, size_t ncols, int lyr) {
 
 	std::vector<double> errout;
 	if (source[src].rotated) {
@@ -1363,7 +1562,7 @@ std::vector<double> SpatRaster::readValuesGDAL(unsigned src, size_t row, size_t
 
 
 
-std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_t scols) {
+std::vector<double> SpatRaster::readGDALsample(size_t src, size_t srows, size_t scols, bool overview) {
 
 	std::vector<double> errout;
 	if (source[src].rotated) {
@@ -1384,7 +1583,9 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 	#if GDAL_VERSION_MAJOR <= 3 && GDAL_VERSION_MINOR < 3
 	// do nothing
 	#else 
-	openops.push_back("OVERVIEW_LEVEL=NONE");
+	if (!overview) {
+		openops.push_back("OVERVIEW_LEVEL=NONE");
+	}
 	#endif
 	
     GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_drivers, openops);
@@ -1398,8 +1599,8 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 		return errout;
 	}
 
-	unsigned ncell = scols * srows;
-	unsigned nl = source[src].nlyr;
+	size_t ncell = scols * srows;
+	size_t nl = source[src].nlyr;
 	std::vector<double> out(ncell*nl);
 	int hasNA;
 	CPLErr err = CE_None;
@@ -1464,8 +1665,105 @@ std::vector<double> SpatRaster::readGDALsample(unsigned src, size_t srows, size_
 }
 
 
+void SpatRaster::readRowColGDAL(size_t src, std::vector<std::vector<double>> &out, size_t outstart, std::vector<int_64> &rows, const std::vector<int_64> &cols) {
 
-std::vector<std::vector<double>> SpatRaster::readRowColGDAL(unsigned src, std::vector<int_64> &rows, const std::vector<int_64> &cols) {
+
+	if (source[src].rotated) {
+		setError("cannot read from rotated files. First use 'rectify'");
+		return;
+	}
+	size_t n = rows.size();
+	if (n < 1) {
+		addWarning("nothing to extract");
+		return;
+	}
+
+    GDALDataset *poDataset = openGDAL(source[src].filename, GDAL_OF_RASTER | GDAL_OF_READONLY, source[src].open_drivers, source[src].open_ops);
+
+    if( poDataset == NULL )  {
+		if (!file_exists(source[src].filename )) {
+			setError("file does not exist: " + source[src].filename);
+		} else {
+			setError("cannot read from " + source[src].filename  );
+		}
+		return;
+	}
+
+	std::vector<size_t> lyrs = source[src].layers;
+	size_t nl = lyrs.size();
+	size_t outend = outstart + nl;
+
+	size_t fnr = nrow() - 1;
+	if (source[src].flipped) {
+		for (size_t i=0; i<n; i++) {
+			rows[i] = fnr - rows[i];
+		}
+	}
+
+	std::vector<int> panBandMap;
+	if (!source[src].in_order()) {
+		panBandMap.reserve(nl);
+		for (size_t i=0; i < nl; i++) {
+			panBandMap.push_back(lyrs[i]+1);
+		}
+	}
+
+	for (size_t i=outstart; i<outend; i++) {
+		out[i] = std::vector<double> (n, NAN);
+	}
+
+	int_64 nr1 = nrow()-1;
+	int_64 nc1 = ncol()-1;
+	CPLErr err = CE_None;
+	std::vector<double> value(nl);
+
+	if (panBandMap.empty()) {
+		for (size_t i=0; i < n; i++) {
+			if ((cols[i] < 0) || (cols[i] > nc1) || (rows[i] < 0) || (rows[i] > nr1) ) continue;
+			err = poDataset->RasterIO(GF_Read, cols[i], rows[i], 1, 1, &value[0], 1, 1, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+			if (err == CE_None) {
+				for (size_t j=0; j<nl; j++) {
+					out[outstart+j][i] = value[j];
+				}
+			} else {
+				break;
+			}
+		}
+	} else {
+		for (size_t i=0; i < n; i++) {
+			if ((cols[i] < 0) || (rows[i] < 0)) continue;
+			err = poDataset->RasterIO(GF_Read, cols[i], rows[i], 1, 1, &value[0], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
+			if (err == CE_None) {
+				for (size_t j=0; j<nl; j++) {
+					out[outstart+j][i] = value[j];
+				}
+			} else {
+				break;
+			}
+		}
+	}
+		//std::vector<double> naflags(nl, NAN);
+
+	if (err == CE_None) {
+		int hasNA;
+		GDALRasterBand *poBand;
+		for (size_t i=0; i<nl; i++) {
+			poBand = poDataset->GetRasterBand(lyrs[i]+1);
+			double naflag = poBand->GetNoDataValue(&hasNA);
+			if (!hasNA) naflag = NAN;
+			NAso(out[outstart+i], n, {naflag}, source[src].scale, source[src].offset, source[src].has_scale_offset, source[src].hasNAflag, source[src].NAflag);
+		}
+	}
+	GDALClose((GDALDatasetH) poDataset);
+	if (err != CE_None ) {
+		setError("cannot read values");
+		return;
+	}
+}
+
+
+/*
+std::vector<std::vector<double>> SpatRaster::readRowColGDAL(size_t src, std::vector<int_64> &rows, const std::vector<int_64> &cols) {
 
 	std::vector<std::vector<double>> errout;
 	if (source[src].rotated) {
@@ -1486,9 +1784,9 @@ std::vector<std::vector<double>> SpatRaster::readRowColGDAL(unsigned src, std::v
 
 	GDALRasterBand *poBand;
 
-	std::vector<unsigned> lyrs = source[src].layers;
-	unsigned nl = lyrs.size();
-	unsigned n = rows.size();
+	std::vector<size_t> lyrs = source[src].layers;
+	size_t nl = lyrs.size();
+	size_t n = rows.size();
 
 	size_t fnr = nrow() - 1;
 	if (source[src].flipped) {
@@ -1507,15 +1805,15 @@ std::vector<std::vector<double>> SpatRaster::readRowColGDAL(unsigned src, std::v
 
 	std::vector<double> out(n * nl, NAN);
 	CPLErr err = CE_None;
-	for (size_t j=0; j < n; j++) {
-		if ((cols[j] < 0) || (rows[j] < 0)) continue;
+	for (size_t i=0; i < n; i++) {
+		if ((cols[i] < 0) || (rows[i] < 0)) continue;
 		if (panBandMap.empty()) {
-			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
+			err = poDataset->RasterIO(GF_Read, cols[i], rows[i], 1, 1, &out[i*nl], 1, 1, GDT_Float64, nl, NULL, 0, 0, 0, NULL);
 		} else {
-			err = poDataset->RasterIO(GF_Read, cols[j], rows[j], 1, 1, &out[j*nl], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
+			err = poDataset->RasterIO(GF_Read, cols[i], rows[i], 1, 1, &out[i*nl], 1, 1, GDT_Float64, nl, &panBandMap[0], 0, 0, 0, NULL);
 		}
 		if (err != CE_None ) {
-			break;
+			break;x`
 		}
 	}
 
@@ -1547,11 +1845,10 @@ std::vector<std::vector<double>> SpatRaster::readRowColGDAL(unsigned src, std::v
 	}
 	return r;
 }
+*/
 
-
-
-
-std::vector<double> SpatRaster::readRowColGDALFlat(unsigned src, std::vector<int_64> &rows, const std::vector<int_64> &cols) {
+/*
+std::vector<double> SpatRaster::readRowColGDALFlat(size_t src, std::vector<int_64> &rows, const std::vector<int_64> &cols) {
 
 	std::vector<double> errout;
 	if (source[src].rotated) {
@@ -1572,9 +1869,9 @@ std::vector<double> SpatRaster::readRowColGDALFlat(unsigned src, std::vector<int
 
 	GDALRasterBand *poBand;
 
-	std::vector<unsigned> lyrs = source[src].layers;
-	unsigned nl = lyrs.size();
-	unsigned n = rows.size();
+	std::vector<size_t> lyrs = source[src].layers;
+	size_t nl = lyrs.size();
+	size_t n = rows.size();
 
 	size_t fnr = nrow() - 1;
 	if (source[src].flipped) {
@@ -1624,12 +1921,11 @@ std::vector<double> SpatRaster::readRowColGDALFlat(unsigned src, std::vector<int
 
 	return out;
 }
-
+*/
 
 
 // ncdf
 
-
 bool ncdf_good_ends(std::string const &s) {
 	std::vector<std::string> end = {"_bnds", "_bounds", "lat", "lon", "longitude", "latitude"};
 	for (size_t i=0; i<end.size(); i++) {
@@ -1673,7 +1969,7 @@ void ncdf_pick_most(std::vector<std::string> &sd, std::vector<std::string> &varn
 
 
 
-bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> options, std::string driver) {
+bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> options, std::string driver, bool noflip) {
 
 	bool ncdf = driver =="netCDF";
 	bool gtiff = driver == "GTiff";
@@ -1724,13 +2020,21 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 	} else {
 		// eliminate sources based on names like "*_bnds" and "lat"
 		std::vector<int> rows, cols;
-		for (size_t i=0; i<info[1].size(); i++) {
+		for (size_t i=0; i<info[1].size(); i++) {			
 			if (ncdf_good_ends(info[1][i])) {
 				sd.push_back(info[0][i]);
 				varname.push_back(info[1][i]);
 				srcname.push_back(info[2][i]);
-				rows.push_back(std::stol(info[3][i]));
-				cols.push_back(std::stol(info[4][i]));
+				try {
+					rows.push_back(std::stol(info[3][i]));
+				} catch(...) {
+					rows.push_back(0);					
+				}
+				try {
+					cols.push_back(std::stol(info[4][i]));
+				} catch(...) {
+					cols.push_back(0);					
+				}
 			}
 		}
 		if (sd.empty()) { // all were removed
@@ -1759,7 +2063,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 	size_t cnt;
 
     for (cnt=0; cnt < sd.size(); cnt++) {
-		if (constructFromFile(sd[cnt], {-1}, {""}, {}, options)) break;
+		if (constructFromFile(sd[cnt], {-1}, {""}, {}, options, noflip)) break;
 	}
 //	source[0].source_name = srcname[cnt];
 
@@ -1770,7 +2074,7 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 	SpatOptions opt;
     for (size_t i=(cnt+1); i < sd.size(); i++) {
 //		printf( "%s\n", sd[i].c_str() );
-		bool success = out.constructFromFile(sd[i], {-1}, {""}, {}, options);
+		bool success = out.constructFromFile(sd[i], {-1}, {""}, {}, options, noflip);
 		if (success) {
 			if (out.compare_geom(*this, false, false, 0.1)) {
 //				out.source	[0].source_name = srcname[i];
@@ -1807,6 +2111,19 @@ bool SpatRaster::constructFromSDS(std::string filename, std::vector<std::string>
 		}
 	}
 
+	std::vector<std::string> metadata = get_metadata(filename, options);
+	if (!metadata.empty()) {
+		std::vector<std::string> tagnames, tagvalues;
+//		get_tags(metadata, "NC_GLOBAL#TAG_", tagnames, tagvalues);
+		get_tags(metadata, "NC_GLOBAL#", tagnames, tagvalues);
+		for (size_t i=0; i<tagnames.size(); i++) {
+			std::string mn = tagnames[i];
+			if (!((mn == "_FillValue") || (mn == "grid_mapping") || (mn == "Conventions") || (mn == "created_by") || (mn == "created_date"))) {
+				addTag(tagnames[i], tagvalues[i]);
+			}
+		}
+	}
+
 	return true;
 }
 
@@ -1856,6 +2173,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 	std::vector<double> raw;
 	raw.reserve(vals.size());
 	for (size_t i=0; i<vals.size(); i++) {
+		
 		double dval;
 		if (get_double(vals[i], dval)) {
 			raw.push_back(dval);
@@ -1870,6 +2188,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 	std::string origin;
 	std::string calendar = "standard";
 	for (size_t i=0; i<metadata.size(); i++) {
+
 		if (!fc) {
 			std::string pattern = "time#calendar=";
 			std::size_t found = metadata[i].find(pattern);
@@ -1934,8 +2253,10 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			}
 		}
 	}
+
 	SpatTime_t offset = 0;
 	if (foundorigin) {
+
 		step = "seconds";
 		out.reserve(raw.size());
 		std::string cal = "366";
@@ -1945,11 +2266,11 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			cal = "365";		
 		} else if (calendar =="gregorian" || calendar =="proleptic_gregorian" || calendar=="standard" || calendar == "julian") {
 			cal = "366";
-		} else {
+		} else if (!(months || years || yearmonths || yearsbp)) {
 			//cal = "366";
 			msg = "unknown calendar (assuming standard): " + calendar;			
 		}
-		
+
 		// this shortcut means that 360/noleap calendars loose only have dates, no time
 		// to be refined
 		if ((hours || minutes || seconds) && (cal == "360")) {
@@ -1973,7 +2294,7 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			}
 			days = true;
 		} 
-		
+	
 		if (days) {
 			step = "days";
 			std::vector<int> ymd = getymd(origin);
@@ -2000,7 +2321,6 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 		} else if (minutes) {
 			if (cal == "365") {
 				std::vector<int> ymd = getymd(origin);
-//				Rcpp::Rcout << origin << std::endl;
 				for (size_t i=0; i<raw.size(); i++) out.push_back(
 						get_time_noleap(ymd[0], ymd[1], ymd[2], ymd[3], ymd[4], 0, raw[i], "minutes")
 					);
@@ -2036,7 +2356,12 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 			}
 		} else if (months) {
 			step = "months";
-			for (size_t i=0; i<raw.size(); i++) out.push_back(get_time(1970, raw[i], 15, 0, 0, 0));
+			// check for 0..11 range
+			int zero = vmin(raw, true) == 0.0;
+			for (size_t i=0; i<raw.size(); i++) {
+				unsigned m = std::ceil(raw[i] + zero);
+				out.push_back(get_time(1970, m, 15, 0, 0, 0));
+			}
 		} else {
 			step = "raw";
 			for (size_t i=0; i<raw.size(); i++) out.push_back(raw[i]);
@@ -2052,6 +2377,9 @@ std::vector<int_64> ncdf_time(const std::vector<std::string> &metadata, std::vec
 //NETCDF_DIM_time=0
 //NETCDF_VARNAME=NVEL
 
+
+
+
 std::vector<std::vector<std::string>> ncdf_names(const std::vector<std::vector<std::string>> &m) {
 
 	std::vector<std::vector<std::string>> out(3);
@@ -2117,22 +2445,29 @@ void SpatRasterSource::set_names_time_ncdf(std::vector<std::string> metadata, st
 	source_name = nms[2][0];
 	source_name_long = nms[2][1];
 
-	if (nms[2][2].empty()) {
-		unit = {""};
-		hasUnit = false;
-	} else {
-		unit = {nms[2][2]};
-		hasUnit = true;
+	if (!hasUnit) {
+		if (nms[2][2].empty()) {
+			unit = {""};
+			hasUnit = false;
+		} else {
+			unit = {nms[2][2]};
+			hasUnit = true;
+		}
+		recycle(unit, nlyr);
 	}
-
-	recycle(unit, nlyr);
+	
 	if (!nms[0].empty()) {
 		std::string step;
-		std::vector<int_64> x = ncdf_time(metadata, nms[0], step, msg);
-		if (x.size() == nlyr) {
-			time = x;
-			timestep = step;
-			hasTime = true;
+		std::vector<int_64> x;
+		try {
+			x = ncdf_time(metadata, nms[0], step, msg);
+			if (x.size() == nlyr) {
+				time = x;
+				timestep = step;
+				hasTime = true;
+			}
+		} catch(...) {
+			msg = "could not extract time scale";
 		}
 	}
 }
