@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -27,6 +27,7 @@
 
 #include "string_utils.h"
 
+
 std::string geomType(OGRLayer *poLayer) {
 	std::string s = "";
     poLayer->ResetReading();
@@ -42,6 +43,8 @@ std::string geomType(OGRLayer *poLayer) {
 }
 
 
+#include "Rcpp.h"
+
 SpatDataFrame readAttributes(OGRLayer *poLayer, bool as_proxy) {
 	SpatDataFrame df;
 
@@ -50,35 +53,36 @@ SpatDataFrame readAttributes(OGRLayer *poLayer, bool as_proxy) {
 	if (nfields == 0) return df;
 
 	OGRFieldType ft;
-    poLayer->ResetReading();
-    OGRFeature *poFeature;
 	OGRFieldDefn *poFieldDefn;
 	df.resize_cols(nfields);
-	bool first = true;
 	unsigned dtype;
 	long longNA = NA<long>::value;
+	SpatTime_t timeNA = NA<SpatTime_t>::value;
 
-    while( (poFeature = poLayer->GetNextFeature()) != NULL ) {
-		if (first) {
-			for (size_t i = 0; i < nfields; i++ ) {
-				poFieldDefn = poFDefn->GetFieldDefn(i);
-				std::string fname = poFieldDefn->GetNameRef();
-				ft = poFieldDefn->GetType();
-				if (ft == OFTReal) {
-					dtype = 0;
-				} else if ((ft == OFTInteger) | (ft == OFTInteger64)) {
-					if (poFieldDefn->GetSubType() == OFSTBoolean) {
-						dtype = 3;
-					} else {
-						dtype = 1;
-					}
-				} else {
-					dtype = 2;
-				}
-				df.add_column(dtype, fname);
+	for (size_t i = 0; i < nfields; i++ ) {
+		poFieldDefn = poFDefn->GetFieldDefn(i);
+		std::string fname = poFieldDefn->GetNameRef();
+		ft = poFieldDefn->GetType();
+		// OFTInteger64 may be too large 
+		if ((ft == OFTReal) || (ft == OFTInteger64)) {
+			dtype = 0;
+		} else if (ft == OFTInteger) {
+			if (poFieldDefn->GetSubType() == OFSTBoolean) {
+				dtype = 3;
+			} else {
+				dtype = 1;
 			}
-			first = false;
+		} else if ((ft == OFTDate) || (ft == OFTDateTime)) {
+			dtype = 4;
+		} else {
+			dtype = 2;
 		}
+		df.add_column(dtype, fname);
+	}
+
+    OGRFeature *poFeature;
+    poLayer->ResetReading();
+    while( (poFeature = poLayer->GetNextFeature()) != NULL ) {
 
 		for (size_t i = 0; i < nfields; i++ ) {
 			poFieldDefn = poFDefn->GetFieldDefn( i );
@@ -109,9 +113,37 @@ SpatDataFrame readAttributes(OGRLayer *poLayer, bool as_proxy) {
 					break;
 				case OFTInteger64:
 					if (not_null) {
-						df.iv[j].push_back(poFeature->GetFieldAsInteger64(i));
+						df.dv[j].push_back(poFeature->GetFieldAsInteger64(i));
 					} else {
-						df.iv[j].push_back(longNA);
+						df.dv[j].push_back(NAN);
+					}
+					break;
+				case OFTDate:  
+					if (i == 0) {
+						df.tv[j].step = "days";
+					}
+					if (not_null) {
+						int pnYear, pnMonth, pnDay, pnHour, pnMinute, pnTZFlag;
+						float pfSecond;
+						poFeature->GetFieldAsDateTime(i, &pnYear, &pnMonth, &pnDay, &pnHour, &pnMinute, &pfSecond, &pnTZFlag);
+						SpatTime_t d = get_time(pnYear, pnMonth, pnDay, 0, 0, 0);
+						df.tv[j].x.push_back(d);
+					} else {
+						df.tv[j].x.push_back(timeNA);
+					}
+					break;
+				case OFTDateTime:
+					if (i == 0) {
+						df.tv[j].step = "seconds";
+					}
+					if (not_null) {
+						int pnYear, pnMonth, pnDay, pnHour, pnMinute, pnTZFlag;
+						float pfSecond;
+						poFeature->GetFieldAsDateTime(i, &pnYear, &pnMonth, &pnDay, &pnHour, &pnMinute, &pfSecond, &pnTZFlag);
+						SpatTime_t d = get_time(pnYear, pnMonth, pnDay, pnHour, pnMinute, (int)pfSecond);
+						df.tv[j].x.push_back(d);
+					} else {
+						df.tv[j].x.push_back(timeNA);
 					}
 					break;
 	//          case OFTString:
@@ -184,10 +216,22 @@ std::string getDs_PRJ(GDALDataset *poDataset) {
 }
 */
 
+SpatGeom emptyGeom() {
+	SpatGeom g;
+	g.gtype = null;
+	g.extent.xmin=NAN;
+	g.extent.xmax=NAN;
+	g.extent.ymin=NAN;
+	g.extent.ymax=NAN;
+	return g;
+}
+
 
 SpatGeom getPointGeom(OGRGeometry *poGeometry) {
 	SpatGeom g(points);
 	if (poGeometry->IsEmpty()) {
+		//SpatPart p(NAN, NAN);
+		//g.addPart(p);
 		return g;
 	}
 	#if GDAL_VERSION_NUM >= GDAL_COMPUTE_VERSION(2,3,0)
@@ -207,6 +251,7 @@ SpatGeom getMultiPointGeom(OGRGeometry *poGeometry) {
 	unsigned ng = poMultipoint->getNumGeometries();
 	std::vector<double> X(ng);
 	std::vector<double> Y(ng);
+	SpatGeom g(points);
 	for (size_t i=0; i<ng; i++) {
 	   	OGRGeometry *poMpGeometry = poMultipoint->getGeometryRef(i);
 		#if GDAL_VERSION_NUM >= GDAL_COMPUTE_VERSION(2,3,0)
@@ -214,12 +259,11 @@ SpatGeom getMultiPointGeom(OGRGeometry *poGeometry) {
 		#else
 			OGRPoint *poPoint = (OGRPoint *) poMpGeometry;
 		#endif
-		X[i] = poPoint->getX();
-		Y[i] = poPoint->getY();
+		double x = poPoint->getX();
+		double y = poPoint->getY();
+		SpatPart p(x, y);
+		g.addPart(p);
 	}
-	SpatPart p(X, Y);
-	SpatGeom g(points);
-	g.addPart(p);
 	return g;
 }
 
@@ -374,18 +418,8 @@ std::vector<std::string> SpatVector::layer_names(std::string filename) {
 	return out;
 }
 
-SpatGeom emptyGeom() {
-	SpatGeom g;
-	g.gtype = null;
-	g.extent.xmin=NAN;
-	g.extent.xmax=NAN;
-	g.extent.ymin=NAN;
-	g.extent.ymax=NAN;
-	return g;
-}
 
-
-bool layerQueryFilter(GDALDataset *&poDS, OGRLayer *&poLayer, std::string &layer, std::string &query, std::vector<double> &extent, SpatVector &filter, std::string &errmsg, std::vector<std::string> &wrms) {
+bool layerQueryFilter(GDALDataset *&poDS, OGRLayer *&poLayer, std::string &layer, std::string &query, std::vector<double> &ext, SpatVector &filter, std::string &errmsg, std::vector<std::string> &wrms) {
 
 	if (query.empty()) {
 		if (layer.empty()) {
@@ -456,15 +490,15 @@ bool layerQueryFilter(GDALDataset *&poDS, OGRLayer *&poLayer, std::string &layer
 		}
 		OGRFeature::DestroyFeature( fFeature );
 		GDALClose(filterDS);
-	} else if (!extent.empty()) {
-		poLayer->SetSpatialFilterRect(extent[0], extent[2], extent[1], extent[3]);
+	} else if (!ext.empty()) {
+		poLayer->SetSpatialFilterRect(ext[0], ext[2], ext[1], ext[3]);
 	}
 
 	return true;
 }
 
 
-bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what) {
+bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string query, std::vector<double> ext, SpatVector filter, bool as_proxy, std::string what) {
 
 	if (poDS == NULL) {
 		setError("dataset is empty");
@@ -476,11 +510,11 @@ bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string que
 	poLayer = poDS->GetLayer(0);
 
 	read_query = query;
-	read_extent = extent;
+	read_extent = ext;
 	std::string errmsg;
 	std::vector<std::string> wrnmsg;
 
-	if (!layerQueryFilter(poDS, poLayer, layer, query, extent, filter, errmsg, wrnmsg)) {
+	if (!layerQueryFilter(poDS, poLayer, layer, query, ext, filter, errmsg, wrnmsg)) {
 		setError(errmsg);
 		return false;
 	} else if (!wrnmsg.empty()) {
@@ -575,6 +609,19 @@ bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string que
 			}
 			addGeom(g);
 			OGRFeature::DestroyFeature( poFeature );
+		} else if (wkbgeom == wkbUnknown) {
+			long long fcnt = poLayer->GetFeatureCount(true);
+			if (fcnt == 0) return true;
+			if (fcnt < 0) {
+				if ( (poFeature = poLayer->GetNextFeature()) != NULL ) {
+					return true;
+				}	
+			} 
+			const char *geomtypechar = OGRGeometryTypeToName(wkbgeom);
+			std::string strgeomtype = geomtypechar;
+			std::string s = "cannot read this geometry type: "+ strgeomtype;
+			setError(s);
+			return false;			
 		} else if (wkbgeom != wkbNone) {
 			const char *geomtypechar = OGRGeometryTypeToName(wkbgeom);
 			std::string strgeomtype = geomtypechar;
@@ -583,6 +630,23 @@ bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string que
 			return false;
 		}
 		geom_count = poLayer->GetFeatureCount();
+		
+		// not checking for multiple geom fields
+        // int nGeomFieldCount = poLayer->GetLayerDefn()->GetGeomFieldCount();
+
+		OGREnvelope oExt;
+		if (poLayer->GetExtent(&oExt, FALSE) == OGRERR_NONE) {
+			extent.xmin = oExt.MinX; 
+            extent.xmax = oExt.MaxX;
+			extent.ymin = oExt.MinY;
+			extent.ymax = oExt.MaxY;
+		} else {
+			extent.xmin = NAN; 
+            extent.xmax = NAN;
+			extent.ymin = NAN;
+			extent.ymax = NAN;			
+		}
+		
 		is_proxy = true;
 		return true;
 	}
@@ -639,6 +703,15 @@ bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string que
 			OGRFeature::DestroyFeature( poFeature );
 		}
 	} else if (wkbgeom == wkbUnknown) {
+
+		long long fcnt = poLayer->GetFeatureCount(true);
+		if (fcnt == 0) return true;
+		if (fcnt < 0) {
+			if ( (poFeature = poLayer->GetNextFeature()) != NULL ) {
+				return true;
+			}	
+		}
+
 		SpatVectorCollection sv;
 		std::vector<double> dempty;
 		SpatVector filter2;
@@ -673,9 +746,17 @@ bool SpatVector::read_ogr(GDALDataset *&poDS, std::string layer, std::string que
 }
 
 
-bool SpatVector::read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what) {
-    //OGRRegisterAll();
-    GDALDataset *poDS = static_cast<GDALDataset*>(GDALOpenEx( fname.c_str(), GDAL_OF_VECTOR, NULL, NULL, NULL ));
+bool SpatVector::read(std::string fname, std::string layer, std::string query, std::vector<double> ext, SpatVector filter, bool as_proxy, std::string what, std::vector<std::string> options) {
+
+	char ** openops = NULL;
+	for (size_t i=0; i<options.size(); i++) {
+		std::vector<std::string> opt = strsplit(options[i], "=");
+		if (opt.size() == 2) {
+			openops = CSLSetNameValue(openops, opt[0].c_str(), opt[1].c_str());
+		}
+	}
+		
+    GDALDataset *poDS = static_cast<GDALDataset*>(GDALOpenEx( fname.c_str(), GDAL_OF_VECTOR, NULL, openops, NULL ));
     if( poDS == NULL ) {
 		if (!file_exists(fname)) {
 			setError("file does not exist: " + fname);
@@ -684,7 +765,7 @@ bool SpatVector::read(std::string fname, std::string layer, std::string query, s
 		}
 		return false;
     }
-	bool success = read_ogr(poDS, layer, query, extent, filter, as_proxy, what);
+	bool success = read_ogr(poDS, layer, query, ext, filter, as_proxy, what);
 	if (poDS != NULL) GDALClose( poDS );
 	source = fname;
 	return success;
