@@ -25,14 +25,12 @@
 
 #ifdef useRcpp
 #include <Rcpp.h>
-// Rcpp::depends(RcppProgress)
-//#include "progress.hpp"
-//#include "progress_bar.hpp"
 #endif
 
 typedef long long int_64;
 
 
+
 class SpatCategories {
 	public:
 		virtual ~SpatCategories(){}
@@ -77,8 +75,8 @@ class SpatRasterSource {
 //		void fsclose();
 
 		size_t ncol, nrow;
-		unsigned nlyr;
-		unsigned nlyrfile = 0;
+		size_t nlyr;
+		size_t nlyrfile = 0;
 		SpatExtent extent;
 		bool extset=false;
 		bool rotated=false;
@@ -102,7 +100,7 @@ class SpatRasterSource {
 		
 		//std::vector<std::string> crs = std::vector<std::string>(2, "");
 		SpatSRS srs;
-		std::vector<unsigned> layers;
+		std::vector<size_t> layers;
 		// layer names 
 		std::vector<std::string> names;
 		// data source (sds) has one "variable name" / long_name
@@ -135,6 +133,8 @@ class SpatRasterSource {
 		std::vector<bool> hasCategories;
 		std::vector<SpatCategories> cats;
 		std::vector<unsigned char> valueType;
+		// 0:double; 1:int; 3:bool
+
 		//std::vector<std::string> dataType;
 
 		std::vector<bool> hasColors;
@@ -158,13 +158,13 @@ class SpatRasterSource {
 		std::vector<double> offset;
 
 //		std::vector<SpatRasterSource> subset(std::vector<unsigned> lyrs);
-		SpatRasterSource subset(std::vector<unsigned> lyrs);
+		SpatRasterSource subset(std::vector<size_t> lyrs);
 //		void getValues(std::vector<double> &v, unsigned lyr, SpatOptions &opt);
-		void appendValues(std::vector<double> &v, unsigned lyr);
+		void appendValues(std::vector<double> &v, size_t lyr);
 		
 		void setRange();
-		void resize(unsigned n);
-		void reserve(unsigned n);
+		void resize(size_t n);
+		void reserve(size_t n);
 		bool in_order();
 		bool combine_sources(const SpatRasterSource &x);
 		bool combine(SpatRasterSource &x);
@@ -174,6 +174,10 @@ class SpatRasterSource {
 		void set_names_time_ncdf(std::vector<std::string> metadata, std::vector<std::vector<std::string>> bandmeta, std::string &msg);
 		void set_names_time_grib(std::vector<std::vector<std::string>> bandmeta, std::string &msg);
 		void set_names_time_tif(std::vector<std::vector<std::string>> bandmeta, std::string &msg);
+		
+		std::vector<std::map<std::string, std::string>> lyrTags;
+		void addLyrTag(size_t slyr, std::string name, std::string value);
+		
 };
 
 
@@ -182,7 +186,7 @@ class BlockSize {
 		virtual ~BlockSize(){}
 		std::vector<size_t> row;
 		std::vector<size_t> nrows;
-		unsigned n;
+		size_t n;
 };
 
 
@@ -202,7 +206,6 @@ class SpatRaster {
 	public:
 
 #ifdef useRcpp
-//		Progress* pbar;
 		SpatProgress pbar;
 		bool progressbar = false;
 #endif
@@ -228,20 +231,19 @@ class SpatRaster {
 		std::string getError() { return msg.getError();}
 		std::string getMessage() { return msg.getMessage();}
 
-		std::map<std::string, std::string> tags;
+		std::map<std::string, std::string> user_tags;
 		bool addTag(std::string name, std::string value);
 		bool removeTag(std::string name);
 		std::string getTag(std::string name);
 		std::vector<std::string> getTags();
 
-		std::vector<std::map<std::string, std::string>> lyrTags;
 		void addLyrTags(std::vector<size_t> lyrs, std::vector<std::string> names, std::vector<std::string> values);
 
 		bool removeLyrTags();
 		bool removeLyrTag(size_t lyr, std::string name);
 		std::string getLyrTag(size_t lyr, std::string name);
 		std::vector<std::string> getLyrTags(std::vector<size_t> lyrs);
-
+		std::vector<std::map<std::string, std::string>> getAllLyrTags();
 		//double NA = std::numeric_limits<double>::quiet_NaN();
 
 		size_t ncol();
@@ -278,6 +280,8 @@ class SpatRaster {
 		bool is_global_lonlat();
 		int ns_polar();
 
+		std::vector<bool> is_flipped();
+
 		std::vector<double> resolution();
 		SpatRaster setResolution(double xres, double yres);
 		double ncell() { return nrow() * ncol(); }
@@ -288,7 +292,7 @@ class SpatRaster {
 		double xres();
 		double yres();
 		std::vector<double> origin();
-		unsigned nlyr();
+		size_t nlyr();
 
 		// only no values allowed with a single SpatRasterSource
 		bool hasValues();
@@ -337,7 +341,7 @@ class SpatRaster {
 		std::vector<int_64> getTime();
 		std::string getTimeStep();
 		std::string getTimeZone();
-		std::vector<std::string> getTimeStr(bool addstep);
+		std::vector<std::string> getTimeStr(bool addstep, std::string timesep);
 		bool setTime(std::vector<int_64> time, std::string step, std::string zone);
 		
 		std::vector<double> getDepth();
@@ -358,9 +362,9 @@ class SpatRaster {
 ////////////////////////////////////////////////////
 
 		SpatRaster();
-		SpatRaster(unsigned nr, unsigned nc, unsigned nl, SpatExtent ext, std::string crs);
-		SpatRaster(std::vector<unsigned> rcl, std::vector<double> ext, std::string crs);
-		SpatRaster(std::vector<std::string> fname, std::vector<int> subds, std::vector<std::string> subdsname, bool multi, std::vector<std::string> drivers, std::vector<std::string> options, std::vector<size_t> x);
+		SpatRaster(size_t nr, size_t nc, size_t nl, SpatExtent ext, std::string crs);
+		SpatRaster(std::vector<size_t> rcl, std::vector<double> ext, std::string crs);
+		SpatRaster(std::vector<std::string> fname, std::vector<int> subds, std::vector<std::string> subdsname, bool multi, std::vector<std::string> drivers, std::vector<std::string> options, std::vector<size_t> xyz, bool noflip);
 		SpatRaster(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options);
 		SpatRaster(SpatRasterSource &s);
 		virtual ~SpatRaster(){}
@@ -375,9 +379,9 @@ class SpatRaster {
         SpatRaster geometry(long nlyrs=-1, bool properties=false, bool time=true, bool units=false, bool tags=false);
 		SpatRaster geometry_opt(long nlyrs, bool properties, bool time, bool units, bool tags, bool datatype, SpatOptions &opt);
 
-		bool constructFromFile(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options);
+		bool constructFromFile(std::string fname, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> drivers, std::vector<std::string> options, bool noflip);
 		bool constructFromFileMulti(std::string fname, std::vector<int> sub, std::vector<std::string> subname, std::vector<std::string> drivers, std::vector<std::string> options, std::vector<size_t> xyz);
-		bool constructFromSDS(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> options, std::string driver);
+		bool constructFromSDS(std::string filename, std::vector<std::string> meta, std::vector<int> subds, std::vector<std::string> subdsname, std::vector<std::string> options, std::string driver, bool noflip);
 
 		
 		//SpatRaster fromFiles(std::vector<std::string> fname, std::vector<int> subds, std::vector<std::string> subdsname, std::string drivers, std::vector<std::string> options);
@@ -390,12 +394,15 @@ class SpatRaster {
 		void combine(SpatRaster &x);
 		
 		SpatRaster subsetSource(size_t snr);
-		SpatRaster subset(std::vector<unsigned> lyrs, SpatOptions &opt);
-		SpatRaster replace(SpatRaster x, unsigned layer, SpatOptions &opt);
+		SpatRaster subset(std::vector<size_t> lyrs, SpatOptions &opt);
+		SpatRaster replace(SpatRaster x, size_t layer, SpatOptions &opt);
 ////////////////////////////////////////////////////
 // helper methods
 ////////////////////////////////////////////////////
 
+
+		std::vector<std::string> getAllFiles();
+
 		void gdalogrproj_init(std::string path);
 
 		bool compare_geom(SpatRaster &x, bool lyrs, bool crs, double tol, bool warncrs=false, bool ext=true, bool rowcol=true, bool res=false);
@@ -425,26 +432,26 @@ class SpatRaster {
 		std::vector<std::vector<int_64>> rowColFromExtent(SpatExtent e);
 		std::vector<std::vector<double>> coordinates(bool narm, bool nall, SpatOptions &opt);
 		
-        std::vector<unsigned> sourcesFromLyrs(std::vector<unsigned> lyrs);
-		int sourceFromLyr(unsigned lyr);
-		std::vector<unsigned> findLyr(unsigned lyr);
-		std::vector<unsigned> getBands();
+        std::vector<size_t> sourcesFromLyrs(std::vector<size_t> lyrs);
+		int sourceFromLyr(size_t lyr);
+		std::vector<size_t> findLyr(size_t lyr);
+		std::vector<size_t> getBands();
 
-        std::vector<unsigned> nlyrBySource();
-        std::vector<unsigned> lyrsBySource();
-        unsigned nsrc();
+        std::vector<size_t> nlyrBySource();
+        std::vector<size_t> lyrsBySource();
+        size_t nsrc();
 
 		SpatRaster makeCategorical(long layer, SpatOptions &opt);
-		bool createCategories(unsigned layer, SpatOptions &opt);
+		bool createCategories(size_t layer, SpatOptions &opt);
 		std::vector<bool> hasCategories();
-		bool setCategories(unsigned layer, SpatDataFrame d, unsigned index);
+		bool setCategories(size_t layer, SpatDataFrame d, size_t index);
 		bool removeCategories(long layer);
 		std::vector<SpatCategories> getCategories();
-		SpatCategories getLayerCategories(unsigned layer);
-		std::vector<std::string> getLabels(unsigned layer);
-		bool setLabels(unsigned layer, std::vector<long> value, std::vector<std::string> labels, std::string name);
-		int getCatIndex(unsigned layer);
-		bool setCatIndex(unsigned layer, int index);
+		SpatCategories getLayerCategories(size_t layer);
+		std::vector<std::string> getLabels(size_t layer);
+		bool setLabels(size_t layer, std::vector<long> value, std::vector<std::string> labels, std::string name);
+		int getCatIndex(size_t layer);
+		bool setCatIndex(size_t layer, int index);
 
 		bool hasLegend();
 		bool setLegend(SpatDataFrame x);
@@ -457,11 +464,11 @@ class SpatRaster {
 
 		//bool setAttrIndex(size_t layer, int i);
 		//std::vector<int> getAttrIndex();
-		//void createAttributes(unsigned layer);
+		//void createAttributes(size_t layer);
 		//std::vector<bool> hasAttributes();
-		//void setAttributes(unsigned layer, SpatDataFrame df);
+		//void setAttributes(size_t layer, SpatDataFrame df);
 		//std::vector<SpatDataFrame> getAttributes();
-		//SpatDataFrame getLayerAttributes(unsigned layer);
+		//SpatDataFrame getLayerAttributes(size_t layer);
 		
 		std::vector<bool> hasColors();
 		std::vector<SpatDataFrame> getColors();
@@ -484,12 +491,12 @@ class SpatRaster {
 		void readValuesWhileWriting(std::vector<double> &out, size_t row, size_t nrows, size_t col, size_t ncols);
 		void readChunkMEM(std::vector<double> &out, size_t src, size_t row, size_t nrows, size_t col, size_t ncols);
 
-		void readBlock(std::vector<double> &v, BlockSize bs, unsigned i){ // inline
+		void readBlock(std::vector<double> &v, BlockSize bs, size_t i){ // inline
 			readValues(v, bs.row[i], bs.nrows[i], 0, ncol());
 		}
 
-		void readBlock2(std::vector<std::vector<double>> &v, BlockSize bs, unsigned i);
-		void readBlockIP(std::vector<double> &x, BlockSize bs, unsigned i);		
+		void readBlock2(std::vector<std::vector<double>> &v, BlockSize bs, size_t i);
+		void readBlockIP(std::vector<double> &x, BlockSize bs, size_t i);		
 		std::vector<double> readExtent(SpatExtent e);
 		bool readStop();
 
@@ -497,7 +504,7 @@ class SpatRaster {
 
 		bool writeStart(SpatOptions &opt, std::vector<std::string> srcnames);
 
-		bool writeBlock(std::vector<double> &v, unsigned i){ // inline
+		bool writeBlock(std::vector<double> &v, size_t i){ // inline
 			// for debugging?
 			// if (bs.row.size() <= i) {
 			//    setError("invalid block number"); return false;	
@@ -523,32 +530,40 @@ class SpatRaster {
 		bool getTempFile(std::string &filename, std::string &driver, SpatOptions& opt);
 
 
-		bool readStartMulti(unsigned src);
-		bool readStopMulti(unsigned src);
+		bool readStartMulti(size_t src);
+		bool readStopMulti(size_t src);
 		bool readValuesMulti(std::vector<double> &out, size_t src, size_t row, size_t nrows, size_t col, size_t ncols);
 
 
 
 		//bool writeStartBinary(std::string filename, std::string datatype, std::string bandorder, bool overwrite);
-		//bool writeValuesBinary(std::vector<double> &vals, unsigned startrow, unsigned nrows, unsigned startcol, unsigned ncols);
+		//bool writeValuesBinary(std::vector<double> &vals, size_t startrow, size_t nrows, size_t startcol, size_t ncols);
 
 		bool writeValuesMem(std::vector<double> &vals, size_t startrow, size_t nrows);
 		bool writeValuesMemRect(std::vector<double> &vals, size_t startrow, size_t nrows, size_t startcol, size_t ncols);
 
 		// binary (flat) source
-		//std::vector<double> readValuesBinary(unsigned src, unsigned row, unsigned nrows, unsigned col, unsigned ncols);
-		//std::vector<double> readSampleBinary(unsigned src, unsigned srows, unsigned scols);
-		//std::vector<std::vector<double>> readCellsBinary(unsigned src, std::vector<double> cells);
+		//std::vector<double> readValuesBinary(size_t src, size_t row, size_t nrows, size_t col, size_t ncols);
+		//std::vector<double> readSampleBinary(size_t src, size_t srows, size_t scols);
+		//std::vector<std::vector<double>> readCellsBinary(size_t src, std::vector<double> cells);
 
 		// gdal source
-		std::vector<double> readValuesGDAL(unsigned src, size_t row, size_t nrows, size_t col, size_t ncols, int lyr = -1);
-		std::vector<double> readGDALsample(unsigned src, size_t srows, size_t scols);
-		std::vector<std::vector<double>> readRowColGDAL(unsigned src, std::vector<int_64> &rows, const std::vector<int_64> &cols);
-		std::vector<double> readRowColGDALFlat(unsigned src, std::vector<int_64> &rows, const std::vector<int_64> &cols);
+		std::vector<double> readValuesGDAL(size_t src, size_t row, size_t nrows, size_t col, size_t ncols, int lyr = -1);
+		std::vector<double> readGDALsample(size_t src, size_t srows, size_t scols, bool overview);
 
-		bool readStartGDAL(unsigned src);
-		bool readStopGDAL(unsigned src);
-		void readChunkGDAL(std::vector<double> &data, unsigned src, size_t row, unsigned nrows, size_t col, unsigned ncols);
+		void readRowColGDAL(size_t src, std::vector<std::vector<double>> &out, size_t outstart, std::vector<int_64> &rows, const std::vector<int_64> &cols);
+
+//		std::vector<double> readRowColGDALFlat(size_t src, std::vector<int_64> &rows, const std::vector<int_64> &cols);
+
+//		std::vector<double> readRowColBlockFlat(size_t src, std::vector<int_64> &rows, std::vector<int_64> &cols);
+//		std::vector<std::vector<double>> readRowColBlock(size_t src, std::vector<int_64> &rows, std::vector<int_64> &cols);
+
+		void readRowColBlock(size_t src, std::vector<std::vector<double>> &out, size_t outstart, std::vector<int_64> &rows, std::vector<int_64> &cols);
+
+
+		bool readStartGDAL(size_t src);
+		bool readStopGDAL(size_t src);
+		void readChunkGDAL(std::vector<double> &data, size_t src, size_t row, size_t nrows, size_t col, size_t ncols);
 
 		bool setWindow(SpatExtent x);
 		bool removeWindow();
@@ -585,14 +600,13 @@ class SpatRaster {
 		SpatRaster rectify(std::string method, SpatRaster aoi, unsigned useaoi, bool snap, SpatOptions &opt);
 		
         std::vector<double> adjacent(std::vector<double> cells, std::string directions, bool include);
-        std::vector<double> adjacentMat(std::vector<double> cells, std::vector<bool> mat, std::vector<unsigned> dim, bool include);
- 		SpatRaster aggregate(std::vector<unsigned> fact, std::string fun, bool narm, SpatOptions &opt);
+        std::vector<double> adjacentMat(std::vector<double> cells, std::vector<bool> mat, std::vector<size_t> dim, bool include);
+ 		SpatRaster aggregate(std::vector<size_t> fact, std::string fun, bool narm, SpatOptions &opt);
 		SpatExtent align(SpatExtent e, std::string snap);
 		SpatRaster rst_area(bool mask, std::string unit, bool transform, int rcmax, SpatOptions &opt);
-
 		std::vector<std::vector<double>> sum_area(std::string unit, bool transform, bool by_value, SpatOptions &opt);
-
 		std::vector<std::vector<double>> sum_area_group(SpatRaster group, std::string unit, bool transform, bool by_value, SpatOptions &opt);
+		SpatRaster surfaceArea(SpatOptions &opt);
 
 		SpatRaster roll(size_t n, std::string fun, std::string type, bool circular, bool narm, SpatOptions &opt);
 
@@ -601,7 +615,7 @@ class SpatRaster {
 		SpatRaster arith(std::vector<double> x, std::string oper, bool reverse, bool falseNA, SpatOptions &opt);
 		SpatRaster arith_m(std::vector<double> x, std::string oper, std::vector<size_t> dim, bool reverse, SpatOptions &opt);
 
-		SpatRaster apply(std::vector<unsigned> ind, std::string fun, bool narm, std::vector<std::string> nms, std::vector<int_64> time, std::string timestep, std::string timezone, SpatOptions &opt);
+		SpatRaster apply(std::vector<size_t> ind, std::string fun, bool narm, std::vector<std::string> nms, std::vector<int_64> time, std::string timestep, std::string timezone, SpatOptions &opt);
 	
 		SpatRaster rapply(SpatRaster x, double first, double last, std::string fun, bool clamp, bool narm, bool circular, SpatOptions &opt);
 		std::vector<std::vector<double>> rappvals(SpatRaster x, double first, double last, bool clamp, bool all, double fill, size_t startrow, size_t nrows, bool circular);
@@ -620,9 +634,10 @@ class SpatRaster {
 		SpatVector as_multipoints(bool narm, bool nall, SpatOptions &opt);
 		SpatRaster atan_2(SpatRaster x, SpatOptions &opt);
 
-		std::vector<std::vector<double>> bilinearValues(const std::vector<double> &x, const std::vector<double> &y);
+
+		void bilinearValues(std::vector<std::vector<double>> &out, const std::vector<double> &x, const std::vector<double> &y);
 		std::vector<double> bilinearCells(const std::vector<double> &x, const std::vector<double> &y);
-		std::vector<double> fourCellsFromXY(const std::vector<double> &x, const std::vector<double> &y);
+		void fourCellsFromXY(std::vector<double> &out, const std::vector<double> &x, const std::vector<double> &y);
 
 		SpatRaster buffer(double d, double background, SpatOptions &opt);
 		SpatRaster clamp(std::vector<double> low, std::vector<double> high, bool usevalue, SpatOptions &opt);
@@ -633,36 +648,52 @@ class SpatRaster {
 		SpatRaster dropLevels();
 
 		SpatRaster cover(SpatRaster x, std::vector<double> value, SpatOptions &opt);
+		SpatRaster cover(std::vector<double> value, SpatOptions &opt);
 
 		SpatRaster crop(SpatExtent e, std::string snap, bool expand, SpatOptions &opt);
 		SpatRaster cropmask(SpatVector &v, std::string snap, bool touches, bool expand, SpatOptions &opt);
 		SpatRaster cum(std::string fun, bool narm, SpatOptions &opt);
-        SpatRaster disaggregate(std::vector<unsigned> fact, SpatOptions &opt);
-		SpatRaster distance(double target, double exclude, bool keepNA, std::string unit, bool remove_zero, bool haversine, SpatOptions &opt);
+        SpatRaster disaggregate(std::vector<size_t> fact, SpatOptions &opt);
 		SpatRaster proximity(double target, double exclude, bool keepNA, std::string unit, bool buffer, double maxdist, bool remove_zero, SpatOptions &opt);
+		SpatRaster fillNA(double missing, double maxdist, int niter, SpatOptions &opt);
 		
-		SpatRaster distance_rasterize(SpatVector p, double target, double exclude, std::string unit, bool haversine, SpatOptions &opt);
-		SpatRaster direction_rasterize(SpatVector p, bool from, bool degrees, double target, double exclude,  SpatOptions &opt);
-		
-		SpatRaster distance_spatvector(SpatVector p, std::string unit, bool haversine, SpatOptions &opt);
-		SpatRaster distance_crds(std::vector<double>& x, std::vector<double>& y, bool haversine, bool skip, bool setNA, std::string unit, SpatOptions &opt);
+		SpatRaster distance(double target, double exclude, bool keepNA, std::string unit, bool remove_zero, std::string method, bool values, double threshold, SpatOptions &opt);
+		SpatRaster nearest(double target, double exclude, bool keepNA, std::string unit, bool remove_zero, std::string method, SpatOptions &opt);
 
-		SpatRaster direction(bool from, bool degrees, double target, double exclude, SpatOptions &opt);
-		SpatRaster direction_vector(SpatVector p, bool from, bool degrees, SpatOptions &opt);
+//		SpatRaster distance_spatvector(SpatVector p, std::string unit, const std::string& method, SpatOptions &opt);
+//		SpatRaster distance_rasterize(SpatVector p, double target, double exclude, std::string unit, const std::string& method, SpatOptions &opt);
+		SpatRaster distance_vector(SpatVector p, bool rasterize, std::string unit, const std::string& method, SpatOptions &opt);
+
+		SpatRaster direction_rasterize(SpatVector p, bool from, bool degrees, double target, double exclude, const std::string& method, SpatOptions &opt);
+		
+		
+		SpatRaster distance_crds(std::vector<double>& x, std::vector<double>& y, const std::string& method, bool skip, bool setNA, std::string unit,double threshold, SpatOptions &opt);
+		SpatRaster distance_crds_vals(std::vector<double>& x, std::vector<double>& y, const std::vector<double>& v, const std::string& method, bool skip, bool setNA, std::string unit, double threshold, SpatOptions &opt);
+
+		SpatRaster dn_crds(std::vector<double>& x, std::vector<double>& y, const std::string& method, bool skip, bool setNA, std::string unit, SpatOptions &opt);
+
+		SpatRaster direction(bool from, bool degrees, double target, double exclude, const std::string& method, SpatOptions &opt);
+		SpatRaster direction_vector(SpatVector p, bool from, bool degrees, const std::string& method, SpatOptions &opt);
 		
 		SpatRaster clumps(int directions, bool zeroAsNA, SpatOptions &opt);
+		SpatRaster patches(size_t directions, SpatOptions &opt);
+
 
 		SpatRaster edges(bool classes, std::string type, unsigned directions, double falseval, SpatOptions &opt);
 		SpatRaster extend(SpatExtent e, std::string snap, double fill, SpatOptions &opt);
-		std::vector<std::vector<std::vector<double>>> extractVector(SpatVector v, bool touches, std::string method, bool cells, bool xy, bool weights, bool exact, SpatOptions &opt);
-		std::vector<double> extractVectorFlat(SpatVector v, std::vector<std::string> funs, bool narm, bool touches, std::string method, bool cells, bool xy, bool weights, bool exact, SpatOptions &opt);
+		std::vector<std::vector<std::vector<double>>> extractVector(SpatVector v, bool touches, bool small, std::string method, bool cells, bool xy, bool weights, bool exact, SpatOptions &opt);
+		std::vector<double> extractVectorFlat(SpatVector v, std::vector<std::string> funs, bool narm, bool touches, bool small, std::string method, bool cells, bool xy, bool weights, bool exact, SpatOptions &opt);
+		std::vector<std::vector<double>> extractBuffer(const std::vector<double> &x, const std::vector<double> &y, double b);
 		
 		
-		std::vector<double> vectCells(SpatVector v, bool touches, std::string method, bool weights, bool exact, SpatOptions &opt);
+//		std::vector<double> extract_interpolate(std::vector<double> x, std::vector<double> y, std::string algo);
+
+		
+		std::vector<double> vectCells(SpatVector v, bool touches, bool small, std::string method, bool weights, bool exact, SpatOptions &opt);
 		std::vector<double> extCells(SpatExtent ext);
 
 		std::vector<std::vector<double>> extractCell(std::vector<double> &cell);
-		std::vector<double> extractCellFlat(std::vector<double> &cell);
+//		std::vector<double> extractCellFlat(std::vector<double> &cell);
 	
 		std::vector<std::vector<double>> extractXY(const std::vector<double> &x, const std::vector<double> &y, const std::string & method, const bool &cells);
 		std::vector<double> extractXYFlat(const std::vector<double> &x, const std::vector<double> &y, const std::string & method, const bool &cells);
@@ -676,14 +707,15 @@ class SpatRaster {
 		std::vector<std::vector<double>> freq(bool bylayer, bool round, int digits, SpatOptions &opt);
 		std::vector<size_t> count(double value, bool bylayer, bool round, int digits, SpatOptions &opt);
 		
-		bool get_aggregate_dims(std::vector<unsigned> &fact, std::string &message);
-		std::vector<unsigned> get_aggregate_dims2(std::vector<unsigned> fact);
-		std::vector<std::vector<double> > get_aggregates(std::vector<double> &in, size_t nr, std::vector<unsigned> dim);
-//		std::vector<double> compute_aggregates(std::vector<double> &in, size_t nr, std::vector<unsigned> dim, std::function<double(std::vector<double>&, bool)> fun, bool narm);
+		bool get_aggregate_dims(std::vector<size_t> &fact, std::string &message);
+		std::vector<size_t> get_aggregate_dims2(std::vector<size_t> fact);
+		std::vector<std::vector<double> > get_aggregates(std::vector<double> &in, size_t nr, std::vector<size_t> dim);
+//		std::vector<double> compute_aggregates(std::vector<double> &in, unsigned nr, std::vector<unsigned> dim, std::function<double(std::vector<double>&, bool)> fun, bool narm);
 
 		SpatDataFrame mglobal(std::vector<std::string> funs, bool narm, SpatOptions &opt);
 
 		SpatDataFrame global(std::string fun, bool narm, SpatOptions &opt);
+		SpatDataFrame globalTF(std::string fun, SpatOptions &opt);
 		SpatDataFrame global_weighted_mean(SpatRaster &weights, std::string fun, bool narm, SpatOptions &opt);
 
 		SpatRaster gridDistance(double m, SpatOptions &opt);
@@ -694,9 +726,9 @@ class SpatRaster {
 		SpatRaster init(std::vector<double> values, SpatOptions &opt);
 		
 		SpatRaster is_in(std::vector<double> m, SpatOptions &opt);
-		std::vector<std::vector<double>> is_in_cells(std::vector<double> m, SpatOptions &opt);
+		std::vector<std::vector<double>> is_in_cells(std::vector<double> m, bool keepvalue, SpatOptions &opt);
 
-		std::vector<std::string> getDataType(bool unique);
+		std::vector<std::string> getDataType(bool unique, bool memtype);
 		std::vector<std::string> dataType();
 
 		SpatRaster isnot(bool falseNA, SpatOptions &opt);
@@ -725,8 +757,11 @@ class SpatRaster {
 		SpatExtent ext_from_rc(int_64 r1, int_64 r2, int_64 c1, int_64 c2);
 		SpatExtent ext_from_cell(double cell);
 
-		std::vector<std::string> make_tiles(SpatRaster x, bool expand, bool narm, std::string filename, SpatOptions &opt);
-		std::vector<std::string> make_tiles_vect(SpatVector x, bool expand, bool narm, std::string filename, SpatOptions &opt);
+		std::vector<double> get_tiles_extent(SpatRaster x, bool expand, std::vector<int> buffer);
+		std::vector<std::string> make_tiles(SpatRaster x, bool expand, std::vector<int> buffer, bool narm, std::string filename, SpatOptions &opt);
+		
+		std::vector<double> get_tiles_extent_vect(SpatVector x, bool expand, std::vector<int> buffer);
+		std::vector<std::string> make_tiles_vect(SpatVector x, bool expand, std::vector<int> buffer, bool narm, std::string filename, SpatOptions &opt);
 
 		SpatRaster mask(SpatRaster &x, bool inverse, double maskvalue, double updatevalue, SpatOptions &opt);
 		SpatRaster mask(SpatRaster &x, bool inverse, std::vector<double> maskvalues, double updatevalue, SpatOptions &opt);
@@ -758,7 +793,7 @@ class SpatRaster {
 		std::vector<std::vector<double>> win_circle(std::vector<double> x, std::vector<double> y, std::vector<double> z, std::vector<double> win, SpatOptions &opt);
 		std::vector<std::vector<double>> win_rect(std::vector<double> x, std::vector<double> y, std::vector<double> z, std::vector<double> win, SpatOptions &opt);
 		
-		std::vector<double> rasterizeCells(SpatVector &v, bool touches, SpatOptions &opt);
+		std::vector<double> rasterizeCells(SpatVector &v, bool touches, bool small, SpatOptions &opt);
 		//std::vector<std::vector<double>> rasterizeCellsWeights(SpatVector &v, bool touches);
 		SpatRaster rasterizeGeom(SpatVector x, std::string unit, std::string count, SpatOptions &opt);
 		SpatRaster rasterizePoints(std::vector<double>&x, std::vector<double> &y, std::string fun, std::vector<double> &values, bool narm, double background, SpatOptions &opt);
@@ -770,17 +805,17 @@ class SpatRaster {
 
 		SpatRaster replaceValues(std::vector<double> from, std::vector<double> to, long nl, bool setothers, double others, bool keepcats, SpatOptions &opt);
 		SpatRaster reclassify(std::vector<std::vector<double>> rcl, unsigned openclosed, bool lowest, bool others, double othersValue, bool bylayer, bool brackets, bool keepcats, SpatOptions &opt);
-		SpatRaster reclassify(std::vector<double> rcl, unsigned nc, unsigned openclosed, bool lowest, bool others, double othersValue, bool bylayer, bool brackets, bool keepcats, SpatOptions &opt);
+		SpatRaster reclassify(std::vector<double> rcl, size_t nc, unsigned openclosed, bool lowest, bool others, double othersValue, bool bylayer, bool brackets, bool keepcats, SpatOptions &opt);
 		//SpatRaster classify_layers(std::vector<std::vector<double>> groups, std::vector<double> id, SpatOptions &opt);
-		//SpatRaster classify_layers(std::vector<double> groups, unsigned nc, std::vector<double> id, SpatOptions &opt);
+		//SpatRaster classify_layers(std::vector<double> groups, size_t nc, std::vector<double> id, SpatOptions &opt);
 
 		SpatRaster intersect(SpatRaster &x, SpatOptions &opt);
 		
-		std::vector<double> readSample(unsigned src, size_t srows, size_t scols);
+		std::vector<double> readSample(size_t src, size_t srows, size_t scols);
 		SpatRaster rotate(bool left, SpatOptions &opt);
 
 		std::vector<size_t> sampleCells(double size, std::string method, bool replace, unsigned seed);
-		SpatRaster sampleRegularRaster(double size);
+		SpatRaster sampleRegularRaster(double size, bool overview);
 		SpatRaster sampleRowColRaster(size_t nr, size_t nc, bool warn);
 		SpatRaster sampleRandomRaster(double size, bool replace, unsigned seed);
 		std::vector<std::vector<double>> sampleRegularValues(double size, SpatOptions &opt);
@@ -790,9 +825,20 @@ class SpatRaster {
 		SpatRaster sort(bool decreasing, bool order, SpatOptions &opt);
 
 		SpatRaster scale(std::vector<double> center, bool docenter, std::vector<double> scale, bool doscale, SpatOptions &opt);
+		SpatRaster scale_linear(double smin, double smax, SpatOptions &opt);
+
 		SpatRaster similarity(std::vector<double> x, SpatOptions &opt);
 
 		SpatRaster terrain(std::vector<std::string> v, unsigned neighbors, bool degrees, unsigned seed, SpatOptions &opt);
+
+    // watershed2 ecor 20210317; EC 20210702 
+		SpatRaster watershed2(int pp_offset,SpatOptions &opt); 
+		SpatRaster pitfinder2(SpatOptions &opt); 
+		SpatRaster NIDP2(SpatOptions &opt); 
+		SpatRaster flowAccu2(SpatOptions &opt); 
+		SpatRaster flowAccu2_weight(SpatRaster weight,SpatOptions &opt);
+	// END watershed2 
+		
 		SpatRaster hillshade(SpatRaster aspect, std::vector<double> angle, std::vector<double> direction, bool normalize, SpatOptions &opt);
 
 		SpatRaster selRange(SpatRaster x, int z, int recycleby, SpatOptions &opt);
@@ -805,8 +851,8 @@ class SpatRaster {
 
 		SpatRaster transpose(SpatOptions &opt);
 		SpatRaster trig(std::string fun, SpatOptions &opt);
-		SpatRaster trim1(double value, unsigned padding, SpatOptions &opt);
-		SpatRaster trim2(double value, unsigned padding, SpatOptions &opt);
+		SpatRaster trim1(double value, size_t padding, SpatOptions &opt);
+		SpatRaster trim2(double value, size_t padding, SpatOptions &opt);
 		std::vector<std::vector<double>> unique(bool bylayer, double digits, bool narm, SpatOptions &opt);
 		SpatRaster project1(std::string newcrs, std::string method, SpatOptions &opt);
 		SpatRaster project2(SpatRaster &x, std::string method, SpatOptions &opt);
@@ -815,6 +861,8 @@ class SpatRaster {
 
 #ifdef useGDAL
 		bool getDSh(GDALDatasetH &rstDS, SpatRaster &out, std::string &filename, std::string &driver, double &naval, bool update, double background, SpatOptions &opt);
+		bool getDShMEM(GDALDatasetH &rstDS, SpatRaster &out, double &naval, double background, SpatOptions &opt);
+		
 		bool open_gdal(GDALDatasetH &hDS, int src, bool update, SpatOptions &opt);
 		bool create_gdalDS(GDALDatasetH &hDS, std::string filename, std::string driver, bool fill, double fillvalue, std::vector<bool> has_so, std::vector<double> scale, std::vector<double> offset, SpatOptions& opt);
 		bool from_gdalMEM(GDALDatasetH hDS, bool set_geometry, bool get_values);
@@ -840,12 +888,9 @@ class SpatRaster {
 		SpatDataFrame zonal(SpatRaster z, SpatRaster g, std::string fun, bool narm, SpatOptions &opt);
 		SpatDataFrame zonal_weighted(SpatRaster x, SpatRaster w,  bool narm, SpatOptions &opt);
 
-		SpatDataFrame zonal_poly(SpatVector x, std::string fun, bool weights, bool exact, bool touches, bool narm, SpatOptions &opt);
-		SpatDataFrame zonal_poly_weighted(SpatVector x, SpatRaster w, bool weights, bool exact, bool touches, bool narm, SpatOptions &opt);
-		
-		std::vector<std::vector<double>> zonal_poly_table(SpatVector x, bool weights, bool exact, bool touches,bool narm, SpatOptions &opt);
-
-
+		SpatDataFrame zonal_poly(SpatVector x, std::string fun, bool weights, bool exact, bool touches, bool small, bool narm, SpatOptions &opt);
+		SpatDataFrame zonal_poly_weighted(SpatVector x, SpatRaster w, bool weights, bool exact, bool touches, bool small, bool narm, SpatOptions &opt);
+		std::vector<std::vector<double>> zonal_poly_table(SpatVector x, bool weights, bool exact, bool touches, bool small, bool narm, SpatOptions &opt);
 
 //		SpatDataFrame zonal_old(SpatRaster x, std::string fun, bool narm, SpatOptions &opt);
 		SpatRaster rgb2col(size_t r,  size_t g, size_t b, SpatOptions &opt);
