@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -21,6 +21,7 @@
 #include "math_utils.h"
 #include "vecmath.h"
 #include "recycle.h"
+#include <unordered_map>
 
 #ifdef useGDAL
 	#include "crs.h"
@@ -307,10 +308,20 @@ bool SpatVector::empty() {
 	return geoms.empty();
 }
 
+bool SpatVector::is_multipoint() {
+	if (geoms[0].gtype != points) return false;
+	for (size_t i=0; i<geoms.size(); i++) {
+		if (geoms[i].parts.size() > 1) {
+			return true;
+		}
+	}
+	return false;
+}
+
 bool SpatVector::is_lonlat() {
 	if (srs.is_lonlat()) {
 		SpatExtent e = getExtent();
-		if ((e.xmin < -181) || (e.xmax > 361) || (e.ymin < -90.001) || (e.ymax > 90.001)) {
+		if ((e.xmin < -361) || (e.xmax > 361) || (e.ymin < -90.001) || (e.ymax > 90.001)) {
 			addWarning("coordinates are out of range for lon/lat");
 		}
 		return true;
@@ -886,12 +897,39 @@ void SpatVector::setPointsDF(SpatDataFrame &x, std::vector<unsigned> geo, std::s
 	df = x;
 }
 
+void SpatVector::setLinesStartEnd(std::vector<double> &x, std::string crs) {
+	size_t n = x.size() / 4;
+	if (n == 0) return;
+	size_t n2 = 2 * n;
+	size_t n3 = 3 * n;
+	SpatGeom g;
+	g.gtype = lines;
+	SpatPart p({x[0], x[n]},{x[n2], x[n3]});
+	g.addPart(p);
+	geoms.resize(n, g);
+	for (size_t i=1; i<n; i++) {
+		geoms[i].parts[0].x[0] = x[i];
+		geoms[i].parts[0].x[1] = x[i+n];
+		geoms[i].parts[0].y[0] = x[i+n2];
+		geoms[i].parts[0].y[1] = x[i+n3];
+		geoms[i].extent.xmin = *std::min_element(geoms[i].parts[0].x.begin(), geoms[i].parts[0].x.end());
+		geoms[i].extent.xmax = *std::max_element(geoms[i].parts[0].x.begin(), geoms[i].parts[0].x.end());
+		geoms[i].extent.ymin = *std::min_element(geoms[i].parts[0].y.begin(), geoms[i].parts[0].y.end());
+		geoms[i].extent.ymax = *std::max_element(geoms[i].parts[0].y.begin(), geoms[i].parts[0].y.end());
+	}
+	computeExtent();
+	setSRS( {crs} );
+}
 
-SpatVector SpatVector::subset_rows(std::vector<int> range) {
+
+
+
+SpatVector SpatVector::subset_rows(std::vector<long> range) {
 
 	SpatVector out;
 	int n = nrow();
 	std::vector<unsigned> r;
+	r.reserve(range.size());
 	for (size_t i=0; i<range.size(); i++) {
 		if ((range[i] >= 0) && (range[i] < n)) {
 			r.push_back(range[i]);
@@ -931,10 +969,9 @@ SpatVector SpatVector::subset_rows(std::vector<unsigned> range) {
 
 
 
-SpatVector SpatVector::subset_rows(int i) {
-	std::vector<int> range(1, i);
-	SpatVector out = subset_rows(range);
-	return out;
+SpatVector SpatVector::subset_rows(long i) {
+	std::vector<long> range(1, i);
+	return subset_rows(range);
 }
 
 
@@ -958,8 +995,8 @@ SpatVector SpatVector::remove_rows(std::vector<unsigned> range) {
 
 
 
-SpatVector SpatVector::subset_cols(std::vector<int> range) {
-	int nc = ncol();
+SpatVector SpatVector::subset_cols(std::vector<long> range) {
+	long nc = ncol();
 	std::vector<unsigned> valid;
 	valid.reserve(range.size());
 	for (size_t i=0; i<range.size(); i++) {
@@ -973,14 +1010,14 @@ SpatVector SpatVector::subset_cols(std::vector<int> range) {
 }
 
 
-SpatVector SpatVector::subset_cols(int i) {
+SpatVector SpatVector::subset_cols(long i) {
 	if (i < 0) {
 		SpatVector out;
 		out.geoms = geoms;
 		out.srs = srs;
 		return out;
 	}
-	std::vector<int> range = {i};
+	std::vector<long> range = {i};
 	return subset_cols(range);
 }
 
@@ -1325,12 +1362,14 @@ SpatVector SpatVector::rotate_longitude(double longitude, bool left) {
 		size_t np = out.geoms[i].size();
 		for (size_t j=0; j<np; j++) {
 			size_t nx = out.geoms[i].parts[j].x.size();
-			for (size_t k=0; k<nx; k++) {
-				if (left) {
+			if (left) {
+				for (size_t k=0; k<nx; k++) {
 					if (out.geoms[i].parts[j].x[k] > longitude) {
 						out.geoms[i].parts[j].x[k] = out.geoms[i].parts[j].x[k] - 360;
 					}
-				} else {
+				}
+			} else {
+				for (size_t k=0; k<nx; k++) {
 					if (out.geoms[i].parts[j].x[k] < longitude) {
 						out.geoms[i].parts[j].x[k] = out.geoms[i].parts[j].x[k] + 360;
 					}
@@ -1340,12 +1379,14 @@ SpatVector SpatVector::rotate_longitude(double longitude, bool left) {
 				size_t nh = out.geoms[i].parts[j].holes.size();
 				for (size_t k=0; k<nh; k++) {
 					size_t nx = out.geoms[i].parts[j].holes[k].x.size();
-					for (size_t h=0; h<nx; h++) {
-						if (left) {
+					if (left) {
+						for (size_t h=0; h<nx; h++) {
 							if (out.geoms[i].parts[j].holes[k].x[h] > longitude) {
 								out.geoms[i].parts[j].holes[k].x[h] = out.geoms[i].parts[j].holes[k].x[h] - 360;
 							} 
-						} else {
+						}
+					} else {
+						for (size_t h=0; h<nx; h++) {
 							if (out.geoms[i].parts[j].holes[k].x[h] < longitude) {
 								out.geoms[i].parts[j].holes[k].x[h] = out.geoms[i].parts[j].holes[k].x[h] + 360;
 							} 
