@@ -24,7 +24,6 @@
 #include "gdal_priv.h"
 #endif
 
-
 enum SpatGeomType { points, lines, polygons, null};
 
 
@@ -144,6 +143,7 @@ class SpatVector {
 		bool could_be_lonlat();
 		std::string type();
 		SpatGeomType getGType(std::string &type);
+		bool is_multipoint();
 
 		//std::vector<std::string> getCRS();
 		//void setCRS(std::vector<std::string> _crs);
@@ -154,6 +154,7 @@ class SpatVector {
 				addWarning("Cannot set SRS to vector: "+ msg);
 				return false;
 			}
+			source = "";
 			return true;	
 		}
 
@@ -178,30 +179,35 @@ class SpatVector {
 		SpatVector project(std::string crs, bool partial);
 		std::vector<double> project_xy(std::vector<double> x, std::vector<double> y, std::string fromCRS, std::string toCRS);
 
-		SpatVector subset_cols(int i);
-		SpatVector subset_cols(std::vector<int> range);
-		SpatVector subset_rows(int i);
-		SpatVector subset_rows(std::vector<int> range);
+		SpatVector subset_cols(long i);
+		SpatVector subset_cols(std::vector<long> range);
+		SpatVector subset_rows(long i);
+		SpatVector subset_rows(std::vector<long> range);
 		SpatVector subset_rows(std::vector<unsigned> range);
 		SpatVector remove_rows(std::vector<unsigned> range);
 
 		void setGeometry(std::string type, std::vector<unsigned> gid, std::vector<unsigned> part, std::vector<double> x, std::vector<double> y, std::vector<unsigned> hole);
 		void setPointsGeometry(std::vector<double> &x, std::vector<double> &y);
 		void setPointsDF(SpatDataFrame &x, std::vector<unsigned> geo, std::string crs, bool keepgeom);
+		void setLinesStartEnd(std::vector<double> &x, std::string crs);
 
 		std::vector<double> area(std::string unit, bool transform, std::vector<double> mask);
 
 		void reserve(size_t n);
 		std::vector<double> length();
-		std::vector<double> distance(SpatVector x, bool pairwise, std::string unit);
-		std::vector<double> pointdistance(const std::vector<double>& px, const std::vector<double>& py, const std::vector<double>& sx, const std::vector<double>& sy, bool pairwise, double m, bool lonlat);
+		std::vector<double> nseg();
+
+		std::vector<double> distance(bool sequential, std::string unit, const std::string method);
+		std::vector<double> distance(SpatVector x, bool pairwise, std::string unit, const std::string method);
+		std::vector<double> pointdistance(const std::vector<double>& px, const std::vector<double>& py, const std::vector<double>& sx, const std::vector<double>& sy, bool pairwise, double m, bool lonlat, std::string method);
 
 //		std::vector<double> pointdistance_seq(const std::vector<double>& px, const std::vector<double>& py, double m, bool lonlat);
 
+//		std::vector<std::vector<double>> get_index(SpatVector &p);
 
-		std::vector<double> distance(bool sequential, std::string unit);
-		std::vector<double> linedistLonLat(SpatVector x);
-
+		std::vector<double> distLonLat(SpatVector p, std::string unit, std::string method, bool transp);
+//		std::vector<double> distLonLat(std::vector<double> x, std::vector<double> y, std::string unit, std::string method, bool transp);
+		//std::vector<double> nearestDistLonLat(std::vector<double> x, std::vector<double> y, std::string unit, std::string method);
 		std::vector<std::vector<size_t>> knearest(size_t k);
 
 		size_t size();
@@ -213,7 +219,7 @@ class SpatVector {
 		SpatVector set_holes(SpatVector x, size_t i);
 		SpatVector remove_duplicate_nodes(int digits);
 		
-		bool read(std::string fname, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what);
+		bool read(std::string fname, std::string layer, std::string query, std::vector<double> ext, SpatVector filter, bool as_proxy, std::string what, std::vector<std::string> options);
 		
 		bool write(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string>);
 
@@ -222,7 +228,7 @@ class SpatVector {
 #ifdef useGDAL
 		GDALDataset* write_ogr(std::string filename, std::string lyrname, std::string driver, bool append, bool overwrite, std::vector<std::string> options);
 		GDALDataset* GDAL_ds();
-		bool read_ogr(GDALDataset *&poDS, std::string layer, std::string query, std::vector<double> extent, SpatVector filter, bool as_proxy, std::string what);
+		bool read_ogr(GDALDataset *&poDS, std::string layer, std::string query, std::vector<double> ext, SpatVector filter, bool as_proxy, std::string what);
 		SpatVector fromDS(GDALDataset *poDS);
 		bool ogr_geoms(std::vector<OGRGeometryH> &ogrgeoms, std::string &message);		
 		bool delete_layers(std::string filename, std::vector<std::string> layers, bool return_error);		
@@ -312,6 +318,8 @@ class SpatVector {
 		std::vector<std::string> wkt();
 		std::vector<std::string> wkb();
 		std::vector<std::string> hex();
+		std::vector<std::vector<unsigned char>> wkb_raw(); 
+		
 		SpatVector from_hex(std::vector<std::string> x, std::string srs);
 		SpatVector make_nodes();
 		SpatVector polygonize();
@@ -324,7 +332,7 @@ class SpatVector {
 		SpatVector snap(double tolerance);
 		SpatVector snapto(SpatVector y, double tolerance);
 		SpatVector thin(double threshold);
-
+		SpatVector split_lines(SpatVector v);
 		SpatVector allerretour();
 		SpatVectorCollection bienvenue();
 		SpatVector aggregate(bool dissolve);
@@ -343,12 +351,16 @@ class SpatVector {
 
 		SpatVector centroid(bool check_lonlat);
 		SpatVector point_on_surface(bool check_lonlat);
+		std::vector<int> pointInPolygon(std::vector<double> &x, std::vector<double> &y);
 
 		SpatVector crop(SpatExtent e, bool wrap);
 		SpatVector crop(SpatVector e);
 		SpatVector voronoi(SpatVector bnd, double tolerance, int onlyEdges);		
-		SpatVector delaunay(double tolerance, int onlyEdges);		
-		SpatVector hull(std::string htype, std::string by="");
+		SpatVector voronoi_sphere(SpatVector bnd, double tolerance, int onlyEdges);
+
+		SpatVector delaunay(double tolerance, int onlyEdges, bool constrained=false);		
+		SpatVector hull(std::string htype, std::string by="", double param=1, bool allowHoles=true, bool tight=true);
+
 		SpatVector intersect(SpatVector v, bool values);
 		SpatVector unite(SpatVector v);
 		SpatVector unite();
@@ -375,11 +387,11 @@ class SpatVector {
 		std::vector<unsigned> equals_exact(SpatVector v, double tol);
 		std::vector<unsigned> equals_exact(bool symmetrical, double tol);
 
-		std::vector<double> geos_distance(SpatVector v, bool parallel, std::string fun);
-		std::vector<double> geos_distance(bool sequential, std::string fun);
+		std::vector<double> geos_distance(SpatVector v, bool parallel, std::string fun, double m);
+		std::vector<double> geos_distance(bool sequential, std::string fun, double m);
 
-		SpatVector nearest_point(SpatVector v, bool parallel);
-		SpatVector nearest_point();
+		SpatVector nearest_point(SpatVector v, bool parallel, const std::string method);
+		SpatVector nearest_point(const std::string method);
 		std::vector<int> nearest_geometry(SpatVector v);
 		
 		SpatVector sample(unsigned n, std::string method, unsigned seed);
