@@ -26,6 +26,10 @@
 #include "NA.h"
 #include <math.h>
 
+#include <map>
+#include <algorithm>
+#include <unordered_map>
+
 
 bool haveFun(std::string fun);
 std::function<double(std::vector<double>&, bool)> getFun(std::string fun);
@@ -67,7 +71,7 @@ static inline std::vector<double> vquantile(std::vector<double> v, const std::ve
         [](const double& value) { return std::isnan(value); }),
         std::end(v));
 
-	if (((!narm) && (v.size() < n)) || (v.size() == 0)) {
+	if (((!narm) && (v.size() < n)) || v.empty()) {
 		return std::vector<double>(probs.size(), NAN);
 	}
 
@@ -498,17 +502,24 @@ T vall(const std::vector<T>& v, bool narm) {
 template <typename T>
 T vany(const std::vector<T>& v, bool narm) {
 	T x = NA<T>::value;
-	for (size_t i=0; i<v.size(); i++) {
-		if (!is_NA(v[i])) {
-			if (v[i] != 0) {
+	x = 0;
+	if (narm) {
+		for (size_t i=0; i<v.size(); i++) {
+			if (!is_NA(v[i])) {
+				if (v[i] != 0) {
+					x = 1;
+					break;
+				} 
+			}
+		}
+	} else {
+		for (size_t i=0; i<v.size(); i++) {
+			if (is_NA(v[i])) {
+				x = NA<T>::value;
+				break;				
+			} else if (v[i] != 0) {
 				x = 1;
-				break;
-			} else {
-				x = 0;				
 			}
-		} else if (!narm) {
-			x = NA<T>::value;
-			break;
 		}
 	}
 	return x;
@@ -552,7 +563,7 @@ std::vector<T> vrange(const std::vector<T>& v, bool narm) {
 
 
 template <typename T>
-T vmodal(std::vector<T>& v, bool narm) {
+T vmodal_old(std::vector<T>& v, bool narm) {
 
 	size_t n = v.size();
     std::vector<unsigned> counts(n, 0);
@@ -579,6 +590,45 @@ T vmodal(std::vector<T>& v, bool narm) {
 }
 
 
+template <typename T>
+T vmodal(std::vector<T>& v, bool narm) {
+
+	if (narm) {
+		std::map<double, size_t> count;
+		for_each( v.begin(), v.end(), [&count]( double val ){
+				if(!std::isnan(val)) count[val]++;
+			}
+		);
+		if (count.size() == 0) return NAN;
+		
+		std::map<double, size_t>::iterator mode =	
+			std::max_element(count.begin(), count.end(),[] (const std::pair<double, size_t>& a, 
+			const std::pair<double, size_t>& b)->bool{ return a.second < b.second; } );
+			
+		return mode->first;
+
+	}  else {
+	
+		std::map<double, size_t> count;
+		for(size_t i=0; i<v.size(); i++) {
+			if (std::isnan(v[i])) {
+				return NAN;
+			} else {
+				count[v[i]]++;
+			}
+		}
+
+		std::map<double, size_t>::iterator mode =	
+			std::max_element(count.begin(), count.end(),[] (const std::pair<double, size_t>& a, 
+			const std::pair<double, size_t>& b)->bool{ return a.second < b.second; } );
+			
+		return mode->first;
+	}
+	}
+
+
+
+
 
 template <typename T>
 std::vector<bool> visna(const std::vector<T>& v) {
@@ -604,6 +654,25 @@ std::vector<bool> visnotna(const std::vector<T>& v) {
 }
 
 
+template <typename T>
+bool vany_notna(const std::vector<T>& v) {
+	for (size_t i=0; i<v.size(); i++) {
+		if (!is_NA(v[i])) {
+			return true;
+		}
+	}
+	return false;
+}
+
+template <typename T>
+bool vany_na(const std::vector<T>& v) {
+	for (size_t i=0; i<v.size(); i++) {
+		if (is_NA(v[i])) {
+			return true;
+		}
+	}
+	return false;
+}
 
 
 template <typename T>
@@ -706,6 +775,40 @@ std::vector<size_t> order(const std::vector<T> &v) {
 */
 
 
+template <typename T>
+double expH(std::vector<T> d, bool narm) {
+  
+	std::unordered_map<T, unsigned int> counts;
+	double s = 0;
+	if (narm) {
+		for (int v : d) {
+			if (!is_NA(v)) {
+				counts[v]++;
+				s++;
+			}
+		}
+	} else {
+		for (int v : d) {
+			if (is_NA(v)) {
+				return NAN;
+			} else {
+				counts[v]++;
+				s++;
+			}
+		}
+	}
+	if (s == 0) {
+		return NAN;
+	} 
+
+	double sump = 0;
+	for (auto const& pair : counts) { 
+		double p = pair.second / s;
+		sump += p * log(p);
+	}
+  
+	return exp(-sump);
+}
 
 #endif
 
