@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2023  Robert J. Hijmans
+// Copyright (c) 2018-2025  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -64,7 +64,7 @@ bool SpatRaster::write_aux_json(std::string filename) {
 		if (f.is_open()) {
 			f << "{" << std::endl;
 			if (wtime) {
-				std::vector<std::string> tstr = getTimeStr(false);
+				std::vector<std::string> tstr = getTimeStr(false, " ");
 				std::string ss = quoted_csv(tstr);
 				f << "\"time\":[" << ss << "]," << std::endl;
 				f << "\"timestep\":\"" << source[0].timestep << "\"";
@@ -287,6 +287,9 @@ bool checkFormatRequirements(const std::string &driver, std::string &filename, s
 			msg = "SAGA filenames must end on '.sdat'";
 			return false;
 		}
+	} else if (driver == "VRT") {
+		msg = "Cannot directly write to VRT (see '?vrt')";
+		return false;
 	}
 
 	return true;
@@ -351,6 +354,13 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 		setError("invalid driver");
 		return (false);
 	}
+	if (driver == "GTiff") {
+		if (nlyr() > 65535) {
+			setError("cannot write more than 65535 layers");
+			return(false);
+		}
+	}
+	
     char **papszMetadata;
     papszMetadata = poDriver->GetMetadata();
     if (!CSLFetchBoolean( papszMetadata, GDAL_DCAP_RASTER, FALSE)) {
@@ -410,7 +420,7 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 			if (is_ratct(source[0].cats[0].d)) {
 				std::fill(hasCT.begin(), hasCT.end(), false);
 			} else if (ct[0].nrow() < 256) {
-				if (opt.datatype_set && (datatype != opt.get_datatype())) {
+				if (opt.datatype_set && (datatype != "INT1U")) {
 					addWarning("change datatype to INT1U to write the color-table");
 				} else {
 					datatype = "INT1U";
@@ -539,8 +549,9 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 		std::vector <std::string> files;
 		if (filelist != NULL) {
 			for (size_t i=0; filelist[i] != NULL; i++) {
-				files.push_back(filelist[i]);
-				std::replace( files[i].begin(), files[i].end(), '\\', '/');
+				std::string thefile = filelist[i];
+				std::replace( thefile.begin(), thefile.end(), '\\', '/');
+				files.push_back(thefile);
 			}
 		}
 		CSLDestroy( filelist );
@@ -570,6 +581,13 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 	std::vector<std::string> nms = getNames();
 	double naflag=NAN;
 	bool hasNAflag = opt.has_NAflag(naflag);
+	
+//	if (driver == "AAIGrid" && std::isnan(naflag)) {
+		// avoid nan as flag
+//		naflag = -3.40282347E+38;
+//		hasNAflag = true;
+// set opt.NAflag?
+//	}
 
 	if (writeRGB) nms = {"red", "green", "blue"};
 
@@ -599,13 +617,29 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 
 	bool scoffwarning = false;
 
-	if (driver == "GTiff") {
+	// if (driver == "GTiff") {
 		std::vector<std::string> m = getTags();
 		if (m.size() > 0) {
 			for (size_t i=0; i<m.size(); i+=2) {
 				poDS->SetMetadataItem(m[i].c_str(), m[i+1].c_str(), "USER_TAGS");
 			}
 		}
+	// }
+
+	std::vector<std::string> tstr, ustr;
+	bool wtime = false;
+	bool have_date_time=false;
+	std::string tstep = getTimeStep();
+	if (hasTime()) {
+		tstr = getTimeStr(false, "T");
+		wtime = true;	
+		have_date_time = true;
+//		have_date_time = (tstep == "seconds") || (tstep == "days") || (tstep == "years") || (tstep == "yearmonths");
+ 	}
+	bool wunit = false;
+	if (hasUnit()) {
+		ustr = getUnit();
+		wunit = true;
 	}
 	
 	for (size_t i=0; i < nlyr(); i++) {
@@ -645,14 +679,25 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 		*/
 		poBand->SetDescription(nms[i].c_str());
 
-		if (driver == "GTiff") {
+		// if (driver == "GTiff") {
 			std::vector<std::string> m = getLyrTags({i});
 			if (m.size() > 0) {
-				for (size_t i=0; i<m.size(); i+=2) {
-					poBand->SetMetadataItem(m[i].c_str(), m[i+1].c_str());
+				for (size_t j=0; j<m.size(); j+=3) {
+					poBand->SetMetadataItem(m[j+1].c_str(), m[j+2].c_str(), "USER_TAGS");
 				}
 			}
-		}
+			if (wtime) {
+				if (have_date_time) {
+					poBand->SetMetadataItem("DATE_TIME", tstr[i].c_str());				
+				} else {
+					poBand->SetMetadataItem("TIMESTAMP", tstr[i].c_str());									
+					poBand->SetMetadataItem("TIMEUNIT", tstep.c_str());									
+				}
+			}
+			if (wunit) {
+				poBand->SetMetadataItem("UNIT", ustr[i].c_str());				
+			}
+		// }
 
 
 		if ((i==0) || (driver != "GTiff")) {
@@ -704,7 +749,7 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 			if (source[0].has_scale_offset[i]) {
 				bool failed = (poBand->SetScale(scale[i])) != CE_None;
 				if (!failed) {
-					failed = ((poBand->SetOffset(offset[i])) != CE_None);
+					failed = (poBand->SetOffset(offset[i])) != CE_None;
 				}
 				if (failed) {
 					source[0].has_scale_offset[i] = false;
@@ -756,8 +801,8 @@ bool SpatRaster::writeStartGDAL(SpatOptions &opt, const std::vector<std::string>
 	source[0].driver = "gdal" ;
 	source[0].filename = filename;
 	source[0].memory = false;
-	write_aux_json(filename);
 
+	if (driver != "GTiff") write_aux_json(filename);
 /*
 	if (append) {
 		GDALClose( (GDALDatasetH) poDS );
@@ -1105,7 +1150,6 @@ bool SpatRaster::update_meta(bool names, bool crs, bool ext, SpatOptions &opt) {
 		return false;
 	}
 	GDALDatasetH hDS;
-	GDALRasterBandH poBand;
 	size_t n=0;
 	for (size_t i=0; i<nsrc(); i++) {
 		if (source[i].memory) continue;
@@ -1116,10 +1160,12 @@ bool SpatRaster::update_meta(bool names, bool crs, bool ext, SpatOptions &opt) {
 		}
 		if (names) {
 			for (size_t b=0; b < source[i].nlyr; b++) {
-				poBand = GDALGetRasterBand(hDS, b+1);
-				GDALSetDescription(poBand, source[i].names[b].c_str());
+				GDALRasterBandH poBand = GDALGetRasterBand(hDS, b+1);
+				if (GDALGetRasterAccess(poBand) == GA_Update) {
+					GDALSetDescription(poBand, source[i].names[b].c_str());
+				}
 			}
-		} 
+		}
 		if (crs) {
 			std::string crs = source[i].srs.wkt;
 			OGRSpatialReference oSRS;
